/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: AgentException Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: AgentExceptions.h,v 1.4 2007/06/11 08:17:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_AGENTEXCEPTIONS_H_
#define GLITE_DATA_AGENTS_AGENTEXCEPTIONS_H_

#include <stdexcept>

namespace glite {
namespace data  {
namespace agents{
    
//------------------------------------------------------------------------------
// Base class
//------------------------------------------------------------------------------
/**
 * Base class for all the Exception in the Agents
 */    
class AgentException : public std::exception{
public:
    /**
     * Constructor
     */
    explicit AgentException(
        const std::string& reason):m_reason(reason){}
    /**
     * Destructor
     */
    virtual ~AgentException() throw(){}  
        
    /** 
     * Returns a C-style character string describing the general cause of
     *  the current error (the same string passed to the ctor). 
     */
    virtual const char* what() const throw(){
        try{
            return m_reason.c_str();
        }catch(...){
            return "";
        }
    }
    
    /** 
     * Returns a C-style character string describing the exception type. 
     */
    virtual const char* type() const throw(){
        return "AgentException";
    }
    
private:
    /**
     * Not Implemented
     */
    AgentException();
    /**
     * Error Reason
     */
    std::string m_reason;
};

//------------------------------------------------------------------------------
// Logic Errors
//------------------------------------------------------------------------------

/**
 * Exception Class for reportinig Logic Errors in the Agent
 *
 * A Logic Error is an error that is detectable before the program executes, 
 * such as violations of logical preconditions. Usually a Logic Error represent 
 * a bug in the code and it is quite difficult for the caller code to handle them
 */    
struct LogicError : public AgentException{
    /**
     * Constructor
     */
    explicit LogicError(
        const std::string& reason):AgentException(reason){}
    /**
     * Destructor
     */
    virtual ~LogicError() throw(){}
        
    /** 
     * exception type. 
     */
    virtual const char* type() const throw(){
        return "LogicError";
    }

};

/**
 * Invalid Argument Exception. This exception can be used when preconditions 
 * on arguments are not satisfied
 */
struct InvalidArgumentException : public LogicError{
    /**
     * Constructor
     */
    explicit InvalidArgumentException(
        const std::string& reason):LogicError(reason){}
    /**
     * Constructor
     */
    InvalidArgumentException(const std::string& method,
                             const std::string& arg, 
                             const std::string& reason);
    /**
     * Destructor
     */
    virtual ~InvalidArgumentException() throw(){}  
        
    /** 
     * exception type. 
     */
    virtual const char* type() const throw(){
        return "InvalidArgumentException";
    }
        
};

/**
 * Null Pointer Exception
 *
 * This exception can be used to signal that the a pointer is invalid 
 */
struct NullPointerException : public LogicError{
    /**
     * Constructor
     */
    explicit NullPointerException(
        const std::string& reason):LogicError(reason){}
    /**
     * Destructor
     */
    virtual ~NullPointerException() throw(){}
    /** 
     * exception type. 
     */
    virtual const char* type() const throw(){
        return "NullPointerException";
    }
};

/**
 * Configuration Exception. This exception can be to signal error during 
 * the configuration 
 */
struct ConfigurationException : public LogicError{
    /**
     * Constructor
     */
    explicit ConfigurationException(
        const std::string& reason):LogicError(reason){}
    /**
     * Constructor
     */
    ConfigurationException(const std::string& component,
                           const std::string& reason);
    /**
     * Destructor
     */
    virtual ~ConfigurationException() throw(){}  
        
    /** 
     * exception type. 
     */
    virtual const char* type() const throw(){
        return "ConfigurationException";
    }
};

/**
 * ConfigParam Exception. This exception can be to signal error during 
 * the configuration 
 */
struct ConfigParamException : public ConfigurationException{
    /**
     * ConfigParamException ErrorType 
     */
    typedef enum{
        MISSING = 0,
        INVALID,
        INVALID_VALUE
    } ErrorType;
    
    /**
     * Constructor
     */
    explicit ConfigParamException(
        const std::string& reason):ConfigurationException(reason){}
    /**
     * Constructor
     */
    ConfigParamException(const std::string& component,
                         const std::string& param_name, 
                               ErrorType    t);
    /**
     * Destructor
     */
    virtual ~ConfigParamException() throw(){}  
        
    /** 
     * exception type. 
     */
    virtual const char* type() const throw(){
        return "ConfigParamException";
    }
};

//------------------------------------------------------------------------------
// Runtime Errors
//------------------------------------------------------------------------------

/**
 * Exception Class for reportinig Runtime Errors in the Agents 
 *
 * A Runtime Error is an error that is detectable only when the program executes
 */
struct RuntimeError : public AgentException{
    /**
     * Constructor
     */
    explicit RuntimeError(
        const std::string& reason):AgentException(reason){}
    /**
     * Destructor
     */
    virtual ~RuntimeError() throw(){}
    /** 
     * exception type. 
     */
    virtual const char* type() const throw(){
        return "RuntimeError";
    }
};

/**
 * Exception Class for reportinig posix Errors 
 *
 * A Runtime Error after a posix call
 */
struct PosixError : public RuntimeError{
    /**
     * Constructor
     */
    PosixError(const std::string& reason, int error);
    /**
     * Destructor
     */
    virtual ~PosixError() throw(){} 
    /**
     * Retrun the posix Errno
     */
    int error() const {return m_error;} 
    /** 
     * exception type. 
     */
    virtual const char* type() const throw(){
        return "PosixError";
    }
private:
    /**
     * The posix Errno
     */
    int m_error; 
};

/**
 * Exception Class for reportinig Execution Errors in the Agents
 *
 * An Execution error can be used to report an error during the execution of a 
 * function. Execution error represent allowed failure conditions that normally 
 * can be handled by the caller code
 */
struct ExecutionError : public RuntimeError{
    /**
     * Constructor
     */
    explicit ExecutionError(
        const std::string& reason):RuntimeError(reason){}
    /**
     * Destructor
     */
    virtual ~ExecutionError() throw(){}  
    /** 
     * exception type. 
     */
    virtual const char* type() const throw(){
        return "ExecutionError";
    }
};

} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_AGENTEXCEPTIONS_H_
