/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Scheduler Agents: Agent Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ActiveObject.h,v 1.6 2008/11/14 16:04:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_EXTRA_ACTIVEOBJECT_H_
#define GLITE_DATA_AGENTS_EXTRA_ACTIVEOBJECT_H_

#include <signal.h>
#include <pthread.h>
#include <map>

#include <boost/utility.hpp>
#include <boost/thread/mutex.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite {
namespace data  {
namespace agents{
     
/**
 * ActiveObject Class. The purpose of this class is to define a class which main
 * function is executed in a dedicated thread
 */
class ActiveObject : boost::noncopyable {
public:

    typedef pthread_t id_type;

    /**
     * Constructor
     * @param name [IN] The Object name
     * @param stop_timeout [IN] The timeout to use to stop the Object. -1 means 
     * default, 0 means infinite
     */
    explicit ActiveObject(const std::string& name, long stop_timeout = -1);
    
    /**
     * Destructor
     */
    virtual ~ActiveObject();

    /**
     * Return ActiveObject Identifier
     */
    id_type id() const { return m_id;}

    /**
     * Start the ActiveObject
     * @return true if the ActiveObject is successfully started
     */
    bool start();

    /**
     * Stop the ActiveObject
     */
    void stop();

protected:

    /**
     * Object Main Function (Template Pattern)
     */
    virtual void svc() throw() = 0;

    /**
     * Handle Event Start. This method would be called on the ActiveObject thread
     * when the Object  is started
     */
    virtual bool onEventStart() throw() {return true;}
    /**
     * Handle Event Stop. This method would be called on the the ActiveObject thread
     * when the Object is going to stop
     */
    virtual bool onEventStop() throw() {return true;}

    /**
     * Derived objects can execute cutom action in order to terminate the 
     * svc function
     */
    virtual void terminate() throw() {}

    /**
     * Check if the Active Object should still run. Derived Classes should call 
     * this method in order to check if the main thread function should be 
     * terminated. This method should be called only in the ActiveObject own 
     * Thread
     */
    bool doRun() const throw() {return m_run;}

    /**
     * Retrun true if the thread is started
     */
    bool isStarted() const throw() {return m_started;}

    /**
     * Enter an Execution Critical Section: the stop signals is suspended until
     * the section is left
     */
    void enterCritical() throw();

    /**
     * Leave an Execution Critical Section
     */
    void leaveCritical() throw();
    
    /**
     * CriticalLock Class. Helperclass for handling the Execution critical 
     * sections
     */
    class CriticalLock : boost::noncopyable {
    public:
        /**
         * Constructor
         */
        explicit CriticalLock(ActiveObject& obj):m_obj(obj),m_acquired(true){
            m_obj.enterCritical();
        }
        /**
         * Release lock
         */
        void release(){
            if(true == m_acquired){
                m_obj.leaveCritical();
                m_acquired = false;
            }
        }
        /**
         * Destructor
         */
        ~CriticalLock(){
            if(true == m_acquired){
                m_obj.leaveCritical();
                m_acquired = false;
            }
        }
    private:
        /**
         * The Object
         */
        ActiveObject& m_obj;
        /**
         * Acquired Flag
         */
        bool m_acquired;
    };
    
private:
    
    friend void * glite::data::agents::object_main(void * data);
    friend void glite::data::agents::stop_object(int signum);
    friend void glite::data::agents::term_object(int signum);
    
    /**
     * Service Main Method
     */
    void run();

    /**
     * Do Init Action
     */
    bool doInit() /* throw () */;
    
    /**
     * Do Fini Action
     */
    bool doFini() /* throw () */;

    /**
     * Stop the service
     */
    void signalStop() /* throw () */;

    /**
     * Terminate the service
     */
    void signalTerm() /* throw () */;

    /**
     * Register an Object
     */
    static void registerObject(ActiveObject* obj);
    /**
     * Deregister an Object
     */
    static void deregisterObject(ActiveObject* obj);
    /**
     * Return an Object
     */
    static ActiveObject * get(id_type id);

private:
    
    /**
     * Default Constructor. Not Implemented
     */
    ActiveObject();

protected:
    
    /**
     * The Logger
     */
    mutable glite::config::Logger m_logger;
    
private:
    
    /**
     * ActiveObject Identifier Property
     */
    id_type m_id;

    /**
     * The Stop Timeout
     */
    unsigned long m_stopTimeout;

    /**
     * Started Flag
     */
    bool m_started;

    /**
     * Action Scheduler Conditional variable for stopping the thread
     */
    pthread_cond_t m_cond;
    /*
     * Lock for Coniditions
     */
    pthread_mutex_t m_lock;
    
    /**
     * Run Flag. Not synchronized since it's used only during the Actve Object 
     * Thread
     */
    bool m_run;
        
    /**
     * Object Map
     */
    static std::map<id_type,ActiveObject *> s_objects;
    
    /**
     * Object Map Lock
     */
    static boost::mutex s_mutex;
};

} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_EXTRA_ACTIVEOBJECT_H_
