/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: TSS Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TSS.h,v 1.3 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_EXTRA_TSS_H_
#define GLITE_DATA_AGENTS_EXTRA_TSS_H_

#include <pthread.h>
#include <boost/utility.hpp>
#include <boost/thread/mutex.hpp>
#include "glite/data/agents/AgentExceptions.h"

namespace glite {
namespace data  {
namespace agents{
     
/**
 * TSS Class. The purpose of this class is to define a class that allows to 
 * store Thread Specific instaces
 */
template<typename T>
class TSS : boost::noncopyable {
public:

    /**
     * Constructor
     */
    TSS();
    
    /**
     * Destructor
     */
    ~TSS();

    /**
     * Register a Thread Specific Object for this type. Please not that this 
     * method will not modify the ownership of the object
     */
    void set(T*) /*throw (LogicError)*/;

    /**
     * Access the Thread Specific Object.
     * @throws LogicError is no thread-specific object are registered for that 
     * thread
     */
    T& get() const /*throw (LogicError)*/;

private:
    
    /**
     * Initialize the Key if not yet initialized
     */
    void init_key() /*throw (LogicError)*/;

    /**
     * Check if the Key is initialized
     */
    void check_key() const /*throw (LogicError)*/;

private:

    typedef pthread_key_t key_type;

    /**
     * TSS key property.
     */
    key_type m_key;

    /**
     * The *First time in* flag (for double-checked locking)
     */
    int m_once;

    /**
     * The nutex used to avoid Race conditions during initialization
     */
    mutable boost::mutex m_mutex;
};

} // end namespace agents
} // end namespace data
} // end namespace glite

#include "glite/data/agents/extra/TSS.inl"

#endif //GLITE_DATA_AGENTS_EXTRA_TSS_H_
