/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: TSS Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TSS.inl,v 1.4 2008/01/07 16:58:29 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
namespace glite {
namespace data  {
namespace agents{
     
/*
 * TSS
 *
 * Constructor
 */
template<typename T>
TSS<T>::TSS():m_key(0),m_once(0){
}
    
/*
 * ~TSS
 *
 * Destructor
 */
template<typename T>
TSS<T>::~TSS(){
    // Don't do anything. Removing the key is expensive (need to keep track of 
    // all the threads that are using the key) 
}

/*
 * init_key
 *
 * Create the key identifiyng the Thread Specific Object 
 */
template<typename T>
void TSS<T>::init_key() /*throw (LogicError)*/{
    // Use Double-Checked locking pattern to avoid locking except during the 
    // initialization
    if(0 == m_once){
        // Acquire the lock in order ot serialize initializations
        boost::mutex::scoped_lock scoped_lock(m_mutex);
        // Double check
        if(0 == m_once){
            int r = pthread_key_create(&m_key,0);
            if(0 != r){
                throw LogicError("Too many keys used");
            }
            // Set the flag
            m_once = 1;
        }
        // Lock Released
    }
}

/*
 * check_key
 *
 * Register a Thread Specific Object for this type. Please not that this 
 * method will not modify the ownership of the object
 */
template<typename T>
void TSS<T>::check_key() const /*throw (LogicError)*/{
    if(0 == m_once){
        throw LogicError("Not Initialized");
    }
}

/*
 * set
 *
 * Register a Thread Specific Object for this type. Please not that this 
 * method will not modify the ownership of the object
 */
template<typename T>
void TSS<T>::set(T* data) /*throw (LogicError)*/{
    // Initialize the Key
    init_key();
    
    // Set the Thread-specific Data (eventually override the previos value)
    int r = pthread_setspecific(m_key,(void*)data);
    if(0 != r){
        throw LogicError("Invalid key");
    }
}

/*
 * get
 *
 * Access the Thread Specific Object.
 */
template<typename T>
T& TSS<T>::get() const /*throw (LogicError)*/{
    // Check if the Key is initialized
    check_key();

    // Get the Thread-specific Data
    T* tss_data = reinterpret_cast<T*>(pthread_getspecific(m_key));
    if(0 == tss_data){
        throw LogicError("Not Initialized");
    }
    return *tss_data;
}

} // end namespace agents
} // end namespace data
} // end namespace glite
