/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: Extra Utility functions
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TempFile.h,v 1.4 2007/04/23 13:40:05 szamsu Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_EXTRA_TEMPFILE_H_
#define GLITE_DATA_AGENTS_EXTRA_TEMPFILE_H_

#include "glite/data/agents/AgentExceptions.h"
#include <string>
#include <boost/utility.hpp>
#include <errno.h>
#include <stdio.h> 
#include <string.h> 
 

namespace glite {
namespace data  {
namespace agents{
   
/**
 * TempFile class. The purpose of this class is to unlink a temporary file 
 * when this object goes out of scope
 */	
class TempFile: boost::noncopyable{
public:
	
    /**
     * Constructor
     */
    explicit TempFile(const std::string& fname) throw ():m_filename(fname){
    }
	 
    /**
     * Destructor
     */
    ~TempFile() throw () {
        unlink();
    }
 
    /**
     * Return The Filename
     */
    const std::string& name() const throw() {return m_filename;}
    
    /**
     * Rename the file and release the owership
     */
    void rename(const std::string& name) /*throw(LogicError,RuntimeError)*/ {
        // Check Preconditions
        if(m_filename.empty()){
            throw LogicError("empty TempFile name");
        }
        if(name.empty()){
            throw InvalidArgumentException("empty destination name");
        }
        // Rename File
        int r = ::rename(m_filename.c_str(),name.c_str());
        if(0 != r){
            std::string reason = (std::string)"Cannot rename temporary file. Error is: " +
                strerror(errno);
            throw RuntimeError(reason);
        }
        // Release Ownership
        this->release();
    }
    
    /**
     * Release the ownership without unlinking the file 
     */
    void release() throw() {
        m_filename.clear();
    }
    
    /**
     * Unlink the file
     */
    void unlink() throw() {
        try{
            if(false == m_filename.empty()){
                ::unlink(m_filename.c_str());
            }
        }catch(...){}
        m_filename.clear();
    }

    /**
     * Generate a new temporary file. 
     */
    static std::string generate(const std::string& prefix, const std::string& dir, int * fd) /*throw(LogicError,RuntimeError)*/{
        // Check Procondition
        if(0 == fd){
            throw NullPointerException("null File Descriptor pointer");
        }
        if(true == prefix.empty()){
            throw InvalidArgumentException("empty Prefix");
        }
        char tmp_proxy[FILENAME_MAX];
        if(false == dir.empty()){
            snprintf(tmp_proxy,FILENAME_MAX,"%s/%s.XXXXXX",dir.c_str(),prefix.c_str());
        } else {
            snprintf(tmp_proxy,FILENAME_MAX,"%s.XXXXXX",prefix.c_str());
        }
        *fd = mkstemp(tmp_proxy);
        if (*fd == -1) {
            std::string reason = (std::string)"Cannot create temporary file <" + 
                tmp_proxy + ">.	Error is: " + strerror(errno);
            throw RuntimeError(reason);
        }
        return tmp_proxy;
    }

private:
    /**
     * Default Constructor. Not implemented
     */
    TempFile();
	
private:
    /**
     * The Name of the temporary file
     */
    std::string m_filename;
};
 
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_EXTRA_TEMPFILE_H_
