/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: XmlPatternsList Class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: XmlPatternsList.h,v 1.3 2007/07/20 09:30:43 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_AGENTS_EXTRA_XMLPATTERNSLIST_H_
#define GLITE_DATA_AGENTS_EXTRA_XMLPATTERNSLIST_H_

#include "glite/data/agents/AgentExceptions.h"

#include <string>
#include <vector>
#include <map>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

namespace glite {
namespace data  {
namespace agents{

/**
 * Exception to signal errors in the xml configuration file.
 */    
struct XmlPatternsException : public ConfigurationException{
    /**
     * Constructor
     */
    explicit XmlPatternsException(
        const std::string& reason):ConfigurationException(reason){}
    /**
     * Destructor
     */
    virtual ~XmlPatternsException() throw(){}
    /** 
     * exception type. 
     */
    virtual const char* type() const throw(){
        return "XmlPatternsException";
    }
};
    
/**
 * XmlPatternsList
 */
class XmlPatternsList{
public:

	/**
     * maximum length of string to match
     * (messages will be truncated after this size)
     */
	static const std::string::size_type MaxMatchLen;

    /**
     * constructor
     * @param fileName [IN] name of xml patterns file.
     */
    XmlPatternsList(
        const char* fileName) /* throws XmlPatternsException */;

    /**
     * destructor
     */
    virtual ~XmlPatternsList();

    /**
     * get the category associated to given message.
     * @param message [IN] the message to be matched against the internal 
     * patterns list
     * @return message category, empty string if not found.
     */
    std::string getCategory(
        const std::string& message);
    
    /**
     * return true if given message matches any of the patterns for the
     * given category
     * @param message [IN] the message to be matched against the patterns list
     * @param category [IN] the category of the patterns list
     * @return true if message matches category patterns, false otherwise.
     */
    bool checkCategory(
        const std::string& message,
        const std::string& category);
    
    /**
     * Interface for the objects used internally to store patterns information.
     */
    class PatternMatch{
    public:
        /**
         * constructor
         */
        PatternMatch(){
        }
        /**
         * destructor
         */
        virtual ~PatternMatch(){
        }
        /**
         * return true if given string matches the internal pattern
         * @param message [IN] the string to match
         * @return true if pattern matches, false otherwise
         */
        virtual bool check(
            const std::string& message) const = 0;
        /**
         * get the value of an optional 'property' node
         * @param name [IN] property name
         * @param value [OUT] property value
         * @return true if a property with given name was found, false otherwise
         */
        virtual bool getProperty(
            const std::string&  name,
            std::string&        value) const = 0;
    };
    
    /**
     * return the PatternMatch object for given message and category or
     * null if no pattern in category applies.
     * @param message [IN] the message to be matched against the patterns list
     * @param category [IN] the category of the patterns list
     * @return const pointer to PatternMatch object if message matches category 
     * patterns, false otherwise.
     */
    const PatternMatch * getPatternMatch(
        const std::string& message,
        const std::string& category);
    
protected:
    
    typedef std::vector< boost::shared_ptr<PatternMatch> > PatternMatches;
    typedef std::map<std::string,PatternMatches> PatternMatchesMap;
    // map category->list of patterns for category
    PatternMatchesMap m_patterns;
};

} // end namespace glite
} // end namespace data
} // end namespace agents

#endif // GLITE_DATA_AGENTS_EXTRA_XMLPATTERNSLIST_H_
