/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Command line options parser helper
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: options.h,v 1.2 2008/10/17 14:54:14 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_AGENTS_EXTRA_OPTIONS_H_
#define GLITE_DATA_AGENTS_EXTRA_OPTIONS_H_

#include <string>
#include <stdexcept>
#include <vector>
#include <map>
#include <algorithm>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <getopt.h>
#include <boost/lexical_cast.hpp>

namespace glite   {
namespace data    {
namespace agents  {
namespace options {

/**
 * list of help strings
 */
typedef std::vector<std::string> HelpStrings;

/**
 * option handler function
 * callback that will be invoked when the command line option is found
 */
typedef void (*Handler)(char *);

/**
 * ISetOption interface
 */
class ISetOption{
public:
    virtual void setValue(char * arg) = 0;
    virtual ~ISetOption(){}
};

/**
 * set the argument from the option value
 */
template<typename ValueType>
class SetArg : public ISetOption{
public:
    SetArg(ValueType& arg) : 
        m_arg(arg){
    }
    virtual void setValue(char * arg){
        m_arg = boost::lexical_cast<ValueType>(arg);
    }
protected:
    ValueType& m_arg;
};

/**
 * set the argument from the option value
 */
template<>
class SetArg<const char *> : public ISetOption{
public:
    typedef const char * ptr;
    SetArg(ptr& arg) : 
        m_arg(arg){
    }
    virtual void setValue(char * arg){
        m_arg = arg;
    }
protected:
    ptr& m_arg;
};

/**
 * set the argument from the option value
 */
template<>
class SetArg<char *> : public ISetOption{
public:
    typedef char * ptr;
    SetArg(ptr& arg) : 
        m_arg(arg){
    }
    virtual void setValue(char * arg){
        m_arg = arg;
    }
protected:
    ptr& m_arg;
};

/**
 * set the argument to a predefined value when the option is found
 */
template<typename ValueType>
class SetValue : public ISetOption{
public:
    SetValue(ValueType& arg,ValueType value) : 
        m_arg(arg),
        m_value(value){
    }
    virtual void setValue(char * arg){
        m_arg = m_value;
    }
protected:
    ValueType& m_arg;
    ValueType m_value;
};

/**
 * invoke the callback when the option is found
 */
class CallHandler : public ISetOption{
public:
    CallHandler(Handler handler) : m_handler(handler){
    }
    virtual void setValue(char * arg){
        m_handler(arg);
    }
protected:
    Handler m_handler;
};
 
/**
 * A single command line option
 */
class Option{
public:
    
    /**
     * set the required argument flag
     * @param helpString [IN,OPTIONAL] the name to display in the help string
     * @return the option object
     */
    inline Option& arg(const std::string& helpString = "");

    /**
     * add an help string
     * @param value [IN]
     * @return the option object
     */
    inline Option& help(const std::string& value);
    
    /**
     * destructor
     */
    inline virtual ~Option();
        
protected:

    /**
     * constructor
     * @param handler [IN] the handler object
     * @param shortOpt [IN] the short option
     * @param longOpt [IN] the long option
     */
    inline Option(
        ISetOption *        handler,
        char                shortOpt,
        const std::string&  longOpt);
    
    // the short option
    char m_short;
    // the long option
    std::string m_long;
    // flag signaling if option needs an argument
    bool m_arg;
    // help string
    HelpStrings m_helpStrings;
    // name to use for the argument value in the help string
    std::string m_helpValueString;
    // option handler
    ISetOption * m_setOption;
};

/**
 * A positional parameter (used for help message)
 */
class Positional{
public:
    
    /**
     * add an help string
     * @param value [IN]
     * @return the positional parameter object
     */
    inline Positional& help(const std::string& value);
    
    /**
     * destructor
     */
    inline virtual ~Positional(){}
    
protected:
    
    /**
     * constructor
     * @param name [IN] parameter name 
     */
    inline Positional(const std::string& name);
    
    std::string m_name;
    // help string
    HelpStrings m_helpStrings;
};

/**
 * Command line arguments handler
 */
class Arguments{
public:
    
    /**
     * Constructor
     */
    inline Arguments(
        int                 argc,
        char *              argv[],
        std::ostream&       out = std::cout,
        std::ostream&       err = std::cerr);
    
    /**
     * Add an option with a short description
     * @param shortOption [IN] the short option 
     * @return the newly created option object
     */
    inline Option& opt(ISetOption * handler,char shortOption);
    
    /**
     * Add an option with a short and a long description
     * @param shortOption [IN] the short option 
     * @param longOption [IN] the long option 
     * @return the newly created option object
     */
    inline Option& opt(ISetOption * handler,char shortOption,const std::string& longOption);
    
    /**
     * Add an option with a long description
     * @param longOption [IN] the long option 
     * @return the newly created option object
     */
    inline Option& opt(ISetOption * handler,const std::string& longOption);
    
    /**
     * Add a positional parameter object
     * @param name [IN] the parameter name
     * @return the newly created positional parameter object
     */
    inline Positional& pos(const std::string& name);
    
    /**
     * destructor
     */
    inline ~Arguments();

    /**
     * Show the help message
     * @param margin [IN,OPTIONAL]
     */
    inline void showHelp(unsigned int margin = 30) const /*throw()*/;
    
    /**
     * parse command line arguments
     * @return true if successful
     */
    inline bool parse() /*throw()*/;
    
protected:
    
    /**
     * internal add method
     */
    inline Option& add(int index,ISetOption * handler,char shortOption,const std::string& longOption);

    // index of last non-character option inserted
    int m_lastIndex;
    
    // argc and argv from command line
    int m_argc;
    char ** m_argv;
    
    // name the program was called with (basename of argv[0])
    std::string m_progName;
    // description of positional parameters for help message
    std::string m_usage;
        
    // map option index -> option object
    typedef std::map<int,Option *> Opts;
    Opts m_opts;
    // vector (to print help in the same order options are added)
    typedef std::vector<Option *> OptsList;
    OptsList m_optsList;
    // positional parameters and help strings (for help message only)
    typedef std::vector<Positional *> Positionals;
    Positionals m_positionals;
    
    // output stream
    std::ostream& m_out;
    // error stream
    std::ostream& m_err;
    
};

//------------------------------------------------------------------------------
//                      Helpers to add options
//------------------------------------------------------------------------------
inline Option& opt(Handler handler,Arguments& args,char sopt,const std::string& lopt){
    return args.opt(new CallHandler(handler),sopt,lopt);
}
inline Option& opt(Handler handler,Arguments& args,char sopt){
    return args.opt(new CallHandler(handler),sopt);
}
inline Option& opt(Handler handler,Arguments& args,const std::string& lopt){
    return args.opt(new CallHandler(handler),lopt);
}

template <typename ValueType>
inline Option& opt(ValueType& arg,Arguments& args,char sopt,const std::string& lopt){
    return args.opt(new SetArg<ValueType>(arg),sopt,lopt);
}        
template <typename ValueType>
inline Option& opt(ValueType& arg,Arguments& args,char sopt){
    return args.opt(new SetArg<ValueType>(arg),sopt);
}        
template <typename ValueType>
inline Option& opt(ValueType& arg,Arguments& args,const std::string& lopt){
    return args.opt(new SetArg<ValueType>(arg),lopt);
}

template <typename ValueType>
inline Option& opt(ValueType& arg,ValueType value,Arguments& args,char sopt,const std::string& lopt){
    return args.opt(new SetValue<ValueType>(arg,value),sopt,lopt);
}        
template <typename ValueType>
inline Option& opt(ValueType& arg,ValueType value,Arguments& args,char sopt){
    return args.opt(new SetValue<ValueType>(arg,value),sopt);
}        
template <typename ValueType>
inline Option& opt(ValueType& arg,ValueType value,Arguments& args,const std::string& lopt){
    return args.opt(new SetValue<ValueType>(arg,value),lopt);
}

} // namespace options
} // namespace agents
} // namespace data
} // namespace glite

#include <glite/data/agents/extra/options.inl>

#endif // GLITE_DATA_AGENTS_EXTRA_OPTIONS_H_

