/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Command line options parser helper
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: options.inl,v 1.2 2008/10/17 14:54:14 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

namespace glite   {
namespace data    {
namespace agents  {
namespace options {

//------------------------------------------------------------------------------
//                                  Exceptions 
//------------------------------------------------------------------------------

namespace details{

/**
 * Exception class for the command line option parser utilities
 */
class OptionsException : public std::exception{
public:
    explicit OptionsException(
        const std::string& reason) : 
        m_reason(reason){
    }
    virtual const char* what() const throw(){
        return m_reason.c_str();
    }
    virtual ~OptionsException() throw(){}  

protected:
    OptionsException(){
    }
    std::string m_reason;
};

/**
 * DuplicateOptionException
 */
class DuplicateOptionException : public OptionsException{
public:
    DuplicateOptionException(const std::string& optName) : 
        OptionsException(),
        m_optName(optName){
        std::stringstream err;
        err << "Duplicate option '" << optName << "' found.";
        m_reason = err.str();
    }
    virtual ~DuplicateOptionException() throw(){}
protected:
    std::string m_optName;    
};

} // details

//------------------------------------------------------------------------------
//                                  Options 
//------------------------------------------------------------------------------

/**
 * constructor
 */
Option::Option(ISetOption * handler,char shortOpt = 0,const std::string& longOpt = "") :
    m_short(shortOpt),
    m_long(longOpt),
    m_arg(false),
    m_helpValueString("VALUE"),
    m_setOption(handler){
}
    
/**
 * set the required argument flag
 */
Option& Option::arg(const std::string& helpString){
    m_arg = true;
    if (!helpString.empty()){
        m_helpValueString = helpString;
    }
    return (*this);
}

/**
 * add an help string
 */
Option& Option::help(const std::string& value){
    m_helpStrings.push_back(value);
    return (*this);
}

/**
 * destructor
 */
Option::~Option(){
    if (m_setOption){
        delete m_setOption;
        m_setOption = 0;
    }
}

//------------------------------------------------------------------------------
//                                  Positional
//------------------------------------------------------------------------------

/**
 * constructor
 */
Positional::Positional(const std::string& name) : m_name(name){
}

/**
 * add an help string
 */
Positional& Positional::help(const std::string& value){
    m_helpStrings.push_back(value);
    return (*this);
}

//------------------------------------------------------------------------------
//                                  Impl objects 
//------------------------------------------------------------------------------

namespace details{

/**
 * OptionImpl
 */
class OptionImpl : public Option{
public:

    /**
     * constructor
     * @param shortOpt [IN] the short option
     * @param longOpt [IN] the long option
     */
    OptionImpl(
        ISetOption *        handler,
        char                shortOpt = 0,
        const std::string&  longOpt = "") : Option (handler,shortOpt,longOpt){
    }
        
    /**
     * hasShortOpt
     * @return true if option has short representation
     */
    bool hasShortOpt () const {
        return (0 != m_short);
    }
    
    /**
     * hasLongOpt
     * @return true if option has long representation
     */
    bool hasLongOpt () const {
        return (!m_long.empty());
    }

    /**
     * isDuplicate
     * @return true if given short option is identical to own
     */
    bool isDuplicate(char value) const{
        return ((0 != value) && (0 != m_short) && (value == m_short));
    }
    
    /**
     * isDuplicate
     * @return true if long short option is identical to own
     */
    bool isDuplicate(const std::string& value) const{
        return ((!value.empty()) && (!m_long.empty()) && (value == m_long));
    }
        
    /**
     * hasArg
     * @return true if option accepts an argument
     */
    bool hasArg() const{
        return m_arg;
    }
    
    /**
     * shortOpt
     * @return the short option
     */
    char shortOpt() const {
        return m_short;
    }
    
    /**
     * longOpt
     * @return the long option
     */
    const std::string& longOpt() const {
        return m_long;
    }
    
    /**
     * helpString
     * @return the help string
     */
    const HelpStrings& helpStrings() const {
        return m_helpStrings;
    }
    
    /**
     * helpValueString
     * @return the help value string
     */
    const std::string& helpValueString() const {
        return m_helpValueString;
    }
    
    /**
     * callHandler
     * call the handler function
     * @param optArg [IN] the option argument (or NULL)
     */
    void callHandler(char * optArg){
        if (m_setOption){
            m_setOption->setValue(optArg);
        }
    }
    
    /**
     * toString
     * @return string with option description
     */
    std::string toString() const{
        std::stringstream ret;    
        if (hasShortOpt()){
            ret << " -" << m_short;
            if (hasLongOpt()){
                ret << ",";
            }
        } 
        if (hasLongOpt()){
            ret << " --" << m_long;
        }
        if (hasArg()){
            ret << " " << helpValueString();
        }
        ret << ": ";
        return ret.str();
    }
    
    /**
     * destructor
     */
    virtual ~OptionImpl(){
    }
        
};

/**
 * PositionalImpl
 */
class PositionalImpl : public Positional{
public:
    
    PositionalImpl(const std::string& name) : Positional(name){
        m_toString = m_name + std::string(": ");
    }

    const std::string& name() const{
        return m_name;
    }
        
    const std::string& toString() const{
        return m_toString;
    }
    
    const HelpStrings& helpStrings() const{
        return m_helpStrings;    
    }
    
    virtual ~PositionalImpl(){}
    
protected:
    std::string m_toString;
};        

} // details

//------------------------------------------------------------------------------
//                                  Options 
//------------------------------------------------------------------------------

/**
 * Constructor
 */
Arguments::Arguments(
    int                 argc,
    char *              argv[],
    std::ostream&       out,
    std::ostream&       err) : 
    // initialize last index with a non-printable character; if for an option
    // a short string is not specified, use something that will not conflict
    // with other options. Assigning each option a non-zero value makes it
    // possible to index the map without using the option index parameter of
    // getopt_long
    m_lastIndex(0x80),
    m_argc(argc),
    m_argv(argv),
    m_out(out),
    m_err(err) { 
    m_progName = argv[0];
    size_t slashPos = m_progName.rfind('/');
    if (std::string::npos != slashPos){
        m_progName = m_progName.substr(slashPos + 1);
    }
}

/**
 * Add an option with a short description
 */
Option& Arguments::opt(ISetOption * handler,char shortOption){
    return add(shortOption,handler,shortOption,"");
}

/**
 * Add an option with a short and a long description
 */
Option& Arguments::opt(ISetOption * handler,char shortOption,const std::string& longOption){
    return add(shortOption,handler,shortOption,longOption);
}

/**
 * Add an option with a long description only
 */
Option& Arguments::opt(ISetOption * handler,const std::string& longOption){
    return add(++m_lastIndex,handler,0,longOption);
}

/**
 * Add a positional parameter object
 * @param name [IN] the parameter name
 * @return the newly created positional parameter object
 */
Positional& Arguments::pos(const std::string& name){
    Positional * positional = new details::PositionalImpl(name);
    m_positionals.push_back(positional);
    return (*positional);
}

/**
 * destructor
 */
Arguments::~Arguments(){
    for (Opts::iterator it = m_opts.begin();it != m_opts.end();++it){
        delete (it->second);
    }
    for (Positionals::iterator it = m_positionals.begin();it != m_positionals.end();++it){
        delete (*it);
    }
}

namespace details{

/**
 * helper method to print help lines
 */
template <typename T>
void printHelpLine(std::ostream& out,const T& opt,unsigned int margin){
    std::string optString = opt.toString();
    out << std::left << std::setw(margin) << optString;
    if (margin && optString.length() > margin){
        out << std::endl << std::left << std::setw(margin) << "";
    }
    const HelpStrings& help = opt.helpStrings();
    for (HelpStrings::const_iterator hit = help.begin();hit != help.end();++hit){
        if (hit != help.begin()){
            out << std::left << std::setw(margin) << "";
        }
        out << (*hit) << std::endl;
    }
}

} // details

/**
 * Show the help message
 */
void Arguments::showHelp(unsigned int margin) const {
    using details::PositionalImpl;
    using details::OptionImpl;
    using details::printHelpLine;
    m_out << "Usage: " << m_progName << " [options] ";
    for (Positionals::const_iterator it = m_positionals.begin();it != m_positionals.end();++it){
        const PositionalImpl * pos = (const PositionalImpl *)(*it);
        m_out << pos->name() << " ";
    }
    m_out << std::endl << "Options:" << std::endl;
    for (OptsList::const_iterator it = m_optsList.begin();it != m_optsList.end();++it){
        const OptionImpl * opt = (const OptionImpl *)(*it);
        printHelpLine(m_out,*opt,margin);
    }
    if (m_positionals.size()){
        m_out << std::endl << "Arguments:" << std::endl;        
        for (Positionals::const_iterator it = m_positionals.begin();it != m_positionals.end();++it){
            const PositionalImpl * pos = (const PositionalImpl *)(*it);
            printHelpLine(m_out,*pos,margin);
        }
    }
    m_out << std::endl;        
}

/**
 * parse command line arguments
 */
bool Arguments::parse() {
    using details::OptionImpl;
    // the string with short options in form "ab:c"
    std::stringstream short_options_stream;
    // array of long options object. Using the short option as an index if 
    // the short option is specified.
    struct option long_options[m_opts.size() + 1];
    // build the short options string and the long options array 
    int i = 0;
    for (Opts::const_iterator it = m_opts.begin();it != m_opts.end();++it,++i){
        const int& index = it->first;
        const OptionImpl* opt = (const OptionImpl*)(it->second);
        if (opt->hasShortOpt()){
            short_options_stream << opt->shortOpt();
            if (opt->hasArg()){
                short_options_stream << ":";
            }
        }
        long_options[i] = (struct option){
            opt->longOpt().c_str(),
            opt->hasArg() ? required_argument : no_argument,
            NULL,
            index
        };
    }
    long_options[i] = (struct option) {NULL,no_argument,NULL,0};
    // actual parsing with getopt_long
    int optind = 0;
    std::string option_string = short_options_stream.str();
    while (-1 != (optind = getopt_long(m_argc,m_argv,option_string.c_str(),long_options,NULL))){
        switch (optind){
        case ':':
            m_err << "missing option argument" << std::endl;
            return false;
        case '?':
            m_err << "unknown option" << std::endl;
            return false;
        default:
            Opts::iterator it = m_opts.find(optind);
            if (it == m_opts.end()){
                // this should never happen
                m_err << "Error parsing command line arguments: unrecognized option (" << optind << ")." << std::endl;
                return false;
            }
            OptionImpl* opt = (OptionImpl*)(it->second);
            opt->callHandler(optarg);
        }
    }
    return true;
}

/**
 * internal add method
 */
Option& Arguments::add(int index,ISetOption * handler,char shortOption,const std::string& longOption){
    using details::OptionsException;
    using details::DuplicateOptionException;
    using details::OptionImpl;
    if (0 == handler){
        // this should never happen
        throw OptionsException("No handler configured for option.");
    }
    // check uniqueness of short and long option
    for (Opts::const_iterator it = m_opts.begin();it != m_opts.end();++it){
        const OptionImpl* opt = (const OptionImpl*)(it->second);
        if (opt->isDuplicate(shortOption)){
            throw DuplicateOptionException(std::string(1,shortOption));
        }
        if (opt->isDuplicate(longOption)){
            throw DuplicateOptionException(longOption);
        }
    }
    if (m_opts.end() != m_opts.find(index)){
        std::stringstream err;
        err << "Duplicate internal index '" << index << "' in Arguments object.";
        throw OptionsException(err.str());
    }
    // create option object
    Option * option = new OptionImpl(handler,shortOption,longOption);
    m_opts[index] = option;
    m_optsList.push_back(option);
    return (*option);
}
        
} // namespace options
} // namespace agents
} // namespace data
} // namespace glite

  
