/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: SDCache class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDCache.h,v 1.6 2008/11/14 16:04:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_SD_SDCACHE_H_
#define GLITE_DATA_AGENTS_SD_SDCACHE_H_

#include "glite/data/agents/sd/SDTypes.h"

#include <vector>
#include <string>
#include <boost/scoped_ptr.hpp>
#include <boost/utility.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite {
namespace data  {
namespace agents{
namespace sd    {

/**
 * Forward declarations
 */
class SDCacheImpl;
class SDConfig;
    
/**
 * SDCache
 */
class SDCache : boost::noncopyable {
public:
    /**
     * StringArray Type
     */
    typedef std::vector<std::string> StringArray;
    /**
     * ServiceArray Type
     */
    typedef std::vector<const Service *> ServiceArray;

    /**
     * Destructor
     */
    ~SDCache();

    /**
     * Set the Validity of an entry in the Cache
     */
    void ttl(unsigned int v);
    /**
     * Set the Increment of the validity of an stale entry in the Cache
     */
    void staleTime(unsigned int v);
    /**
     * Set the time that should elapse before considering an entry obsolete
     */
    void obsoleteTime(unsigned int v);
    /**
     * Set the time that should elapse before considering a negative entry obsolete
     */
    void negativeObsoleteTime(unsigned int v);
    
    /**
     * Insert a Service in the Cache
     */
    void insert(const Service& service);
    void insert(const Service& service, const std::string& vo_name);
    void insert(const Service& service, const StringArray& vo_list);

    /**
     * Store an association between two services in the Cache. If the services
     * are not there, they will be added, but no VO would be specified
     */
    void associate(const Service& service, const Service& associated_service);

    /**
     * Insert a Property for a given Service and VO
     */
    void insertProperty(const Service& service, const std::string& name, const std::string& value);
    void insertProperty(const Service& service, const std::string& name, const std::string& value, const std::string& vo_name);
    void insertProperty(const Service& service, const std::string& name, const std::string& value, const StringArray& vo_list);
    
    /**
     * Clear the Cache
     */
    void clear();

    /**
     * Purge obsolete entries from the cache
     */
    void purge();

    /**
     * Get the service by Name and VO (enforce authorization)
     */
    const Service * getByName(const std::string& name);
    const Service * getByName(const std::string& name, const std::string& vo_name);
    const Service * getByName(const std::string& name, const StringArray& vo_list);

    /**
     * Get the list of services by type
     */
    ServiceArray getByType(const std::string& type);
    ServiceArray getByType(const std::string& type, const std::string& vo_name);
    ServiceArray getByType(const std::string& type, const StringArray& vo_list);

    /**
     * Get the list of services by host and type
     */
    ServiceArray getByHost(const std::string& type, const std::string& host);
    ServiceArray getByHost(const std::string& type, const std::string& host,const std::string& vo_name);
    ServiceArray getByHost(const std::string& type, const std::string& host,const StringArray& vo_list);

    /**
     * Get the list of services by site and type
     */
    ServiceArray getBySite(const std::string& type, const std::string& site);
    ServiceArray getBySite(const std::string& type, const std::string& site,const std::string& vo_name);
    ServiceArray getBySite(const std::string& type, const std::string& site,const StringArray& vo_list);

    /**
     * Get the list of associated services by type
     */
    ServiceArray getAssociated(const std::string& name, const std::string& assoc_type);
    ServiceArray getAssociated(const std::string& name, const std::string& assoc_type, const std::string& vo_name);
    ServiceArray getAssociated(const std::string& name, const std::string& assoc_type, const StringArray& vo_list);

    /**
     * Get the list of associated services by type and site
     */
    ServiceArray getAssociatedOnSite(const std::string& name, const std::string& assoc_type, const std::string& assoc_site);
    ServiceArray getAssociatedOnSite(const std::string& name, const std::string& assoc_type, const std::string& assoc_site, const std::string& vo_name);
    ServiceArray getAssociatedOnSite(const std::string& name, const std::string& assoc_type, const std::string& assoc_site, const StringArray& vo_list);

    /**
     * Get a Property for a given Service. If no property is found, the second 
     * field of the pair is set to false
     */
    std::pair<std::string,bool> getProperty(const std::string& srv_name, const std::string& prop_name);
    std::pair<std::string,bool> getProperty(const std::string& srv_name, const std::string& prop_name,const std::string& vo_name);
    std::pair<std::string,bool> getProperty(const std::string& srv_name, const std::string& prop_name,const StringArray& vo_list);

    
    // -------------------------------------------------------------------------
    // Missing services, properties & associations
    //
    // Cache also the information that a service, a property or an association
    // is missing: This would allow not to query evry time the information
    // system for data that would never be there
    // -------------------------------------------------------------------------

    /**
     * Remeber that a Service is not listed in the information system
     */
    void rememberMissing(const std::string& type);
    void rememberMissing(const std::string& type, const StringArray& vo_list);
    void rememberMissing(const std::string& type, const std::string& source);
    void rememberMissing(const std::string& type, const std::string& source, const StringArray& vo_list);

    /**
     * Remeber that a ServiceProeprty is not listed in the information system
     */
    void rememberMissingProperty(const std::string& service_name, const std::string& prop_name);
    void rememberMissingProperty(const std::string& service_name, const std::string& prop_name, const StringArray& vo_list);

    /**
     * Remeber that a Service doesn't have associated services of a given type
     * listed in the information system
     */
    void rememberMissingAssociation(const std::string& service_name, const std::string& type);
    void rememberMissingAssociation(const std::string& service_name, const std::string& type, const StringArray& vo_list);
    void rememberMissingAssociation(const std::string& service_name, const std::string& type, const std::string& site);
    void rememberMissingAssociation(const std::string& service_name, const std::string& type, const std::string& site, const StringArray& vo_list);

    /**
     * Check if a Service is not listed in the information system
     */
    bool isServiceMissing(const std::string& type);
    bool isServiceMissing(const std::string& type, const StringArray& vo_list);
    bool isServiceMissing(const std::string& type, const std::string& source);
    bool isServiceMissing(const std::string& type, const std::string& source, const StringArray& vo_list);
    /**
     * Check if a Service Property is not listed in the information system
     */
    bool isPropertyMissing(const std::string& service_name, const std::string& prop_name);
    bool isPropertyMissing(const std::string& service_name, const std::string& prop_name, const StringArray& vo_list);
    /**
     * Remeber that a Service doesn't have associated services of a given type
     * listed in the information system
     */
    bool isAssociationMissing(const std::string& service_name, const std::string& type);
    bool isAssociationMissing(const std::string& service_name, const std::string& type, const StringArray& vo_list);
    bool isAssociationMissing(const std::string& service_name, const std::string& type, const std::string& site);
    bool isAssociationMissing(const std::string& service_name, const std::string& type, const std::string& site, const StringArray& vo_list);

protected:

    friend class glite::data::agents::sd::SDConfig;
        
    /**
     * Constructor
     */
    SDCache();

private:
    /**
     * The Logger
     */
    glite::config::Logger m_logger;
        
    /**
     * Pimpl
     */
    boost::scoped_ptr<class SDCacheImpl> m_impl;
};

} // end namespace sd
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_SD_SDCACHE_H_
