/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: SDConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDConfig.h,v 1.5 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_SD_SDCONFIG_H_
#define GLITE_DATA_AGENTS_SD_SDCONFIG_H_

#include <string>
#include <boost/scoped_ptr.hpp>
#include <boost/utility.hpp>

namespace glite {
namespace data  {
namespace agents{
namespace sd    {
/*
 * Forward Declaration
 */
class SDCache;
    
/**
 * SDConfig Class
 */
class SDConfig : boost::noncopyable{
public:

    /**
     * Return the SDConfig singleton instance
     * @return the SDConfig singleton instance
     */
    static SDConfig& instance() /*throw ()*/; 
   
    // =========================================================================
    // SDConfig Methods
    // =========================================================================

    /**
     * Return the SDCache instance
     * @return the SDCache instance
     */
    SDCache * cache() {return m_cache.get();} 

         /**
     * Get SRM Service Type Identifier
     * @return SRM Service Type Identifier
     */
    const std::string& srm() const {return m_srmType;} 

    /**
     * Get GridFTP Service Type Identifier
     * @return GridFTP Service Type Identifier
     */
    const std::string& gridftp() const {return m_gridftpType;} 

    /**
     * Get legacy GridFTP Service Type Identifier
     * @return legacy GridFTP Service Type Identifier
     */
    const std::string& legacyGridftp() const {return m_legacyGridftpType;} 

    /**
     * Get MyProxy Service Type Identifier
     * @return MyProxy Service Type Identifier
     */
    const std::string& myproxy() const {return m_myproxyType;} 

    /**
     * Get FileTransfer Service Type Identifier
     * @return FileTransfer Service Type Identifier
     */
    const std::string& fileTransfer() const {return m_fileTransferType;} 

    /**
     * Get FilePlacementStats Service Type Identifier
     * @return FilePlacementStats Service Type Identifier
     */
    const std::string& fileTransferStats() const {return m_fileTransferStatsType;} 

    /**
     * Get ChannelManagement Service Type Identifier
     * @return ChannelManagement Service Type Identifier
     */
    const std::string& channelManagement() const {return m_channelManagementType;} 

    /**
     * Get ChannelAgent Service Type Identifier
     * @return ChannelAgent Service Type Identifier
     */
    const std::string& channelAgent() const {return m_channelAgentType;} 

    /**
     * Get BDII Service Type Identifier
     * @return BDII Service Type Identifier
     */
    const std::string& bdii() const {return m_bdiiType;} 

    /**
     * Get VOMS Service Type Identifier
     * @return VOMS Service Type Identifier
     */
    const std::string& voms() const {return m_vomsType;} 

    /**
     * Get SEIndex Service Type Identifier
     * @return SEIndex Service Type Identifier
     */
    const std::string& seIndex() const {return m_seIndexType;} 

    /**
     * Get Fireman Service Type Identifier
     * @return fireman Service Type Identifier
     */
    const std::string& fireman() const {return m_firemanType;} 

    /**
     * Get SEMountPoint Property Name
     * @return SEMountPoint Property Name
     */
    const std::string& sePathProperty() const {return m_sePathProperty;} 
    
    /**
     * Get ChannelAgent Source Property Name
     * @return ChannelAgent Source Property Name
     */
    const std::string& channelAgentSourceProperty() const {return m_channelAgentSourceProperty;} 

    /**
     * Get ChannelAgent Destination Property Name
     * @return ChannelAgent Destination Property Name
     */
    const std::string& channelAgentDestinationProperty() const {return m_channelAgentDestinationProperty;} 
     
    /**
     * Get MyProxy FTSMode Property Name
     * @return MyProxy FTS Mode Property Name
     */
    const std::string& myproxyFtsModeProperty() const {return m_myproxyFtsModeProperty;} 

    /**
     * Get MyProxy FTSMode Retriever Property value
     * @return MyProxy FTS Mode Retriever Property value
     */
    const std::string& myproxyFtsModeRetriever() const {return m_myproxyFtsModeRetriever;} 

    /**
     * Get MyProxy FTSMode Renewer Property value
     * @return MyProxy FTS Mode Renewer Property value
     */
    const std::string& myproxyFtsModeRenewer() const {return m_myproxyFtsModeRenewer;} 

protected:
        
    /**
     * enableCache
     */
    void enableCache(bool b = true); 
    
private:
    
    /*
     * Declare factory methods as friends
     */
    friend class ServiceDiscoveryConfig;

    /**
     * Constructor
     */
    SDConfig();
        
    /**
     * Destructor
     */
    ~SDConfig();

private:
    
    /**
     * The SRM Service Type
     */
    std::string m_srmType;
    
    /**
     * The GridFTP Service Type
     */
    std::string m_gridftpType;
    
    /**
     * The Legacy GridFTP Service Type
     */
    std::string m_legacyGridftpType;
    
    /**
     * The MyProxy Service Type
     */
    std::string m_myproxyType;
    
    /**
     * The FileTransfer Service Type
     */
    std::string m_fileTransferType;
    
    /**
     * The FileTransferStats Service Type
     */
    std::string m_fileTransferStatsType;
    
    /**
     * The ChannelManagement Service Type
     */
    std::string m_channelManagementType;
    
    /**
     * The ChannelAgent Service Type
     */
    std::string m_channelAgentType;
    
    /**
     * The BDII Service Type
     */
    std::string m_bdiiType;
    
    /**
     * The VOMS Service Type
     */
    std::string m_vomsType;
    
    /**
     * The SEIndex Service Type
     */
    std::string m_seIndexType;
    
    /**
     * The Fireman Service Type
     */
    std::string m_firemanType;
    
    /**
     * The SE MountPoint Property
     */
    std::string m_sePathProperty;
        
    /**
     * The ChannelAgent Source Property
     */
    std::string m_channelAgentSourceProperty;
        
    /**
     * The ChannelAgent Destination Property
     */
    std::string m_channelAgentDestinationProperty;
    
    /**
     * The Myproxy FTS Mode Property
     */
    std::string m_myproxyFtsModeProperty;
    
    /**
     * The Myproxy FTS Mode Retriever Property value
     */
    std::string m_myproxyFtsModeRetriever;
    
    /**
     * The Myproxy FTS Mode Renewer Property value
     */
    std::string m_myproxyFtsModeRenewer;
    
    /**
     * The SDCache instance
     */
    boost::scoped_ptr<class SDCache> m_cache;
};

} // end namespace sd
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_SD_SDCONFIG_H_
