/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: ServiceDiscovery Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ServiceDiscovery.h,v 1.3 2008/11/14 16:04:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_SD_SERVICEDISCOVERY_H_
#define GLITE_DATA_AGENTS_SD_SERVICEDISCOVERY_H_

#include "glite/data/agents/sd/SDTypes.h"

#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite {
namespace data  {
namespace agents{
namespace sd    {

/**
 * ServiceDiscovery Client
 */
class ServiceDiscovery : boost::noncopyable {
public:

    typedef std::vector<boost::shared_ptr<Service> > ServiceArray;

    /**
     * Constructor
     */
    ServiceDiscovery();
        
    /**
     * Destructor
     */
    ~ServiceDiscovery();

    /**
     * Get the service with the given name
     * @param name [IN] the Service identifier
     * @retrun the Service instance that matches the given criteria
     * @throws ServiceDiscoveryException if no Service is found
     * @throws LogicError if precondition are not met
     */
    Service * getServiceByName(const std::string& name) /*throw (LogicError, ServiceDiscoveryException)*/;

    /**
     * Return a list of services of the given type on the given host
     * @param type [IN] the Service type
     * @param host [IN] the Service host
     * @param port [IN] the Service port (optional)
     * @param vo_name [IN] the VO name (optional)
     * @param services [OUT] the Service instances that match the given criteria
     * @throws ServiceDiscoveryException if no Service is found
     * @throws LogicError if precondition are not met
     */
    void getServicesOnHost(const std::string& type, const std::string& host, unsigned int port, const std::string& vo_name, ServiceArray& services) /*throw (LogicError, ServiceDiscoveryException)*/;

    /**
     * Return a list of services of the given type in the given site
     * @param type [IN] the Service type
     * @param site [IN] the Service site
     * @param vo_name [IN] the VO name (optional)
     * @param services [OUT] the Service instances that match the given criteria
     * @throws ServiceDiscoveryException if no Service is found
     * @throws LogicError if precondition are not met
     */
    void getServicesOnSite(const std::string& type, const std::string& site, const std::string& vo_name, ServiceArray& services) /*throw (LogicError, ServiceDiscoveryException)*/;

    /**
     * Return a list of services of the given type
     * @param type [IN] the Service type
     * @param vo_name [IN] the VO name (optional)
     * @param services [OUT] the Service instances that match the given criteria
     * @throws ServiceDiscoveryException if no Service is found
     * @throws LogicError if precondition are not met
     */
    void getServicesByType(const std::string& type, const std::string& vo_name, ServiceArray& services) /*throw (LogicError, ServiceDiscoveryException)*/;

    /**
     * Get the associated service of the given type
     * @param service [IN] the associated Service
     * @param type [IN] the requested Service type
     * @param site [IN] the requiested site name (optional)
     * @param services [OUT] the Service instances that match the given criteria
     * @throws ServiceDiscoveryException if no Service is found
     * @throws LogicError if precondition are not met
     */
    void getAssociatedServices(const Service& service, const std::string& type, const std::string& site, ServiceArray& services) /*throw (LogicError, ServiceDiscoveryException)*/;

    /**
     * Get the value of a service property
     * @param service [IN] the Service instance
     * @param prop_name [IN] the property name
     * @throws ServiceDiscoveryException if no Property is found
     */
    std::string getProperty(const Service& service, const std::string& prop_name) /*throw (ServiceDiscoveryException)*/;

private:
    /**
     * The Logger
     */
    glite::config::Logger m_logger;
};

} // end namespace sd
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_SD_SERVICEDISCOVERY_H_
