/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: SD Utility Methods 
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: sd-utility.h,v 1.8 2008/01/07 16:58:29 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_SD_SD_UTILITY_H_
#define GLITE_DATA_AGENTS_SD_SD_UTILITY_H_

#include "glite/data/agents/sd/SDTypes.h"
#include <string>
#include <vector>
#include <functional>

namespace glite {
namespace data  {
namespace agents{
namespace sd    {

/**
 * Service Selection predicates: when a list of services is returned, use that
 * predicates to select the correct one
 */    
struct SelectPred : public std::unary_function<const std::vector<const Service*>&,const Service*>{
public:    
    /**
     * call operator 
     * select service from list according to implemented criteria
     */ 
    const Service* operator()(const std::vector<const Service*>& services) throw (){
        if (services.empty()){
            return 0;
        }
        return select(services);
    }
    /**
     * type
     * @return predicate type (for debugging)
     */ 
    virtual const char * type() = 0;
    /**
     * destructor
     */
    virtual ~SelectPred(){};
protected:
    /**
     * select
     * select service from list according to implemented criteria; to be overridden by derived classes
     */ 
    virtual const Service* select(const std::vector<const Service*>& services) throw () = 0;
};

/**
 * Predicate to select the first Service
 */    
class SelectFirst : public SelectPred {
public:
    virtual const char * type(){return "SelectFirst";}
    virtual ~SelectFirst(){};
protected:
    virtual const Service* select(const std::vector<const Service*>& services) throw() {
        return *(services.begin());
    }
};

/**
 * Predicate to select the Service by version
 */
class SelectByVersion : public SelectPred {
public:
    SelectByVersion(const std::string& version);
    virtual const char * type(){return "SelectByVersion";}
    virtual ~SelectByVersion(){};
protected:
    virtual const Service* select(const std::vector<const Service*>& services) throw();
private:
    SelectByVersion();
    unsigned int m_major;
    unsigned int m_minor;
    unsigned int m_patch;
};

/**
 * Predicate to select the Service by endpoint port and path and then version
 */
class SelectByPortAndVersion : public SelectPred {
public:
    SelectByPortAndVersion(unsigned int port, const std::string& path, const std::string& version):m_port(port),m_path(path),m_version(version){}
    virtual const char * type(){return "SelectByPortAndVersion";}
    virtual ~SelectByPortAndVersion(){};
protected:
    virtual const Service* select(const std::vector<const Service*>& services) throw();
private:
    SelectByPortAndVersion();
    unsigned int m_port;
    std::string  m_path;
    std::string  m_version;
};

/**
 * Get a service instance from an Source Name, that can be either a site name 
 * or a service host name. In case more than one services are available on the 
 * given source, the first one is returned. It's up to the caller to delete the 
 * returned instance. In case no service is found, 0 is returned
 * @throws LogicError is preconditions are not met 
 */
Service * get_service(const std::string& source, 
                      const std::string& type, 
                      const std::string& vo_name) /*throw (LogicError)*/;

/**
 * Get a service instance from an Source Name, that can be either a site name 
 * or a service host name. A predicate can be specified in order to select the 
 * service in case more services are available on the given source. It's up to 
 * the caller to delete the returned instance. In case no service is found, 0 
 * is returned
 * @throws LogicError is preconditions are not met 
 */
Service * get_service(const std::string& source, 
                      const std::string& type, 
                      const std::string& vo_name,
                            SelectPred & pred) /*throw (LogicError)*/;

/**
 * Get a service instance of a given type
 * In case more than one services are available on the 
 * given source, the first one is returned. It's up to the caller to delete the 
 * returned instance. In case no service is found, 0 is returned
 * @throws LogicError is preconditions are not met 
 */
Service * get_service_by_type(const std::string& type, 
                              const std::string& vo_name) /*throw (LogicError)*/;

/**
 * Get a service instance of the given type. A predicate can be specified in 
 * order to select the service to return in case more services are available. 
 * It's up to the caller to delete the returned instance. In case no service is 
 * found, 0 is returned
 * @throws LogicError is preconditions are not met 
 */
Service * get_service_by_type(const std::string& type, 
                              const std::string& vo_name,
                                    SelectPred & pred) /*throw (LogicError)*/;

/**
 * Return the Site Name from an Source Name, that can be either a site name or a 
 * service host name. In case of more than one services is available on the 
 * given source, the first one is returned. In case no service is found, the 
 * source name is returned
 * @throws LogicError is preconditions are not met 
 */    
std::string get_site_name(const std::string& source, 
                          const std::string& type, 
                          const std::string& vo_name) /*throw (LogicError)*/;

/**
 * Get a site name from an Source Name, that can be either a site name or a 
 * service host name. A predicate can be specified in order to select the 
 * service name ti return in case of more services are available on the given 
 * source. In case no service is found, the source name is returned
 * @throws LogicError is preconditions are not met 
 */    
std::string get_site_name(const std::string& source, 
                          const std::string& type, 
                          const std::string& vo_name,
                                SelectPred & pred) /*throw (LogicError)*/;
    
/**
 * Get the service of type <service_type> that is associated with 
 * the service on host <assoc_service_host> of type <assoc_service_type>. In 
 * case the endpoint is not found, an empty string is returned. In case more 
 * than one services are available the first one is returned. It's up to the 
 * caller to delete the returned instance.
 * In case no service is found, 0 is returned
 * @throws LogicError is preconditions are not met 
 */
Service * get_associated_service(const std::string& assoc_srv_host,
                                 const std::string& assoc_srv_type, 
                                 const std::string& srv_type,
                                 const std::string& vo_name) /*throw (LogicError)*/;

/**
 * Get the service of type <service_type> that is associated with 
 * the service on host <assoc_service_host> of type <assoc_service_type>. In 
 * case the endpoint is not found, an empty string is returned. A predicate can 
 * be specified in order to select the service to return in case more services  
 * are available. It's up to the caller to delete the returned instance.
 * In case no service is found, 0 is returned. 
 * @throws LogicError is preconditions are not met 
 */
Service * get_associated_service(const std::string& assoc_srv_host,
                                 const std::string& assoc_srv_type, 
                                 const std::string& srv_type,
                                 const std::string& vo_name, 
                                       SelectPred & pred) /*throw (LogicError)*/;

/**
 * Return a the value of a property for a given service. In case no property is 
 * an empty string is returned
 */
std::string get_property(const Service& srv, 
                         const std::string& prop_name, 
                         const std::string& vo_name) /*throw ()*/;

/**
 * Return a the value of of the Se Path for teh given SE. In case no property is 
 * an empty string is returned
 */
std::string get_se_path(const sd::Service& srv, 
                        const std::string& vo_name, 
                        const std::string& storage_class) /*throw ()*/;

} // End namespace sd
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_SD_SD_UTILITY_H_
