/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: url utility Methods
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: url-utility.h,v 1.2 2007/01/24 12:08:21 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_URL_UTILITY_H_
#define GLITE_DATA_AGENTS_URL_UTILITY_H_

#include <string>
#include <vector>

namespace glite {
namespace data  {
namespace agents{
    
/**
 * url escape a string
 */
std::string url_escape(const std::string& str);

/**
 * url escape a string
 */
std::string url_escape(std::string::const_iterator begin,
                       std::string::const_iterator end);

/**
 * path escape a string
 */
std::string path_escape(const std::string& str);

/**
 * path escape a string
 */
std::string path_escape(std::string::const_iterator begin,
                        std::string::const_iterator end);

/**
 * unescape a string with values hex encoded
 */
std::string hex_unescape(const std::string& str);

/**
 * unescape part of a string with values hex encoded
 */
std::string hex_unescape(std::string::const_iterator begin,
                         std::string::const_iterator end);

/**
 * Parse an url and extract the hostname, port, protocol, path and quuery string. 
 * Please note that this function is not generic and is intended to be used just
 * in the cases used by the FTA (services endpoint, SURLs and TURLs). The url 
 * should be in the format <protocol>://<host>[:port][/<path>[?<query_string]]
 */
void parse_url(const std::string& url,
                     std::string& hostname) /*throw (LogicError)*/;

/**
 * Parse an url and extract hostname, port, protocol, path and quuery string. 
 * Please note that this function is not generic and is intended to be used just
 * in the cases used by the FTA (services endpoint, SURLs and TURLs). The url 
 * should be in the format <protocol>://<host>[:port][/<path>[?<query_string]]
 */
void parse_url(const std::string & url,
                     std::string & hostname,
                     unsigned int& port,
                     std::string & protocol, 
                     std::string & path, 
                     std::string & query_str) /*throw (LogicError)*/;

/**
 * Parse an url and extract hostname, port, protocol and path. Please note that 
 * this function is not generic and is intended to be used just in the cases 
 * used by the FTA (services endpoint, SURLs and TURLs). The url should be in 
 * the format <protocol>://<host>[:port][/<path>
 */
void parse_url(const std::string & url,
                     std::string & hostname,
                     unsigned int& port,
                     std::string & protocol, 
                     std::string & path) /*throw (LogicError)*/;

/**
 * Parse an url and extract hostname, port and protocol. Please note that this 
 * function is not generic and is intended to be used just in the cases used by 
 * the FTA (services endpoint, SURLs and TURLs). The url should be in the format
 * <protocol>://<host>[:port][/<path>[?<query_string]]
 */
void parse_url(const std::string & url,
                     std::string & hostname,
                     unsigned int& port,
                     std::string & protocol) /*throw (LogicError)*/;

/**
 * Parse a query string and return the array of key value pairs. The query 
 * string should be in the format <key_1>=<value_1>&<key_2>=<value_3>...
 * Key an values are stored already unescaped
 */
void parse_query_string(const std::string& query_str,
                      std::vector<std::pair<std::string,std::string> >& values);

/**
 * Create an URL string
 */
std::string create_url(const std::string & protocol, 
                       const std::string & hostname,
                             unsigned int  port = 0, 
                       const std::string & path = "", 
                       const std::string & query_string = "", 
                             bool         escape_str = false) /*throw (LogicError)*/;

/**
 * Create an URL string
 */
std::string create_url(const std::string & protocol, 
                       const std::string & hostname,
                             unsigned int  port, 
                       const std::string & path, 
                       const std::vector<std::pair<std::string,std::string> >& values, 
                             bool          escape_str = false) /*throw (LogicError)*/;

/**
 * Parse an network endpoint and extract hostname, port. The endpoint should be 
 * in the format <host>[:port]
 */
void parse_network_endpoint(const std::string & str,
                                  std::string & hostname,
                                  unsigned int& port) /*throw (LogicError)*/;

/**
 * Parse an endpoint and extract hostname, port. The endpoint could be either an
 * URL (<protocol>://<host>[:port][/<path>]) or a network endpoint.
 */
void parse_endpoint(const std::string & str,
                          std::string & hostname,
                          unsigned int& port) /*throw (LogicError)*/;

} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_URL_UTILITY_H_
