/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Agent Exception Classes
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: AgentExceptions.cpp,v 1.4 2008/01/07 16:58:29 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/AgentExceptions.h"
#include <sstream>
#include <stdio.h>
#include <string.h>
#include <errno.h>

using namespace glite::data::agents;
    

//------------------------------------------------------------------------------
// Logic Errors
//------------------------------------------------------------------------------

/*
 * InvalidArgumentException
 *
 * Constructor
 */
InvalidArgumentException::InvalidArgumentException(const std::string& method,const std::string& arg, const std::string& reason):
    LogicError((std::string)"Invalid argument " + arg + " in method " + method 
               + ": " + reason){
}

/*
 * ConfigurationException
 *
 * Constructor
 */
ConfigurationException::ConfigurationException(const std::string& component,const std::string& reason):
    LogicError((std::string)"Configuration failed for component " + component 
               + ": " + reason){
}

namespace {
/*
 * get_config_param_error_reason
 *
 * Create the reason string for a ConfigParamException
 */
std::string get_config_param_error_reason(ConfigParamException::ErrorType t, const std::string& component, const std::string& param){
    std::stringstream reason;
    switch(t){
        case ConfigParamException::MISSING:
            reason << "Missing Parameter " << component << "." << param;
        break;
        case ConfigParamException::INVALID:
            reason << "Invalid format for parameter " << component << "." << param;
        break;
        case ConfigParamException::INVALID_VALUE:
            reason << "Invalid value for parameter " << component << "." << param;
        break;
        default:
            reason << "Unkwon error for parameter " << component << "." << param;
        break;
    }
    return reason.str();
}
/*
 * get_reason_from_errno
 *
 * Create the reason string for a posix error
 */
std::string get_reason_from_errno(const std::string& descr, int error){
    std::stringstream reason;
    const char * str_error = strerror(error);
    reason << descr;
    if(0 != str_error){
        reason << ": " << str_error;
    }  else {
        reason << ": (unknown reason). Code:" << error; 
    }
    return reason.str();
}


} // End global namespace

/*
 * ConfigParamException
 *
 * Constructor
 */
ConfigParamException::ConfigParamException(const std::string& component, const std::string& param_name, ErrorType t):
    ConfigurationException(get_config_param_error_reason(t,component,param_name)){
}
    
/*
 * PosixError
 *
 * Constructor
 */
PosixError::PosixError(const std::string& reason, int error):
    RuntimeError(get_reason_from_errno(reason,error)),
    m_error(error){
}
