/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Guid Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Guid.cpp,v 1.6 2008/11/14 16:04:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/Guid.h"
#include "glite/data/agents/AgentExceptions.h"

#include <glite/data/config/service/Logger.hxx>
#include <iostream>
#include <string>
#include <sstream>
#include <cstdlib>

using namespace glite::data::agents;
using namespace std;

namespace {
const char * const COMPONENT_NAME = "guid-utility";
}

/*
 * Guid
 *
 * Constructor
 */
Guid::Guid(){
    uuid_generate(m_id);
}

/*
 * Guid
 *
 * Constructor
 */
Guid::Guid(const std::string& str){
    // Create the GUID from the String
    try{
        fromString(str);
    } catch(const LogicError& exc){
        _log_error(COMPONENT_NAME,"Failed to parse Guid. Reason is " << exc.what());
    }
}

/*
 * Guid
 *
 * Constructor
 */
Guid::Guid(const Guid& other){
    // Copy a Guid
    uuid_copy(this->m_id,other.m_id);
}

/*
 * operator=
 *
 * Assignment Operator
 */
void Guid::operator=(const Guid& other) /*throw()*/{
    // Copy a Guid
    uuid_clear(this->m_id);
    uuid_copy(this->m_id,other.m_id);
}

/*
 * operator ==
 *
 * Comparison Operator
 */
bool Guid::operator==(const Guid& other) const /*throw ()*/{
    return (uuid_compare(this->m_id,other.m_id) == 0)?true:false; 
}

/*
 * operator <
 *
 * Comparison Operator
 */
bool Guid::operator<(const Guid& other) const /*throw ()*/{
    return (uuid_compare(this->m_id,other.m_id) < 0)?true:false; 
}

/*
 * operator >
 *
 * Comparison Operator
 */
bool Guid::operator>(const Guid& other) const /*throw ()*/{
    return (uuid_compare(this->m_id,other.m_id) > 0)?true:false; 
}

/*
 * isNull
 *
 * Chekc if the GUID is null
 */
bool Guid::isNull() const /*throw ()*/{
    return (uuid_is_null(m_id) == 1)?true:false;
}

/*
 * str
 *
 * Create a String
 */
std::string Guid::str() const /*throw ()*/{
    char id_str[37];
    uuid_unparse(m_id, id_str);
    return std::string(id_str);
}

/*
 * fromString
 *
 * Create a Guid from a String
 */
void Guid::fromString(const std::string& s) /*throw (LogicError)*/{
    // Clear the GUID
    uuid_clear(m_id);
    if(false == s.empty()){
        // Parse the String
        int result = uuid_parse( s.c_str(), m_id);
        if(-1 == result){
            throw LogicError("Invalid GUID String");
        }
    }
}

/*
 * hash
 *
 * compute the has of the Guid
 */
unsigned short Guid::hash() const /*throw ()*/ {

    // Compute 16-bit  hash value of the GUID.
    short  c0 = 0, c1 = 0, x, y;
    unsigned char *c = (unsigned char *) &m_id;
    unsigned int l = sizeof(m_id)/sizeof(unsigned char);
    for(unsigned int i = 0; i < l; i++) {
        c0 += *c++;
        c1 += c0;
    }

    //  Calculate the value for "First octet" of the hash
    x = -c1 % 255;
    if(x < 0) {
        x += 255;
    }

    // Calculate the value for "second octet" of the hash
    y = (c1 - c0) % 255;
    if(y < 0) {
        y += 255;
    }
    return (unsigned short)((y << 8) + x);
}

/*
 * crossSumHash
 *
 * compute the crossSum of the hash of the guid
 */
unsigned int Guid::crossSumHash() const /*throw()*/{

    unsigned int csum = 0;

    // compute the hash value
    unsigned short h = this->hash();
    std::stringstream ss;
    ss << h;
    std::string s = ss.str();
    std::string::iterator it;
    char c[2];
    c[0] = 0;
    c[1] = 0;
    for(it = s.begin(); it != s.end(); ++it){
        c[0] = (*it);
        csum += atoi(c);
        csum %= 10;
    }
    return csum;
}
