/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: CredService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CredService.cpp,v 1.9 2008/11/17 10:21:08 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/cred/CredService.h"
#include "glite/data/agents/cred/CredServiceException.h"
#include "glite/data/agents/cred/cred-utility.h"
#include "glite/data/agents/extra/TempFile.h"
#include "glite/data/agents/extra/Handle.h"

#include <glite/data/config/service/Logger.hxx>

#ifdef __cplusplus
extern "C"{
#endif //__cplusplus
// Fix Warning
#ifdef IOV_MAX 
#undef IOV_MAX
#endif //IOV_MAX
#include "globus_gsi_credential.h"
#ifdef __cplusplus
}
#endif //__cplusplus
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <boost/scoped_ptr.hpp>

using namespace glite::data::agents;
using namespace glite::data::agents::cred;

using boost::scoped_ptr;

namespace {
const char * const TMP_DIRECTORY = "/tmp";
}

/*
 * CredService
 *
 * Constructor
 */
CredService::CredService():
    m_logger("cred-service"){
}

/*
 * CredService
 *
 * Constructor
 */
CredService::CredService(const glite::config::Logger& l):
    m_logger(l){
}
   
/**
 * get
 * Get The proxy Certificate for the requested user
 */
void CredService::get(
    const std::string&  userDn, 
    const std::string&  id, 
    std::string&        filename) /*throw (LogicError, CredServiceException)*/{

    // Check Preconditions
    if(userDn.empty()){
        m_log_error("Invalid User DN specified");
        throw InvalidArgumentException("Invalid User DN specified");
    }
    if(id.empty()){
        m_log_error("Invalid credential id specified");
        throw InvalidArgumentException("Invalid credential id specified");
    }
    
    // Get the filename to be for the given DN
    std::string fname = getFileName(userDn,id);
    // Post-Condition Check: filename length should be max (FILENAME_MAX - 7)
    if(fname.length() > (FILENAME_MAX - 7)){
        m_log_error("Invalid credential file name generated: length exceeded");
        throw LogicError("Invalid credential file name generated");
    }
    
    // Check if the Proxy Certificate is already there and it's valid
    if(true == isValidProxy(fname)){
        filename = fname;
        m_log_debug("Proxy Certificate is already on file " << filename);
        return;
    }
    
    // Create a Temporary File
    Handle h;
    std::string tmp_proxy_fname;
    try{
        tmp_proxy_fname = TempFile::generate("cred",TMP_DIRECTORY,h.get());
    } catch(const AgentException& exc){
        m_log_error("Cannot create temporary file: " << exc.what());
        throw CredServiceException("Cannot create file for proxy certificate");
    }
    // RAII
    TempFile tmp_proxy(tmp_proxy_fname);

    // Get certificate form MyProxy Server
    try{
        getNewCertificate(userDn,id,tmp_proxy.name());
    } catch(...){
        throw;
    }

    // Rename the Temporary File    
    try{
        tmp_proxy.rename(fname);
    } catch(const AgentException& exc){
        m_log_error("Cannot rename temporary file <" << tmp_proxy.name() << ">: " << exc.what());
        throw CredServiceException("Cannot create file for proxy certificate");
    }
    // Return the proxy certificate File name
    filename = fname;
}

/*
 * isValidProxy
 *
 * Check if the Proxy Already Exists and is Valid. 
 */
bool CredService::isValidProxy(const std::string& filename){

    // Check if it's valid
    time_t lifetime = get_proxy_lifetime(filename);
    if(lifetime < 0){
        m_log_debug("Proxy Certificate expired");
        return false;
    }
    m_log_debug("Lifetime       : " << lifetime);
    m_log_debug("Min Valid  time: " << this->minValidityTime());

    // casting to unsigned long is safe, condition lifetime < 0 already checked
    if(this->minValidityTime() >= (unsigned long)lifetime){ 
        m_log_debug("Proxy Certificate should be renewed");
        return false;
    }
    m_log_debug("Proxy Certificate is still valid");
    return true;
}
