/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Cred Utility Functions
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: cred-utility.cpp,v 1.12 2008/11/17 10:21:08 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/cred/cred-utility.h"
#include "glite/data/agents/cred/CredServiceFactory.h"
#include "glite/data/agents/sd/sd-utility.h"
//#include "SDConfig.h"
#include <glite/data/config/service/Logger.hxx>

#ifdef __cplusplus
extern "C"{
#endif //__cplusplus
// Fix Warning
#ifdef IOV_MAX 
#undef IOV_MAX
#endif //IOV_MAX
#include "globus_gsi_credential.h"
#ifdef __cplusplus
}
#endif //__cplusplus

#include <boost/scoped_ptr.hpp>
#include <unistd.h>


using boost::scoped_ptr;

using glite::data::agents::sd::get_associated_service;
using glite::data::agents::sd::Service;
//using glite::data::agents::sd::SDConfig;

namespace {
const char * const CRED_MODULE = "agents-cred";    
}

namespace glite    {
namespace data     {
namespace agents   {
namespace cred     {
    
/*
 * get_proxy_dn
 *
 * Get Proxy DN
 */
std::string get_proxy_dn(const std::string& filename) /*throw (CredServiceException)*/{
    
    std::string dn;
    
    // Activate Module
    globus_module_activate(GLOBUS_GSI_CREDENTIAL_MODULE);
    
    // Check if it's valid
    globus_gsi_cred_handle_t        proxy_handle = 0;
    globus_gsi_cred_handle_attrs_t  handle_attrs = 0;
    try {
        // Init handle attributes     
        globus_result_t result = globus_gsi_cred_handle_attrs_init(&handle_attrs);
        if(0 != result){
            throw CredServiceException("Cannot Init Handle Attributes");
        }
        // Init handle
        result = globus_gsi_cred_handle_init(&proxy_handle,handle_attrs);
        if(0 != result){
            throw CredServiceException("Cannot Init Handle");
        }
        if(false == filename.empty()){
            _log_trace(CRED_MODULE,"Reading proxy certificate " << filename);
            // Load Proxy
            result = globus_gsi_cred_read_proxy(proxy_handle,const_cast<char *>(filename.c_str()));
            if(0 != result){
                throw CredServiceException("Cannot Load Proxy File");
            }
        } else {
            _log_trace(CRED_MODULE,"Reading default certificate");
            // Read Default
            result = globus_gsi_cred_read(proxy_handle,0);
            if(0 != result){
                throw CredServiceException("Cannot Read Proxy");
            }
        }
        _log_trace(CRED_MODULE,"Get the Subject Name for agent certificate");
        char * subject_name = 0;
        result = globus_gsi_cred_get_subject_name(proxy_handle,&subject_name);
        if(0 != result){
            throw CredServiceException("Cannot get Subject Name");
        }
        if(0 != subject_name){
            dn = subject_name;
            free(subject_name);
        }
    } catch(...){
        // Destroy handles
        if(0 != proxy_handle){
            globus_gsi_cred_handle_destroy(proxy_handle);
        }
        if(0 != handle_attrs){
            globus_gsi_cred_handle_attrs_destroy(handle_attrs);
        }
        // Deactivate Module
        globus_module_deactivate(GLOBUS_GSI_CREDENTIAL_MODULE);
        throw;
    }
    
    // Destroy handles
    if(0 != proxy_handle){
        globus_gsi_cred_handle_destroy(proxy_handle);
    }
    if(0 != handle_attrs){
        globus_gsi_cred_handle_attrs_destroy(handle_attrs);
    }
    // Deactivate Module
    globus_module_deactivate(GLOBUS_GSI_CREDENTIAL_MODULE);

    return dn;
}

/*
 * get_proxy_cert
 *
 * Get Proxy Certificate File Name for the given job
 */
std::string get_proxy_cert(const std::string& user_dn,
                           const std::string& user_cred,
                           const std::string& vo_name,
                                 std::string& cred_service_endpoint,
                           const std::string& assoc_service,
                           const std::string& assoc_service_type,
                                 bool         disable_delegation,
                           const std::string& cred_service_type) /*throw (LogicError, CredServiceException)*/ {
    std::string proxy_file;

    // Check if the job contains information for retrieving delegated credential 
    if((disable_delegation == false) && 
       (false == user_dn.empty()) ) {
        _log_trace(CRED_MODULE,"Get the Proxy Certificate for that user");
           
        // Get CertProxy Service                           
        scoped_ptr<CredService> cred_service;
        try {
            // Get the Proxy Credential Factory for the given type
            CredServiceFactory& factory = CredServiceFactory::instance(cred_service_type);
            
            // Check Params
            if(true == cred_service_endpoint.empty()){
                // Get the appropriate Selector
                scoped_ptr<sd::SelectPred> pred(factory.getServiceSelector(vo_name));
                // Check For Associated Service host
                if((false == assoc_service.empty()) && (false == assoc_service_type.empty())){
                    _log_trace(CRED_MODULE,"Finding Assoc Service <" << assoc_service << "> ");
                    scoped_ptr<Service> assoc_srv;
                    if(0 == pred.get()){
                        assoc_srv.reset(get_associated_service(assoc_service, 
                            assoc_service_type,
                            factory.getServiceType(),
                            vo_name));
                    } else {
                        assoc_srv.reset(get_associated_service(assoc_service, 
                            assoc_service_type,
                            factory.getServiceType(),
                            vo_name,
                            *pred));
                    }
                    if(0 != assoc_srv.get()){
                        cred_service_endpoint = assoc_srv->endpoint;
                    }
                }
            }
            if(false == cred_service_endpoint.empty()){
                _log_trace(CRED_MODULE,"Cred server endpoint is <" << cred_service_endpoint << ">");
            } else {
                _log_trace(CRED_MODULE,"No proxy server specified");
            }
            // Create Instance
            cred_service.reset(factory.create(cred_service_endpoint));
            // Get Proxy Certificate
            cred_service->get(user_dn,user_cred,proxy_file);
            _log_trace(CRED_MODULE,"Proxy Certificate is " << proxy_file);
        } catch(const CredServiceException& exc){
            _log_trace(CRED_MODULE,"Cannot Retrieve Proxy Certificate");
            throw;
        }
    } else {
        // Don't Use Delegated Credentials
        _log_trace(CRED_MODULE,"Delegated Credentials not used");
    }
    return proxy_file;
}

/*
 * get_proxy_lifetime
 *
 * Return the proxy lifetime. In case of other errors, returns -1
 */
time_t get_proxy_lifetime(const std::string& filename) /*throw ()*/ {
    
    time_t lifetime = (time_t)-1;

    // Check that the file Exists
    int result = access(filename.c_str(), R_OK);
    if(0 != result){
        _log_trace(CRED_MODULE,"Requested Proxy doesn't exist. A new one should be created. Reason is " << strerror(errno));
        return lifetime;
    }
    // Check if it's valid
    globus_gsi_cred_handle_t        proxy_handle = 0;
    globus_gsi_cred_handle_attrs_t  handle_attrs = 0;
    try {
        // Init handle attributes     
        globus_result_t result = globus_gsi_cred_handle_attrs_init(&handle_attrs);
        if(0 != result){
            throw RuntimeError("Cannot Init Handle Attributes");
        }
        // Init handle
        result =  globus_gsi_cred_handle_init(&proxy_handle,handle_attrs);
        if(0 != result){
            throw RuntimeError("Cannot Init Handle");
        }
        // Load Proxy
        result = globus_gsi_cred_read_proxy(proxy_handle,const_cast<char *>(filename.c_str()));
        if(0 != result){
            throw RuntimeError("Cannot Load Proxy File");
        }
        // Get Lifetime (in seconds)
        result = globus_gsi_cred_get_lifetime(proxy_handle,&lifetime);
        if(0 != result){
            throw RuntimeError("Cannot Get Proxy Lifetime");
        }
    } catch(const RuntimeError& exc) {
        _log_trace(CRED_MODULE,"Cannot Check Proxy Validity. Reason is: " << exc.what());
        lifetime = (time_t)-1;
    }
    // Destroy handles
    if(0 != proxy_handle){
        globus_gsi_cred_handle_destroy(proxy_handle);
    }
    if(0 != handle_attrs){
        globus_gsi_cred_handle_attrs_destroy(handle_attrs);
    }
    return lifetime;
}

} // end namespace cred
} // end namespace agents
} // end namespace data
} // end namespace glite
