/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: MyProxyClient Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: MyProxyClient.cpp,v 1.6 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "MyProxyClient.h"
#include "MyProxyClientFactory.h"
#include "MyProxyClientException.h"
#include "glite/data/agents/url-utility.h"
#include "glite/data/agents/extra/utility.h"

#include <glite/data/config/service/Logger.hxx>

#ifdef __cplusplus
extern "C"{
#endif //__cplusplus
#include "myproxy.h"
#include "myproxy_delegation.h"
#ifdef __cplusplus
}
#endif //__cplusplus
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

using namespace glite::data::agents;
using namespace glite::data::agents::cred;
using namespace glite::data::agents::cred::myproxy;


namespace {
const char * const PROXY_NAME_PREFIX         = "x509up_dn";
const char * const PROXY_NAME_PREFIX_HASHED  = "x509up_h";
}

/*
 * MyProxyClient
 *
 * Constructor
 */
MyProxyClient::MyProxyClient(const MyProxyClientFactory& f, const std::string& server):
    CredService(f.logger()),
    m_factory(f),
    m_server(server){
}
    
/*
 * ~MyProxyClient
 *
 * Destructor
 */
MyProxyClient::~MyProxyClient(){
}

/**
 * getFromServer
 * Get the Cerificate From the MyProxy Server
 */
void MyProxyClient::getNewCertificate(
    const std::string& userDn, 
    const std::string& passphrase, 
    const std::string& filename) /*throw (MyProxyClientException)*/{
        
    myproxy_socket_attrs_t *socket_attrs    = 0;
    myproxy_request_t      *client_request  = 0;
    myproxy_response_t     *server_response = 0;

    try{
        // Clear MyProxy Error Context
        verror_clear();

        // Create myproxy_socket_attrs
        socket_attrs = (myproxy_socket_attrs_t*)malloc(sizeof(myproxy_socket_attrs_t));
        memset(socket_attrs, 0, sizeof(myproxy_socket_attrs_t));
        
        // Create myproxy_request_t
        client_request = (myproxy_request_t*) malloc(sizeof(myproxy_request_t));
        memset(client_request, 0, sizeof(myproxy_request_t));
        
        // Create myproxy_response_t
        server_response = (myproxy_response_t*)malloc(sizeof(myproxy_response_t));
        memset(server_response, 0, sizeof(myproxy_response_t));
    
        // Set Defaults
        myproxy_set_delegation_defaults(socket_attrs, client_request);
       
        // Set user DN
        client_request->username = strdup(userDn.c_str());
        // Set user Passphrase
        strncpy(client_request->passphrase,passphrase.c_str(),MAX_PASS_LEN);
                              
        // Set Proxy Lifeime                          
        client_request->proxy_lifetime = m_factory.proxyLifetime();
        
        // Get Server Host
        std::string server_host;
        unsigned int server_port = 0;
        parseServerEndpoint(server_host,server_port);
        if(true == server_host.empty()){
            // Get Server From ServiceDiscovery
            m_factory.getServer(server_host,server_port);
        }    
        const char * server = (false == server_host.empty()) ? server_host.c_str(): socket_attrs->pshost;
        if ((0 == server) || (0 == strlen(server))) {
            m_log_error("No MyProxy Server Host Name specified");
            throw MyProxyClientException("Invalid MyProxy Server");
        }
        
        // Set HostName
        socket_attrs->pshost = strdup(server);
                       
        // Set Port
        if(0 == server_port){
            // Check if the default host name contains a port number
            char * p = strchr(socket_attrs->pshost, ':');
            if(0 != p) {
                *p++ = '\0';
                socket_attrs->psport = atoi(p);
            } else {
              // Use the default port
              socket_attrs->psport = MYPROXY_SERVER_PORT;
            }
        } else {
            // Use the configured port
            socket_attrs->psport = server_port;
        }   
        m_log_debug("Contacting MyProxy Server on " << socket_attrs->pshost << ":" << socket_attrs->psport);
        m_log_debug("For dn " << client_request->username << " - filename " << filename);
        
        // Get the Proxy from the server
#if (MYPROXY_VERSION_MAJOR == 0) && (MYPROXY_VERSION_MINOR == 6) && (MYPROXY_VERSION_MICRO == 2)
        int r = myproxy_get_delegation(socket_attrs, client_request, server_response, const_cast<char*>(filename.c_str()));
#else        
        int r = myproxy_get_delegation(socket_attrs, client_request, 0 , server_response, const_cast<char*>(filename.c_str()));
#endif        
        if (r == 1) {
            const char * err = verror_get_string();
            const char * reason = (0!=err)?err:"(null)";
            m_log_error("Failed to get the proxy from the MyProxyServer for DN " << client_request->username << ". Reason is:[" << verror_get_errno() << ":" << verror_get_value() << "] " << reason);
            std::string error_reason = (std::string)"Failed to get proxy certificate from " +
                ((0 != socket_attrs->pshost)?socket_attrs->pshost:
                 (0 != getenv("MYPROXY_SERVER"))?getenv("MYPROXY_SERVER"):"unknown server") +
                " . Reason is " + ((0 != err)?err:"(null)"); 
            throw MyProxyClientException(error_reason.c_str());
        }
    } catch(...){
        // Release Memory
        myproxy_free(socket_attrs, client_request, server_response);
        throw;
    }

    // Release Memory
    myproxy_free(socket_attrs, client_request, server_response);
}

    
/*
 * getFileName
 *
 * Generate a Name for the Certificate Proxy File
 */
std::string MyProxyClient::getFileName(const std::string& userDn, const std::string& /*Not used*/) /*throw(LogicError)*/{

    std::string filename;
    
    // Encode the DN
    std::string encoded_dn =  urlEncode(userDn);

    // Compute Max length
    unsigned long filename_max = pathconf(m_factory.repository().c_str(), _PC_NAME_MAX);
    long max_length = (filename_max - 7 - strlen(PROXY_NAME_PREFIX));
    if(max_length <= 0){
        m_log_error("Failed to generate the proxy file name: prefix name (" << PROXY_NAME_PREFIX << ") too long for " << m_factory.repository());
        throw LogicError("Cannot generate proxy file name: prefix too long");
    }
    
    // Check the length of the filename
    // casting is safe, condition lifetime < 0 already checked
    if(encoded_dn.length() < (std::string::size_type)max_length){
        // Filename is Repository + Proxy Name Prefix + the userDN URL encoded
        filename = m_factory.repository() + PROXY_NAME_PREFIX + encoded_dn;
    } else {
        // Hash the DN
        unsigned long h = hash_string(userDn);
        // Copy the Hash and the DN
        std::stringstream ss;
        ss << h;
        std::string h_str = ss.str();
        if(h_str.length() > (std::string::size_type)max_length){
             m_log_error("Failed to generate the proxy file name: hash (" << h_str << ") too long for " << m_factory.repository());
           throw LogicError("Cannot generate proxy file name: hash too long");
        } else {
            // Generate the filename using the hash
            filename = m_factory.repository() + PROXY_NAME_PREFIX_HASHED + h_str;
            if(h_str.length() < (std::string::size_type)max_length){
                filename.append(encoded_dn.substr(0,(max_length - h_str.length())));
            }
        }
    }
    return filename;
}

/*
 * minValidityTime
 *
 * Returns the validity time that the cached copy of the certificate should 
 * have. 
 */
unsigned long MyProxyClient::minValidityTime() /* throw() */{
    return m_factory.minValidityTime();
}


/*
 * urlEncode
 *
 * Url Encode a string
 */
std::string MyProxyClient::urlEncode(const std::string& str){

    std::string encoded;
    encoded.reserve(3*str.length());
    
    // for each character
    std::string::const_iterator s_it;
    for(s_it = str.begin(); s_it != str.end(); ++s_it){
        if (isalnum((*s_it))){
            encoded.push_back(tolower((*s_it)));
        } else {
            char e[4];
            // Note: use the 'X' character instead of '%' since it seems globus
            // don't like it. In any case all the chars are always converted
            // to lower case, so there's no ambiguity
            //snprintf(e,4, "%%%02x",(*s_it));
            snprintf(e,4, "X%02x",(*s_it));
            encoded.append(e,3);
        }        
    }
    return encoded;
}

/*
 * parseServerEndpoint
 *
 * Parse the server endpoint. It can be in the format: 
 *      myproxy://<host>[:<port>]
 *      myproxy:<host>[:<port>]
 *      <host>[:<port>]
 */
void MyProxyClient::parseServerEndpoint(std::string& server_host,unsigned int& server_port){
        
    if(true == m_server.empty()){
        m_log_debug("No job endpoint specified");
        return;
    }

    m_log_debug("job specified myproxy endpoint is <" << m_server << ">");

    // Get Server Host and port
    parse_endpoint(m_server,server_host,server_port);

    m_log_debug("job specified myproxy server is <" << server_host << ":" << server_port << ">");
}
