/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: MyProxyClientFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: MyProxyClientFactory.cpp,v 1.7 2008/11/14 16:04:48 ptedesco Exp $ 
 *  Release: $Name:  $
 *
 */
#include "MyProxyClientFactory.h"
#include "MyProxyClient.h"
#include "MyProxyClientException.h"
#include <glite/data/config/service/Logger.hxx>
#include "glite/data/config/service/ParamValue.hxx"
#include "glite/data/agents/AgentExceptions.h"
#include "glite/data/agents/sd/sd-utility.h"
#include "glite/data/agents/sd/sd-predicates.h"
#include "glite/data/agents/url-utility.h"
#include "glite/data/agents/sd/SDConfig.h"

// Fix Warning
#ifdef IOV_MAX 
#undef IOV_MAX
#endif //IOV_MAX
#include "globus_gsi_credential.h"

#include <sys/types.h>
#include <dirent.h>
#include <boost/scoped_ptr.hpp>

using namespace glite::data::agents;
using namespace glite::data::agents::sd;
using namespace glite::data::agents::cred;
using namespace glite::data::agents::cred::myproxy;
using namespace glite::config;

using boost::scoped_ptr;

/*
 * Constants
 */
namespace {
const char * const MYPROXY_CLIENT_NAME           = "cred-myproxy";
                                                                                                                          
// Properties Names
const char * const SERVER_PROPERTY_NAME          = "Server";
const char * const PORT_PROPERTY_NAME            = "Port";
const char * const PROXYLIFETIME_PROPERTY_NAME   = "ProxyLifetime";
const char * const REPOSITORY_PROPERTY_NAME      = "Repository";
const char * const MINVALIDITYTIME_PROPERTY_NAME = "MinValidityTime";
                                                                                                                          
// Properties Default  Values
const unsigned int  DEFAULT_PORT                 = 0;            // Use the default one
const unsigned long DEFAULT_PROXYLIFETIME        = 60 * 60 * 12; // 12 hours
const unsigned long DEFAULT_MINVALIDITYTIME      = 60 * 60;      // 1 hour
const char *  const DEFAULT_REPOSITORY           = "/tmp/";
const char *  const MYPROXY_CREDSERVICETYPE      = "myproxy";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new MyProxyClientFactory();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    MyProxyClientFactory * factory = dynamic_cast<MyProxyClientFactory*>(component);
    if(0 != factory){
        delete factory;
    }
}
                                                                                
} // End extern "C"
    
/*
 * MyProxyClientFactory
 *
 * Constructor
 */
MyProxyClientFactory::MyProxyClientFactory():CredServiceFactory(),
        ComponentConfiguration(MYPROXY_CLIENT_NAME), 
        m_logger(MYPROXY_CLIENT_NAME),
        m_port(DEFAULT_PORT),
        m_proxyLifetime(DEFAULT_PROXYLIFETIME),
        m_minValidityTime(DEFAULT_MINVALIDITYTIME),
        m_repository(DEFAULT_REPOSITORY),
        m_myproxyType(MYPROXY_CREDSERVICETYPE){
    this->registerFactory(MYPROXY_CREDSERVICETYPE);
}
                                                                                                                          
/*
 * ~MyProxyClientConfig
 *
 * Destructor
 */
MyProxyClientFactory::~MyProxyClientFactory(){
    this->deregisterFactory(MYPROXY_CREDSERVICETYPE);
}

/*
 * init
 *
 * Initialize the Component
 */
int MyProxyClientFactory::init(const Params& params){
                                                                                                                          
    try{
        Params::const_iterator it;
        
        // Get MyProxy Server Host
        if((it = params.find(SERVER_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),SERVER_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_server = pv->getValue();
        } 
        
        // Get My Proxy Server Port
        if((it = params.find(PORT_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),PORT_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_port = atol(pv->getValue().c_str());
        }
        
        // Get Proxy Lifetime
        if((it = params.find(PROXYLIFETIME_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),PROXYLIFETIME_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_proxyLifetime = atol(pv->getValue().c_str());
        }
        
        // Get Proxy Lifetime
        if((it = params.find(MINVALIDITYTIME_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),MINVALIDITYTIME_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_minValidityTime = atol(pv->getValue().c_str());
        }
        
        // Get Proxy Repository
        if((it = params.find(REPOSITORY_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),REPOSITORY_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            // Check That Repository is Valid
            DIR * dir = opendir(pv->getValue().c_str());
            if(0 == dir){
                throw ConfigParamException(getName(),REPOSITORY_PROPERTY_NAME,ConfigParamException::INVALID_VALUE);
            } else {
                closedir(dir);
                m_repository = pv->getValue();
            }
        }
        // Check thet repository end with a slash
        if('/' != *(m_repository.rbegin())){
            m_repository.push_back('/');
        }
        
        // Log Configuration Values
        m_log_info(getName() << " Initialized. Initialization Parameters are");
        m_log_info("Server          : " << m_server);
        m_log_info("Port            : " << m_port);
        m_log_info("ProxyLifetime   : " << m_proxyLifetime);
        m_log_info("MinValidityTime : " << m_minValidityTime);
        m_log_info("Repository      : " << m_repository);
        
        // Activate Globus GSI Credential Module
        globus_module_activate(GLOBUS_GSI_CREDENTIAL_MODULE);    
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        throw;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        throw;
    }
    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int MyProxyClientFactory::config(const Params& params){
    // Nothing to do
    return 0;
}

/*
 * fini
 *
 * Finalize the Component
 */
int MyProxyClientFactory::fini(){
    // Deativate Globus GSI Credential Module
    globus_module_deactivate(GLOBUS_GSI_CREDENTIAL_MODULE);

    return 0;
}

/*
 * getTag
 *
 * Return the tag to be used to search the myproxy server in the job param
 */
const std::string& MyProxyClientFactory::getType() /*throw ()*/{
    return m_myproxyType;    
}

/*
 * getServiceType
 *
 * Return the Cred Service Type to be used to query ServiceDiscovery.
 */
const std::string& MyProxyClientFactory::getServiceType() /*throw ()*/{
    return SDConfig::instance().myproxy();
}

/*
 * getServiceSelector
 *
 * Return the SD Selector predicate for choosing the correct CredService Service
 * when more than one are available
 */
SelectPred * MyProxyClientFactory::getServiceSelector(const std::string& vo_name) /*throw()*/{
    return new SelectMyProxyForRetrieval(vo_name);
}


/*
 * create
 *
 * Create a CredService instance
 */
CredService * MyProxyClientFactory::create(const std::string& endpoint) /*throw (LogicError)*/{
    // Return new Instance
    return new MyProxyClient(*this, endpoint);
}

/*
 * getServer
 * 
 * Return the information on the host and port to use to contact the MyProxy 
 * Server
 */
void MyProxyClientFactory::getServer(std::string& server_host, unsigned int& server_port) const{
    
    // Set Server Host
    server_host = m_server;
    server_port = m_port;
    if(true == server_host.empty()){
        try{ 
            // User the correct predicate
            SelectMyProxyForRetrieval pred("");
            m_log_debug("Get Default Myproxy Server");
            // No Associated Service found: Get the first entry from SD
            scoped_ptr<Service> svc(get_service_by_type(
                SDConfig::instance().myproxy(),"",pred));
            if(0 == svc.get()){
                throw MyProxyClientException("No service found");
            }
            m_log_debug("MyProxy endpoint is " << svc->endpoint);
            // Get Server Host and port
            server_host = svc->hostname;
            server_port = svc->port;
        } catch(const MyProxyClientException& exc){
            m_log_debug("No MyProxy Server registered. Reason: " << exc.what());
            m_log_debug("Use the default one");
        }
    }
    m_log_debug("Using MyProxy Server <" << server_host << ":" << server_port << ">");
}
