/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: MyProxyClientFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: MyProxyClientFactory.h,v 1.5 2008/11/17 10:21:08 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_COMMON_CRED_MYPROXYCLIENTFACTORY_H_
#define GLITE_DATA_AGENTS_COMMON_CRED_MYPROXYCLIENTFACTORY_H_

#include <glite/data/config/service/Logger.hxx>
#include "glite/data/agents/cred/CredServiceFactory.h"
#include "glite/data/config/service/ComponentConfiguration.hxx"

namespace glite  {
namespace data   {
namespace agents {
namespace cred   {
namespace myproxy{

/**
 * Configure MyProxy Client library
 */
class MyProxyClientFactory : public CredServiceFactory, 
                             public glite::config::ComponentConfiguration{
public:

    /**
     * Constructor
     */
    MyProxyClientFactory();
        
    /**
     * Destructor
     */
    virtual ~MyProxyClientFactory();

    // =========================================================================
    // CertProxyServiceFactory Methods
    // =========================================================================

    /**
     * Return the CredService Type.
     * @return the type of the CredService
     */
    virtual const std::string& getType() /*throw ()*/; 

    /**
     * Return the CertProxy Service Type to be used to query ServiceDiscovery.
     * @return the CertProxy Service Type to be used to query ServiceDiscovery 
     */
    virtual const std::string& getServiceType() /*throw ()*/;

    /**
     * Return the SD Selector predicate for choosing the correct CredService Service
     * when more than one are available
     * @return the SD Service Selector. 0 means use the default
     */
    virtual sd::SelectPred * getServiceSelector(const std::string& vo_name) /*throw()*/;

    /**
     * Create a CredService instance
     * @param endpoint [IN] the service hostname and port to contact. If this 
     * parameter is empty, ServiceDiscovery will be used to find the endpoint 
     * of the Server to contact
     * @return a new instance of a CredService
     * @throws LogicError in case of error
     */
    virtual CredService * create(const std::string& endpoint) /*throw (LogicError)*/; 

    // ============================================m_associatedService=============================
    // ComponentConfiguration Methods
    // =========================================================================

    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finalize the Component
     */
    virtual int fini();

    // =========================================================================
    // Properties
    // =========================================================================
    
    /**
     * Get The Logger Object
     */
    const glite::config::Logger& logger() const {return m_logger;}

    /**
     * Get Proxy Lifetime Property
     */
    unsigned long proxyLifetime() const { return m_proxyLifetime;}
    
    /**
     * Get Minimum Valibity Time for a Proxy Certificate 
     */
    unsigned long minValidityTime() const { return m_minValidityTime;}
    
    /**
     * Get Proxy  Certificates Repository Property
     */
    const std::string& repository() const { return m_repository;}

    // =========================================================================
    // Methods
    // =========================================================================
    
    /**
     * Return the information on the host and port to use to contact the MyProxy 
     * Server
     */
    void getServer(std::string& server_host, unsigned int& server_port) const; 
   
private:
    /**
     * The Logger
     */
    mutable glite::config::Logger m_logger;
    
    /**
     * MyProxy Server Host
     */
    std::string m_server;
    
    /**
     * MyProxy Server Port
     */
    unsigned int m_port;
    
    /**
     * Proxy Certificate Lifetime
     */
    unsigned long m_proxyLifetime;

    /**
     * Minimum Valibity Time for a Proxy Certificate 
     */
    unsigned long m_minValidityTime;
    
    /**
     * Proxy Certificates Repository
     */
    std::string m_repository;

    /**
     * MyProxy job param tag
     */
    std::string m_myproxyType;

    /** @link dependency 
     * @label creates*/
    /*# MyProxyClient lnkMyProxyClient; */

};

} // end namespace myproxy
} // end namespace cred
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_COMMON_CRED_MYPROXYCLIENTFACTORY_H_
