/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: OracleDAOComponent Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: OracleDAOComponent.cpp,v 1.7 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleDAOComponent.h"
#include "OracleDAOConfig.h"
#include "glite/data/config/service/ParamValue.hxx"
#include "glite/data/agents/AgentExceptions.h"
#include <glite/data/config/service/Logger.hxx>

#include <time.h>
#include <memory>
#include <boost/scoped_ptr.hpp>
#include <algorithm>
#include <functional>


using namespace glite::data::agents;
using namespace glite::data::agents::dao::oracle;
using namespace glite::config;


using boost::scoped_ptr;

/*
 * Constants
 */
namespace {
const char * const ORACLE_DAO_NAME                = "agents-dao-oracle";
                                                                                                                          
// Properties Names
const char * const USER_PROPERTY_NAME             = "User";
const char * const PASSWORD_PROPERTY_NAME         = "Password";
const char * const CONNECTSTRING_PROPERTY_NAME    = "ConnectString";
const char * const STMTCACHESIZE_PROPERTY_NAME    = "StatementCacheSize";
const char * const THREADED_PROPERTY_NAME         = "Threaded";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new OracleDAOComponent();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    OracleDAOComponent * o = dynamic_cast<OracleDAOComponent *>(component); 
    if(0 != o){
        delete o;
    }
}
                                                                                
} // End extern "C"

/*
 * OracleDAOComponent
 *
 * Constructor
 */
OracleDAOComponent::OracleDAOComponent():
    ComponentConfiguration(ORACLE_DAO_NAME), 
    m_logger(ORACLE_DAO_NAME){
}

/*
 * ~OracleDAOComponent
 *
 * OracleDAOComponent
 */
OracleDAOComponent::~OracleDAOComponent(){
}

/*
 * init
 *
 * Initialize the Component
 */
int OracleDAOComponent::init(const Params& params){

    // Better to set the TZ, since it will be used in the conversions
    tzset();
    
    // Get OracleDAOConfig instance    
    OracleDAOConfig& dao_config = OracleDAOConfig::instance();
    try{
        Params::const_iterator it;
        
        // Get DB ConnectString
        if((it = params.find(CONNECTSTRING_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CONNECTSTRING_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            // Set Connect String
            dao_config.connectString(pv->getValue());
        } else {
            throw ConfigParamException(getName(),CONNECTSTRING_PROPERTY_NAME, ConfigParamException::MISSING);
        }
        
        // Get DB User
        if((it = params.find(USER_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),USER_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            // Set User Name
            dao_config.user(pv->getValue());
        } else {
            throw ConfigParamException(getName(),USER_PROPERTY_NAME, ConfigParamException::MISSING);
        }
        
        // Get DB Password
        if((it = params.find(PASSWORD_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),PASSWORD_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            // Set Password
            dao_config.password(pv->getValue());
        } else {
            throw ConfigParamException(getName(),PASSWORD_PROPERTY_NAME, ConfigParamException::MISSING);
        }

        // Get Statement Cache Size
        if((it = params.find(STMTCACHESIZE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),STMTCACHESIZE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            // Set Statement Cache Size
            dao_config.stmtCacheSize(atol(pv->getValue().c_str()));
        } 
        
        // Threaded
        if((it = params.find(THREADED_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),THREADED_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            // Copy string without case
            std::string tm = pv->getValue();
            std::transform(tm.begin(),tm.end(),tm.begin(),tolower);
            dao_config.threadingModel(tm);
        }

        // Initialize DAO Config
        dao_config.init();
        
        // Log Configuration Values
        m_log_info(getName() << " Initialized");
        
    } catch(const LogicError& exc) {
        m_log_error("Initialization Logic Error for " << getName() << ": " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error("Unexpected Exception during " << getName() << " initialization: " << exc.what());
        throw;
    } catch(...) {
        m_log_error("Unhandled Exception during " << getName() << " initialization");
        throw;
    }
    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int OracleDAOComponent::config(const Params& params){

    // Nothing to do    
    m_log_info(getName() << " Configured");
    return 0;
}

/*
 * fini
 *
 * Finalize the Component
 */
int OracleDAOComponent::fini(){

    // Finalize DAOConfig
    OracleDAOConfig::instance().fini();
    m_log_info(getName() << " Finalized");
    
    return 0;
}
