/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: OracleDAOConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: OracleDAOConfig.cpp,v 1.9 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleDAOConfig.h"
#include "glite/data/agents/cred/cred-utility.h"
#include "glite/data/agents/dao/oracle/StatementPtr.h"
#include "glite/data/agents/dao/oracle/ResultSetPtr.h"
#include "glite/data/agents/cred/CredServiceException.h"

#include <glite/data/config/service/Logger.hxx>
#include <memory>
#include <boost/scoped_ptr.hpp>
#include <stdio.h>
#include <string.h>

using namespace glite::data::agents::dao::oracle;


using glite::data::agents::cred::get_proxy_dn;
using glite::data::agents::cred::CredServiceException;

using boost::scoped_ptr;
using oracle::occi::Environment;
using oracle::occi::SQLException;
using oracle::occi::ResultSet;

/*
 * Constants
 */
namespace {
const char * const ORACLE_DAO_NAME    = "agents-dao-oracle";
    
const char * const SCHEMA_VERS_MAJOR  = "MAJOR";
const char * const SCHEMA_VERS_MINOR  = "MINOR";
const char * const SCHEMA_VERS_PATCH  = "PATCH";
        
const char * const NO_THREADS         = "none";
const char * const THREADED_MUTEXED   = "mutexed";
const char * const THREADED_UNMUTEXED = "unmutexed";

const unsigned int DEFAULT_STMTCACHESIZE   = 0;
const char * const DEFAULT_THREADING_MODEL = NO_THREADS;
}

/*
 * instance
 *
 * Return the singleton Instance
 */
OracleDAOConfig& OracleDAOConfig::instance() /*throw ()*/ {
    static OracleDAOConfig s_instance;
    return s_instance;
}

/*
 * OracleDAOConfig
 *
 * Constructor
 */
OracleDAOConfig::OracleDAOConfig():DAOContextFactory(),
    m_logger(ORACLE_DAO_NAME),
    m_stmtCacheSize(DEFAULT_STMTCACHESIZE),
    m_threadingModel(DEFAULT_THREADING_MODEL),
    m_environment(0){
}

/*
 * ~OracleDAOConfig
 *
 * Destructor
 */
OracleDAOConfig::~OracleDAOConfig(){
}

/*
 * init
 *
 * Initialize the Component
 */
void OracleDAOConfig::init()/*throw (DAOLogicError)*/{
    try{                                                                                                                          
        // Log Initialization Values
        m_log_info("Initializig " << ORACLE_DAO_NAME << ". Initialization Parameters are:");
        m_log_info("ConnectString       : " << m_connectString);
        m_log_info("User                : " << m_user);
        m_log_info("Password            : " << "******");
        m_log_info("StatementCacheSize  : " << m_stmtCacheSize);
        m_log_info("Threading Model     : " << m_threadingModel);
        
        // Initialize The Environment
        try {
            if(m_threadingModel == THREADED_UNMUTEXED){
                m_environment = Environment::createEnvironment(Environment::THREADED_UNMUTEXED);
            } else if(m_threadingModel == THREADED_MUTEXED){
                m_environment = Environment::createEnvironment(Environment::THREADED_MUTEXED);
            } else {
                m_environment = Environment::createEnvironment(Environment::DEFAULT);
            }
            if(0 == m_environment){
                throw ConfigurationException(ORACLE_DAO_NAME,"Cannot Create Environment - NullPointer returned");
            }
        } catch(const SQLException& exc){
            m_log_error("Got Exception in Creating The Connection: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
            throw ConfigurationException(ORACLE_DAO_NAME,"Cannot create environment");
        }
        m_log_info("Oracle DAO Initialized");
    
        // get the Agent DN
        try {
            m_agentDn = get_proxy_dn();
            m_log_info("Agent DN is <" << m_agentDn << ">");
        } catch(const CredServiceException& exc){
            m_log_warn("No Agent DN set. Reason is " << exc.what());
        }
    
    } catch(const LogicError& exc) {
        m_log_error("Logic Initialization error: " << exc.what());
        throw DAOLogicError(exc.what());
    } catch(...) {
        m_log_error("Unhandled Exception during Oracle DAO initialization");
        throw;
    }
}

/*
 * fini
 *
 * Finalize the Component
 */
void OracleDAOConfig::fini(){
    
    if(0 != m_environment){
       try {
            // Finalize Environment
            Environment::terminateEnvironment(m_environment);
            m_environment = 0;
        }  catch(const SQLException& exc){
            m_log_error("Got Exception in terminating the Environment: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        }
    }
}

/*
 * createContext
 *
 * Initialize a Oracle Context
 */
OracleDAOContext * OracleDAOConfig::createContext() /*throw (DAOLogicError, DAOException)*/{
    if(0 == m_environment){
        m_log_error("Cannot Create DAO Context since the Environment is not initialized");
        throw DAOLogicError("Environment not initilized");
    }

    // Create the Oracle DAO context Object
    std::auto_ptr<OracleDAOContext> ctx(new OracleDAOContext(*m_environment,m_agentDn));

    // Initialize the Oracle DAO context Object
    try{
        ctx->init(m_user,m_password,m_connectString,m_stmtCacheSize);
        m_log_info("DAO Context Initialized");
    } catch(const DAOException& exc){
        m_log_error("Cannot Create DAO Context. Reason is " << exc.what());
        throw;            
    } catch(const DAOLogicError& exc){
        m_log_alert("Got <" << exc.what() << "> in createContext.This code should never be executed");
        throw;
    }
    // Return Context                     
    return ctx.release();
}

/*
 * checkContext
 *
 * Check that a Oracle Context is still valid, otherwise revalidate it
 */
void OracleDAOConfig::checkContext(OracleDAOContext& ctx) /*throw (DAOLogicError, DAOException)*/{
    if(0 == m_environment){
        m_log_error("Cannot Create DAO Context since the Environment is not initialized");
        throw DAOLogicError("Environment not initilized");
    }
    // Check the Oracle DAO context Object
    try{
        if(false == ctx.isAlive()){
            // Dispose Context
            ctx.dispose();
        
            // Initialize Connection
            ctx.init(m_user,m_password,m_connectString,m_stmtCacheSize);
            
            // Check if now the connection is alive
            if(false == ctx.isAlive()){
                m_log_warn("Invalid DAO Context after reinitialization");
                // Dispose Context
                ctx.dispose();
                throw DAOException("Cannot Establish valid DAO Context");
            }
        }
    } catch(const DAOException& exc){
        m_log_error("Cannot Check DAO Context. Reason is " << exc.what());
        throw;            
    }catch(const DAOLogicError& exc){
        m_log_alert("Got <" << exc.what() << "> in checkContext.This code should never be executed");
        throw;
    }
}


/*
 * checkSchema
 *
 * Check if the the DB Schema is compatible with the requested one
 */
void OracleDAOConfig::checkSchema(DAOContext& daoctx, const std::string& table, unsigned int major,unsigned int minor,unsigned int patch) /*throw (DAOLogicError, DAOException)*/{
    // Check preconditions
    OracleDAOContext * ctx = dynamic_cast<OracleDAOContext*>(&daoctx);
    if(0 == ctx){
        throw DAOLogicError("Invalid DAOContext");
    }
    if(strcspn(table.c_str(),"\'\";-\\\n\r" ) != table.length()){
        m_log_error("<" << table << "> is not a valid table name");
        throw DAOLogicError("Invalid table name");
    }
    
    try {
        // Query the schema version
        std::stringstream query_stmt;
        query_stmt << "SELECT " 
                   << SCHEMA_VERS_MAJOR << ", "
                   << SCHEMA_VERS_MINOR << ", "
                   << SCHEMA_VERS_PATCH << " "
                   << "FROM " << table << " WHERE ROWNUM<=1";
        m_log_trace("Executing Query " << query_stmt.str());

        // Execute Statement To Get The Schema Version
        StatementPtr stmt(ctx->connection(), ctx->connection().createStatement(query_stmt.str()));
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(ResultSet::DATA_AVAILABLE == rs->status()){
            // Fetch Query Result
            rs->next();
            
            // Get Schema Version Numbers        
            int schema_major = rs->getInt(1);
            int schema_minor = rs->getInt(2);
            int schema_patch = rs->getInt(3);
                
            m_log_debug("Got Version " << schema_major << "." << schema_minor << "." << schema_patch);
                
            // Check if the Version is compatible
            if(!((schema_major == (int)major) && (schema_minor >= (int)minor))){
                std::stringstream reason;
                reason << "Incompatible DB Schema Version found: Got " 
                       << schema_major << "." << schema_minor << "." << schema_patch << " requested "
                       << major << "." << minor << "." << patch;
                std::string reason_str = reason.str();
                m_log_debug(reason.str());
                throw DAOException(reason_str);
            } else {
                m_log_debug("DB Schema Version is compatible");
            }
        } else {
            m_log_warn("No Schema Version. Continue Anyway");
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Executing Query Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Check Schema Version - SQL Exception");
    }
}
