/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: OracleDAOContext Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: OracleDAOContext.cpp,v 1.14 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "glite/data/agents/dao/oracle/StatementPtr.h"
#include "glite/data/agents/dao/oracle/StreamPtr.h"
#include "OracleDAOConfig.h"

#include <glite/data/config/service/Logger.hxx>
#include "occi.h"
#include <boost/scoped_ptr.hpp>
#include <boost/algorithm/string/compare.hpp>
#include <algorithm>

using namespace glite::data::agents::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::longlong;
using boost::scoped_ptr;

namespace {
const char * const ORACLE_DAOCONTEXT_NAME = "agents-dao-oracle-ctx";
const char * const LONGLONG_FMT           = "99999999999999999999";
const std::string BOOLEAN_TRUE_STR  = "Y";
const std::string BOOLEAN_FALSE_STR = "N";
} // end global namespace

/*
 * OracleDAOContext
 *
 * Constructor
 */
OracleDAOContext::OracleDAOContext(Environment& env, const std::string& dn) throw ():DAOContext(dn),
    m_logger(ORACLE_DAOCONTEXT_NAME),
    m_env(env),
    m_connection(0){
}

/*
 * ~OracleDAOContext
 *
 * Destructor
 */
OracleDAOContext::~OracleDAOContext(){
   dispose();
}

/*
 * check
 *
 * validate the context
 */
void OracleDAOContext::check() /*throw (DAOLogicError, DAOException)*/ {
    return OracleDAOConfig::instance().checkContext(*this);
}

/*
 * start
 *
 * Start a Transaction
 */
void OracleDAOContext::start() /*throw (DAOLogicError, DAOException)*/{
    
    // Nothing To Do. Default Autocommit is already set to false
    // Statement::setAutoCommit(FALSE);
    
    m_log_trace("Transaction Started");
}

/*
 * commit
 *
 * Commit a Transaction
 */
void OracleDAOContext::commit() /*throw (DAOLogicError, DAOException)*/{
    if(0 == m_connection){
        throw DAOLogicError("No connection pointer set");
    }
    
    m_log_trace("Committing Transaction");

    try{
        // Commit
        m_connection->commit();
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Commit: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Commit Transaction Failed");
    }

    m_log_trace("Transaction Committed");
}

/*
 * rollback
 *
 * Rollback a Transaction
 */
void OracleDAOContext::rollback(){

    m_log_info("Rollback Transaction");

    try{
        // Rollback
        if(0 != m_connection){
            m_connection->rollback();
            m_log_trace("Transaction Rolled Back");
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Rollback: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        // Ignore Error        
    }
}

/*
 * kill
 *
 * Kill context
 */
void OracleDAOContext::kill(){

    m_log_info("Killing Context");

    try{
        if(0 != m_connection){
            // Get Service Context
            OCIEnv *envhp       = m_env.getOCIEnvironment();
            OCIServer * srv_ctx = m_connection->getOCIServer();
            //OCISvcCtx * svc_ctx = m_connection->getOCIServiceContext();

            OCIError *errhp = (OCIError *)0;
            OCIHandleAlloc((dvoid *)envhp, (dvoid **)&errhp, OCI_HTYPE_ERROR, 0, 0);

            //OCIBreak(service_ctx,errhp);
            if(OCI_SUCCESS != OCIBreak(srv_ctx,errhp)){
                // Failed To Kill context
                text errbuf[512];
                sb4 errcode;
                OCIErrorGet ((dvoid *) errhp, 1, NULL, &errcode, errbuf, sizeof(errbuf), OCI_HTYPE_ERROR);
                m_log_warn("Failed to Kill Context. Reason is " << errbuf);
            } else {
                m_log_info("Context Killed");
            }
            if (errhp){
                OCIHandleFree(errhp, OCI_HTYPE_ERROR );
            }

        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Kill: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        // Ignore Error        
    }
}

/*
 * init
 *
 * Initialize the Connection
 */
void OracleDAOContext::init(const std::string& user,
                            const std::string& password,
                            const std::string& connectString,
                                 unsigned long stmtCacheSize) /*throw (DAOLogicError, DAOException)*/{
    
    // Check Preconditions
    if(0 != m_connection){
        throw DAOLogicError("Connection object already exists");
    }
    try{
        m_log_debug("Initializing connection.");
        // Initialize Connection Object
        m_connection = m_env.createConnection(user,password,connectString);
        m_log_debug("Setting statements cache size.");
        // SetStatement Cache Size
        m_connection->setStmtCacheSize(stmtCacheSize);
    } catch(const SQLException& exc){
        m_connection = 0;
        m_log_error("Cannot connect to DB: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        std::stringstream reason;
        reason << "Cannot connect to DB. Reason is " << exc.what();
        throw DAOException(reason.str());
    }
}

/*
 * dispose
 *
 * Destroy the connection
 */
void OracleDAOContext::dispose() {
    // Check Preconditions
    if(0 == m_connection){
        // Connection Already Deleted
        return;
    }
    
    try{
        // Finalize the Connection Object
        m_env.terminateConnection(m_connection);
        m_log_debug("Connection terminated.");
    } catch(const SQLException& exc){
        m_log_error("An error occurs during Connection termination: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
    }

    // Conection    
    m_connection = 0;
}

/*
 * check
 *
 * Check the connection
 */
bool OracleDAOContext::isAlive() {
    bool result = false;
    
    // Check Precondition
    if(0 == m_connection){
        m_log_warn("No Connection established");
        return false;
    }
    
    try{
        m_log_debug("Checking Connection");
        
        // Create Statement
        StatementPtr stmt(*m_connection, m_connection->createStatement());
        // Execute a Simple Query
        stmt->execute("SELECT SYSDATE FROM DUAL");
   
        // Connection is still Alive
        result = true;
        m_log_debug("Connection is valid");
    } catch(const SQLException& exc){
        m_log_error("Error Executing Check Connection Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_warn("Connection has been dropped");
        result = false;
    } 
    return result;
}

/*
 * toTimeT
 *
 * Convert a Timestamp into a time value
 */
time_t OracleDAOContext::toTimeT(const Timestamp& timestamp) /*throw ()*/{
    time_t t = (time_t)-1;
    
    try{
        if(!timestamp.isNull()){
            int year;
            unsigned int month,day,hour,minute,second,fs;
            int tz_hour = 0;
            int tz_minute = 0;
            // Get Date
            timestamp.getDate(year,month,day);
            // Get Time
            timestamp.getTime(hour,minute,second,fs);
            // Get TimeZone Offset
            timestamp.getTimeZoneOffset(tz_hour,tz_minute);
            
            struct tm tmp_tm;
            tmp_tm.tm_sec   = second;
            tmp_tm.tm_min   = minute - tz_minute;
            tmp_tm.tm_hour  = hour - tz_hour;
            tmp_tm.tm_mday  = day;
            tmp_tm.tm_mon   = (month >= 1)   ?month - 1   :0;
            tmp_tm.tm_year  = (year  >= 1900)?year  - 1900:0;
            tmp_tm.tm_wday  = 0;
            tmp_tm.tm_yday  = 0;
            // As spotted by D.Smith, the DST shouln't be considered in this 
            // case, since the passed value already consider the offset
            //tmp_tm.tm_isdst = -1;
            tmp_tm.tm_isdst = 0;
        
            // Get Time Value
            t = mktime(&tmp_tm);
            if((time_t)-1 == t){
                m_log_error("Cannot Convert Timestamp " << timestamp.toText("dd/mm/yyyy hh:mi:ss [tzh:tzm]",0));
            } else {
                t -= timezone;
                // Don't consider the timezone (see before)
                //if(1 == tmp_tm.tm_isdst){
                //    t += 3600;
                //}
            }
            m_log_trace("Timestamp is " << timestamp.toText("dd/mm/yyyy hh:mi:ss [tzh:tzm]",0) << " => Timevalue is " << t);
        }
    } catch(const SQLException& exc){
        m_log_error("Cannot Convert Timestamp: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
    }
    
    return t;
}

/*
 * toTimestamp
 *
 * Convert a time value into a Timestamp 
 */
Timestamp OracleDAOContext::toTimestamp(time_t t) /*throw ()*/{
    Timestamp timestamp;
    try{    
        struct tm * tmp_tm = gmtime(&t);
        if(0 != tmp_tm){
            timestamp = Timestamp(&m_env,                  // Environment
                                   tmp_tm->tm_year + 1900, // Year
                                   tmp_tm->tm_mon + 1,     // Month
                                   tmp_tm->tm_mday,        // Day
                                   tmp_tm->tm_hour,        // Hour
                                   tmp_tm->tm_min,         // Minute
                                   tmp_tm->tm_sec,         // Second
                                   0,                      // Fraction of Seconds
                                   0,                      // tz hour
                                   0);                     // tz minute

            m_log_trace("Timevalue is " << t << " => Timestamp is " << timestamp.toText("dd/mm/yyyy hh:mi:ss [tzh:tzm]",0));
        }
    } catch(const SQLException& exc){
        m_log_error("Cannot Convert Time Value: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
    }
    
    return timestamp;
}

/*
 * toLongLong
 *
 * Convert a Number into a longlong
 * Note: this function use a temporary string. It not efficient, but as far as 
 * I know, Number doesn't support long long types
 */
longlong OracleDAOContext::toLongLong(const Number& number) /*throw()*/{
    longlong n = -1;
    try{    
        if(!number.isNull()){
            std::string n_str = number.toText(&m_env,LONGLONG_FMT);
#ifdef WIN32
            n = _atoi64(n_str.c_str());
#else
            n = atoll(n_str.c_str());
            m_log_trace("LongLong string value is " << n_str << " = >LongLong value is " << n);
#endif  
        }
    } catch(const SQLException& exc){
        m_log_error("Cannot Convert Number: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
    }
    return n;
}

/*
 * toNumber
 *
 * Convert a longlong into a Number
 * Note: this function use a temporary string. It not efficient, but as far as 
 * I know, Number doesn't support long long types
 */
Number OracleDAOContext::toNumber(longlong n) /*throw()*/{
    std::string n_str;    
#ifdef WIN32
    char tmp_buffer[66];
    _i64toa(n,tmp_buffer,10);
    n_str = tmp_buffer;
#else
    std::stringstream str;
    str << n;
    n_str = str.str();
#endif    

    Number number(0);
    try{    
        number.fromText(&m_env,n_str,LONGLONG_FMT);
        m_log_trace("Long value is " << n << " => LongLong string value is " << number.toText(&m_env, LONGLONG_FMT));
    } catch(const SQLException& exc){
        m_log_error("Cannot Convert LongLong: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
    }
    return number;
}

/*
 * toBoolean
 *
 * Convert a Char into a boolean
 */
bool OracleDAOContext::toBoolean(const std::string& str) const /*throw ()*/{
    if(str.empty()){
        return false;
    }
    return std::equal(str.begin(), str.end(),BOOLEAN_TRUE_STR.begin(),boost::is_iequal());
}

/*
 * toBoolean
 *
 * Convert a Boolean into a String
 */
const std::string& OracleDAOContext::toBoolean(bool b) const /*throw ()*/{
    return (b == true)?BOOLEAN_TRUE_STR:BOOLEAN_FALSE_STR;
}


/*
 * toString
 *
 * Read a Clob into a String
 */
void OracleDAOContext::toString(Clob clob, std::string& str) const /*throw (DAOException)*/{
    // Check if the Clob is Null
    if(clob.isNull()){
        str.clear();
        return;
    }

    try{
        // Open the Clob
        clob.open(OCCI_LOB_READONLY);
        int len = clob.length();

        // reserve some space on the string
        str.resize(len, 0);
        char * buffer = &(*(str.begin())); 

        // Get the stream
        StreamPtr<Clob> instream(clob, clob.getStream(1,0));
        instream->readBuffer(buffer, len);

        // Close the Clob and the related stream        
        clob.close();
    } catch(const SQLException&  exc){
        //Close the Clob
        try{
            clob.close();
        }catch(...){}
        std::string reason = (std::string)"Failed to read clob: " + exc.getMessage();
        throw DAOException(reason);
    }
}
