/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 *
 *  GLite Data Transfer Agents: JobStatements Class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 *  Version info: $Id: Statements.cpp,v 1.1 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 *
 */

#include <glite/data/agents/dao/oracle/Statements.h>
#include <glite/data/agents/dao/oracle/StatementPtr.h>

#include <sstream>

namespace glite   {
namespace data    {
namespace agents  {
namespace dao     {
namespace oracle  {

using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;

using namespace ::oracle::occi;

/**
 * constructor
 * @param logger_instance [IN] logger initialization parameter
 */
Statements::Statements(const char* logger_instance) : 
    m_logger(logger_instance){
}

/**
 * create_statement
 * helper function for statements creations
 * @param ctx [IN] the oracle DAO context
 * @param tag [IN] the statement cache tag
 * @param ptr [IN] pointer to function that returns the statement string
 * @param name [IN] statement name (for errors logging)
 */
Statement* Statements::create_statement(
    OracleDAOContext&       ctx,
    const char*             tag,
    create_statement_string ptr,
    const char*             name){
        
    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt = ptr();
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            m_log_trace("Retrieving cached Statement: " << tag);
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating " << name << " Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare statement - Nullpointer");
    }
    return stmt;
}

} // End namespace oracle
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

