/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: ActiveObject Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ActiveObject.cpp,v 1.8 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/extra/ActiveObject.h"
#include <glite/data/config/service/Logger.hxx>

#include <unistd.h>

using namespace glite::data::agents;


namespace {
const unsigned long DEFAULT_STOP_TIMEOUT = 100;
const unsigned int  DEFINED_STOP_SIGNAL  = SIGUSR1;
const unsigned int  DEFINED_TERM_SIGNAL  = SIGTERM;    
}
/*
 * Define Class Attributes
 */
std::map<ActiveObject::id_type,ActiveObject *> ActiveObject::s_objects;
boost::mutex ActiveObject::s_mutex;

/*
 * registerObject
 *
 * Register an ActiveObject
 */
void ActiveObject::registerObject(ActiveObject* obj){
    // Acquire Lock
    boost::mutex::scoped_lock scoped_lock(s_mutex);
    
    std::map<ActiveObject::id_type,ActiveObject *>::iterator it = s_objects.find(obj->id());
    if(it == s_objects.end()){
        s_objects[obj->id()] = obj;
    }
}

/*
 * deregisterObject
 *
 * Deregister an ActiveObject
 */
void ActiveObject::deregisterObject(ActiveObject * obj){
    // Acquire Lock
    boost::mutex::scoped_lock scoped_lock(s_mutex);

    std::map<ActiveObject::id_type,ActiveObject *>::iterator it = s_objects.find(obj->id());
    if(it != s_objects.end()){
        s_objects.erase(it);
    }
}

/*
 * getAgent
 *
 * Retrun an Agent
 */
ActiveObject * ActiveObject::get(id_type id){
    ActiveObject * obj = 0;
    
    // Acquire Lock
    boost::mutex::scoped_lock scoped_lock(s_mutex);

    std::map<ActiveObject::id_type,ActiveObject *>::iterator it = s_objects.find(id);
    if(it != s_objects.end()){
        obj = (*it).second;
    }
    
    return obj;
}

/*
 * agent_main
 *
 * Agent Thread Main
 */
namespace glite{
namespace data{
namespace agents{
void * object_main(void * data){
    ActiveObject * o = reinterpret_cast<ActiveObject*>(data);
    o->run();
    return 0;
}
                                                 
/*
 * stop_object
 *
 * Stop the Object
 */
void stop_object(int signum){
    // Get thread Id
    pthread_t id = pthread_self();
    // Unset the run flag
    ActiveObject * obj = ActiveObject::get(id);
    if(0 != obj){
        obj->signalStop();
    }
}

/*
 * term_object
 *
 * Terminate the object
 */
void term_object(int signum){
    // Get thread Id
    pthread_t id = pthread_self();
    // Unset the run flag
    ActiveObject * obj = ActiveObject::get(id);
    if(0 != obj){
        obj->signalTerm();
    }
}
    
} //end agents
} //end data
} //end glite

/*
 * Agent
 *
 * Constructor
 */
ActiveObject::ActiveObject(const std::string& name, long stop_timeout):
    m_logger(name),
    m_id((id_type)-1),
    m_stopTimeout((-1 == stop_timeout)?DEFAULT_STOP_TIMEOUT:stop_timeout),
    m_started(false),
    m_run(false){

    pthread_mutex_init(&m_lock,0);
    pthread_cond_init(&m_cond,0);
}

/*
 * ~ActiveObject
 *
 * Destructor
 */
ActiveObject::~ActiveObject(){
    // Stop The ActiveObject Thread
    stop();
        
    pthread_cond_destroy(&m_cond);
    pthread_mutex_destroy(&m_lock);
}

/*
 * start
 *
 * Start the scheduler Thread
 */
bool ActiveObject::start(){

    if((id_type)-1 == m_id){
        // Wait the Main Thread Is Started
        int check_locked = pthread_mutex_trylock(&m_lock);
	if(check_locked != 0)
	   m_log_debug("Cannot lock mutex: ActiveObject::start");
        pthread_t id;
        pthread_create(&id, NULL, object_main,this); 
        // Wait Condition
        pthread_cond_wait(&m_cond,&m_lock);
        
        m_started = m_run;
        
        pthread_mutex_unlock(&m_lock);
        m_log_debug("ActiveObject started");
    }
    return m_started;
}

/*
 * Stop 
 *
 * Stop the Scheduler Thread
 */
void ActiveObject::stop(){
    
    if(true == m_started){
        // Set run Flag
        m_log_debug("Stopping ActiveObject");
            
        int check_locked = pthread_mutex_trylock(&m_lock);
	if(check_locked != 0)
	   m_log_debug("Cannot lock mutex: ActiveObject::stop");

        pthread_t id = m_id;

        // Stop the thread
        pthread_kill(id, DEFINED_STOP_SIGNAL);

        unsigned int t = m_stopTimeout;
        if(t > 0){
            // Timed Wait
            struct timespec timeout;
            time(&timeout.tv_sec);
            timeout.tv_nsec = 0L;
            timeout.tv_sec += t;
            
            m_log_debug("Waiting ActiveObject shutdown");
            int ret =  pthread_cond_timedwait(&m_cond, &m_lock, &timeout);
            if(0 != ret){
                m_log_warn("ActiveObject " << id << " didn't stop in " << t << " seconds. Send TERM signal");
                // Terminate the thread
                pthread_kill(id, DEFINED_TERM_SIGNAL);
 
                // Blocking Wait
                pthread_cond_wait(&m_cond,&m_lock);
            }
        } else {
            m_log_debug("Waiting ActiveObject shutdown");
            // Blocking Wait
            pthread_cond_wait(&m_cond,&m_lock);
        }

        // Wait the thread is stopped
        pthread_join(id,0);

        m_started = false;
        
        pthread_mutex_unlock(&m_lock);
        m_log_debug("ActiveObject stopped");
    }
}

/*
 * run
 *
 * Object Main Method
 */
void ActiveObject::run(){
    
    m_log_debug("ActiveObject main method");
    
    // Set thread Id
    m_id = pthread_self();
    
    // Add the Scheduler 
    registerObject(this);
    
    // Set Thread Signal
    sigset_t signal_mask;
    sigset_t blocked_signal_mask;
    sigemptyset(&signal_mask);
    sigemptyset(&blocked_signal_mask);
    sigaddset(&blocked_signal_mask, DEFINED_STOP_SIGNAL);
    sigaddset(&signal_mask, DEFINED_STOP_SIGNAL);
    sigaddset(&signal_mask, DEFINED_TERM_SIGNAL);
    pthread_sigmask(SIG_SETMASK, &signal_mask, NULL);
                                                                                               
    struct sigaction sig_stop,sig_term;
    sig_stop.sa_handler = stop_object;
    sig_stop.sa_flags = 0;
    sig_term.sa_handler = term_object;
    sig_term.sa_flags = 0;
    sigaction(DEFINED_STOP_SIGNAL,&sig_stop,NULL);
    sigaction(DEFINED_TERM_SIGNAL,&sig_term,NULL);

    // Execute Init Action
    m_run = doInit();
    
    int check_locked = pthread_mutex_trylock(&m_lock);
    if(check_locked != 0)
	   m_log_debug("Cannot lock mutex: ActiveObject::run");
    pthread_cond_signal(&m_cond);
    pthread_mutex_unlock(&m_lock);
    
    m_log_debug("ActiveObject " << m_id << " Started");
    
    // Unblock signal mask
    pthread_sigmask(SIG_UNBLOCK, &signal_mask, NULL);

    try{
        // Call Template Method
        this->svc();
    } catch(const std::exception& exc){
        m_log_fatal("Failed To Run ActiveObject (Exception). Reason is " << exc.what());
    } catch(...){
        m_log_fatal("Cannot To Run ActiveObject. Unknown Reason");
    }
    
    // Execute Fini Action
    doFini();
    
    // Deregister the Object
    deregisterObject(this);

    // Signal the condition
    check_locked = pthread_mutex_trylock(&m_lock);
    if(check_locked != 0)
	 m_log_debug("Cannot lock mutex: ActiveObject::run");
    pthread_cond_signal(&m_cond);
    pthread_mutex_unlock(&m_lock);

    m_log_info("ActiveObject " << m_id << " Stopped");
    
    m_id = (id_type)-1;
}

/*
 * signalStop
 *
 * Signal Stop on the Object's Thread
 */
void ActiveObject::signalStop(){
    
    m_log_debug("Stop Signal Received");
    m_run = false;
}

/*
 * signalTerm
 *
 * Signal Term on the Object's Thread
 */
void ActiveObject::signalTerm(){
    try{
        m_log_debug("Term Signal Received");

        // Call hook for termination
        this->terminate();
    } catch(const std::exception& exc){
        m_log_fatal("Got Exception (" << exc.what() << ") in terminate");
    } catch(...){
        m_log_fatal("Got Unknown Exception in terminate");
    }
    m_run = false;
}

/*
 * enterCritical
 *
 * Enter an Execution Critical Section: the stop signals is suspended until
 * the section is left
 */
void ActiveObject::enterCritical() throw() {

    // Set Thread Signal Mask
    sigset_t blocked_signal_mask;
    sigemptyset(&blocked_signal_mask);
    sigaddset(&blocked_signal_mask, DEFINED_STOP_SIGNAL);
    
    // Block Signal
    pthread_sigmask(SIG_BLOCK, &blocked_signal_mask, NULL);
    m_log_trace("Stop Signal Blocked");
}

/*
 * leaveCritical
 *
 * Leave an Execution Critical Section
 */
void ActiveObject::leaveCritical() throw() {

    // Set Thread Signal Mask
    sigset_t blocked_signal_mask;
    sigemptyset(&blocked_signal_mask);
    sigaddset(&blocked_signal_mask, DEFINED_STOP_SIGNAL);
    
    // Unblock Signal
    pthread_sigmask(SIG_UNBLOCK, &blocked_signal_mask, NULL);
    
    m_log_trace("Stop Signal Restored");
}

/*
 * doInit
 *
 * Do Init Action
 */
bool ActiveObject::doInit() /* throw () */{
    bool result  = true;
    try{
        result = this->onEventStart();
    } catch(const std::exception& exc){
        m_log_fatal("Got Exception (" << exc.what() << ") in notifying Start Event");
        result = false;
    } catch(...){
        m_log_fatal("Got Unknown Exception in notifying Start Event");
        result = false;
    }
    return result;
}

/*
 * doFini
 *
 * Do Fini Action
 */
bool ActiveObject::doFini() /* throw () */{
    bool result = true;
    try{
        result = this->onEventStop();
    } catch(const std::exception& exc){
        m_log_fatal("Got Exception (" << exc.what() << ") in notifying Start Event");
        result = false;
    } catch(...){
        m_log_fatal("Got Unknown Exception in notifying Start Event");
        result = false;
    }
    
    return result;
}
