/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: PatternList Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PatternList.cpp,v 1.4 2007/03/15 09:07:15 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/extra/PatternList.h"

#include <boost/regex.hpp>
#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>

using namespace glite::data::agents;

const std::string::size_type PatternList::MaxMatchLen = 1024;

/*
 * PatternList
 *
 * Constructor
 */
PatternList::PatternList(const char * const p[], unsigned int s){
    for (unsigned int i = 0; i < s; ++i){
	try{
	    add(p[i]);
	} catch(const boost::bad_expression& ){
	    // Ignore error
	}
    }
}

/*
 * PatternList
 *
 * Constructor
 */
PatternList::PatternList(const std::vector<std::string>& p){
    std::vector<std::string>::const_iterator it;
    for (it = p.begin(); it != p.end(); ++it){
	try{
	    add((*it).c_str());
	} catch(const boost::bad_expression& ){
	    // Ignore error
	}
    }
}

/*
 * add
 *
 * add a pattern
 */
void PatternList::add(const char * pattern) /*throw (boost::bad_expression)*/{
    //compile the Regular expression
    boost::shared_ptr<boost::regex> e(
	 new boost::regex(pattern, boost::regex::perl|boost::regex::icase));
    // Store it in the list
    m_patterns.push_back(e);
}

/*
 * match
 *
 * Return true if the string matches one of the patterns
 */
bool PatternList::match(const std::string& str, bool partial_match){
	if (MaxMatchLen < str.length()) {
		return match(str.substr(0,MaxMatchLen),partial_match);
	}
    bool result = false;

    try{
        boost::match_results<std::string::const_iterator> what;
        boost::match_flag_type flags = boost::match_default;
        if(true == partial_match){ 
        flags |= boost::match_partial;
        }
        // For each pattern
        std::vector<boost::shared_ptr<boost::regex> >::iterator it;
        for(it = m_patterns.begin(); it != m_patterns.end();++it){
           boost::regex& re = *(*it);
           if(0 != boost::regex_match(str,what,re, flags)){
           result = true;
           break;
           }
        }
    } catch (const std::exception&) {
        // if an exception is caught, just return false
    }
    
    return result;
}
