/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: PatternList Class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: XmlPatternsList.cpp,v 1.5 2007/11/05 11:29:25 ptedesco Exp $
 *  Release: $Name:  $
 */
#include "glite/data/agents/extra/XmlPatternsList.h"

#include <boost/regex.hpp>
#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>
#include <fstream>
#include <iostream>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include <stdlib.h>

using namespace glite::data::agents;
using boost::scoped_ptr;
using boost::shared_ptr;

const std::string::size_type XmlPatternsList::MaxMatchLen = 1024;

namespace{

//------------------------------------------------------------------------------
// Utilitites
//------------------------------------------------------------------------------

/**
 * areEqual
 * compare an xmlChar* with a char*
 * @param first [IN] xml string
 * @param second [IN] string
 * @return true if the two strings are equal, false otherwise  
 */
bool areEqual(const xmlChar* first,const char* second){
    return (!xmlStrcmp(first,(const xmlChar*)second));
}

/**
 * isElement
 * return true if node is an element with given name 
 */
bool isElement(const xmlNodePtr node,const char* name){
    return ((XML_ELEMENT_NODE == node->type) &&
        (!xmlStrcmp(node->name,(const xmlChar*)name)));
}

//------------------------------------------------------------------------------
// Helper classes
//------------------------------------------------------------------------------

/**
 * XmlString
 * helper class for xmlChar* handling
 */
class XmlString{
 public:
    /**
     * constructor
     * @param value [IN] the xml string
     */
    XmlString(xmlChar* value) : m_string(value){
    }
    /**
     * destructor
     */
    virtual ~XmlString(){
        xmlFree(m_string);
    }
    
    /**
     * equality and inequality operators
     */
    bool operator == (const XmlString& other){
        return (!xmlStrcmp(m_string,other.m_string));
    }
    bool operator != (const XmlString& other){
        return (xmlStrcmp(m_string,other.m_string));
    }
    bool operator == (const xmlChar* value){
        return (!xmlStrcmp(m_string,value));
    }
    bool operator != (const xmlChar* value){
        return (xmlStrcmp(m_string,value));
    }
    bool operator == (const char* value){
        return (!xmlStrcmp(m_string,(const xmlChar*)value));
    }
    bool operator != (const char* value){
        return (xmlStrcmp(m_string,(const xmlChar*)value));
    }
    std::string str(){
        return std::string((const char*)m_string);
    }
    bool isNull(){
        return (0 == m_string);
    }
protected:
    xmlChar * m_string;
};

/**
 * XmlAttribute
 */
class XmlAttribute : public XmlString{
public:
    /**
     * constructor
     * @param node [IN] Xml node
     * @param name [IN] attribute name
     */
    XmlAttribute(
        xmlNodePtr  node,
        const char* name) : XmlString(xmlGetProp(node,(const xmlChar*)name)){
    }
    /**
     * destructor
     */
    virtual ~XmlAttribute(){
    }
};


/**
 * XmlNodeValue
 */
class XmlNodeValue : public XmlString{
public:
    /**
     * constructor
     * @param doc [IN] the xml document
     * @param node [IN] the xml node
     */
    XmlNodeValue(
        xmlDocPtr   doc,
        xmlNodePtr  node) : XmlString(xmlNodeListGetString(doc, node->xmlChildrenNode, 1)){
        }
    /**
     * destructor
     */
    virtual ~XmlNodeValue(){
    }
};

//------------------------------------------------------------------------------
// Implementation of PatternMatch class
//------------------------------------------------------------------------------

/**
 * PatternMatchImpl
 */
class PatternMatchImpl : public XmlPatternsList::PatternMatch{
public:
    /**
     * constructor
     * @param doc [IN] xml document
     * @param node [IN] the pattern node
     */
    PatternMatchImpl(
        xmlDocPtr   doc,
        xmlNodePtr  patternNode);
    /**
     * destructor
     */
    virtual ~PatternMatchImpl(){
    }
    /**
     * return true if given string matches the internal pattern
     * @param message [IN] the string to match
     * @return true if pattern matches, false otherwise
     */
    virtual bool check(
        const std::string& message) const{
        return (this->*m_check)(message);    
    }

    /**
     * get the value of an optional 'property' node
     * @param name [IN] property name
     * @param value [OUT] property value
     * @return true if a property with given name was found, false otherwise
     */
    virtual bool getProperty(
        const std::string&  name,
        std::string&        value) const{
        PropertiesMap::const_iterator it = m_properties.find(name);
        if (m_properties.end() == it){
            return false;
        }else{
            value = it->second;
            return true;
        }
    }
    
protected:
    
    // the map of properties
    typedef std::map<std::string,std::string> PropertiesMap;
    PropertiesMap m_properties;
    
    // function to be used for pattern matching (match, search)
    typedef bool (PatternMatchImpl::*CheckFunction)(const std::string&) const;
    CheckFunction m_check;
    
    /**
     * match internal pattern against given message 
     */
    bool match(const std::string& message) const{
        boost::match_results<std::string::const_iterator> what;
        boost::match_flag_type flags = boost::match_default;
        boost::regex& re = *(m_regex.get());
        return boost::regex_match(message,what,re,flags);
    }

    /**
     * search internal pattern in given message 
     */
    bool search(const std::string& message) const{
        boost::match_results<std::string::const_iterator> what;
        boost::match_flag_type flags = boost::match_default | boost::match_any;
        boost::regex& re = *(m_regex.get());
        return boost::regex_search(message.begin(),message.end(),what,re,flags);
    }
    // the regex object
    scoped_ptr<boost::regex> m_regex;
    // the pattern string
    std::string m_pattern;
};

/**
 * constructor
 * @param doc [IN] xml document
 * @param node [IN] the pattern node
 */
PatternMatchImpl::PatternMatchImpl(
    xmlDocPtr   doc,
    xmlNodePtr  patternNode) : 
    XmlPatternsList::PatternMatch(),
    m_check(0){
    
    XmlAttribute type(patternNode,"type");
    if (type.isNull() || // not specified -> use default (search)
        (type == "search")){
        m_check = &PatternMatchImpl::search;
    } else if (type == "match"){
        m_check = &PatternMatchImpl::match;
    } else {
        throw XmlPatternsException("Error parsing xml patterns file: invalid pattern type value.");
    }
    
    for (xmlNodePtr node = patternNode->xmlChildrenNode;0 != node;node = node->next){
        if (isElement(node,"value")){
            if (0 != m_regex.get()){
                throw XmlPatternsException("Error parsing xml patterns file: more than one value node specified.");
            }
            XmlNodeValue pattern(doc,node);
            m_pattern = pattern.str();
            m_regex.reset(new boost::regex(pattern.str(), boost::regex::perl|boost::regex::icase));
        } else if (isElement(node,"property")){
            XmlAttribute name(node,"name");
            if (name.isNull()){
                throw XmlPatternsException("Error parsing xml patterns file: no name specified for property.");
            }
            std::string str_name = name.str();
            PropertiesMap::const_iterator it = m_properties.find(str_name);
            if (m_properties.end() != it){
                throw XmlPatternsException("Error parsing xml patterns file: property defined more than once.");
            }
            XmlNodeValue value(doc,node);
            m_properties[str_name]=value.str();
        }
    }
    if (0 == m_check){
       throw XmlPatternsException("Error parsing xml patterns file: no value node defined."); 
    }
}


} // namespace 

//------------------------------------------------------------------------------
// XmlPatternsList implementation
//------------------------------------------------------------------------------

/**
 * constructor
 */
XmlPatternsList::XmlPatternsList(
    const char* fileName){

    char * glite_location = getenv("GLITE_LOCATION");
    if (0 == glite_location){
        throw XmlPatternsException("Error opening xml patterns file: GLITE_LOCATION environment variable is not defined.");
    }
    std::string file_path(glite_location);
    if ('/' != file_path[file_path.length()-1]){
        file_path+='/';
    }
    file_path += "etc/";
    file_path += std::string(fileName);
    
    std::ifstream ifs(file_path.c_str());
    if(ifs.is_open()){
    	ifs.close();
    }
    else{
    	file_path = "/etc/" + std::string(fileName);
    }   
    
    // check that document is well formed and contains the expected nodes even
    // if it should have been validated against a schema.
    // open the document
    shared_ptr<xmlDoc> doc(xmlParseFile(file_path.c_str()),xmlFreeDoc);
    if (0 == doc.get()){
        std::string reason = "Parsing document '" + file_path + "' failed. Check that the file exists, is accessible and contains valid xml.";
        throw XmlPatternsException(reason);
    }
    // get the root element
    xmlNodePtr root = xmlDocGetRootElement(doc.get());
    if (0 == root){ // empty document
        std::string reason = "Patterns file '" + file_path + "' is empty.";
        throw XmlPatternsException(reason);
    }
    if (!isElement(root,"patterns")){ // unexpected root element
        std::string reason = "Patterns file '" + file_path + "' contains an invalid root element.";
        throw XmlPatternsException(reason);
    }
    // iterate through 'list' nodes
    for (xmlNodePtr listNode = root->xmlChildrenNode;0 != listNode;listNode = listNode->next){
        if (isElement(listNode,"list")){ 
            XmlAttribute category(listNode,"category");
            if (category.isNull()){ // category attribute (mandatory) is missing
                std::string reason = "Patterns file '" + file_path + 
                    "' contains invalid xml (missing mandatory attribute 'category' from 'list' node).";
                throw XmlPatternsException(reason);
            }
            PatternMatches listMatches;
            for (xmlNodePtr patternNode = listNode->xmlChildrenNode;0 != patternNode;patternNode = patternNode->next){
                if (isElement(patternNode,"pattern")){
                    listMatches.push_back(shared_ptr<PatternMatch>(new PatternMatchImpl(doc.get(),patternNode)));
                }
            }
            m_patterns[category.str()] = listMatches;
        }
    }
}

/**
 * destructor
 */
XmlPatternsList::~XmlPatternsList(){
}

/**
 * get the category associated to given message.
 * @param message [IN] the message to be matched against the internal 
 * patterns list
 * @return message category, empty string if not found.
 */
std::string XmlPatternsList::getCategory(
    const std::string& message){
    PatternMatchesMap::const_iterator it;
    for (it=m_patterns.begin();it!=m_patterns.end();++it){
        const std::string& category = it->first;
        if(checkCategory(message,category)){
            return category;
        }
    }
    return "";
}

/**
 * return true if given message matches any of the patterns for the
 * given category
 * @param message [IN] the message to be matched against the patterns list
 * @param category [IN] the category of the patterns list
 * @return true if message matches category patterns, false otherwise.
 */
bool XmlPatternsList::checkCategory(
    const std::string& message,
    const std::string& category){
    return (0 != getPatternMatch(message,category));
}


/**
 * return the PatternMatch object for given message and category or
 * null if no pattern in category applies.
 * @param message [IN] the message to be matched against the patterns list
 * @param category [IN] the category of the patterns list
 * @return const pointer to PatternMatch object if message matches category 
 * patterns, false otherwise.
 */
const XmlPatternsList::PatternMatch * XmlPatternsList::getPatternMatch(
    const std::string& message,
    const std::string& category){

	if (MaxMatchLen < message.length()) {
		return getPatternMatch(message.substr(0,MaxMatchLen),category);
	}
    
    PatternMatchesMap::const_iterator pm_it = m_patterns.find(category);
    if (m_patterns.end() == pm_it){
        return 0;
    }
    const PatternMatches& patterns = pm_it->second;
    PatternMatches::const_iterator it = patterns.begin();
    for (;it!=patterns.end();++it){
        const PatternMatch * match = it->get();
        if (match->check(message)){
            return match;
        }
    }
    return 0;
}
