/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: Utility Functions Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: utility.cpp,v 1.10 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 */
#include "glite/data/agents/extra/utility.h"
#include "glite/data/agents/sd/SDConfig.h"
#include "glite/data/agents/sd/sd-utility.h"
#include "glite/data/agents/url-utility.h"
#include "glite/data/agents/AgentExceptions.h"

#include <glite/data/config/service/Logger.hxx>

#include <boost/scoped_ptr.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>
#include <algorithm>
#include <ctype.h>
#include <libgen.h>

using namespace glite::data::agents;
using namespace glite::data::agents::sd;

using boost::scoped_ptr;

namespace {
const char * const MODULE_NAME           = "agents-extra";
const char * const SRM_PROTOCOL          = "srm";
const char * const SFN_PROTOCOL          = "sfn";
const char * const GSIFTP_PROTOCOL       = "gsiftp";
const char * const UNKNOWN_SITE          = "UNKNOWN";

const unsigned long KILOBYTE_SIZE  = 1024;
const unsigned long MEGABYTE_SIZE  = 1024 * 1024;
const unsigned long GIGABYTE_SIZE  = 1024 * 1024 * 1024;
}

/*
 * get_site_for_surl
 *
 * get the name of the site where the SURL is located
 */
std::string glite::data::agents::get_site_for_surl(const std::string& surl, const std::string& vo_name, bool enable_unknown) /*throw (LogicError, RuntimeError)*/{

    std::string  hostname;
    std::string  protocol;
    unsigned int port = 0;
    try{
        // Extract Hostname
        parse_url(surl,hostname,port,protocol);
    } catch(const LogicError& exc){
        _log_error(MODULE_NAME,"Invalid SURL (" << surl << "): Reason is " << exc.what());
        throw;
    }
    _log_debug(MODULE_NAME,"Hostname for SURL " << surl << " is <" << protocol << ">://<" << hostname << ">:<" << port << ">");
    
    // Get ServiceDiscovery 
    SDConfig& sd_config = SDConfig::instance();

    // Find the Site of the SE that owns the SURL
    std::string site;
    std::string service_type;
    
    if((protocol == SRM_PROTOCOL) || (protocol == SFN_PROTOCOL)){ 
        service_type = sd_config.srm();
    } else if(protocol == GSIFTP_PROTOCOL){
        service_type = sd_config.gridftp();
    } else {
        // Unsupported Protocol Type
        _log_error(MODULE_NAME,"Unsupported Protocol Type (" << protocol << ") for SURL " << surl);
        std::string error_reason = (std::string)"Unknwon SURL/TURL protocol: " + protocol;
        throw RuntimeError(error_reason);
    }
    
    // Find Service
    scoped_ptr<Service> s(get_service(hostname,service_type,vo_name));
    if(0 != s.get()){
        // Get service site
        site = s->site;
    } else {
        if(protocol == GSIFTP_PROTOCOL){ 
            // Check For Legacy GRIDFTP Service Type
            s.reset(get_service(hostname,sd_config.legacyGridftp(),vo_name));
        }
        if(0 != s.get()){
            // Get service site
            site = s->site;
        } else {
            if(true == enable_unknown){
                _log_debug(MODULE_NAME,"Cannot Find SRM for host " << hostname);
                _log_info(MODULE_NAME,"Associating Unknown Site (" << UNKNOWN_SITE << ")");
                site = UNKNOWN_SITE;
            } else {
                _log_error(MODULE_NAME,"Cannot Find Site for host " << hostname);
                std::string error_reason = (std::string)"No site found for host " + hostname;
                throw RuntimeError(error_reason);
            }
        }
    }
    _log_debug(MODULE_NAME,"Site for SURL " << surl << " is " << site);
      
    return site;    
}

/*
 * str_to_size
 *
 * Convert a string representing a size (in bytes) to an integer. Examples of the allowed formats are:
 *   122
 *   100 KB
 *   1.2 MB (or without the space 1.2MB)
 *   .5 GB
 */
unsigned long glite::data::agents::str_to_size(std::string str) /*throw (LogicError)*/{

    unsigned long size      = 0;
    unsigned long unit_size = 1;

    // trim beginning  white spaces
    boost::trim(str);
    if(true == str.empty()){
        // Empty string. Retrun 0
        return size;
    }

    try{
        if(str.at(0) == '.'){
           // String starts with a ".". Append a 0
           str.insert(str.begin(),'0');
        }
        // check if the string starts with a number
        if(isdigit(str.at(0))){
            // Check if the string has only digit
            if(boost::all(str,boost::is_digit())){
                size = boost::lexical_cast<unsigned long>(str);
            } else {
                // Check if the string has a dot
                std::string::size_type dot_pos = str.find('.');

                // find the last element that is not a digit
                std::string::iterator num_end = (dot_pos != std::string::npos)?num_end = str.begin() + dot_pos + 1: str.begin();
                for(; num_end != str.end(); ++num_end) {
                    if(0 == isdigit(*num_end)){
                        break;
                    }
                }
                // Check if the string has the unit
                std::string unit;
                if(num_end != str.end()){
                    unit = str.substr(std::distance(str.begin(),num_end));
                    boost::trim(unit); 
                }
                if(unit.length() >= 2){
                    if(unit == "KB"){
                        unit_size = KILOBYTE_SIZE;
                    } else if(unit == "MB"){
                        unit_size = MEGABYTE_SIZE;
                    } else if(unit == "GB"){
                        unit_size = GIGABYTE_SIZE;
                    } else {
                        throw LogicError("Invalid string size representation: unknown unit");
                    }
                } else {
                    if(dot_pos != std::string::npos){
                        throw LogicError("Decimal number used to express a size");
                    } else {
                        throw LogicError("Invalid string size representation: string contains invalid characters");
                    }
                }

                std::string num_units_str = str.substr(0, std::distance(str.begin(),num_end));
                double num_units = boost::lexical_cast<double>(num_units_str);
                size = (unsigned long) (num_units * unit_size);
            } 
        } else {
            // Invalid String
            throw LogicError("Invalid string size representation: string doesn't start with a digit or .");
        }
    } catch(const boost::bad_lexical_cast&){
        throw LogicError("Invalid string to number conversion");
    }

    return size;
}

    
/*
 * hash_string
 *
 * hash a string (for the moment use SDBM algorithm, when boost 1.33 will be 
 * used, switch to boost::hash)
 */
unsigned long glite::data::agents::hash_string(const std::string& str) throw() {
    unsigned long hash = 0;
    int c;
    std::string::const_iterator it;
    for(it = str.begin();it != str.end(); ++it){
        c = *it;
        hash = c + (hash << 6) + (hash << 16) - hash;
    }
    return hash;
}

/*
 * get_dirname
 *
 * Get the parent Directory name
 */
std::string glite::data::agents::get_dirname(const std::string& dir_path) throw (){
    char * parent_name_str = strdup(dir_path.c_str());
    ::dirname(parent_name_str);
    std::string parent_name = parent_name_str;
    free(parent_name_str);
    return parent_name;
}
