/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Python Helper Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PyHelper.cpp,v 1.4 2008/11/14 16:04:48 ptedesco Exp $ 
 *  Release: $Name:  $
 */
#include "glite/data/agents/python/PyHelper.h"
#include "glite/data/agents/python/PythonException.h"
#include "UtilsModule.h"
#include "SDModule.h"
#include <glite/data/config/service/Logger.hxx>

using namespace glite::data::agents::python;


namespace py = boost::python;

/*
 * Constants
 */
namespace {
const char * const MODULE_PYTHON_NAME   = "agents-python";
    
const char * const ENV_PYTHONPATH       = "PYTHONPATH";
#ifdef _WINDOWS
const char * const PATH_SEPARATOR       = ";";
#else
const char * const PATH_SEPARATOR       = ":";
#endif
}     
    
/*
 * instance
 *
 * Return the singleton Instance
 */
PyHelper& PyHelper::instance() {
    static PyHelper the_instance;
    return the_instance;
}

/*
 * Python
 *
 * Constructor
 */
PyHelper::PyHelper():m_logger(MODULE_PYTHON_NAME){
}
                                                                                                                          
/*
 * ~PyHelper
 *
 * Destructor
 */
PyHelper::~PyHelper(){
}


/*
 * setPath
 *
 * extend PYTHONPATH by adding additional path
 */
void PyHelper::setPath(const std::string& p) /*throw ()*/{
    const char * path = getenv(ENV_PYTHONPATH);
    if(0 == path){
        setenv(ENV_PYTHONPATH,p.c_str(),1);
    } else {
        std::string new_path = p + PATH_SEPARATOR + path;
        setenv(ENV_PYTHONPATH,new_path.c_str(),1);
    }
}

/*
 * import
 *
 * Import a module
 */
py::object PyHelper::import(const std::string& module_name) /*throw (PythonException)*/{
    py::object module;
    try{
         module = py::object(py::handle<>(py::borrowed(
            PyImport_ImportModule(const_cast<char*>(module_name.c_str())))));
    } catch(const std::exception& exc){
        m_log_error("Error importing module " << module_name << ". Reason is: " << exc.what() << " ");
        throw PythonException(exc.what());
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = lastError();
        m_log_error("Failed to import module " << module_name << ". Error " << err.first << " - " << err.second);
        throw PythonException("Failed to import module");
    }
    if((0 == module.ptr()) || (Py_None == module.ptr())){
        m_log_error("Failed to import module " << module_name);
        throw PythonException("Failed to import module");
    }
    return module;
}        

/*
 * getFunction
 *
 * Return a function object
 */
py::object PyHelper::getFunction(py::object& module, const std::string& function_name) /*throw (PythonException)*/{
    py::object function;
    try{
        // Get Dictionary
        py::object dict((py::handle<>(py::borrowed(PyModule_GetDict(module.ptr())))));
        if((0 != dict.ptr()) && (Py_None != dict.ptr())){
            // Get Function Object        
            function =  py::object(py::handle<>(py::borrowed(PyDict_GetItemString(
                            dict.ptr(),const_cast<char*>(function_name.c_str())))));
        }
    } catch(const std::exception& exc){
        m_log_error("Error getting function " << function_name << ". Reason is: " << exc.what() << " ");
        throw PythonException(exc.what());      
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = lastError();
        m_log_error("Failed to get function " << function_name << ". Error is " << err.first << " - " << err.second);
        throw PythonException("Failed to get function");
    }
    if((0 == function.ptr()) || (Py_None == function.ptr())){
        m_log_error("Failed to get function " << function_name);
        throw PythonException("Failed to get function");
    }
    return function;
}        


/*
 * init
 *
 * Initialize the Python Interpreter
 */
void PyHelper::init() /*throw (std::PythonException)*/ {

    try {
        // Initailizae Python Interpreter
        Py_Initialize();

        // Register the module with the interpreter
    	if(-1 == PyImport_AppendInittab(const_cast<char*>(UTILS_MODULE_NAME), init_sd)){
            throw PythonException("Failed to add glite.data.agents.utils to the interpreter's builtin modules");
        }
        // Register the module with the interpreter
    	if(-1 == PyImport_AppendInittab(const_cast<char*>(SD_MODULE_NAME), init_sd)){
            throw PythonException("Failed to add glite.data.agents.sd to the interpreter's builtin modules");
        }
        
        // Retrieve the main module
    	m_mainModule = py::object(py::handle<>(py::borrowed(PyImport_AddModule("__main__"))));
        if((0 == m_mainModule.ptr()) || (Py_None == m_mainModule.ptr())){
            throw PythonException("Failed to get main module");
        }
        // Retrieve the main module's namespace
        m_global = m_mainModule.attr("__dict__");
        if((0 == m_global.ptr()) || (Py_None == m_global.ptr())){
            throw PythonException("Failed to get global dictionary");
        }
    } catch(const PythonException& exc){
        m_log_error("Error Initializing the Python interpreter. Reason is: " << exc.what() << " ");
        throw ;
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = lastError();
        m_log_error("Error Initializing the Python interpreter. Error is " << err.first << " - " << err.second);
        throw PythonException("Failed to Initialize interpreter");
    } catch(const std::exception& exc){
        m_log_error("Error Initializing the Python interpreter. Reason is: " << exc.what() << " ");
        throw PythonException(exc.what());
    }
}


/*
 * fini
 *
 * Finalize the Python Interpreter
 */
void PyHelper::fini() /*throw ()*/ {

    try {
        // Initailizae Python Interpreter
        Py_Finalize();
    } catch(...){
        m_log_warn("Error Finalizing the Python interpreter");
        throw;
    }
}

/*
 * lastError
 *
 * Translate Python Error in type, value
 */
std::pair<std::string, std::string> PyHelper::lastError() /*throw ()*/{
    
    std::pair<std::string,std::string> err;
    PyObject *errobj, *errdata, *errtraceback, *pystring;
 
    // Get latest python error
    PyErr_Fetch(&errobj, &errdata, &errtraceback);
 
    pystring = 0;
    if((errobj != 0) && ((pystring = PyObject_Str(errobj)) != 0) && (PyString_Check(pystring))) {
        err.first  = PyString_AsString(pystring);
    } else {
        err.first = "Unknown Type";
    }
    Py_XDECREF(pystring);
 
    pystring = 0;
    if((errdata != 0) && ((pystring = PyObject_Str(errdata)) != 0) && (PyString_Check(pystring))) {
        err.second = PyString_AsString(pystring);
    } else {
        err.second =  "Unknown Reason";
    }
    Py_XDECREF(pystring);
 
    Py_XDECREF(errobj);
    Py_XDECREF(errdata);         /* caller owns all 3 */
    Py_XDECREF(errtraceback);    /* already NULL'd out */

    return err;
}
