/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: PythonConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PythonConfig.cpp,v 1.4 2008/11/14 16:04:48 ptedesco Exp $ 
 *  Release: $Name:  $
 *
 */
#include "PythonConfig.h"
#include "glite/data/agents/python/PyHelper.h"
#include <glite/data/config/service/Logger.hxx>
#include "glite/data/config/service/ParamValue.hxx"
#include "glite/data/agents/AgentExceptions.h"

using namespace glite::data::agents::python;
using namespace glite::config;


/*
 * Constants
 */
namespace {
const char * const MODULE_PYTHON_NAME        = "agents-python";
    
// Properties Names
const char * const PYTHONPATH_PROPERTY_NAME  = "PythonPath";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new PythonConfig();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    PythonConfig * s = dynamic_cast<PythonConfig*>(component);
    if(0 != s){
        delete s;
    }
}
                                                                                
} // End extern "C"

/*
 * PythonConfig
 *
 * Constructor
 */
PythonConfig::PythonConfig():
    ComponentConfiguration(MODULE_PYTHON_NAME), 
    m_logger(MODULE_PYTHON_NAME){
}
                                                                                                                          
/*
 * ~PythonConfig
 *
 * Destructor
 */
PythonConfig::~PythonConfig(){
}

/*
 * init
 *
 * Initialize the Component
 */
int PythonConfig::init(const Params& params){
    
    try{
        Params::const_iterator it;
    
        // Get Python Path
        std::string python_path;
        if((it = params.find(PYTHONPATH_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),PYTHONPATH_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            python_path = pv->getValue();
            PyHelper::instance().setPath(python_path);
        }
        
        // Initialize the interpreter
        PyHelper::instance().init();
        
        // Log Initialization Values
        m_log_info(getName() << " Initialized. Initialization Parameters are");
        if(!python_path.empty()){
            m_log_info("PythonPath    : " << python_path);
        }
    } catch(const LogicError& exc) {
        m_log_error("Initialization Logic Error for " << getName() << ": " << exc.what());
        return -1;
    } catch(const RuntimeError& exc) {
        m_log_error("Initialization Runtime Error for " << getName() << ": " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error("Unexpected Exception during " << getName() << " initialization: " << exc.what());
        throw;
    } catch(...) {
        m_log_error("Unhandled Exception during " << getName() << " initialization");
        throw;
    }
    return 0;
        
}

/*
 * config
 *
 * Configure the Component
 */
int PythonConfig::config(const Params& params){

    // Nothing to so
    m_log_info(getName() << " Configured");
    return 0;
}


/*
 * fini
 *
 * Configure the Component
 */
int PythonConfig::fini(){

    // Finalize The interpreter
    PyHelper::instance().fini();
    
    m_log_info(getName() << " Finalized");
    return 0;
}
