/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: Define the SD Class Wrapper Module
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDModule.cpp,v 1.4 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "SDModule.h"
#include "glite/data/agents/sd/sd-utility.h"

#include <boost/shared_ptr.hpp>
#include <vector>
#include <string>

using namespace glite::data::agents::python;

/*
 * Python Module Name
 */    
const char * const glite::data::agents::python::SD_MODULE_NAME = "_sd";

namespace glite {
namespace data  {
namespace agents{
namespace python{
    
/*
 * get_sd_version
 *
 * Return SD Version
 */
const char * get_sd_schema_version(){
    return ""; 
}

/*
 * get_sd_service
 *
 * Return a Service Object
 */
boost::shared_ptr<sd::Service> get_sd_service(const std::string& source, const std::string& type, const std::string& vo_name = "") {
    return boost::shared_ptr<sd::Service>(sd::get_service(source,type,vo_name));
}

/*
 * get_sd_service_by_type
 *
 * Return a Service Object by type
 */
boost::shared_ptr<sd::Service> get_sd_service_by_type(const std::string& type, const std::string& vo_name = ""){
    return boost::shared_ptr<sd::Service>(sd::get_service_by_type(type,vo_name));
}

/*
 * get_sd_associated_service
 *
 * Return an associated Service Object of the given type
 */
boost::shared_ptr<sd::Service> get_sd_associated_service(const std::string& assoc_srv_host,const std::string& assoc_srv_type, const std::string& srv_type,const std::string& vo_name = ""){
    return boost::shared_ptr<sd::Service>(sd::get_associated_service(assoc_srv_host,assoc_srv_type,srv_type,vo_name));
}

/*
 * get_sd_site_name
 *
 * Return an the site name for a given Source (Host/Site)
 */
std::string get_sd_site_name(const std::string& source, const std::string& type, const std::string& vo_name = ""){
    return sd::get_site_name(source,type,vo_name);
}

/*
 * get_sd_property
 *
 * Returna Service Object Property
 */
std::string get_sd_property(const sd::Service& srv, const std::string& prop_name, const std::string& vo_name = ""){
    return get_property(srv,prop_name,vo_name);
}

/**
 * Create and return the wrapper for the Job class
 */
void Service_class() {

    // Create the Python type object for Job Class
    static boost::python::object s_Service_class(
      boost::python::class_<sd::Service, boost::shared_ptr<sd::Service> >("Service")
        .def(boost::python::init<const std::string& // Name
                ,const std::string&                 // Type
                ,const std::string&                 // Endpoint
                ,const std::string&                 // version
                ,const std::string&                 // site
                ,const std::string&                 // hostname
                ,boost::python::optional< 
                  unsigned int                      // port
                 >
               >())
       .def(boost::python::init<const sd::Service&>())
       .def_readonly ("name"     , &sd::Service::name)
       .def_readonly ("type"     , &sd::Service::type)
       .def_readonly ("endpoint" , &sd::Service::endpoint)
       .def_readonly ("version"  , &sd::Service::version)
       .def_readonly ("site"     , &sd::Service::site)
       .def_readonly ("hostname" , &sd::Service::hostname)
       .def_readonly ("port"     , &sd::Service::port)
    );
}

} // end namespace python
} // end namespace agents
} // end namespace data
} // end namespace glite


// Overloads
BOOST_PYTHON_FUNCTION_OVERLOADS(get_sd_service_overloads, get_sd_service, 2, 3)
BOOST_PYTHON_FUNCTION_OVERLOADS(get_sd_service_by_type_overloads, get_sd_service_by_type, 1, 2)
BOOST_PYTHON_FUNCTION_OVERLOADS(get_sd_associated_service_overloads, get_sd_associated_service, 3, 4)
BOOST_PYTHON_FUNCTION_OVERLOADS(get_sd_property_overloads, get_sd_property, 2, 3)
BOOST_PYTHON_FUNCTION_OVERLOADS(get_sd_site_name_overloads, get_sd_site_name, 2, 3)

/*
 * Init Python Module
 */
BOOST_PYTHON_MODULE(_sd){

    // Declare GetSchemaVersion   
    boost::python::def("SchemaVersion",get_sd_schema_version);

    // Declare the wrapped classes
    Service_class();

    // SD functions wrapper
    boost::python::def("GetService"          ,get_sd_service, get_sd_service_overloads());
    boost::python::def("GetServiceByType"    ,get_sd_service_by_type,get_sd_service_by_type_overloads());
    boost::python::def("GetAssociatedService",get_sd_associated_service,get_sd_associated_service_overloads());
    boost::python::def("GetServiceProperty"  ,get_sd_property, get_sd_property_overloads());
    boost::python::def("GetSiteName"         ,get_sd_site_name,get_sd_site_name_overloads());
    
} // end module glite_fts
