/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Define the Utils Wrapper Module
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: UtilsModule.cpp,v 1.6 2008/01/07 16:58:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "UtilsModule.h"
#include "glite/data/agents/url-utility.h"
#include "glite/data/agents/Url.h"
#include "glite/data/agents/Guid.h"
#include "glite/data/agents/cred/cred-utility.h"
#include "glite/data/agents/cred/CredServiceException.h"

#include <boost/shared_ptr.hpp>
#include <log4cpp/Category.hh>
#include <vector>
#include <string>

using namespace glite::data::agents::python;

/*
 * Python Module Name
 */    
const char * const glite::data::agents::python::UTILS_MODULE_NAME = "_utils";

namespace glite {
namespace data  {
namespace agents{
namespace python{
    
/*
 * log_entry
 *
 * Insert a log entry
 */
void log_entry(const char * component,log4cpp::Priority::PriorityLevel p, const char * msg){
    log4cpp::Category::getInstance(component).log(p,msg);
}

/*
 * log_debug
 *
 * Insert a debug log entry
 */
void log_debug(const char * component, const char * msg){
    log_entry(component,log4cpp::Priority::DEBUG,msg);
}
/*
 * log_info
 *
 * Insert a info log entry
 */
void log_info(const char * component, const char * msg){
    log_entry(component,log4cpp::Priority::INFO,msg);
}
/*
 * log_warn
 *
 * Insert a warn log entry
 */
void log_warn(const char * component, const char * msg){
    log_entry(component,log4cpp::Priority::WARN,msg);
}
/*
 * log_error
 *
 * Insert a erro log entry
 */
void log_error(const char * component, const char * msg){
    log_entry(component,log4cpp::Priority::ERROR,msg);
}

/**
 * Create and return the wrapper for the Priority::PriorityLevel enum
 */
void LogPriority_enum() {

    // Expose log4cpp::Priority as LogPriority
    static boost::python::object s_LogPriority_enum(
      boost::python::enum_<log4cpp::Priority::PriorityLevel>("LogPriority")
        .value("Debug"    , log4cpp::Priority::DEBUG)
        .value("Info"     , log4cpp::Priority::INFO)
        .value("Notice"   , log4cpp::Priority::NOTICE)
        .value("Warn"     , log4cpp::Priority::WARN)
        .value("Error"    , log4cpp::Priority::ERROR)
        .value("Critical" , log4cpp::Priority::CRIT)
        .value("Alert"    , log4cpp::Priority::ALERT)
        .value("Fatal"    , log4cpp::Priority::FATAL)
        .value("Emergency", log4cpp::Priority::EMERG)
        .export_values()
    );
}

/**
 * Create and return the wrapper for the Url class
 */
void Url_class() {
    
    // Create the Python type object for Transfer Class
    static boost::python::object s_Url_class(
      boost::python::class_<Url, boost::shared_ptr<Url> >("Url")
       .def(boost::python::init<const std::string>())
       .def(boost::python::init<const Url&>())
       .def_readwrite("schema", &Url::schema)
       .def_readwrite("host"  , &Url::host)
       .def_readwrite("port"  , &Url::port)
       .def_readwrite("path"  , &Url::path)
       .def_readwrite("query" , &Url::query)
       .def("isGood" , &Url::isGood)
       .def("str"    , &Url::str)
       .def("__str__", &Url::str)
    );
}

/**
 * Operator used by the Str Function
 */
std::ostream& operator<<(std::ostream& s,Guid& g){
    s << g.str();
    return s;
}

/**
 * Create and return the wrapper for the Guid class
 */
void Guid_class() {
    
    // Create the Python type object for Guid Class
    static boost::python::object s_Guid_class(
      boost::python::class_<Guid, boost::shared_ptr<Guid> >("Guid")
       .def(boost::python::init<const std::string>())
       .def(boost::python::init<const Guid&>())
       .def("isNull"      , &Guid::isNull)
       .def("hash"        , &Guid::hash)
       .def("crossSumHash", &Guid::crossSumHash)
       .def("__str__"     , &Guid::str)
       .def(boost::python::self < boost::python::self)
    );
}

/**
 * Get Proxy Certificate File Name for the given user
 */
std::string get_user_proxy_cert(const std::string& user_dn, const std::string& user_cred, const std::string& endpoint, const std::string& cred_type) /*throw ()*/{
    std::string proxy_file;
    std::string cred_server_endpoint = endpoint;
    try{
        proxy_file = cred::get_proxy_cert(user_dn,user_cred,"",cred_server_endpoint,"","",false,cred_type);
    }catch(const cred::CredServiceException& exc){
        log_error(UTILS_MODULE_NAME,exc.what());
    }catch(const LogicError& exc){
        log_error(UTILS_MODULE_NAME,exc.what());
    }
    return proxy_file;
}

} // end namespace python
} // end namespace agents
} // end namespace data
} // end namespace glite

BOOST_PYTHON_FUNCTION_OVERLOADS(get_user_proxy_cert_overload, get_user_proxy_cert, 2, 4)

/*
 * Init Python Module
 */
BOOST_PYTHON_MODULE(_utils){

    // Log4Cpp wrapper
    LogPriority_enum();
    boost::python::def("Log",log_entry);
    boost::python::def("LogDebug",log_debug);
    boost::python::def("LogInfo" ,log_info);
    boost::python::def("LogWarn" ,log_warn);
    boost::python::def("LogError",log_error);
    
    // Url wrapper
    Url_class();
    
    // Guid class
    Guid_class();
   
    // Get Proxy Certificate File Name
    boost::python::def("GetProxyFileName",get_user_proxy_cert, get_user_proxy_cert_overload());
    
} // end module utils
