/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agent Common: SDCache Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDCache.cpp,v 1.7 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/sd/SDCache.h"
#include "SDCacheImpl.h"
#include <glite/data/config/service/Logger.hxx>

using namespace glite::data::agents;
using namespace glite::data::agents::sd;


/*
 * Constants
 */
namespace {
const char * const SDCACHE_NAME = "transfer-agent-sd-cache";
} // End global namespace

/*
 * SDCache
 *
 * Constructor
 */
SDCache::SDCache():
    m_logger(SDCACHE_NAME),
    m_impl(new SDCacheImpl(m_logger)){
}
    
/*
 * ~SDCache
 *
 * Destructor
 */
SDCache::~SDCache(){
}

/*
 * ttl
 *
 * Set the Validity of an entry in the Cache
 */
void SDCache::ttl(unsigned int v){
    m_impl->ttl(v);
}
/*
 * staleTime
 *
 * Set the Increment of the validity of an stale entry in the Cache
 */
void SDCache::staleTime(unsigned int v){
    m_impl->staleTime(v);
}
/*
 * obsoleteTime
 *
 * Set the time that should elapse before considering an entry obsolete
 */
void SDCache::obsoleteTime(unsigned int v){
    m_impl->obsoleteTime(v);
}
/*
 * negativeObsoleteTime
 *
 * Set the time that should elapse before considering a negative result obsolete
 */
void SDCache::negativeObsoleteTime(unsigned int v){
    m_impl->negativeObsoleteTime(v);
}

/*
 * insert
 *
 * Insert a Service in the Cache
 */
void SDCache::insert(const Service& service){
    StringArray none;
    m_impl->insert(service,none);
}
void SDCache::insert(const Service& service, const std::string& vo_name){
    StringArray vo_list;
    vo_list.push_back(vo_name);
    m_impl->insert(service,vo_list);
}
void SDCache::insert(const Service& service, const StringArray& vo_list){
    m_impl->insert(service,vo_list);
}

/*
 * associate
 *
 * Store a service association into the Cache
 */
void SDCache::associate(const Service& service, const Service& associated_service){
    m_impl->associate(service,associated_service);
}

/*
 * insertProperty
 *
 * Store a property into the Cache
 */
void SDCache::insertProperty(const Service& service, const std::string& name, const std::string& value){
    StringArray none;
    m_impl->insertProperty(service,name,value,none);
}
void SDCache::insertProperty(const Service& service, const std::string& name, const std::string& value, const std::string& vo_name){
    StringArray vo_list;
    vo_list.push_back(vo_name);
    m_impl->insertProperty(service,name,value,vo_list);
}
void SDCache::insertProperty(const Service& service, const std::string& name, const std::string& value, const StringArray& vo_list){
    m_impl->insertProperty(service,name,value,vo_list);
}

/*
 * clear
 *
 * clear the Cache
 */
void SDCache::clear(){
    m_impl->clear();
}

/*
 * purge
 *
 * purge the Cache
 */
void SDCache::purge(){
    m_impl->purge();
}

/*
 * getByName
 *
 * Get the service by Name and VO (enforce authorization)
 */
const Service * SDCache::getByName(const std::string& name){
    StringArray none;
    return m_impl->getByName(name,none);
}
const Service * SDCache::getByName(const std::string& name, const std::string& vo_name){
    StringArray vo_list;
    vo_list.push_back(vo_name);
    return m_impl->getByName(name,vo_list);
}
const Service * SDCache::getByName(const std::string& name, const StringArray& vo_list){
    return m_impl->getByName(name,vo_list);    
}

/*
 * getByType
 *
 * Get the list of services by type
 */
SDCache::ServiceArray SDCache::getByType(const std::string& type){
    StringArray none;
    return m_impl->getByType(type,none);
}
SDCache::ServiceArray SDCache::getByType(const std::string& type, const std::string& vo_name){
    StringArray vo_list;
    vo_list.push_back(vo_name);
    return m_impl->getByType(type,vo_list);
}
SDCache::ServiceArray SDCache::getByType(const std::string& type, const StringArray& vo_list){
    return m_impl->getByType(type,vo_list);
}

/*
 * getByHost
 *
 * Get the list of services by host and type
 */
SDCache::ServiceArray SDCache::getByHost(const std::string& type, const std::string& host){
    StringArray none;
    return m_impl->getByHost(type,host,none);
}
SDCache::ServiceArray SDCache::getByHost(const std::string& type, const std::string& host,const std::string& vo_name){
    StringArray vo_list;
    vo_list.push_back(vo_name);
    return m_impl->getByHost(type,host,vo_list);
}
SDCache::ServiceArray SDCache::getByHost(const std::string& type, const std::string& host,const StringArray& vo_list){
    return m_impl->getByHost(type,host,vo_list);
}

/*
 * getBySite
 *
 * Get the list of services by site and type
 */
SDCache::ServiceArray SDCache::getBySite(const std::string& type, const std::string& site){
    StringArray none;
    return m_impl->getBySite(type,site,none);
}
SDCache::ServiceArray SDCache::getBySite(const std::string& type, const std::string& site,const std::string& vo_name){
    StringArray vo_list;
    vo_list.push_back(vo_name);
    return m_impl->getBySite(type,site,vo_list);
}
SDCache::ServiceArray SDCache::getBySite(const std::string& type, const std::string& site,const StringArray& vo_list){
    return m_impl->getBySite(type,site,vo_list);
}

/*
 * getAssociated
 *
 * Get the list of associated services by type
 */
SDCache::ServiceArray SDCache::getAssociated(const std::string& name, const std::string& assoc_type){
    StringArray none;
    return m_impl->getAssociated(name,assoc_type,none);
}
SDCache::ServiceArray SDCache::getAssociated(const std::string& name, const std::string& assoc_type, const std::string& vo_name){
    StringArray vo_list;
    vo_list.push_back(vo_name);
    return m_impl->getAssociated(name,assoc_type,vo_list);
}
SDCache::ServiceArray SDCache::getAssociated(const std::string& name, const std::string& assoc_type, const StringArray& vo_list){
    return m_impl->getAssociated(name,assoc_type,vo_list);
}

/*
 * getAssociated
 *
 * Get the list of associated services by type and site
 */
SDCache::ServiceArray SDCache::getAssociatedOnSite(const std::string& name, const std::string& assoc_type, const std::string& assoc_site){
    StringArray none;
    return m_impl->getAssociated(name,assoc_type,assoc_site,none);
}
SDCache::ServiceArray SDCache::getAssociatedOnSite(const std::string& name, const std::string& assoc_type, const std::string& assoc_site, const std::string& vo_name){
    StringArray vo_list;
    vo_list.push_back(vo_name);
    return m_impl->getAssociated(name,assoc_type,assoc_site,vo_list);
}
SDCache::ServiceArray SDCache::getAssociatedOnSite(const std::string& name, const std::string& assoc_type, const std::string& assoc_site, const StringArray& vo_list){
    return m_impl->getAssociated(name,assoc_type,assoc_site,vo_list);
}

/*
 * getProperty
 *
 * Get a Property for a given Service. If no property is found, the second 
 * field of the pair is set to false
 */
std::pair<std::string,bool> SDCache::getProperty(const std::string& srv_name, const std::string& prop_name){
    StringArray none;
    return m_impl->getProperty(srv_name,prop_name,none);
}
std::pair<std::string,bool> SDCache::getProperty(const std::string& srv_name, const std::string& prop_name,const std::string& vo_name){
    StringArray vo_list;
    vo_list.push_back(vo_name);
    return m_impl->getProperty(srv_name,prop_name,vo_list);
}
std::pair<std::string,bool> SDCache::getProperty(const std::string& srv_name, const std::string& prop_name,const StringArray& vo_list){
    return m_impl->getProperty(srv_name,prop_name,vo_list);
}

/*
 * rememberMissing
 *
 * Remeber that a Service is not listed in the information system
 */
void SDCache::rememberMissing(const std::string& type){
    StringArray none;
    return m_impl->rememberMissing(type,"",none);
}
void SDCache::rememberMissing(const std::string& type, const StringArray& vo_list){
    return m_impl->rememberMissing(type,"",vo_list);
}
void SDCache::rememberMissing(const std::string& type, const std::string& source){
    StringArray none;
    return m_impl->rememberMissing(type,source,none);
}
void SDCache::rememberMissing(const std::string& type, const std::string& source, const StringArray& vo_list){
    return m_impl->rememberMissing(type,source,vo_list);
}

/*
 * rememberMissingProperty
 *
 * Remeber that a ServiceProeprty is not listed in the information system
 */
void SDCache::rememberMissingProperty(const std::string& service_name, const std::string& prop_name){
    StringArray none;
    return m_impl->rememberMissingProperty(service_name,prop_name,none);
}
void SDCache::rememberMissingProperty(const std::string& service_name, const std::string& prop_name, const StringArray& vo_list){
    return m_impl->rememberMissingProperty(service_name,prop_name,vo_list);
}

/*
 * rememberMissingAssociation
 *
 * Remeber that a Service doesn't have associated services of a given type
 * listed in the information system
 */
void SDCache::rememberMissingAssociation(const std::string& service_name, const std::string& type){
    StringArray none;
    return m_impl->rememberMissingAssociation(service_name,type,"",none);
}
void SDCache::rememberMissingAssociation(const std::string& service_name, const std::string& type, const StringArray& vo_list){
    return m_impl->rememberMissingAssociation(service_name,type,"",vo_list);
}
void SDCache::rememberMissingAssociation(const std::string& service_name, const std::string& type, const std::string& site){
    StringArray none;
    return m_impl->rememberMissingAssociation(service_name,type,site,none);
}
void SDCache::rememberMissingAssociation(const std::string& service_name, const std::string& type, const std::string& site, const StringArray& vo_list){
    return m_impl->rememberMissingAssociation(service_name,type,site,vo_list);
}

/*
 * isServiceMissing
 *
 * Check if a Service is not listed in the information system
 */
bool SDCache::isServiceMissing(const std::string& type){
    StringArray none;
    return m_impl->isServiceMissing(type,"",none);
}
bool SDCache::isServiceMissing(const std::string& type, const StringArray& vo_list){
    return m_impl->isServiceMissing(type,"",vo_list);
}
bool SDCache::isServiceMissing(const std::string& type, const std::string& source){
    StringArray none;
    return m_impl->isServiceMissing(type,source,none);
}
bool SDCache::isServiceMissing(const std::string& type, const std::string& source, const StringArray& vo_list){
    return m_impl->isServiceMissing(type,source,vo_list);
}

/*
 * isPropertyMissing
 *
 * Check if a Service Property is not listed in the information system
 */
bool SDCache::isPropertyMissing(const std::string& service_name, const std::string& prop_name){
    StringArray none;
    return m_impl->isPropertyMissing(service_name,prop_name,none);
}
bool SDCache::isPropertyMissing(const std::string& service_name, const std::string& prop_name, const StringArray& vo_list){
    return m_impl->isPropertyMissing(service_name,prop_name,vo_list);
}

/*
 * isAssociationMissing
 *
 * Remeber that a Service doesn't have associated services of a given type
 * listed in the information system
 */
bool SDCache::isAssociationMissing(const std::string& service_name, const std::string& type){
    StringArray none;
    return m_impl->isAssociationMissing(service_name,type,"",none);
}
bool SDCache::isAssociationMissing(const std::string& service_name, const std::string& type, const StringArray& vo_list){
    return m_impl->isAssociationMissing(service_name,type,"",vo_list);
}
bool SDCache::isAssociationMissing(const std::string& service_name, const std::string& type, const std::string& site){
    StringArray none;
    return m_impl->isAssociationMissing(service_name,type,site,none);
}
bool SDCache::isAssociationMissing(const std::string& service_name, const std::string& type, const std::string& site, const StringArray& vo_list){
    return m_impl->isAssociationMissing(service_name,type,site,vo_list);
}
