/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common : SDCache Implementation Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDCacheImpl.cpp,v 1.13 2008/11/17 10:21:08 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "SDCacheImpl.h"
#include "glite/data/agents/AgentExceptions.h"
#include "glite/data/agents/sd/ServiceDiscovery.h"
#include "glite/data/agents/sd/ServiceDiscoveryException.h"
#include <boost/tuple/tuple.hpp>
#include <boost/scoped_ptr.hpp>

#include <glite/data/config/service/Logger.hxx>

using namespace glite::data::agents;
using namespace glite::data::agents::sd;
using namespace boost::multi_index;


namespace {
const char * const EMPTY_VO_NAME                  = "";
    
const unsigned int DEFAULT_TTL                    =  3600; // 1 hour
const unsigned int DEFAULT_STALE_TIME             =   900; // 15 minutes
const unsigned int DEFAULT_OBSOLETE_TIME          = 10800; // 3 hour
const unsigned int DEFAULT_NEGATIVE_OBSOLETE_TIME =   300; // 5 minutes
    
/**
 * Update Service Entry
 */    
struct update_service{
    /**
     * Constructor
     */
    update_service(const Service& s):m_service(s){}

    /**
     * Call Operator
     */
    void operator()(SDCacheImpl::ServiceEntry& e){
        if(0 != e.srv.get()){
            e.srv->type      = m_service.type;
            e.srv->endpoint  = m_service.endpoint;
            e.srv->version   = m_service.version;
            e.srv->site      = m_service.site;
            e.srv->hostname  = m_service.hostname;
            e.srv->port      = m_service.port;
        } else {
            e.srv.reset(new Service(m_service));
        }
        // Covert the site always to upper case
        std::transform(e.srv->site.begin(),e.srv->site.end(),e.srv->site.begin(),toupper);
    }
private:
    /**
     * The new service to copy from
     */
    const Service& m_service;
};

/**
 * Update Property Entry
 */    
struct update_property{
    /**
     * Constructor
     */
    update_property(const std::string& v):m_value(v){}

    /**
     * Call Operator
     */
    void operator()(SDCacheImpl::Property& p){
        p.value = m_value;
    }
private:
    /**
     * The new property value to update
     */
    const std::string& m_value;
};

/**
 * Is an entry obsolete ?
 */
struct is_obsolete : public std::unary_function<const SDCacheImpl::CacheEntry&, bool>{
    /**
     * Constructor
     */
    is_obsolete(time_t t):m_obsoleteTime(t){}

    /**
     * Call Operator
     */
    bool operator()(const SDCacheImpl::CacheEntry& e) const {
        if(e.creationTime < m_obsoleteTime){
            return true;
        }
        return false;
    }
private:
    /**
     * The Obsolete threshold
     */
    time_t m_obsoleteTime;
};

/**
 * Remove obsolete entries from a cache table
 */
template<typename C>
unsigned int remove_obsolete(C& t, time_t obsolete_time){

    unsigned int count = 0;
    
    // Create Obsolete predicate
    is_obsolete pred(obsolete_time);
    
    // For each element
    typename C::iterator it;
    do{
        // Check if it's obsolete
        it = std::find_if(t.begin(),t.end(),pred);
        if(it != t.end()){
            // Remove it from cache
            t.erase(it);
            ++count;
        } else {
            break;
        }
    } while(1);

    return count;
}

}
/*
 * SDCache
 *
 * Constructor
 */
SDCacheImpl::SDCacheImpl(const glite::config::Logger& logger):
    m_logger(logger),
    m_ttl(DEFAULT_TTL),
    m_staleTime(DEFAULT_STALE_TIME),
    m_obsoleteTime(DEFAULT_OBSOLETE_TIME),
    m_negativeObsoleteTime(DEFAULT_NEGATIVE_OBSOLETE_TIME){
}
    
/*
 * ~SDCache
 *
 * Destructor
 */
SDCacheImpl::~SDCacheImpl(){
}

/*
 * insertServiceEntry
 *
 * Insert a ServiceEntry in the Cache
 */
const SDCacheImpl::ServiceEntry * SDCacheImpl::insertServiceEntry(const Service& service) /*throw()*/{
    
    m_log_debug("Inserting entry for service : name='" << service.name << "' type='" << service.type << "' endpoint='" << service.endpoint << "' version='" << service.version << "' site='" << service.site << "' hostname='" << service.hostname << "' port='" << service.port << "'");
    
    const ServiceEntry * srv_entry = 0;

    ServiceEntry new_entry;
    new_entry.srv.reset(new Service(service));
    // Convert always the site name to uppercase
    std::transform(new_entry.srv->site.begin(),new_entry.srv->site.end(),new_entry.srv->site.begin(),toupper);
    
    // Try To insert service
    std::pair<ServiceTable::iterator,bool> result = 
        m_serviceTable.insert(new_entry);
    srv_entry = &*(result.first);
    if(true == result.second) {
        // New entry inserted in cachein the list
        m_log_debug("Service <" << srv_entry->srv->name << "> inserted in cache");
    } else {
        // Service already there: update it
        m_serviceTable.modify(result.first, update_service(service));
        m_log_debug("Service entry <" << srv_entry->srv->name << "> updated in cache");
    }
    // Refresh the Service entry's properties
    time(&(srv_entry->creationTime));
    srv_entry->validity = m_ttl;

    return srv_entry;
}

/*
 * insertVOEntry
 *
 * Insert a VO entry in the Cache
 */
const SDCacheImpl::VirtualOrganizationEntry * SDCacheImpl::insertVOEntry(const std::string& vo) /*throw()*/{
    const VirtualOrganizationEntry * vo_entry = 0;
    
    VirtualOrganizationEntry new_entry;
    new_entry.vo.reset(new VirtualOrganization(vo));
    // Try to insert entry
    std::pair<VirtualOrganizationTable::iterator, bool> result = 
        m_voTable.insert(new_entry);
    vo_entry = &*(result.first);
    if(true == result.second){
        // New entry inserted
        m_log_debug("VO <" << vo_entry->vo->name << "> inserted in cache");
    }
    // Refresh the VO entry's properties
    time(&(vo_entry->creationTime));
    vo_entry->validity = m_ttl;
    
    return vo_entry;
}
  
/*
 * insert
 *
 * Insert a Service in the Cache
 */
void SDCacheImpl::insert(
    const Service&      service, 
    const StringArray&  vo_list) /*throw()*/{
    
    m_log_debug("insert : service '" << service.name << "', vo_list[" << vo_list.size() << "]");
            
    // Insert Service Entry (Or refresh the existing one)
    const ServiceEntry * srv_entry = insertServiceEntry(service);
    
    // Insert VO entries
    std::vector<const VirtualOrganizationEntry *> vos;
    if(false == vo_list.empty()){
        vos.reserve(vo_list.size());
        // For each VO
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            // Insert VO Entry
            const VirtualOrganizationEntry * vo_entry = insertVOEntry(*vo_it);
            vos.push_back(vo_entry);
        }
    } else {
        // Use the fake "empty" VO
        const VirtualOrganizationEntry * vo_entry = insertVOEntry(EMPTY_VO_NAME);
        vos.push_back(vo_entry);
    }        

    // For each VO, Store the association Service - VO
    std::vector<const VirtualOrganizationEntry*>::iterator it;
    for(it = vos.begin(); it != vos.end();++it){
        // Save Association Service - VO
        const VOServiceEntry * vo_service_entry = 0;
        
        // Try to insert a new entry in cache
        VOServiceEntry new_entry(srv_entry->srv,(*it)->vo);
        std::pair<VOServiceTable::iterator, bool> result = 
            m_voServiceTable.insert(new_entry);
        vo_service_entry = &*(result.first);
        if(true == result.second){
            // New entry inserted in cache
            m_log_debug("Association Service <" << vo_service_entry->srv->name << "> - VO <" << vo_service_entry->vo->name << "> inserted in cache");
        }
        // Refresh the VOService entry's properties
        time(&(vo_service_entry->creationTime));
        vo_service_entry->validity = m_ttl;
    }        
}

/*
 * associate
 *
 * Store an association between two services in the Cache
 */
void SDCacheImpl::associate(
    const Service& service, 
    const Service& associated_service){

    m_log_debug("associate : service '" << service.name << "', service '" << associated_service.name << "'");
    // Insert Service in Cache
    const ServiceEntry * srv_entry = insertServiceEntry(service);
    // Insert Associated service in Cache
    const ServiceEntry * assoc_srv_entry = insertServiceEntry(associated_service);
    
    // Try To insert the Association
    std::pair<AssociationTable::iterator, bool> result = 
        m_associationTable.insert(Association(srv_entry->srv,assoc_srv_entry->srv));
    const Association * assoc_entry = &*(result.first);
    // Store the Association between the two services
    if(true == (result.second)){ 
        m_log_debug("Association Service <" << assoc_entry->srv->name << "> - Service <" << assoc_entry->associated->name << "> inserted in cache");
    }
    // Refresh the Association entry's properties
    time(&(assoc_entry->creationTime));
    assoc_entry->validity = m_ttl;
}

/*
 * insertProperty
 *
 * Insert a Property
 */
void SDCacheImpl::insertProperty(
    const Service&      service, 
    const std::string&  prop_name, 
    const std::string&  value, 
    const StringArray&  vo_list){
    
    m_log_debug("insertProperty : service '" << service.name << "', property '" << prop_name << "', value '" << value << ", vo_list[" << vo_list.size() << "]");
    
    // Insert Service Entry if not exists
    const ServiceEntry * srv_entry = insertServiceEntry(service);
    
    // Insert VO entries
    std::vector<const VirtualOrganizationEntry*> vos;
    if(false == vo_list.empty()){
        vos.reserve(vo_list.size());
        // For each VO
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            // Insert VO Entry
            const VirtualOrganizationEntry * vo_entry = insertVOEntry(*vo_it);
            vos.push_back(vo_entry);
        }
    } else {
        // Use the fake "empty" VO
        const VirtualOrganizationEntry * vo_entry = insertVOEntry(EMPTY_VO_NAME);
        vos.push_back(vo_entry);
    }        

    // For Each Property, store the association between (Service;VO;name;value)
    std::vector<const VirtualOrganizationEntry*>::iterator it;
    for(it = vos.begin(); it != vos.end();++it){
        // Try To Store New Entry
        Property new_entry(srv_entry->srv,(*it)->vo,prop_name,value); 
        
        std::pair<PropertyTable::iterator,bool> result = m_propertyTable.insert(new_entry);
        const Property * prop_entry = &*(result.first);
        if(true == result.second){ 
            m_log_debug("Property Service <" << prop_entry->srv->name << ">:VO <" << prop_entry->vo->name << ">:Name <" << prop_entry->name << ">:Value <" << prop_entry->value << "> inserted in cache");
        } else {
            // Property already there: update it
            m_propertyTable.modify(result.first, update_property(value));
        }
        // Refresh the Property entry's properties
        time(&(prop_entry->creationTime));
        prop_entry->validity = m_ttl;
    }
}

/*
 * reset
 *
 * Reset the Cache
 */
void SDCacheImpl::clear(){

    // Cleanup the tables
    m_missingAssocServiceTable.clear();
    m_missingPropertyTable.clear();
    m_missingServiceTable.clear();
    m_propertyTable.clear();
    m_associationTable.clear();
    m_voServiceTable.clear();
    m_serviceTable.clear();
    m_voTable.clear();
    m_log_debug("Cache cleaned");
}

/*
 * purge
 *
 * Purge obsolete entries from the cache
 */
void SDCacheImpl::purge(){
    
    // Get Current Time
    time_t current, obsolete, missing_obsolete;
    time(&current);
    obsolete         = current - m_obsoleteTime;
    missing_obsolete = current - m_negativeObsoleteTime;
    
    unsigned int removed = 0;

    // Remove Obsolete Missing Association Entries
    removed = remove_obsolete<MissingAssocServiceTable>(m_missingAssocServiceTable,missing_obsolete);
    m_log_debug("Removed " << removed << " obsolete Missing Association Entries");
    // Remove Obsolete Missing Property Entries
    removed = remove_obsolete<MissingPropertyTable>(m_missingPropertyTable,missing_obsolete);
    m_log_debug("Removed " << removed << " obsolete Missing Property Entries");
    // Remove Obsolete Missing Service Entries
    removed = remove_obsolete<MissingServiceTable>(m_missingServiceTable,missing_obsolete);
    m_log_debug("Removed " << removed << " obsolete Missing Service Entries");
    // Remove Obsolete Property Entries
    removed = remove_obsolete<PropertyTable>(m_propertyTable,obsolete);
    m_log_debug("Removed " << removed << " obsolete Property Entries");
    // Remove Obsolete Association Entries
    removed = remove_obsolete<AssociationTable>(m_associationTable,obsolete);
    m_log_debug("Removed " << removed << " obsolete Association Entries");
    // Remove Obsolete VOService Entries
    removed = remove_obsolete<VOServiceTable>(m_voServiceTable,obsolete);
    m_log_debug("Removed " << removed << " obsolete VO-Service Entries");
    // Remove Obsolete VO Entries
    removed = remove_obsolete<VirtualOrganizationTable>(m_voTable,obsolete);
    m_log_debug("Removed " << removed << " obsolete VO Entries");
    // Remove Obsolete Service Entries
    removed = remove_obsolete<ServiceTable>(m_serviceTable,obsolete);
    m_log_debug("Removed " << removed << " obsolete Service Entries");
}

/*
 * getByName
 *
 * Get the service by Name and VO (enforce authorization)
 */
const Service * SDCacheImpl::getByName(const std::string& name, const StringArray& vo_list){
    
    const VOServiceEntry * svc = 0;
    typedef VOServiceTable::index<id_vo>::type  service_set_by_id_vo;
    service_set_by_id_vo& indx = m_voServiceTable.get<id_vo>();
    if(false == vo_list.empty()){
        // Loop for each VO. This may be not the best way of using multi-index, but 
        // usually there is just one vo so it should be fine
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            service_set_by_id_vo::iterator it = indx.find(boost::make_tuple(name,*vo_it));
            if(it != indx.end()){
                svc = &(*it);
                break;
            }           
        }            
    } 
    if(0 == svc){
        // Check for the EMPTY VO
        service_set_by_id_vo::iterator it = indx.find(boost::make_tuple(name,EMPTY_VO_NAME));
        if(it != indx.end()){
            svc = &(*it);
        }
    }
    if((0 == svc) || (0 == svc->srv)){
        m_log_debug("Service <" << name << "> not found in cache");
        return 0;
    } 

    // Check the validity of the entry
    time_t current;
    time(&current);
    if(current <= (svc->validity + svc->creationTime)){
        // Entry is valid: nothing to do
    } else {
        // Entry is stale: try to refresh it
        updateEntry(svc->srv->name);
    }
    
    return svc->srv.get();
}

/*
 * getByType
 *
 * Get the list of services by type
 */
SDCacheImpl::ServiceArray SDCacheImpl::getByType(const std::string& type, const StringArray& vo_list){

    std::set<const Service *> service_set;
    std::vector<const VOServiceEntry *> entries;
    
    SDCacheImpl::ServiceArray srv_array;
    typedef VOServiceTable::index<type_vo>::type  service_set_by_type_vo;
    service_set_by_type_vo& indx = m_voServiceTable.get<type_vo>();
    service_set_by_type_vo::iterator it0,it1;
    if(false == vo_list.empty()){
        // Loop for each VO. This may be not the best way of using multi-index, but 
        // usually there is just one vo so it should be fine
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            boost::tuples::tie(it0,it1) = indx.equal_range(boost::make_tuple(type,*vo_it));
            for(;it0 != it1;++it0){
                if(service_set.insert((*it0).srv.get()).second == true){
                    // Store Service
                    srv_array.push_back((*it0).srv.get());
                    entries.push_back(&(*it0));
                }
            }           
        }            
    } 
    // Check always for the Empty VO
    boost::tuples::tie(it0,it1) = indx.equal_range(boost::make_tuple(type,EMPTY_VO_NAME));
    for(;it0 != it1;++it0){
        // Avoid sduplicated entries
        if(service_set.insert((*it0).srv.get()).second == true){
            srv_array.push_back((*it0).srv.get());
            entries.push_back(&(*it0));
        }
    }
    if(srv_array.empty()){
        m_log_debug("No " << type << " services found in cache");
    } else {
        time_t current;
        time(&current);
        std::vector<const VOServiceEntry *>::iterator it;
        for(it = entries.begin(); it != entries.end(); ++it){
            // Check the validity of the entry
            if(current <= ((*it)->validity + (*it)->creationTime)){
                // Entry is valid: nothing to do
            } else {
                // Entry is stale: try to refresh it
                updateEntry((*it)->srv->name);
            }
        }
    }
    
    return srv_array;    
}

/*
 * getByHost
 *
 * Get the list of services by host and type
 */
SDCacheImpl::ServiceArray SDCacheImpl::getByHost(
    const std::string& type, 
    const std::string& host,
    const StringArray& vo_list){
    
    std::set<const Service *> service_set;
    std::vector<const VOServiceEntry *> entries;
    
    // transform hostname in lower case
    std::string hostname = host;
    std::transform(hostname.begin(),hostname.end(),hostname.begin(),tolower);
            
    SDCacheImpl::ServiceArray srv_array;
    typedef VOServiceTable::index<type_host_vo>::type  service_set_by_type_host_vo;
    service_set_by_type_host_vo& indx = m_voServiceTable.get<type_host_vo>();
    service_set_by_type_host_vo::iterator it0,it1;
    if(false == vo_list.empty()){
        // Loop for each VO. This may be not the best way of using multi-index, but 
        // usually there is just one vo so it should be fine
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            boost::tuples::tie(it0,it1) = indx.equal_range(boost::make_tuple(type,hostname,*vo_it));
            for(;it0 != it1;++it0){
                if(service_set.insert((*it0).srv.get()).second == true){
                    srv_array.push_back((*it0).srv.get());
                    entries.push_back(&(*it0));
                }
            }           
        }            
    } 
    // Check always for the empty VO
    boost::tuples::tie(it0,it1) = indx.equal_range(boost::make_tuple(type,hostname,EMPTY_VO_NAME));
    for(;it0 != it1;++it0){
        // Avoid duplicated entries
        if(service_set.insert((*it0).srv.get()).second == true){
            srv_array.push_back((*it0).srv.get());
            entries.push_back(&(*it0));
        }
    }
    if(srv_array.empty()){
        m_log_debug("No " << type << " services on host " << host << " found in cache");
    } else {
        time_t current;
        time(&current);
        std::vector<const VOServiceEntry *>::iterator it;
        for(it = entries.begin(); it != entries.end(); ++it){
            // Check the validity of the entry
            if(current <= ((*it)->validity + (*it)->creationTime)){
                // Entry is valid: nothing to do
            } else {
                // Entry is stale: try to refresh it
                updateEntry((*it)->srv->name);
            }
        }
    }
    
    return srv_array;    
}

/**
 * Get the list of services by site and type
 */
SDCacheImpl::ServiceArray SDCacheImpl::getBySite(const std::string& type, const std::string& site,const StringArray& vo_list){
    
    std::set<const Service *> service_set;
    std::vector<const VOServiceEntry *> entries;
    
    // convert site to upper_case
    std::string site_upr = site;
    std::transform(site_upr.begin(),site_upr.end(),site_upr.begin(),toupper);

    SDCacheImpl::ServiceArray srv_array;
    typedef VOServiceTable::index<type_site_vo>::type  service_set_by_type_site_vo;
    service_set_by_type_site_vo& indx = m_voServiceTable.get<type_site_vo>();
    service_set_by_type_site_vo::iterator it0,it1;
    if(false == vo_list.empty()){
        // Loop for each VO. This may be not the best way of using multi-index, but 
        // usually there is just one vo so it should be fine
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            boost::tuples::tie(it0,it1) = indx.equal_range(boost::make_tuple(type,site_upr,*vo_it));
            for(;it0 != it1;++it0){
                if(service_set.insert((*it0).srv.get()).second == true){
                    srv_array.push_back((*it0).srv.get());
                    entries.push_back(&(*it0));
                }
            }           
        }            
    } 
    // Check always for the empty VO
    boost::tuples::tie(it0,it1) = indx.equal_range(boost::make_tuple(type,site_upr,EMPTY_VO_NAME));
    for(;it0 != it1;++it0){
        // Avoid duplicated entries
        if(service_set.insert((*it0).srv.get()).second == true){
            srv_array.push_back((*it0).srv.get());
            entries.push_back(&(*it0));
        }
    }
    if(srv_array.empty()){
        m_log_debug("No " << type << " services on site " << site << " found in cache");
    } else {
        time_t current;
        time(&current);
         std::vector<const VOServiceEntry *>::iterator it;
        for(it = entries.begin(); it != entries.end(); ++it){
            // Check the validity of the entry
            if(current <= ((*it)->validity + (*it)->creationTime)){
                // Entry is valid: nothing to do
            } else {
                // Entry is stale: try to refresh it
                updateEntry((*it)->srv->name);
            }
        }
    }
    
    return srv_array;
}


/*
 * getAssociated
 *
 * Get the list of services of the given type that are associated with the given 
 * service
 */
SDCacheImpl::ServiceArray SDCacheImpl::getAssociated(const std::string& name, const std::string& assoc_srv_type, const StringArray& vo_list){

    std::vector<const Association *> entries;
    
    SDCacheImpl::ServiceArray assoc_srv_array;
    
    // Get the service by name
    const Service * srv = this->getByName(name, vo_list);
    if(0 == srv){
        // No service found or VO not authorized for this service
        return assoc_srv_array;
    }

    typedef AssociationTable::index<assoc_type>::type assoc_set_by_type;
    assoc_set_by_type& indx = m_associationTable.get<assoc_type>();
    assoc_set_by_type::iterator it0,it1;
    boost::tuples::tie(it0,it1) = indx.equal_range(boost::make_tuple(srv->name,assoc_srv_type));
    if(true == vo_list.empty()){
        // No VO specified. Return all the services
        for(;it0 != it1;++it0){
            assoc_srv_array.push_back((*it0).associated.get());
            entries.push_back(&(*it0));
        }
    } else {
        // Check if the VOs are authorized to see the service
        for(;it0 != it1;++it0){
            const Service * srv = this->getByName((*it0).associated->name, vo_list);
            if(0 != srv){
                assoc_srv_array.push_back((*it0).associated.get());
                entries.push_back(&(*it0));
            }
        }
    }
    if(assoc_srv_array.empty()){
        m_log_debug("No " << assoc_srv_type << " associated services for <" << name << "> found in cache");
        if(it0 != it1){
            // No association visible for that VO: remember missing association
            this->rememberMissingAssociation(name,assoc_srv_type,"",vo_list);
        }
    } else {
        bool update_assoc = false;
        time_t current;
        time(&current);
        std::vector<const Association *>::iterator it;
        for(it = entries.begin(); it != entries.end(); ++it){
            // Check the validity of the entry
            if(current <= ((*it)->validity + (*it)->creationTime)){
                // Entry is valid: nothing to do
            } else {
                // Entry is stale: All the association entries would be 
                // refreshed aftwerward in one go, so just reduce the validity time
                // (Missing association should become obsolete faster)
                update_assoc = true;                
                (*it)->validity = m_staleTime;
            }
        }
        if(true == update_assoc){
            // At least one entry is stale
            updateAssociations(srv, assoc_srv_type, "");
        }
    }

    return assoc_srv_array;
}

/*
 * getAssociated
 *
 * Get the list of services of the given type that are associated with the given
 * service and are in the given site 
 */
SDCacheImpl::ServiceArray SDCacheImpl::getAssociated(const std::string& name, const std::string& assoc_srv_type, const std::string& assoc_srv_site,  const StringArray& vo_list){

    std::vector<const Association *> entries;

    SDCacheImpl::ServiceArray assoc_srv_array;
    
    // Get the service by name
    const Service * srv = this->getByName(name, vo_list);
    if(0 == srv){
        // No service found or VO not authorized for this service
        return assoc_srv_array;
    }

    // convert site to upper_case
    std::string assoc_srv_site_upr = assoc_srv_site;
    std::transform(assoc_srv_site_upr.begin(),assoc_srv_site_upr.end(),assoc_srv_site_upr.begin(),toupper);

    typedef AssociationTable::index<assoc_type_site>::type assoc_set_by_type_site;
    assoc_set_by_type_site& indx = m_associationTable.get<assoc_type_site>();
    assoc_set_by_type_site::iterator it0,it1;
    boost::tuples::tie(it0,it1) = indx.equal_range(boost::make_tuple(srv->name,assoc_srv_type, assoc_srv_site_upr));
    if(true == vo_list.empty()){
        // No VO specified. Return all the services
        for(;it0 != it1;++it0){
            assoc_srv_array.push_back((*it0).associated.get());
            entries.push_back(&(*it0));
        }
    } else {
        // Check if the VOs are authorized to see the service
        for(;it0 != it1; ++it0){
            const Service * srv = this->getByName((*it0).associated->name, vo_list);
            if(0 != srv){
                assoc_srv_array.push_back(srv);
                entries.push_back(&(*it0));
            }
        }
    }
    if(assoc_srv_array.empty()){
        m_log_debug("No " << assoc_srv_type << " associated services for <" << name << "> on site " << assoc_srv_site << " found in cache");
        if(it0 != it1){
            // No association visible for that VO: remember missing association
            this->rememberMissingAssociation(name,assoc_srv_type,assoc_srv_site,vo_list);
        }
    } else {
        bool update_assoc = false;
        time_t current;
        time(&current);
        std::vector<const Association *>::iterator it;
        for(it = entries.begin(); it != entries.end(); ++it){
            // Check the validity of the entry
            if(current <= ((*it)->validity + (*it)->creationTime)){
                // Entry is valid: nothing to do
            } else {
                // Entry is stale: All the association entries would be 
                // refreshed aftwerward in one go, so just reduce the validity time
                // (Missing association should become obsolete faster)
                update_assoc = true;                
                (*it)->validity = m_staleTime;
            }
        }
        if(true == update_assoc){
            // At least one entry is stale
            updateAssociations(srv, assoc_srv_type, assoc_srv_site_upr);
        }
    }
    
    return assoc_srv_array;
}

/*
 * getProperty
 *
 * Get a Property for a given Service. If no property is found, the second 
 * field of the pair is set to false;
 */
std::pair<std::string,bool> SDCacheImpl::getProperty(const std::string& srv_name, const std::string& property_name,const StringArray& vo_list){
    
    std::vector<const Property *> entries;

    std::pair<std::string,bool> result;
    result.second = false;
    
    typedef PropertyTable::index<prop_name>::type prop_set_by_name;
    prop_set_by_name& indx = m_propertyTable.get<prop_name>();
    if(false == vo_list.empty()){
        // Loop for each VO. This may be not the best way of using multi-index, 
        // but usually there is just one vo so it should be fine
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            prop_set_by_name::iterator it = indx.find(boost::make_tuple(srv_name,*vo_it,property_name));
            if(it != indx.end()){
                result.first  = (*it).value;
                result.second = true;
                entries.push_back(&(*it));
                break;
            }           
        }            
    } 
    if(false == result.second) {
        // Check for the Empty VO
        prop_set_by_name::iterator it = indx.find(boost::make_tuple(srv_name,EMPTY_VO_NAME,property_name));
        if(it != indx.end()){
            result.first  = (*it).value;
            result.second = true;
            entries.push_back(&(*it));
        }
    }
    
    if(false == result.second){
        m_log_debug("Property <" << property_name << "> for service for <" << srv_name << "> not found in cache");
    } else {
        time_t current;
        time(&current);
        std::vector<const Property *>::iterator it;
        for(it = entries.begin(); it != entries.end(); ++it){
            // Check the validity of the entry
            if(current <= ((*it)->validity + (*it)->creationTime)){
                // Entry is valid: nothing to do
            } else {
                // Entry is stale: try to refresh it
                updateProperty(*it);
            }
        }
    }
    
    return result;
}


/*
 * rememberMissing
 *
 * Remeber that a Service is not listed in the information system
 */
void SDCacheImpl::rememberMissing(const std::string& type, const std::string& source, const StringArray& vo_list){
    time_t current;
    time(&current);        
    if(true == vo_list.empty()){
        // Remeber Missing Service
        std::pair<MissingServiceTable::iterator, bool> result = 
            m_missingServiceTable.insert(MissingService(type,source,EMPTY_VO_NAME));
        const MissingService * entry = &*(result.first);
        if(true == result.second){
            m_log_debug("Missing Service <" << type << ">:<" << source << "> recorded in cache");
        } else {
            // Missing Service entry already recorded
            m_log_debug("Missing Service <" << type << ">:<" << source << "> already in cache");
        }
        // Refresh the entry's properties
        entry->creationTime = current;
        entry->validity = m_ttl;
    } else {    
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            const std::string& vo_name = (*vo_it);
            // Remeber Missing Service
            std::pair<MissingServiceTable::iterator, bool> result = 
                m_missingServiceTable.insert(MissingService(type,source,vo_name));
            const MissingService * entry = &*(result.first);
            if(true == result.second){
                m_log_debug("Missing Service <" << type << ">:<" << source << ">:<" << vo_name << "> recorded in cache");
            } else {
                // Missing Service entry already recorded
                m_log_debug("Missing Service <" << type << ">:<" << source << ">:<" << vo_name << "> already in cache");
            } 
            // Refresh the entry's properties
            entry->creationTime = current;
            entry->validity = m_ttl;
        }
    }
}

/*
 * rememberMissingProperty
 *
 * Remeber that a ServiceProeprty is not listed in the information system
 */
void SDCacheImpl::rememberMissingProperty(const std::string& service_name, const std::string& prop_name, const StringArray& vo_list){
    time_t current;
    time(&current);        
    if(true == vo_list.empty()){
        // Remeber Missing Property
        std::pair<MissingPropertyTable::iterator, bool> result = 
            m_missingPropertyTable.insert(MissingProperty(service_name,prop_name,EMPTY_VO_NAME));
        const MissingProperty * entry = &*(result.first);
        if(true == result.second){
            m_log_debug("Missing Property <" << service_name << ">:<" << prop_name << "> recorded in cache");
        } else {
            // Missing Property entry already recorded
            m_log_debug("Missing Property <" << service_name << ">:<" << prop_name << "> already in cache");
        }    
        // Refresh the entry's properties
        entry->creationTime = current;
        entry->validity = m_ttl;
    } else {        
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            const std::string& vo_name = (*vo_it);
            // Remeber Missing Property
            std::pair<MissingPropertyTable::iterator, bool> result = 
                m_missingPropertyTable.insert(MissingProperty(service_name,prop_name,vo_name));
            const MissingProperty * entry = &*(result.first);
            if(true == result.second){
                m_log_debug("Missing Property <" << service_name << ">:<" << prop_name << ">:<" << vo_name << "> recorded in cache");
            } else {
                // Missing Property entry already recorded
                m_log_debug("Missing Property <" << service_name << ">:<" << prop_name << ">:<" << vo_name << "> already in cache");
            } 
            // Refresh the entry's properties
            entry->creationTime = current;
            entry->validity = m_ttl;
        }
    }
}

/*
 * rememberMissingAssociation
 *
 * Remeber that a Service doesn't have associated services of a given type
 * listed in the information system
 */
void SDCacheImpl::rememberMissingAssociation(const std::string& service_name, const std::string& type, const std::string& site, const StringArray& vo_list){
    time_t current;
    time(&current);        
    if(true == vo_list.empty()){
        // Remeber Missing Association
        std::pair<MissingAssocServiceTable::iterator , bool> result = 
            m_missingAssocServiceTable.insert(MissingAssocService(service_name,type,site,EMPTY_VO_NAME));
        const MissingAssocService * entry = &*(result.first);
        if(true == result.second){
            m_log_debug("Missing Association <" << service_name << ">:<" << type << ">:<" << site << "> recorded in cache");
        } else {
            // Missing Association entry already recorded
            m_log_debug("Missing Association <" << service_name << ">:<" << type << ">:<" << site << "> already in cache");
        }    
        // Refresh the entry's properties
        entry->creationTime = current;
        entry->validity = m_ttl;
    } else {        
        StringArray::const_iterator vo_it;
        for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
            const std::string& vo_name = (*vo_it);
            // Remeber Missing Property
            std::pair<MissingAssocServiceTable::iterator , bool> result = 
                m_missingAssocServiceTable.insert(MissingAssocService(service_name,type,site,vo_name));
            const MissingAssocService * entry = &*(result.first);
            if(true == result.second){
                m_log_debug("Missing Association <" << service_name << ">:<" << type << ">:<" << site << ">:<" << vo_name << "> recorded in cache");
            } else {
                // Missing Association entry already recorded
                m_log_debug("Missing Association <" << service_name << ">:<" << type << ":><" << site << ">:<" << vo_name << "> already in cache");
            } 
            // Refresh the entry's properties
            entry->creationTime = current;
            entry->validity = m_ttl;
        }
    }
}

/*
 * isServiceMissing
 *
 * Check if a Service is not listed in the information system
 */
bool SDCacheImpl::isServiceMissing(const std::string& type, const std::string& source, const StringArray& vo_list){
    
    bool result = true;
    
    // Check if service is missing
    typedef MissingServiceTable::index<id>::type  set_by_id;
    // Check if entry exists
    set_by_id& index = m_missingServiceTable.get<id>();
    
    // Check first if it's missing for any VO
    set_by_id::iterator srv_it = index.find(boost::make_tuple(type,source,EMPTY_VO_NAME));
    if(srv_it == index.end()) {
        if(vo_list.empty()){
            result = false;
        } else {
            // Service is not recorded as missing for any vo, check for each VO Name
            StringArray::const_iterator vo_it;
            for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
                const std::string& vo_name = (*vo_it);
                set_by_id::iterator srv_it = index.find(boost::make_tuple(type,source,vo_name));
                if(srv_it == index.end()) {
                    // Service is not recorded as missing for that VO
                    result = false;
                    break;
                } 
            } // end for each VO
        }
    }    
    if(true == result){
        m_log_debug("Service <" << type << ">:<" << source << "> not listed in the information system");
    }
    return result;
}

/*
 * isPropertyMissing
 *
 * Check if a Service Property is not listed in the information system
 */
bool SDCacheImpl::isPropertyMissing(const std::string& service_name, const std::string& prop_name, const StringArray& vo_list){
    bool result = true;
    
    // Check if service is missing
    typedef MissingPropertyTable::index<id>::type  set_by_id;
    // Check if entry exists
    set_by_id& index = m_missingPropertyTable.get<id>();
    
    // Check first if it's missing for any VO
    set_by_id::iterator srv_it = index.find(boost::make_tuple(service_name,prop_name,EMPTY_VO_NAME));
    if(srv_it == index.end()) {
        if(vo_list.empty()){
            result = false;
        } else {
            // Property is not recorded as missing for any vo, check for each VO Name
            StringArray::const_iterator vo_it;
            for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
                const std::string& vo_name = (*vo_it);
                set_by_id::iterator srv_it = index.find(boost::make_tuple(service_name,prop_name,vo_name));
                if(srv_it == index.end()) {
                    // Proeprty is not recorded as missing for that VO
                    result = false;
                    break;
                } 
            } // end for each VO
        }
    }    
    if(true == result){
        m_log_debug("Property <" << service_name << ">:<" << prop_name << "> not listed in the information system");
    }
    return result;
}

/**
 * Remeber that a Service doesn't have associated services of a given type
 * listed in the information system
 */
bool SDCacheImpl::isAssociationMissing(const std::string& service_name, const std::string& type, const std::string& site, const StringArray& vo_list){
    bool result = true;
    
    // Check if service is missing
    typedef MissingAssocServiceTable::index<id>::type  set_by_id;
    // Check if entry exists
    set_by_id& index = m_missingAssocServiceTable.get<id>();
    
    // Check first if it's missing for any VO
    set_by_id::iterator srv_it = index.find(boost::make_tuple(service_name,type,site,EMPTY_VO_NAME));
    if(srv_it == index.end()) {
        if(vo_list.empty()){
            result = false;
        } else {
            // Association is not recorded as missing for any vo, check for each VO Name
            StringArray::const_iterator vo_it;
            for(vo_it = vo_list.begin(); vo_it != vo_list.end();++vo_it){
                const std::string& vo_name = (*vo_it);
                set_by_id::iterator srv_it = index.find(boost::make_tuple(service_name,type,site,vo_name));
                if(srv_it == index.end()) {
                    // Association is not recorded as missing for that VO
                    result = false;
                    break;
                } 
            } // end for each VO
        }
    }
    if(true == result){
        m_log_debug("Association <" << service_name << ">:<" << type << ">:<" << site << "> not listed in the information system");
    }
    return result;
}

/**
 * Update an Entry in the cache
 */
void SDCacheImpl::updateEntry(const std::string& svc_name) /*throw()*/{

    const ServiceEntry * svc = 0;
    typedef ServiceTable::index<name>::type  service_set_by_name;
    service_set_by_name& indx = m_serviceTable.get<name>();
    service_set_by_name::iterator it = indx.find(svc_name);
    if(it == indx.end()){
        // Service not in cache: this should never happen, so ignore this case
        return;
    }
    svc = &*(it);
    // Refresh The Entry Using ServiceDiscovery
    ServiceDiscovery serviceDiscovery;
    try{
        try{
            // Get the entry by name: If successfull, SD will insert it 
            // into the cache
            boost::scoped_ptr<Service> s(
                serviceDiscovery.getServiceByName(svc_name));
            m_log_debug("Entry for service <" << svc_name << "> successfully updated");
        } catch(const ServiceDiscoveryException& exc){
            m_log_warn("Failed to refresh service <" << svc_name << ">. Error is " << exc.what());
            throw;
        } catch(const LogicError& exc){
            m_log_warn("LogicError in refreshing service <" << svc_name << ">. Error is " << exc.what());
            throw;
        }
    } catch(...){
        time_t current;
        time(&current);
        int diff = current - svc->creationTime;
        // Keep the entry in the cache, but reduce teh validity time
        m_log_info("Entry for service <" << svc_name << "> is stale: keep it valid for next " << m_staleTime << " seconds. This entry was created " << diff << " seconds ago");
        // Reduce the validity time
        svc->validity = m_staleTime;
    }
}

/*
 * Update an Association in the Cache
 */
void SDCacheImpl::updateAssociations(const Service * svc, const std::string& assoc_srv_type, const std::string& assoc_srv_site) /*throw()*/{
    if(0 == svc){
        // Invalid Service: this should never happen, so ignore this case
        return;
    }
    // Refresh The Entry Using ServiceDiscovery
    ServiceDiscovery serviceDiscovery;
    try{
        try{
            // Get the entry by name: If successfull, SD will insert it 
            // into the cache
            ServiceDiscovery:: ServiceArray services;             
            serviceDiscovery.getAssociatedServices(*svc, assoc_srv_type, 
                assoc_srv_site, services);
            m_log_debug("Entries for associated service to <" << svc->name << "> successfully updated");
        } catch(const ServiceDiscoveryException& exc){
            m_log_warn("Failed to refresh associated service to <" << svc->name << ">. Error is " << exc.what());
            throw;
        } catch(const LogicError& exc){
            m_log_warn("LogicError in refreshing associated service to <" << svc->name << ">. Error is " << exc.what());
            throw;
        }
    } catch(...){
        // Keep the entry in the cache, but reduce teh validity time
        m_log_info("Entries for associated services to <" << svc->name << "> are stale: keep them valid for next " << m_staleTime << " seconds.");
    }
}

/*
 * updateProperty
 *
 * Update a Property in the Cache
 */
void SDCacheImpl::updateProperty(const Property* prop_entry) /*throw()*/{
    if((0 == prop_entry) || (0 == prop_entry->srv)){
        // Invalid Property: this should never happen, so ignore this case
        return;
    }
    // Refresh The Entry Using ServiceDiscovery
    ServiceDiscovery serviceDiscovery;
    try{
        try{
            // Get the entry by name: If successfull, SD will insert it 
            // into the cache
            boost::scoped_ptr<Service> s(
                serviceDiscovery.getServiceByName(prop_entry->srv->name));
            m_log_debug("Entry for service <" << prop_entry->srv->name << "> successfully updated");
            if(0 != s.get()){
                // Get the property: If successfull, SD will also refresh it 
                // into the cache
                std::string p = serviceDiscovery.getProperty(*s,prop_entry->name);
                m_log_debug("Entry for service <" << prop_entry->srv->name << "> property <" << prop_entry->name << "> successfully updated");
            }
        } catch(const ServiceDiscoveryException& exc){
            m_log_warn("Failed to refresh service <" << prop_entry->srv->name << "> property <" << prop_entry->name << ">. Error is " << exc.what());
            throw;
        } catch(const LogicError& exc){
            m_log_warn("LogicError in refreshing service <" << prop_entry->srv->name << "> property <" << prop_entry->name << ">. Error is " << exc.what());
            throw;
        }
    } catch(...){
        time_t current;
        time(&current);
        int diff = current - prop_entry->creationTime;
        // Keep the entry in the cache, but reduce the validity time
        m_log_info("Entry for service <" << prop_entry->srv->name << "> property <" << prop_entry->name << "> is stale: keep it valid for next " << m_staleTime << " seconds. This entry was created " << diff << " seconds ago");
        // Reduce the validity time
        prop_entry->validity = m_staleTime;
    }
}
