/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common : SDCache Implementation Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDCacheImpl.h,v 1.8 2008/11/17 10:21:08 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_AGENTS_SD_SDCACHEIMPL_H_
#define GLITE_DATA_AGENTS_SD_SDCACHEIMPL_H_

#include "glite/data/agents/sd/SDTypes.h"
#include <vector>
#include <string>
#include <boost/multi_index_container.hpp>
#include <boost/multi_index/member.hpp>
#include <boost/multi_index/mem_fun.hpp>
#include <boost/multi_index/composite_key.hpp>
#include <boost/multi_index/ordered_index.hpp>
#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite {
namespace data  {
namespace agents{
namespace sd    {

/**
 * SDCache Implementation (Pimpl Idiom)
 */
class SDCacheImpl : boost::noncopyable {
public:

    typedef std::vector<std::string> StringArray;

    typedef std::vector<const Service *> ServiceArray;

    /**
     * Constructor
     */
    SDCacheImpl(const glite::config::Logger& logger);
        
    /**
     * Destructor
     */
    ~SDCacheImpl();
    
    /**
     * Set the Validity of an entry in the Cache
     */
    void ttl(unsigned int v){m_ttl = v;}
    /**
     * Set the Increment of the validity of an stale entry in the Cache
     */
    void staleTime(unsigned int v){m_staleTime = v;}
    /**
     * Set the time that should elapse before considering an entry obsolete
     */
    void obsoleteTime(unsigned int v) {m_obsoleteTime = v;}
    /**
     * Set the time that should elapse before considering a negative result obsolete
     */
    void negativeObsoleteTime(unsigned int v) {m_negativeObsoleteTime = v;}
    
    /**
     * Insert a Service in the Cache
     */
    void insert(const Service& service, const StringArray& vo_list);

    /**
     * Store an association between two services in the Cache. If the services
     * are not there, they will be added, but no VO would be specified
     */
    void associate(const Service& service, const Service& associated_service);

    /**
     * Insert a Property for a given Service and VO
     */
    void insertProperty(const Service& service, const std::string& name, const std::string& value, const StringArray& vo_list);
    
    /**
     * Cleat the Cache
     */
    void clear();
    
    /**
     * purge obsolete entries
     */
    void purge();

    /**
     * Get the service by Name and VO (enforce authorization)
     */
    const Service * getByName(const std::string& name, const StringArray& vo_list);

    /**
     * Get the list of services by type
     */
    ServiceArray getByType(const std::string& type, const StringArray& vo_list);

    /**
     * Get the list of services by host and type
     */
    ServiceArray getByHost(const std::string& type, const std::string& host,const StringArray& vo_list);

    /**
     * Get the list of services by site and type
     */
    ServiceArray getBySite(const std::string& type, const std::string& site,const StringArray& vo_list);

    /**
     * Get the list of services of the given type that are associated with the 
     * given service
     */
    ServiceArray getAssociated(const std::string& name, const std::string& assoc_type, const StringArray& vo_list);

    /**
     * Get the list of services of the given type that are associated with the 
     * given service and are in the given site 
     */
    ServiceArray getAssociated(const std::string& name, const std::string& assoc_srv_type, const std::string& assoc_srv_site,  const StringArray& vo_list);

    /**
     * Get a Property for a given Service. If no property is found, the second 
     * field of the pair is set to false;
     */
    std::pair<std::string,bool> getProperty(const std::string& srv_name, const std::string& prop_name,const StringArray& vo_list);

    // -------------------------------------------------------------------------
    // Missing services, properties & associations
    //
    // Cache also the information that a service, a property or an association
    // is missing: This would allow not to query evry time the information
    // system for data that would never be there
    // -------------------------------------------------------------------------

    /**
     * Remeber that a Service is not listed in the information system
     */
    void rememberMissing(const std::string& type, const std::string& source, const StringArray& vo_list);

    /**
     * Remeber that a ServiceProeprty is not listed in the information system
     */
    void rememberMissingProperty(const std::string& service_name, const std::string& prop_name, const StringArray& vo_list);

    /**
     * Remeber that a Service doesn't have associated services of a given type
     * listed in the information system
     */
    void rememberMissingAssociation(const std::string& service_name, const std::string& type, const std::string& site, const StringArray& vo_list);

    /**
     * Check if a Service is not listed in the information system
     */
    bool isServiceMissing(const std::string& type, const std::string& source, const StringArray& vo_list);
    /**
     * Check if a Service Property is not listed in the information system
     */
    bool isPropertyMissing(const std::string& service_name, const std::string& prop_name, const StringArray& vo_list);
    /**
     * Remeber that a Service doesn't have associated services of a given type
     * listed in the information system
     */
    bool isAssociationMissing(const std::string& service_name, const std::string& type, const std::string& site, const StringArray& vo_list);
    
public:

    typedef boost::shared_ptr<Service>                   ServicePtr;
    typedef boost::shared_ptr<const Service>             ServiceCPtr;
    typedef boost::shared_ptr<VirtualOrganization>       VirtualOrganizationPtr;
    typedef boost::shared_ptr<const VirtualOrganization> VirtualOrganizationCPtr;

    /**
     * CacheEntry Data: set properties as mutable, so without affecting the
     * entry in the cache
     */
    struct CacheEntry {
        mutable time_t  creationTime;
        mutable unsigned int validity;
    };
    
    /**
     * ServiceEntry Data
     */
    struct ServiceEntry: public CacheEntry{
        ServicePtr srv;
    };
    
    /**
     * VirtualOrganizationEntry Data
     */
    struct VirtualOrganizationEntry : public CacheEntry {
        VirtualOrganizationPtr vo;
    };

    /**
     * Association Data
     */
    struct Property : public CacheEntry {
        
       ServiceCPtr             srv;
       VirtualOrganizationCPtr vo;
       std::string             name;
       std::string             value;
        
       Property(const ServicePtr& s, 
                const VirtualOrganizationPtr& v, 
                const std::string& n, 
                const std::string& val):srv(s),vo(v),name(n),value(val){}
    };
    
    /**
     * VOServiceEntry Data
     */
    struct VOServiceEntry : public CacheEntry {

        ServiceCPtr             srv;
        VirtualOrganizationCPtr vo;
        
        VOServiceEntry(const ServicePtr& s, const VirtualOrganizationPtr& v):srv(s),vo(v){}
    };
    
    /**
     * Association Data
     */
    struct Association : public CacheEntry {
        
       ServiceCPtr srv;
       ServiceCPtr associated;
        
       Association(const ServicePtr& s, const ServicePtr& a):srv(s),associated(a){}
    };
    
    /*
     * Entry used to define services not listed in the information system
     * (validity time is not used)
     */
    struct MissingService : public CacheEntry {
        std::string const type;
        std::string const source;
        std::string const vo_name;
        /*
         * Constructor
         */
        MissingService(const std::string& t,const std::string& s,const std::string& n):
            type(t),source(s),vo_name(n){}
    };

    /*
     * Entry used to define properties not defined for a service
     */
    struct MissingProperty  : public CacheEntry {
        std::string const service_name;
        std::string const property_name;
        std::string const vo_name;

        /*
         * Cosntructor
         */
        MissingProperty(const std::string& s, const std::string& p, const std::string& n):
            service_name(s),property_name(p),vo_name(n){}
    };
    
    
    /*
     * Entry used to define associated services not listed in the information
     * system
     */
    struct MissingAssocService : public CacheEntry {
        std::string const assoc_srv_name;
        std::string const srv_type;
        std::string const srv_site;
        std::string const vo_name;

        /*
         * Constructor
         */
        MissingAssocService(const std::string& a,  const std::string& t, const std::string& s, const std::string& n):
            assoc_srv_name(a),srv_type(t),srv_site(s),vo_name(n){}
    };
    
private:

    /*
     * tags for accessing several indices defined below
     */
    struct name{};
    struct id{};
    struct id_vo{};
    struct type_vo{};
    struct type_host_vo{};
    struct type_site_vo{};
    struct assoc_id{};
    struct assoc_type{};
    struct assoc_type_site{};
    struct prop_id{};
    struct prop_name{};

    /*
     * Key Extractor Template Class (for composite keys)
     */
    template<class KE1,class KE2>
    struct key_from_key {
    public:
        typedef typename KE1::result_type result_type;
        /**
         * Constructor
         */
        key_from_key(const KE1& k1=KE1(),const KE2& k2=KE2()):m_key1(k1),m_key2(k2){}
        /**
         * Result Type
         */
        template<typename T> result_type operator()(T& arg)const { return m_key1(m_key2(arg)); }
    private:
        /**
         * First Key Extractor
         */
        KE1 m_key1;
        /**
         * Second Key Extractor
         */
        KE2 m_key2;
    };

    /*
     * Define VO Table with
     *  - a unique index sorted by VirtualOrganization::name
     */
    typedef boost::multi_index::multi_index_container<
      VirtualOrganizationEntry,
      boost::multi_index::indexed_by<
        boost::multi_index::ordered_unique<boost::multi_index::tag<name>,
            key_from_key<
                BOOST_MULTI_INDEX_MEMBER(VirtualOrganization,std::string,name),
                BOOST_MULTI_INDEX_MEMBER(VirtualOrganizationEntry,VirtualOrganizationPtr,vo)
            >
        >
      >
    > VirtualOrganizationTable;

    /*
     * Define a Service Table with
     *  - a unique index sorted by Service::name
     *  - a non-unique index sorted by Service::type
     *  - a non-unique index sorted by Service::hostname
     *  - a non-unique index sorted by Service::site
     */
    typedef boost::multi_index::multi_index_container<
      ServiceEntry,
      boost::multi_index::indexed_by<
        boost::multi_index::ordered_unique<boost::multi_index::tag<name>,
            key_from_key<
                BOOST_MULTI_INDEX_MEMBER(Service,std::string,name),
                BOOST_MULTI_INDEX_MEMBER(ServiceEntry,ServicePtr,srv)
            >
        >,
        boost::multi_index::ordered_non_unique<
            key_from_key<
                BOOST_MULTI_INDEX_MEMBER(Service,std::string,type),
                BOOST_MULTI_INDEX_MEMBER(ServiceEntry,ServicePtr,srv)
            >
        >,
        boost::multi_index::ordered_non_unique<
            key_from_key<
                BOOST_MULTI_INDEX_MEMBER(Service,std::string,hostname),
                BOOST_MULTI_INDEX_MEMBER(ServiceEntry,ServicePtr,srv)
            >
        >,
        boost::multi_index::ordered_non_unique<
            key_from_key<
                BOOST_MULTI_INDEX_MEMBER(Service,std::string,site),
                BOOST_MULTI_INDEX_MEMBER(ServiceEntry,ServicePtr,srv)
            >
        >
      >
    > ServiceTable;
   
    /*
     * VO Service Table Name Composite Key
     */
    struct id_key: boost::multi_index::composite_key<
      VOServiceEntry,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,name),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,ServiceCPtr,srv)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(VirtualOrganization,const std::string,name),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,VirtualOrganizationCPtr,vo)
      >
    >{};
    
    /*
     * VO Service Table Type Composite Key
     */
    struct type_key: boost::multi_index::composite_key<
      VOServiceEntry,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,type),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,ServiceCPtr,srv)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(VirtualOrganization,const std::string,name),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,VirtualOrganizationCPtr,vo)
      >
    >{};

    /*
     * VO Service Table Host Composite Key
     */
    struct host_key: boost::multi_index::composite_key<
      VOServiceEntry,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,type),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,ServiceCPtr,srv)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,hostname),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,ServiceCPtr,srv)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(VirtualOrganization,const std::string,name),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,VirtualOrganizationCPtr,vo)
      >
    >{};
    /*
     * VO Service Table Site Composite Key
     */
    struct site_key: boost::multi_index::composite_key<
      VOServiceEntry,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,type),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,ServiceCPtr,srv)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,site),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,ServiceCPtr,srv)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(VirtualOrganization,const std::string,name),
        BOOST_MULTI_INDEX_MEMBER(VOServiceEntry,VirtualOrganizationCPtr,vo)
      >
    >{};

    /*
     * Define a multi_index_container of ServiceEntries with following indices:
     *   - a unique index sorted by Service::name, VO::name
     *   - a non-unique index sorted by Service::type, Service::hostname, VO::name
     *   - a non-unique index sorted by Service::type, Service::site, VO::name
     */
    typedef boost::multi_index::multi_index_container<
      VOServiceEntry,
      boost::multi_index::indexed_by<
        /* primary index sorted by service name (for the same vo) */
        boost::multi_index::ordered_unique<boost::multi_index::tag<id_vo>, id_key>,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<type_vo>, type_key>,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<type_host_vo>, host_key>,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<type_site_vo>, site_key>
      >
    > VOServiceTable;

    /*
     * Associations Table Name Composite Key
     */
    struct assoc_id_key: boost::multi_index::composite_key<
      Association,
      BOOST_MULTI_INDEX_MEMBER(Association,ServiceCPtr,srv),
      BOOST_MULTI_INDEX_MEMBER(Association,ServiceCPtr,associated)
    >{};
    
    /*
     * Associations Service Table Type Composite Key
     */
    struct assoc_type_key: boost::multi_index::composite_key<
      Association,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,name),
        BOOST_MULTI_INDEX_MEMBER(Association,ServiceCPtr,srv)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,type),
        BOOST_MULTI_INDEX_MEMBER(Association,ServiceCPtr,associated)
      >
    >{};

    /*
     * Associations Service Table Site Composite Key
     */
    struct assoc_site_key: boost::multi_index::composite_key<
      Association,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,name),
        BOOST_MULTI_INDEX_MEMBER(Association,ServiceCPtr,srv)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,type),
        BOOST_MULTI_INDEX_MEMBER(Association,ServiceCPtr,associated)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,site),
        BOOST_MULTI_INDEX_MEMBER(Association,ServiceCPtr,associated)
      >
    >{};

    /*
     * Define a multi_index_container of Associations with following indices:
     *   - a unique index sorted by Association::srv, Association::associated
     *   - a non-unique index sorted by Association::srv, associated Service::type
     *   - a non-unique index sorted by Association::srv, associated Service::type, associated Service::site
     */
    typedef boost::multi_index::multi_index_container<
      Association,
      boost::multi_index::indexed_by<
        /* primary index sorted by service name - service name */
        boost::multi_index::ordered_unique<boost::multi_index::tag<assoc_id>, assoc_id_key>,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<assoc_type>, assoc_type_key>,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<assoc_type_site>, assoc_site_key>
      >
    > AssociationTable;
    
    /*
     * Property Table Name Composite Key
     */
    struct prop_name_key: boost::multi_index::composite_key<
      Property,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(Service,const std::string,name),
        BOOST_MULTI_INDEX_MEMBER(Property,ServiceCPtr,srv)
      >,
      key_from_key<
        BOOST_MULTI_INDEX_MEMBER(VirtualOrganization,const std::string,name),
        BOOST_MULTI_INDEX_MEMBER(Property,VirtualOrganizationCPtr,vo)
      >,
      BOOST_MULTI_INDEX_MEMBER(Property,const std::string,name)
    >{};

    /*
     * Define a multi_index_container of Properties with following indices:
     *   - a unique index sorted by Property::srv, Property::VO, Property::name
     *   - a unique index sorted by Service::name, VO::name, Property::name
     */
    typedef boost::multi_index::multi_index_container<
      Property,
      boost::multi_index::indexed_by<
        /* primary index sorted by Service::name, VO::name, Property::name */
        boost::multi_index::ordered_unique<boost::multi_index::tag<prop_name>, prop_name_key>
      >
    > PropertyTable;

    // -------------------------------------------------------------------------
    // Missing services, properties & associations
    //
    // -------------------------------------------------------------------------

    /*
     * Missing Table Name Composite Key
     */
    struct missing_id_key: boost::multi_index::composite_key<
      MissingService,
      BOOST_MULTI_INDEX_MEMBER(MissingService,const std::string,type),
      BOOST_MULTI_INDEX_MEMBER(MissingService,const std::string,source),
      BOOST_MULTI_INDEX_MEMBER(MissingService,const std::string,vo_name)
    >{};

    /*
     * Define tables to store missing entries (in order not to query all the
     * time for services that don't exist)
     */
    typedef boost::multi_index::multi_index_container<
      MissingService,
      boost::multi_index::indexed_by<
        boost::multi_index::ordered_unique<boost::multi_index::tag<id>, missing_id_key>
      >
    > MissingServiceTable;

    /*
     * Missing Table Name Composite Key
     */
    struct missing_prop_id_key: boost::multi_index::composite_key<
      MissingProperty,
      BOOST_MULTI_INDEX_MEMBER(MissingProperty,const std::string,service_name),
      BOOST_MULTI_INDEX_MEMBER(MissingProperty,const std::string,property_name),
      BOOST_MULTI_INDEX_MEMBER(MissingProperty,const std::string,vo_name)
    >{};

    /*
     * Define tables to store missing properties (in order not to query all the
     * time for properties that don't exist)
     */
    typedef boost::multi_index::multi_index_container<
      MissingProperty,
      boost::multi_index::indexed_by<
        boost::multi_index::ordered_unique<boost::multi_index::tag<id>, missing_prop_id_key>
      >
    > MissingPropertyTable;

    /*
     * Missing Table Name Composite Key
     */
    struct missing_assoc_id_key: boost::multi_index::composite_key<
      MissingAssocService,
      BOOST_MULTI_INDEX_MEMBER(MissingAssocService,const std::string,assoc_srv_name),
      BOOST_MULTI_INDEX_MEMBER(MissingAssocService,const std::string,srv_type),
      BOOST_MULTI_INDEX_MEMBER(MissingAssocService,const std::string,srv_site),
      BOOST_MULTI_INDEX_MEMBER(MissingAssocService,const std::string,vo_name)
    >{};

    /*
     * Define tables to store missing associated services (in order not to
     * query all the time for services that don't exist)
     */
    typedef boost::multi_index::multi_index_container<
      MissingAssocService,
      boost::multi_index::indexed_by<
        boost::multi_index::ordered_unique<boost::multi_index::tag<id>, missing_assoc_id_key>
      >
    > MissingAssocServiceTable;

private:

    /**
     * Insert a Service Entry
     */
    const ServiceEntry * insertServiceEntry(const Service& service) /*throw()*/;

    /**
     * Insert a VO Entry
     */
    const VirtualOrganizationEntry * insertVOEntry(const std::string& vo) /*throw()*/;

    /*
     * Update An Entry in the Cache
     */
    void updateEntry(const std::string& svc_name) /*throw()*/;

    /*
     * Update an Association in the Cache
     */
    void updateAssociations(const Service * svc, const std::string& assoc_srv_type, const std::string& assoc_srv_site) /*throw()*/;

    /*
     * Update a Property in the Cache
     */
    void updateProperty(const Property* prop_entry) /*throw()*/;

private:
    
    /**
     * The Logger
     */
    glite::config::Logger m_logger;
        
    /**
     * The VO Table
     */
    VirtualOrganizationTable m_voTable;
    /**
     * The Service Table
     */
    ServiceTable m_serviceTable;
    /**
     * The VO Service Table
     */
    VOServiceTable m_voServiceTable;
    /**
     * The Service Associations Table
     */
    AssociationTable m_associationTable;
    /**
     * The Service Property Table
     */
    PropertyTable m_propertyTable;
    /**
     * The Missing Services Table
     */
    MissingServiceTable m_missingServiceTable;
    /**
     * The Missing Properties Table
     */
    MissingPropertyTable m_missingPropertyTable;
    /**
     * The Missing AssocService Table
     */
    MissingAssocServiceTable m_missingAssocServiceTable;
    
    /**
     * The Validity of an entry in the Cache
     */
    unsigned int m_ttl;
    /**
     * The Increment of the validity of an stale entry in the Cache
     */
    unsigned int m_staleTime;
    /**
     * The time that should elapse before considering an entry obsolete
     */
    unsigned int m_obsoleteTime;    
    /**
     * The time that should elapse before considering a negative result obsolete
     */
    unsigned int m_negativeObsoleteTime;    
};

} // end namespace sd
} // end namespace agents
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_AGENTS_SD_SDCACHEIMPL_H_
