/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: ServiceDiscovery Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ServiceDiscovery.cpp,v 1.7 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/sd/ServiceDiscovery.h"
#include "glite/data/agents/sd/SDCache.h"
#include "glite/data/agents/sd/ServiceDiscoveryException.h"
#include "glite/data/agents/url-utility.h"
#include "glite/data/agents/sd/SDConfig.h"

#include "sd.h"
#include "glite/data/glite-util.h"

#include <glite/data/config/service/Logger.hxx>

#include <boost/scoped_ptr.hpp>

using namespace glite::data::agents;
using namespace glite::data::agents::sd;

using boost::scoped_ptr;
using boost::shared_ptr;

namespace {
/*
 * Constants
 */
const char * const SD_NAME = "agent-sd";

/*
 * create_vo_list
 *
 * Create a VO List with just one entry
 */ 
SDVOList * create_vo_list(const char * vo_name){
    if(0 == vo_name){
        return 0;
    }
    
    SDVOList *volist = (SDVOList *)calloc(1, sizeof(SDVOList));
    volist->names = (char**)malloc(1 * sizeof(char*));
    volist->numNames = 1;
    volist->names[0] = strdup(vo_name);
    return volist;
}

/*
 * create_vo_list
 *
 * delete a VO List
 */ 
void free_vo_list(SDVOList * vo_list){
    if(0 == vo_list){
        return;
    }
    if(0 != vo_list->names){
        for(int i = 0; i < vo_list->numNames; ++i){
            if(0 != vo_list->names[i]){
                free(vo_list->names[i]);
            }
        }
        free(vo_list->names);
    }
    free(vo_list);
}

/*
 * convert_to_upper
 *
 * Convert a String to uppercase
 */
void convert_to_upper(std::string& str) {
    std::transform(str.begin(),str.end(),str.begin(),toupper);
}

/*
 * convert_to_lower
 *
 * Convert a String to lower
 */
void convert_to_lower(std::string& str) {
    std::transform(str.begin(),str.end(),str.begin(),tolower);
}

/*
 * getService
 *
 * Instantiate a service and store in into the Cache
 */
Service * create_service(const SDService * sd_service) /*throw ()*/{

    _log_debug(SD_NAME,"Creating service : name='" << sd_service->name << "' type='" << sd_service->type << "' endpoint='" << sd_service->endpoint << "' version='" << sd_service->version << "'");
    
    std::string site;
    SDVOList   * vos  = 0;
    // Get Service Details for the Site and VO List
    SDException exception;
    SDServiceDetails * sd_details = SD_getServiceDetails(sd_service->name, &exception);
    if(0 == sd_details){
        const char * service_name = (0 != sd_service->name)?sd_service->name:"(null)";
        _log_debug(SD_NAME,"No service details for the service <" << service_name << ">. Reason is " << str_char(exception.reason));
        SD_freeException(&exception);
    } else {
        if(0 != sd_details->site){
            site = sd_details->site;
            convert_to_upper(site);
        }
        vos  = sd_details->vos;
    }

    // Get hostname and port number
    std::string endpoint = (0!= sd_service->endpoint)?sd_service->endpoint:"";
    std::string hostname;
    unsigned int port = 0;
    parse_endpoint(endpoint,hostname, port);

    // Create the Service
    Service * service = new Service(
        sd_service->name,
        sd_service->type,
        endpoint,
        (0!= sd_service->version)?sd_service->version:"",
        site,
        hostname,
        port);

    // Store the service into the Cache
    SDCache * cache = SDConfig::instance().cache();
    if(0 != cache){
        // Create StringArray for VO List
        SDCache::StringArray vo_list;
        if(0 != vos){
            for(int i = 0; i < vos->numNames; ++i){
                const char * vo_name = vos->names[i];
                if(0 != vo_name){
                    vo_list.push_back(vo_name);
                }
            }
        }
        // Insert the service into the cache
        cache->insert(*service,vo_list);
    }
    _log_trace(SD_NAME,"Service Name     is " << service->name);
    _log_trace(SD_NAME,"Service Type     is " << service->type);
    _log_trace(SD_NAME,"Service Endpoint is " << service->endpoint);
    _log_trace(SD_NAME,"Service Version  is " << service->version);
    _log_trace(SD_NAME,"Service Site     is " << service->site);
    _log_trace(SD_NAME,"Service Hostname is " << service->hostname);
    _log_trace(SD_NAME,"Service Port     is " << service->port);

    // Free Memory
    if(0 != sd_details){
        SD_freeServiceDetails(sd_details);
    }
    
    return service;
}

} //End global namespace 

/*
 * ServiceDiscovery
 *
 * Constructor
 */
ServiceDiscovery::ServiceDiscovery():
    m_logger(SD_NAME){
}
    
/*
 * ~ServiceDiscovery
 *
 * Destructor
 */
ServiceDiscovery::~ServiceDiscovery(){
}
    
/*
 * getServiceByName
 *
 * Get the enpoint of a service of the given name
 */
Service * ServiceDiscovery::getServiceByName(const std::string& name) /*throw (LogicError, ServiceDiscoveryException)*/{
    if(name.empty()){
        m_log_error("Null name specified in getServiceByName");
        throw InvalidArgumentException("Null name specified");
    }

    // Get Service
    SDException exception;
    SDService * sd_service = SD_getService(name.c_str(), &exception);
    if(0 == sd_service){
        std::string error_reason = 
            (std::string) "No service with the name " + name;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        SD_freeException(&exception);
        throw ServiceDiscoveryException(error_reason.c_str());
    }
    
    // Instantiate the service and store it into the Cache
    Service * service = create_service(sd_service);
    
    // Free Memory
    SD_freeService(sd_service);
    
    return service;
}

/*
 * getServicesOnHost
 *
 * Get the list of services of the given type on the given host
 */
void ServiceDiscovery::getServicesOnHost(
    const std::string& type, 
    const std::string& host, 
    unsigned int port, 
    const std::string& vo_name, 
    ServiceArray& services) /*throw (LogicError, ServiceDiscoveryException)*/{
        
    m_log_debug("getServicesOnHost : type " << type << " host " << host << " port " << port << " vo " << vo_name);

    if(type.empty()){
        m_log_error("Null type specified in getServicesOnHost");
        throw InvalidArgumentException("null type specified");
    }

    // Get the vo List
    // Disable it due to a memory leak
    //SDVOList *vos = check_voms_proxy();
    SDVOList *vos = 0;
    if((0 == vos) && (false == vo_name.empty())){
        vos = create_vo_list(vo_name.c_str());
    }

    std::string host_lc = host;    
    convert_to_lower(host_lc);

    // List Services
    SDException exception;
    SDServiceList * service_list = SD_listServicesByHost(type.c_str(), host.c_str(), vos, &exception);
    if(0 == service_list){
        std::string error_reason = 
            (std::string) "No service matching the criteria: Type=" + type + ";Host=" + host;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        SD_freeException(&exception);
        free_vo_list(vos);
        // Register the missing service in the cache
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            SDCache::StringArray vo_list;
            vo_list.push_back(vo_name);
            cache->rememberMissing(type,host,vo_list);
        }
        throw ServiceDiscoveryException(error_reason.c_str());
    }
    free_vo_list(vos);
    m_log_trace("Got " << service_list->numServices << " services of type " << type << " - host " << host);
        
    // For each Service
    for(int i = 0; i < service_list->numServices; ++i){
        // Get the service
        SDService * sd_service = service_list->services[i];
        if(0 == sd_service){
            continue;
        }
        m_log_trace("Checking service " << sd_service->endpoint);

        // Instantiate the service and store it into the Cache
        shared_ptr<Service> s(create_service(sd_service));
        // listServicesByHost doesn't exactly match the hostname: filter out some result
        if(s->hostname == host_lc){ 
            // Store the Service in the List
            services.push_back(s);
        } else {
            m_log_trace("Service " << s->name << " not on host " << host << " but in " << s->hostname);
        }
    } 
    // Free Memory
    SD_freeServiceList(service_list);
    
    if(true == services.empty()){
        std::string error_reason = 
            (std::string) "No service matching the criteria: Type=" + type+ "; Host=" + host;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        // Register the missing service in the cache
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            SDCache::StringArray vo_list;
            vo_list.push_back(vo_name);
            cache->rememberMissing(type,host,vo_list);
        }
        throw ServiceDiscoveryException(error_reason.c_str());
    }        
}

/*
 * getServicesOnSite
 *
 * Get the list of services of the given type on the given site
 */
void ServiceDiscovery::getServicesOnSite(
    const std::string& type, 
    const std::string& site, 
    const std::string& vo_name, 
    ServiceArray& services) /*throw (LogicError, ServiceDiscoveryException)*/{
        
    m_log_debug("getServicesOnSite : type " << type << " site " << site << " vo " << vo_name);

    if(type.empty()){
        m_log_error("Null type specified in getServicesOnSite");
        throw InvalidArgumentException("null type specified");
    }

    // convert site to upper_case
    std::string site_upr = site;
    convert_to_upper(site_upr);

    // Get the vo List
    // Disable it due to a memory leak
    //SDVOList *vos = check_voms_proxy();
    SDVOList *vos = 0;
    if((0 == vos) && (false == vo_name.empty())){
        vos = create_vo_list(vo_name.c_str());
    }
    
    // List Services
    SDException exception;
    SDServiceList * service_list = SD_listServices(type.c_str(), 0, vos, &exception);
    if(0 == service_list){
        std::string error_reason = 
            (std::string) "No service matching the criteria: Type=" + type;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        SD_freeException(&exception);
        free_vo_list(vos);
        // Register the missing service in the cache
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            SDCache::StringArray vo_list;
            vo_list.push_back(vo_name);
            cache->rememberMissing(type,site,vo_list);
        }
        throw ServiceDiscoveryException(error_reason.c_str());
    }
    free_vo_list(vos);
    m_log_trace("Got " << service_list->numServices << " services of type " << type);
        
    // For each Service
    for(int i = 0; i < service_list->numServices; ++i){
        // Get the service
        SDService * sd_service = service_list->services[i];
        if(0 == sd_service){
            continue;
        }
        m_log_trace("Checking service " << sd_service->endpoint);

        // Instantiate the service and store it into the Cache
        shared_ptr<Service> s(create_service(sd_service));
           
        // Check the service endpoint
        if(s->site == site_upr){
            m_log_trace("service " << sd_service->name << " matchs <" << site_upr << ">");
            // Store the Service in the List
            services.push_back(s);
        }
    } 
    // Free Memory
    SD_freeServiceList(service_list);
    
    if(true == services.empty()){
        std::string error_reason = 
            (std::string) "No service matching the criteria: Type=" + type + "; Site=" + site_upr;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        // Register the missing service in the cache
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            SDCache::StringArray vo_list;
            vo_list.push_back(vo_name);
            cache->rememberMissing(type,site,vo_list);
        }
        throw ServiceDiscoveryException(error_reason.c_str());
    }        
}

/*
 * getServiceByType
 *
 * Return a list of services of the given type
 */
void ServiceDiscovery::getServicesByType(
    const std::string& type, 
    const std::string& vo_name, 
    ServiceArray& services) /*throw (LogicError, ServiceDiscoveryException)*/{
        
    m_log_debug("getServicesByType : type " << type);
        
    if(type.empty()){
        m_log_error("Null type specified in getServicesByType");
        throw InvalidArgumentException("null type specified");
    }
    
    // Get the vo List
    // Disable it due to a memory leak
    //SDVOList *vos = check_voms_proxy();
    SDVOList *vos = 0;
    if((0 == vos) && (false == vo_name.empty())){
        vos = create_vo_list(vo_name.c_str());
    }
    
    // List Services
    SDException exception;
    SDServiceList * service_list = SD_listServices(type.c_str(), 0, vos, &exception);
    if(0 == service_list){
        std::string error_reason = 
            (std::string) "No service matching the criteria: Type=" + type;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        SD_freeException(&exception);
        free_vo_list(vos);
        // Register the missing service in the cache
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            SDCache::StringArray vo_list;
            vo_list.push_back(vo_name);
            cache->rememberMissing(type,vo_list);
        }
        throw ServiceDiscoveryException(error_reason.c_str());
    }
    free_vo_list(vos);
    m_log_trace("Got " << service_list->numServices << " services of type " << type);
        
    // For each Service
    for(int i = 0; i < service_list->numServices; ++i){
        // Get the service
        SDService * sd_service = service_list->services[i];
        if(0 == sd_service){
            continue;
        }
        // Instantiate the service and store it into the Cache
        shared_ptr<Service> s(create_service(sd_service));
        // Store the Service in the List
        services.push_back(s);
    } 
    // Free Memory
    SD_freeServiceList(service_list);
    
    if(true == services.empty()){
        std::string error_reason = 
            (std::string) "No service matching the criteria: Type=" + type;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        // Register the missing service in the cache
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            SDCache::StringArray vo_list;
            vo_list.push_back(vo_name);
            cache->rememberMissing(type,vo_list);
        }
        throw ServiceDiscoveryException(error_reason.c_str());
    }
}

/**
 * Get the associated service of the given type
 */
void ServiceDiscovery::getAssociatedServices(
    const Service& service, 
    const std::string& type, 
    const std::string& site, 
    ServiceArray& services) /*throw (LogicError, ServiceDiscoveryException)*/{

    m_log_debug("getAssociatedServices : service " << service.name);
        
    if(type.empty()){
        m_log_error("Null type specified in getAssociatedService");
        throw InvalidArgumentException("Null type specified");
    }
    
    // Convert site to upper case
    std::string site_upr = site;
    convert_to_upper(site_upr);

    // List Services
    SDException exception;
    SDServiceList * service_list = SD_listAssociatedServices(
        service.name.c_str(), type.c_str(),(site_upr.empty())?0:site_upr.c_str(),0,&exception);
    if(0 == service_list){
        std::string error_reason = (std::string) "No services of type " + type + " associated with service " + service.name;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        SD_freeException(&exception);
        // Register the missing association in the cache
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            cache->rememberMissingAssociation(service.name,type,site);
        }
        throw ServiceDiscoveryException(error_reason.c_str());
    }

    // For each Service
    for(int i = 0; i < service_list->numServices; ++i){
        // Get the service
        SDService * sd_service = service_list->services[i];
        if(0 == sd_service){
            continue;
        }
        // Instantiate the service and store it into the Cache
        shared_ptr<Service> s(create_service(sd_service));
        // Store the Service in the List
        services.push_back(s);
        
        // Store Association
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            // Store the association into the cache
            cache->associate(service,*s);
        }
    } 
    // Free Memory
    SD_freeServiceList(service_list);
    
    // Check Return Value
    if(true == services.empty()){
        std::string error_reason = (std::string) "No services of type " + type + " associated with service " + service.name;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        // Register the missing association in the cache
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            cache->rememberMissingAssociation(service.name,type,site);
        }
        throw ServiceDiscoveryException(error_reason.c_str());
    }
}

/*
 * getProperty
 *
 * Get the value of a service property
 */
std::string ServiceDiscovery::getProperty(const Service& service, const std::string& prop_name) /*throw (ServiceDiscoveryException)*/{
    // Get Service Data
    SDException exception;
    char * data = SD_getServiceDataItem(service.name.c_str(), prop_name.c_str(), &exception);
    if(0 == data){
        std::string error_reason = (std::string) "No property found for the service " + service.name;
        m_log_debug(error_reason << ". Reason is " << str_char(exception.reason));
        SD_freeException(&exception);
        // Register the missing proeprty in the cache
        SDCache * cache = SDConfig::instance().cache();
        if(0 != cache){
            cache->rememberMissingProperty(service.name,prop_name);
        }
        throw ServiceDiscoveryException(error_reason.c_str());
    }

    // Get Property
    std::string property = data;
    free(data);
    
    // Store Property
    SDCache * cache = SDConfig::instance().cache();
    if(0 != cache){
        // Store the property into the cache
        cache->insertProperty(service,prop_name,property);
    }
    
    m_log_trace("Service Property " << prop_name << " is " << property);
    
    return property;
}
