/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: ServiceDiscoveryConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ServiceDiscoveryConfig.cpp,v 1.10 2008/11/14 16:04:48 ptedesco Exp $ 
 *  Release: $Name:  $
 *
 */
#include "ServiceDiscoveryConfig.h"
#include "glite/data/agents/sd/SDConfig.h"
#include "glite/data/agents/sd/SDCache.h"
#include <glite/data/config/service/Logger.hxx>
#include "glite/data/config/service/ParamValue.hxx"
#include "glite/data/agents/AgentExceptions.h"
#include <algorithm>
#include <functional>

using namespace glite::data::agents;
using namespace glite::data::agents::sd;
using namespace glite::config;


/*
 * Constants
 */
namespace {
const char * const SD_NAME                                = "agents-sd";
                                                                                                                          
// Properties Names
const char * const SRMSERVICETYPE_PROPERTY_NAME           = "SRM_ServiceType";
const char * const GRIDFTPSERVICETYPE_PROPERTY_NAME       = "GridFTP_ServiceType";
const char * const MYPROXYSERVICETYPE_PROPERTY_NAME       = "MyProxy_ServiceType";
const char * const FILETRANSFERTYPE_PROPERTY_NAME         = "FileTransfer_ServiceType";
const char * const FILETRANSFERSTATS_PROPERTY_NAME        = "FileTransferStats_ServiceType";
const char * const CHANNELMANAGEMENTTYPE_PROPERTY_NAME    = "ChannelManagement_ServiceType";
const char * const CHANNELAGENTTYPE_PROPERTY_NAME         = "ChannelAgent_ServiceType";
const char * const BDIITYPE_PROPERTY_NAME                 = "BDII_ServiceType";
const char * const VOMSTYPE_PROPERTY_NAME                 = "VOMS_ServiceType";
const char * const SEINDEXTYPE_PROPERTY_NAME              = "SEIndex_ServiceType";
const char * const FIREMANTYPE_PROPERTY_NAME              = "Fireman_ServiceType";
    
const char * const SEPATH_PROPERTY_NAME                          = "SEPath_Property";
const char * const CHANNELAGENTSOURCEPROPERTY_PROPERTY_NAME      = "ChannelAgentSource_Property";
const char * const CHANNELAGENTDESTINATIONPROPERTY_PROPERTY_NAME = "ChannelAgentDestination_Property";

const char * const MYPROXYFTSMODEPROPERTY_PROPERTY_NAME          = "MyProxyFtsMode_Property";
const char * const MYPROXYFTSMODERETRIEVER_PROPERTY_NAME         = "MyProxyFtsMode_Retriever";
const char * const MYPROXYFTSMODERENEWER_PROPERTY_NAME           = "MyProxyFtsMode_Renewer";

const char * const CACHEENABLED_PROPERTY_NAME                    = "EnableCache";
const char * const CACHE_TTL_PROPERTY_NAME                       = "Cache_TTL";
const char * const CACHE_STALETIME_PROPERTY_NAME                 = "Cache_StaleTime";
const char * const CACHE_OBSOLETETIME_PROPERTY_NAME              = "Cache_ObsoleteTime";
const char * const CACHE_NEGATIVEOBSOLETETIME_PROPERTY_NAME      = "Cache_NegativeObsoleteTime";
                                                                                                                          
// Properties Default  Values
const bool   DEFAULT_CACHE_ENABLED_PROPERTY         = true;
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new ServiceDiscoveryConfig();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    ServiceDiscoveryConfig * s = dynamic_cast<ServiceDiscoveryConfig*>(component);
    if(0 != s){
        delete s;
    }
}
                                                                                
} // End extern "C"

/*
 * ServiceDiscoveryConfig
 *
 * Constructor
 */
ServiceDiscoveryConfig::ServiceDiscoveryConfig():
        ComponentConfiguration(SD_NAME), 
        m_logger(SD_NAME){
}
                                                                                                                          
/*
 * ~ServiceDiscoveryConfig
 *
 * Destructor
 */
ServiceDiscoveryConfig::~ServiceDiscoveryConfig(){
}

/*
 * init
 *
 * Initialize the Component
 */
int ServiceDiscoveryConfig::init(const Params& params){
    
    SDConfig& sd_config = SDConfig::instance();
    
    try{
        Params::const_iterator it;
    
        // Get SRM Service Type
        if((it = params.find(SRMSERVICETYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),SRMSERVICETYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_srmType = pv->getValue();
        }
        // Get GridFTP Service Type
        if((it = params.find(GRIDFTPSERVICETYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),GRIDFTPSERVICETYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_gridftpType = pv->getValue();
        }
        // Get MyProxy Service Type
        if((it = params.find(MYPROXYSERVICETYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),MYPROXYSERVICETYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_myproxyType = pv->getValue();
        } 
        // Get FileTransfer Service Type
        if((it = params.find(FILETRANSFERTYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),FILETRANSFERTYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_fileTransferType = pv->getValue();
        } 
        // Get FileTransferStats Service Type
        if((it = params.find(FILETRANSFERSTATS_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),FILETRANSFERSTATS_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_fileTransferStatsType = pv->getValue();
        } 
        // Get ChannelManagement Service Type
        if((it = params.find(CHANNELMANAGEMENTTYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CHANNELMANAGEMENTTYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_channelManagementType = pv->getValue();
        } 
        // Get ChannelAgent Service Type
        if((it = params.find(CHANNELAGENTTYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CHANNELAGENTTYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_channelAgentType = pv->getValue();
        } 
        // Get BDII Service Type
        if((it = params.find(BDIITYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),BDIITYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_bdiiType = pv->getValue();
        } 
        // Get VOMS Service Type
        if((it = params.find(VOMSTYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),VOMSTYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_vomsType = pv->getValue();
        } 
        // Get SEIndex Service Type
        if((it = params.find(SEINDEXTYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),SEINDEXTYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_seIndexType = pv->getValue();
        } 
        // Get Fireman Service Type
        if((it = params.find(FIREMANTYPE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),FIREMANTYPE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_firemanType = pv->getValue();
        } 
        
        // Get SE MountPoint
        if((it = params.find(SEPATH_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),SEPATH_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_sePathProperty = pv->getValue();
        } 
        // Get ChannelAgent Source
        if((it = params.find(CHANNELAGENTSOURCEPROPERTY_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CHANNELAGENTSOURCEPROPERTY_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_channelAgentSourceProperty = pv->getValue();
        } 
        // Get ChannelAgent Destination
        if((it = params.find(CHANNELAGENTDESTINATIONPROPERTY_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CHANNELAGENTDESTINATIONPROPERTY_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_channelAgentDestinationProperty = pv->getValue();
        } 
        // Get MyProxy FtsMode 
        if((it = params.find(MYPROXYFTSMODEPROPERTY_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),MYPROXYFTSMODEPROPERTY_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_myproxyFtsModeProperty = pv->getValue();
        } 
        // Get MyProxy FtsMode Retriever 
        if((it = params.find(MYPROXYFTSMODERETRIEVER_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),MYPROXYFTSMODERETRIEVER_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_myproxyFtsModeRetriever = pv->getValue();
        } 
        // Get MyProxy FtsMode Renewer 
        if((it = params.find(MYPROXYFTSMODERENEWER_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),MYPROXYFTSMODERENEWER_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            sd_config.m_myproxyFtsModeRenewer = pv->getValue();
        } 
        
        // Get Cache Enabled property
        bool enable_cache = DEFAULT_CACHE_ENABLED_PROPERTY;
        if((it = params.find(CACHEENABLED_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CACHEENABLED_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            // Copy string without case
            std::string val = pv->getValue();
            std::transform(val.begin(),val.end(),val.begin(),tolower);
                                                                                                                                                                                 
            if((0 == val.compare("true")) || (0 == val.compare("yes"))){
                enable_cache = true;
            } else {
                enable_cache = false;
            }
        }
        sd_config.enableCache(enable_cache);

        // Log Initialization Values
        m_log_info(getName() << " Initialized with parameters:");
        m_log_info("SRM Service Type                 : " << sd_config.srm());
        m_log_info("GridFTP Service Type             : " << sd_config.gridftp());
        m_log_info("Legacy GridFTP Service Type      : " << sd_config.legacyGridftp());
        m_log_info("MyProxy Service Type             : " << sd_config.myproxy());
        m_log_info("FileTransfer Service Type        : " << sd_config.fileTransfer());
        m_log_info("FileTransferStats Service Type   : " << sd_config.fileTransferStats());
        m_log_info("ChannelManagement Service Type   : " << sd_config.channelManagement());
        m_log_info("ChannelAgent Service Type        : " << sd_config.channelAgent());
        m_log_info("BDII Service Type                : " << sd_config.bdii());
        m_log_info("VOMS Service Type                : " << sd_config.voms());
        m_log_info("SEIndex Service Type             : " << sd_config.seIndex());
        m_log_info("Fireman Service Type             : " << sd_config.fireman());
        m_log_info("SEPath Property                  : " << sd_config.sePathProperty());
        m_log_info("ChannelAgent Source Property     : " << sd_config.channelAgentSourceProperty());
        m_log_info("ChannelAgent Destination Property: " << sd_config.channelAgentDestinationProperty());
        m_log_info("MyProxy FtsMode Property         : " << sd_config.myproxyFtsModeProperty());
        m_log_info("MyProxy FtsMode Retriever        : " << sd_config.myproxyFtsModeRetriever());
        m_log_info("MyProxy FtsMode Renewer          : " << sd_config.myproxyFtsModeRenewer());
        m_log_info("Cache                            : " << str_enabled(enable_cache));
        
    } catch(const ConfigurationException& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        throw;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        throw;
    }
    return 0;
        
}

/*
 * config
 *
 * Configure the Component
 */
int ServiceDiscoveryConfig::config(const Params& params){
    // Clean Cache
    SDCache * cache = SDConfig::instance().cache();
    if(0 != cache){
        cache->clear();
    }
    
    try{
        unsigned int cache_ttl              = 0;
        unsigned int stale_time             = 0;
        unsigned int obsolete_time          = 0;
        unsigned int negative_obsolete_time = 0;
        
        Params::const_iterator it;
        if(0 != cache){
            // Get Cache entry validity time
            if((it = params.find(CACHE_TTL_PROPERTY_NAME)) != params.end()){
                ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
                if(0 == pv){
                    throw ConfigParamException(getName(),CACHE_TTL_PROPERTY_NAME,ConfigParamException::INVALID);
                }
                cache_ttl = atoi(pv->getValue().c_str());
            }
            // Get Cache entry stale time
            if((it = params.find(CACHE_STALETIME_PROPERTY_NAME)) != params.end()){
                ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
                if(0 == pv){
                    throw ConfigParamException(getName(),CACHE_STALETIME_PROPERTY_NAME,ConfigParamException::INVALID);
                }
                stale_time = atoi(pv->getValue().c_str());
            }
            // Get Cache entry obsolete time
            if((it = params.find(CACHE_OBSOLETETIME_PROPERTY_NAME)) != params.end()){
                ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
                if(0 == pv){
                    throw ConfigParamException(getName(),CACHE_OBSOLETETIME_PROPERTY_NAME,ConfigParamException::INVALID);
                }
                obsolete_time = atoi(pv->getValue().c_str());
            }
            // Get Cache entry obsolete time
            if((it = params.find(CACHE_NEGATIVEOBSOLETETIME_PROPERTY_NAME)) != params.end()){
                ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
                if(0 == pv){
                    throw ConfigParamException(getName(),CACHE_NEGATIVEOBSOLETETIME_PROPERTY_NAME,ConfigParamException::INVALID);
                }
                negative_obsolete_time = atoi(pv->getValue().c_str());
            }
            // Set Cache proeprties
            if(0 != cache_ttl){
                cache->ttl(cache_ttl);
            }
            if(0 != stale_time){
                cache->staleTime(stale_time);
            }
            if(0 != obsolete_time){
                cache->obsoleteTime(obsolete_time);
            }
            if(0 != negative_obsolete_time){
                cache->negativeObsoleteTime(negative_obsolete_time);
            }
        }
        // Log Configuration Values
        m_log_info(getName() << " Configured with parameters:");
        if(0 != cache_ttl){
            m_log_info("Cache TTL                         : " << cache_ttl);
        } else {
            m_log_info("Cache TTL                         : use default");
        }
        if(0 != stale_time){
            m_log_info("Cache Entry Stale Time            : " << stale_time);
        } else {
            m_log_info("Cache Entry Stale Time            : use default");
        }
        if(0 != obsolete_time){
            m_log_info("Cache Entry Obsolete Time         : " << obsolete_time);
        } else {
            m_log_info("Cache Entry Obsolete Time         : use default");
        }
        if(0 != negative_obsolete_time){
            m_log_info("Cache Negative Entry Obsolete Time: " << negative_obsolete_time);
        } else {
            m_log_info("Cache Negative Entry Obsolete Time: use default");
        }
    } catch(const ConfigurationException& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        throw;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        throw;
    }
    return 0;
}
