/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: SD Utility Functions
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: sd-utility.cpp,v 1.12 2009/12/07 13:51:04 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/sd/sd-utility.h"
#include "glite/data/agents/sd/ServiceDiscovery.h"
#include "glite/data/agents/sd/SDCache.h"
#include "glite/data/agents/sd/ServiceDiscoveryException.h"
#include "glite/data/agents/url-utility.h"
#include "glite/data/agents/sd/SDConfig.h"

#include "sd.h"
#include "glite/data/glite-util.h"

#include <glite/data/config/service/Logger.hxx>

#include <boost/scoped_ptr.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string/split.hpp>
#include <boost/algorithm/string/classification.hpp>
#include <boost/algorithm/string/case_conv.hpp>
#include <algorithm>

using boost::scoped_ptr;

namespace {
const char * const SDUTILITY_NAME = "transfer-agent-sd";
    
/**
 * Predicate used to convert an array of object in an array of pointers to the
 * same objects
 */    
template <typename X>
class return_ptr : std::unary_function<const X*,const X&>{
public:
    const X* operator()(const boost::shared_ptr<X>& x){return x.get();}
};

/*
 * create_vo_list
 *
 * Create a VO List with just one entry
 */ 
void create_vo_list(const std::string& vo_name,std::vector<std::string>& vo_list){
    
    // Get VO list From VOMS attributes
    // Disable it due to a memory leak
    //SDVOList *vos = check_voms_proxy();
    SDVOList *vos = 0;
    if(0 != vos){
        for(int i = 0; i < vos->numNames; i++){
            if(0 != vos->names[i]){
                vo_list.push_back(vos->names[i]);
                free(vos->names[i]);
            }                
        }
        free(vos->names);
        free(vos);
    } 
    if(vo_list.empty()){
        // No VO found in the certificate in use. Use the Default one
        vo_list.push_back(vo_name);
    }
}

/**
 *  normalize slashes in a path
 */
std::string normalize_path(const std::string& path) throw() {
    std::string new_path = path;
    // TODO for the moment remove only the last one.
    if(!new_path.empty() && *(new_path.rbegin()) == '/') {
        new_path.erase(new_path.size() - 1);
    }
    boost::to_lower(new_path);
    return new_path;
}

} // end global namespace

namespace glite {
namespace data  {
namespace agents{
namespace sd    {

//-----------------------------------------------------------------------------
//                              SelectByVersion
//-----------------------------------------------------------------------------

/**
 * Predicate to select the Service by version
 */
SelectByVersion::SelectByVersion(const std::string& version):m_major(0),m_minor(0),m_patch(0){
    std::vector<std::string> ver_nums;
    boost::split( ver_nums, version, boost::is_any_of(".") );
    if(ver_nums.size() >= 1){
        try{
            m_major = boost::lexical_cast<unsigned int>(ver_nums.at(0));
        }catch(...){}
    }
    if(ver_nums.size() >= 2){
        try{
            m_minor = boost::lexical_cast<unsigned int>(ver_nums.at(1));
        }catch(...){}
    }
    if(ver_nums.size() >= 3){
        try{
            m_patch = boost::lexical_cast<unsigned int>(ver_nums.at(2));
        }catch(...){}
    }
}

/*
 * Select by version predicate
 */
const Service* SelectByVersion::select(const std::vector<const Service*>& services) throw() {
    _log_trace(SDUTILITY_NAME,"SelectByVersion major='" << m_major << "' minor='" << m_minor << "' patch='" << m_patch << "'");
    const Service * s = 0;
    const Service * last_saved = 0;
    unsigned int saved_major = 0, saved_minor = 0, saved_patch = 0; 
    std::vector<const Service*>::const_iterator it;
    for(it = services.begin(); it != services.end(); ++it){
        const Service * svc = (*it);           
        // Get service version
        unsigned int major = 0, minor = 0, patch = 0; 
        std::vector<std::string> ver_nums;
        _log_trace(SDUTILITY_NAME,"checking " << svc->version);
        boost::split( ver_nums, svc->version, boost::is_any_of(".") );
        if(ver_nums.size() >= 1){
            try{
                major = boost::lexical_cast<unsigned int>(ver_nums.at(0));
            }catch(...){}
        }
        if(ver_nums.size() >= 2){
            try{
                minor = boost::lexical_cast<unsigned int>(ver_nums.at(1));
            }catch(...){}
        }
        if(ver_nums.size() >= 3){
            try{
                patch = boost::lexical_cast<unsigned int>(ver_nums.at(2));
            }catch(...){}
        }
        // Check if the major is the same
        if(major == m_major){
            if(minor == m_minor){
                // same minor, good candidate
                if(patch == m_patch){
                    // exact match
                    s = svc;
                    break;
                } else {
                    // different patch
                    if(0 == last_saved){
                        last_saved  = svc;
                        saved_major = major;
                        saved_minor = minor;
                        saved_patch = patch;
                    } else {
                        if((saved_minor == minor) &&
                           (saved_patch > patch)){
                            // keep saved
                        } else {
                            last_saved  = svc;
                            saved_major = major;
                            saved_minor = minor;
                            saved_patch = patch;
                        }
                    }
                }
            } else {
                if(0 == last_saved){
                    last_saved = svc;
                    saved_major = major;
                    saved_minor = minor;
                    saved_patch = patch;
                } else {
                    if((saved_minor == m_minor) ||
                      ((saved_minor >= minor) &&
                       (saved_patch > patch))){
                            // keep saved
                    } else {
                        last_saved  = svc;
                        saved_major = major;
                        saved_minor = minor;
                        saved_patch = patch;
                    }
                }
            }
        } else {
            // Invalid major, skip it
        }
    }// end for
    if(0 == s){
        if(0 == last_saved){
            s = *(services.begin());
        } else {
            s = last_saved;
        }
    }
    return s;
}

//-----------------------------------------------------------------------------
//                              SelectByPortAndVersion
//-----------------------------------------------------------------------------

/*
 * Select by port and version predicate
 */
const Service* SelectByPortAndVersion::select(const std::vector<const Service*>& services) throw() {
    _log_trace(SDUTILITY_NAME,"SelectByPortAndVersion port='" << m_port << "' path='" << m_path << "' version='" << m_version << "'");
    std::vector<const Service*> filtered_list;
    std::string norm_path = normalize_path(m_path);
    _log_trace(SDUTILITY_NAME,"path '" << m_path << "' normalized to '" << norm_path << "'");
    std::vector<const Service*>::const_iterator it;
    for(it = services.begin(); it != services.end(); ++it) {
        const Service * svc = (*it);
        _log_trace(SDUTILITY_NAME,"checking if service '" << svc->endpoint << "' matches.");
        std::string host, protocol, path;
        unsigned int port = 0;
        try {
            // parse service endpoint
            parse_url(svc->endpoint,host,port,protocol, path);
            _log_trace(SDUTILITY_NAME,"endpoint '" << svc->endpoint << "' parsed to host='" << 
                host << "', port='" << port << "', protocol='" << protocol << 
                "', path='" << path << "'");

            path = normalize_path(path);
            _log_trace(SDUTILITY_NAME,"service path normalized to '" << path << "'");
            // compare port
            if((0 != m_port) && (0 != port)) {
                _log_trace(SDUTILITY_NAME,"matching ports");
                if(m_port != port) {
                    continue;
                }
            }
            // compare service path
            if((false == norm_path.empty())/* && (false == path.empty())*/) {
                _log_trace(SDUTILITY_NAME,"matching paths");
                if(norm_path != path) {
                    continue;
                }
            }
            _log_trace(SDUTILITY_NAME,"adding '" << svc->endpoint << "' to matching services");
            // Add service to the filtered list
            filtered_list.push_back(svc);
        } catch(...) {
            // ignore all errors
        }
    }
    if(false == filtered_list.empty()) {
        if(1 == filtered_list.size()) {
            _log_trace(SDUTILITY_NAME,"got only 1 service -> return it");
            return *(filtered_list.begin());
        }
        // Apply filter on preferred version
        _log_trace(SDUTILITY_NAME,"got " << filtered_list.size() << " services -> select by version");
        SelectByVersion pred(m_version);
        return pred(filtered_list);
    }
    _log_trace(SDUTILITY_NAME,"no services found -> return null");
    return 0;
}

//-----------------------------------------------------------------------------
//                     service discovery utility functions
//-----------------------------------------------------------------------------

/*
 * get_service
 *
 * Return the Service for the given type and source. A source can be either a 
 * site name of a Service hostname
 */    
Service * get_service(const std::string& source, const std::string& type, const std::string& vo_name) /*throw (LogicError)*/{
    SelectFirst pred;
    return get_service(source,type,vo_name, pred);
}

/**
 * get_service
 * Get service for given type and source. A source can be either a site name or
 * a service host name
 * @param source [IN]
 * @param type [IN] 
 * @param vo_name [IN] 
 * @param pred [IN] 
 * @return first matching service if one or more service metching the specified 
 * criteria is found, null otherwise.
 */
Service * get_service(
    const std::string&  source, 
    const std::string&  type, 
    const std::string&  vo_name,
    SelectPred&         pred) /*throw (LogicError)*/{
    
    _log_trace(SDUTILITY_NAME,"source='" << source << "' type='" << type << "' vo='" << vo_name << "' pred type='" << pred.type() << "'");    
        
    Service * service = 0;
    bool missing = false;
    
    // Get Cache
    SDCache * cache = SDConfig::instance().cache();
    if(0 != cache){
        // Create VO List        
        SDCache::StringArray vo_list;
        create_vo_list(vo_name,vo_list);
        // Try to find the service on the cache by hostname
        SDCache::ServiceArray services = cache->getByHost(type,source,vo_list);
        if(true == services.empty()){
            // Try to find the service on the cache by site
            services = cache->getBySite(type,source,vo_list);
        }
        if(false == services.empty()){
            // At least a service of the given type exists: Use the predicate to
            // select the right one
            const Service * s = pred(services);
            if(0 != s){
                service = new Service(*s);
            }
        }
        if(0 == service){
            // Check if the service is not listed in the information system
            // (Avoid duplicate queries)
            missing = cache->isServiceMissing(type,source,vo_list);
        }
    }
    if((0 == service) && (false == missing)){
        // No Object in Cache or cache disabled
        ServiceDiscovery sd;
        // Try to handle the source as an SE hostname
        ServiceDiscovery::ServiceArray services;
        try{
            sd.getServicesOnHost(type,source,0,vo_name,services);
        } catch(const ServiceDiscoveryException& exc){
            // No service found for the given hostname
        }
        if(true == services.empty()){
            // No service found for the given hostname. Consider source as a
            // site name
            try{
                sd.getServicesOnSite(type,source,vo_name,services);
            } catch(const ServiceDiscoveryException& exc){
                // No service found for the given site
            }
        }
        if(false == services.empty()){
            // At least a service of the given type exists: Use the predicate to
            // select the right one
            SDCache::ServiceArray const_services;
            const_services.resize(services.size(),0);
            std::transform(services.begin(), services.end(),const_services.begin(),return_ptr<Service>());
            const Service * s = pred(const_services);
            if(0 != s){
                service = new Service(*s);
            }
        }
    }
    return service;
}

/*
 * get_service_by_type
 *
 * Get a service instance from an Source Name, that can be either a site name 
 * or a service host name. In case of more than one services is available on the 
 * given source, the first one is returned. It's up to the caller to delete the 
 * returned instance. In case no service is found, 0 is returned
 */
Service * get_service_by_type(const std::string& type, const std::string& vo_name) /*throw (LogicError)*/{
    SelectFirst pred;
    return get_service_by_type(type,vo_name, pred);
}

/*
 * get_service_by_type
 *
 * Get a service instance of the given type. A predicate can be specified in 
 * order to select the service in case more than one are available. It's up to 
 * the caller to delete the returned instance. In case no service is found, 0 
 * is returned
 */
Service * get_service_by_type(const std::string& type, const std::string& vo_name,SelectPred& pred) /*throw (LogicError)*/{
    
    _log_trace(SDUTILITY_NAME,"type='" << type << "' vo='" << vo_name << "' pred type='" << pred.type() << "'");
    
    Service * service = 0;
    bool missing = false;
   
    // Get Cache
    SDCache * cache = SDConfig::instance().cache();
    if(0 != cache){
        // Create VO List        
        SDCache::StringArray vo_list;
        create_vo_list(vo_name,vo_list);
        // Try to find the service on the cache by hostname
        SDCache::ServiceArray services = cache->getByType(type,vo_list);
        if(false == services.empty()){
            // At least a service of the given type exists: Use the predicate to
            // select the right one
            const Service * s = pred(services);
            if(0 != s){
                service = new Service(*s);
            }
        }
        if(0 == service){
            // Check if the service is not listed in the information system
            // (Avoid duplicate queries)
            missing = cache->isServiceMissing(type,vo_list);
        }
    }
    if((0 == service) && (false == missing)){
        // Cache disabled or No object found in cache and Query never performed
        ServiceDiscovery sd;
        // Try to handle the source as an SE hostname
        ServiceDiscovery::ServiceArray services;
        try{
            sd.getServicesByType(type,vo_name,services);
        } catch(const ServiceDiscoveryException& exc){
            // No service found for the given hostname
        }
        if(false == services.empty()){
            // At least a service of the given type exists: Use the predicate to
            // select the right one
            SDCache::ServiceArray const_services;
            const_services.resize(services.size(),0);
            std::transform(services.begin(), services.end(),const_services.begin(),return_ptr<Service>());
            const Service * s = pred(const_services);
            if(0 != s){
                service = new Service(*s);
            }
        }
    }
    return service;
}

/*
 * get_site_name
 *
 * Return the Site Name starting from the source name. A source can be either a 
 * site name of a Service hostname
 */    
std::string get_site_name(const std::string& source, const std::string& type, const std::string& vo_name) /*throw (LogicError)*/{
    // Get Service
    scoped_ptr<Service> service(get_service(source,type,vo_name));
    if(0 == service.get()){
        // No service found for the given source: consider source as site name
        return source;
    }
    return service->site;
}

/*
 * get_site_name
 *
 * Return the Site Name starting from the source name. A source can be either a 
 * site name of a Service hostname
 */    
std::string get_site_name(const std::string& source, const std::string& type, const std::string& vo_name,SelectPred& pred) /*throw (LogicError)*/{
    // Get Service
    scoped_ptr<Service> service(get_service(source,type,vo_name,pred));
    if(0 == service.get()){
        // No service found for the given source: consider source as site name
        return source;
    }
    return service->site;
}

/*
 * get_associated_service
 *
 * Get the service of type <service_type> that is associated with 
 * the service on host <assoc_service_host> of type <assoc_service_type>. In 
 * case the endpoint is not found, an empty string is returned. In case more 
 * than one services are available the first one is returned. It's up to the 
 * caller to delete the returned instance.
 * In case no service is found, 0 is returned
 */
Service * get_associated_service(const std::string& assoc_srv_host,const std::string& assoc_srv_type, const std::string& srv_type,const std::string& vo_name) /*throw (LogicError)*/{
    SelectFirst pred;
    return get_associated_service(assoc_srv_host,assoc_srv_type,srv_type, vo_name, pred);  
}

/*
 * get_associated_service
 *
 * Get the service of type <service_type> that is associated with 
 * the service on host <assoc_service_host> of type <assoc_service_type>. In 
 * case the endpoint is not found, an empty string is returned. A predicate can 
 * be specified in order to select the service to return in case more services  
 * are available. It's up to the caller to delete the returned instance.
 * In case no service is found, 0 is returned
 */
Service * get_associated_service(const std::string& assoc_srv_host,const std::string& assoc_srv_type, const std::string& srv_type,const std::string& vo_name, SelectPred& pred) /*throw (LogicError)*/{

    Service * service = 0;
    bool missing = false;

    // First find the Service
    scoped_ptr<Service> srv(get_service(assoc_srv_host, assoc_srv_type, vo_name));
    if(0 == srv.get()){
        // No Service Found
        return 0;
    }
    
    // Get Cache
    SDCache * cache = SDConfig::instance().cache();
    if(0 != cache){
        // Create VO List        
        SDCache::StringArray vo_list;
        create_vo_list(vo_name,vo_list);
        // Try to find the associated service on the cache
        SDCache::ServiceArray assoc_services = cache->getAssociated(srv->name,srv_type,vo_list);
        if(false == assoc_services.empty()){
            // At least a service of the given type exists: Use the predicate to
            // select the right one
            const Service * s = pred(assoc_services);
            if(0 != s){
                service = new Service(*s);
            }
        }
        if(0 == service){
            // Check if the service is not listed in the information system
            // (Avoid duplicate queries)
            missing = cache->isAssociationMissing(srv->name,srv_type,vo_list);
        }
    }
    if((0 == service) && (false == missing)){
        // Cache disabled or no entry found in cache and Query never performed
        ServiceDiscovery sd;
        // Try to handle the source as an SE hostname
        ServiceDiscovery::ServiceArray assoc_services;
        try{
            sd.getAssociatedServices(*srv, srv_type, "", assoc_services);
        } catch(const ServiceDiscoveryException& exc){
            // No associated services found
        }
        if(false == assoc_services.empty()){
            // At least a service of the given type exists: Use the predicate to
            // select the right one
            SDCache::ServiceArray const_services;
            const_services.resize(assoc_services.size(),0);
            std::transform(assoc_services.begin(), assoc_services.end(),const_services.begin(),return_ptr<Service>());
            const Service * s = pred(const_services);
            if(0 != s){
                service = new Service(*s);
            }
        }
    }
    
    return service;
}

/*
 * get_property
 *
 * Return a the value of a property for a given service. In case no proeprty is 
 * an empty string is returned
 */
std::string get_property(const Service& srv, const std::string& prop_name, const std::string& vo_name) /*throw (LogicError)*/{
    
    std::pair<std::string,bool> property_value;
    property_value.second = false;
    bool missing = false;

    // Get Cache
    SDCache * cache = SDConfig::instance().cache();
    if(0 != cache){
        // Please note that the VO is not used since Service Discovery doesn't
        // propvide VO specific properties
        // Create VO List        
        // SDCache::StringArray vo_list;
        // create_vo_list(vo_name,vo_list);
        
        // Try to find the property on the cache 
        property_value = cache->getProperty(srv.name,prop_name);
        if(false == property_value.second){
            // Check if the service is not listed in the information system
            // (Avoid duplicate queries)
            missing = cache->isPropertyMissing(srv.name,prop_name);
        }
    }
    if((false == property_value.second) && (false == missing)){
        // Cache disabled or query never performed
        ServiceDiscovery sd;
        // Try to handle the source as an SE hostname
        try{
            property_value.first = sd.getProperty(srv, prop_name);
            property_value.second = true;
        } catch(const ServiceDiscoveryException& exc){
            // No property found
        }
    }
    
    return property_value.first;
}

/*
 * get_se_path
 *
 * Get the se path property
 */
std::string get_se_path(const sd::Service& srv, const std::string& vo_name, const std::string& storage_class) {
    
    const std::string& mp_name = SDConfig::instance().sePathProperty();
    
    // TODO: Consider also the Storage Type
    
    // Try To get the Property as <VO>:SEMountPoint    
    std::string prop_name = vo_name + ":" + mp_name;
    std::string property = get_property(srv, prop_name, vo_name);
    if(true == property.empty()){
        // Legacy Check: Try to get the Property as SEMountPoint
        prop_name = mp_name;
        property = get_property(srv, prop_name, vo_name);
    }
    if(true == property.empty()){
        // No Property Found
        _log_error(SDUTILITY_NAME,"No Path found for Service <" << srv.name << ">");
    }
    return property;
}

} // End namespace sd
} // End namespace agents
} // End namespace data
} // End namespace glite
