/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: SURL Utility Functions
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: surl-utility.cpp,v 1.5 2007/01/24 12:08:22 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/surl-utility.h"
#include "glite/data/agents/url-utility.h"
#include "glite/data/agents/AgentExceptions.h"
#include <ctype.h>

#include <boost/algorithm/string.hpp>

using namespace glite::data::agents;

namespace{
const char * const SFN_KEY      = "SFN";
const char * const SRM_PROTOCOL = "srm";
    
/*
 * get_surl_values
 *
 * Translate the SURL into its fully qualified form:
 *  srm://<host>:<port>/srm/managerv1?SFN=<file_path> 
 */
void get_surl_values(const std::string& surl,const std::string& srm_endpoint,
                            std::string& surl_hostname,  unsigned int& surl_port,
                            std::string& surl_protocol,  std::string& surl_path,
                            std::string& surl_query_str, std::string& sfn,
                            std::string& sfn_qr_str) /*throw (LogicError)*/{

    // Parse SURL
    surl_port = 0;
    parse_url(surl,surl_hostname,surl_port,surl_protocol,surl_path,surl_query_str);
    
    // Parse endpoint
    std::string     srm_hostname;
    unsigned int    srm_port = 0;
    std::string     srm_protocol;
    std::string     srm_path;
    parse_url(srm_endpoint,srm_hostname,srm_port,srm_protocol,srm_path);
    
    // Check if the hostname is the same
    if(surl_hostname != srm_hostname){
        // The hostname is different. Check if the surl's one is the same as 
        // the host one, but without the domain.
        if(surl_hostname.length() < srm_hostname.length()){
            if(0 == srm_hostname.compare(0,surl_hostname.length(),surl_hostname)){
                // missing domain from SURL. Add it
                surl_hostname = srm_hostname;
            } else {
                throw LogicError("Invalid SURL: Hostname mismatch");
            }
        } else {
            throw LogicError("Invalid SURL: Hostname mismatch");
        }
    }
    // Check port
    if(surl_port <= 0) {
        if(srm_port > 0){
            surl_port = srm_port;
        }
    }
    // Does the SURL has a Query String?
    bool qs_has_sfn = false;
    if(false == surl_query_str.empty()){
        // Check That the query string contains the SFN
        std::vector<std::pair<std::string,std::string> >values;
        parse_query_string(surl_query_str,values);
        std::vector<std::pair<std::string,std::string> >::iterator v_it;
        for(v_it = values.begin(); v_it != values.end(); ++v_it){
            if((*v_it).first == SFN_KEY){
                if(*((*v_it).second.begin()) == '/'){
                    sfn = (*v_it).second;
                } else {
                    sfn = (std::string)"/" + (*v_it).second;
                }
                qs_has_sfn = true;
                break;
            }
        }
    } 
    
    // Check if the path is the same
    if(false == surl_path.empty()){     
        // Check if the SFN is set
        if(true == sfn.empty()){
            // No SFN specified in the Query String or not query string at all
            if(*(surl_path.begin()) == '/'){
                sfn = surl_path;
            } else {
                sfn = (std::string)"/" + surl_path;
            }
            surl_path = srm_path;
        }
    } else {
        // The path is empty. Take the path from the srm endpoint
        surl_path = srm_path;
    }
    if(true == sfn.empty()){
        throw LogicError("Invalid SURL: No SFN specified");
    }

    // Generate the query string
    if(false == qs_has_sfn){
        sfn_qr_str = (std::string)SFN_KEY + "=" + path_escape(sfn);
    } else {
        sfn_qr_str = "";
    }
}
} // End global namespace

namespace glite {
namespace data  {
namespace agents{
        
/*
 * fully_qualify_surl
 *
 * Translate the SURL into its fully qualified form:
 *  srm://<host>:<port>/srm/managerv1?SFN=<file_path> 
 */
std::string fully_qualify_surl(const std::string& surl,const std::string& srm_endpoint)/*throw (LogicError)*/{
    
    // Parse SURL values
    std::string     surl_hostname;
    unsigned int    surl_port;
    std::string     surl_protocol;
    std::string     surl_path;
    std::string     surl_query_str;
    std::string     sfn;
    std::string     sfn_qr_str;
    get_surl_values(surl,srm_endpoint,surl_hostname,surl_port,surl_protocol,surl_path,surl_query_str,sfn,sfn_qr_str);
    // Check if it's a SURL
    std::transform(surl_protocol.begin(),surl_protocol.end(),surl_protocol.begin(),tolower);
    if(surl_protocol != SRM_PROTOCOL){
        // Not an SURL, don't modify the value
        return surl;
    }
    if(false == sfn_qr_str.empty()){
        if(true == surl_query_str.empty()){
            surl_query_str = sfn_qr_str;
        } else {
            sfn_qr_str.push_back('&');
            surl_query_str.insert(surl_query_str.begin(),sfn_qr_str.begin(),sfn_qr_str.end());
        }
    }
    // Create fully qualified SURL
    return create_url(surl_protocol,surl_hostname,surl_port,surl_path,surl_query_str,false);
}

/*
 * compact_surl
 *
 * Translate the SURL into its compact form:
 *  srm://<host>[:<port>]/<file_path>
 */
std::string compact_surl(const std::string& surl,const std::string& srm_endpoint,bool with_port) /*throw (LogicError)*/{
    
    // Parse SURL values
    std::string     surl_hostname;
    unsigned int    surl_port;
    std::string     surl_protocol;
    std::string     surl_path;
    std::string     surl_query_str;
    std::string     sfn;
    std::string     sfn_qr_str;
    get_surl_values(surl,srm_endpoint,surl_hostname,surl_port,surl_protocol,surl_path,surl_query_str,sfn,sfn_qr_str);
    // Check if it's a SURL
    std::transform(surl_protocol.begin(),surl_protocol.end(),surl_protocol.begin(),tolower);
    if(surl_protocol != SRM_PROTOCOL){
        // Not an SURL, don't modify the value
        return surl;
    }
    
    // Don't specify port
    if(false == with_port){
        surl_port = 0;
    }
    
    // remove starting slash
    if(*(sfn.begin()) == '/'){
        sfn.erase(sfn.begin());
    }
    
    if(true == sfn_qr_str.empty()){
        // String SFN from query string
        std::vector<std::pair<std::string,std::string> >values;
        parse_query_string(surl_query_str,values);
        std::vector<std::pair<std::string,std::string> >::iterator v_it;
        for(v_it = values.begin(); v_it != values.end(); ++v_it){
            if((*v_it).first == SFN_KEY){
                values.erase(v_it);
                break;
            }
        }
        // Create fully qualified SURL
        return create_url(surl_protocol,surl_hostname,surl_port,sfn,values);
    } else {
        // Create fully qualified SURL
        return create_url(surl_protocol,surl_hostname,surl_port,sfn,surl_query_str);
    }
}

/*
 * path_from_surl
 * 
 * Extract the file path from a surl
 */
std::string path_from_surl(const std::string& surl) /*throw (LogicError)*/ {
    std::string file_path;

    std::string  hostname;
    unsigned int port = 0;
    std::string  protocol;
    std::string  path;
    std::string  query_str;

    // Parse surl
    parse_url(surl,hostname,port,protocol,path,query_str);

    // Check query string
    if(query_str.empty()){
	// SURL in compact form: take the path
	file_path = path;
	// Be sure the path start with '/'
	if(*(file_path.begin()) != '/'){
	    file_path.insert(0,"/");
	}
    } else {
	// Parse Query string
	std::vector<std::pair<std::string,std::string> > values;
	parse_query_string(query_str,values);

	// Take the SFN
	std::vector<std::pair<std::string,std::string> >::iterator it;
	for(it = values.begin(); it != values.end(); ++it){
	    std::string& tag = (*it).first;
	    boost::to_upper(tag);
	    if(tag == SFN_KEY){
		file_path = (*it).second;
		break;
	    }
	}
    }
    return file_path;
}

} // End namespace agents
} // End namespace data
} // End namespace glite
