/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: URL utility Functions
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: url-utility.cpp,v 1.6 2008/01/07 16:58:29 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/url-utility.h"
#include "glite/data/agents/AgentExceptions.h"

#include <algorithm>
#include <sstream>
#include <cctype>

#include <boost/regex.hpp>

namespace{
    
/**
 * url parsing utility
 */
class UrlParser{
public:
    
    /**
     * instance
     * @return singleton instance
     */
    static UrlParser& instance(){
        static UrlParser s_instance;
        return s_instance;
    }

    /**
     * parse an URL
     * @param url       [IN] the url to parse
     * @param hostname  [OUT] the host name
     * @param port      [OUT] the port
     * @param protocol  [OUT] the protocol
     * @param path      [OUT] the path
     * @param query_str [OUT] the query string
     * @return true if url was successfully parsed
     */
    bool parse(
        const std::string&  url,
        std::string&        hostname,
        unsigned int&       port,
        std::string&        protocol, 
        std::string&        path, 
        std::string&        query_str){
       
        boost::match_results<std::string::const_iterator> matches;
        boost::match_flag_type flags = boost::match_default | boost::match_not_dot_newline;
        bool is_match = boost::regex_match(url,matches,m_re,flags);
        if (is_match){
            if (matches.length() < 11){
                throw glite::data::agents::InvalidArgumentException("Failed parsing URL");
            }
            hostname = matches[4];
            std::string str_port = matches[6];
            port = atoi(str_port.c_str());
            protocol = matches[2];
            path = matches[8];
            query_str = matches[10];
        }
        return is_match;
    }

    ~UrlParser(){}
private:
    
    /**
     * constructor
     */
    UrlParser():m_re(
        // schema
        //  [1] = (schema) + "://"
        //  [2] = (schema) = any character not in [ \\ / ? # : ], one or more repetitions
        "(([^\\\\/?#:]+)://)?"
        // authority
        //  [3] = (authority with port) = (authority) + : + (port)
        //  [4] = (authority)
        //  [5] = : + (port)
        //  [6] = (port)
        "(([^\\\\/?#:]*)(:([^\\\\/?#]*))?)?"
        // path :
        //  [7] = /(path)
        //  [8] = (path)
        "(/([^\\\\?#]*))?"
        // query :
        //  [9] = ? + (query)
        //  [10] = (query)
        "(\\?([^#]*))?"
    ){
    }
    // the regular expression object
    boost::regex m_re;
};

/**
 * str_escape
 * Escape a string
 */
std::string str_hex_escape(std::string::const_iterator b, std::string::const_iterator e,const std::string& allowed){
    std::string escaped;
    escaped.reserve(3 * std::distance(b,e));
    
    // for each character
    std::string::const_iterator s_it;
    for(s_it = b; s_it != e; ++s_it){
        if((0 != isalnum((*s_it))) || 
           (0 != strchr(allowed.c_str(),(*s_it)))){
            escaped.push_back(*s_it);
        } else {
            char c[4];
            snprintf(c,4, "%%%02X",(*s_it));
            escaped.append(c,3);
        }        
    }
    
    return escaped;
}

} // end global namespace

namespace glite {
namespace data  {
namespace agents{

/*
 * url_escape
 *
 * url Escape a string
 */
std::string url_escape(const std::string& str) {
    return url_escape(str.begin(),str.end());
}

/*
 * url_escape
 *
 * url Escape a string
 */
std::string url_escape(std::string::const_iterator begin, std::string::const_iterator end) {
    return str_hex_escape(begin, end,"$-_.+!*'(),");
}

/*
 * path_escape
 *
 * Path Escape a string
 */
std::string path_escape(const std::string& str) {
    return path_escape(str.begin(),str.end());
}

/*
 * path_escape
 *
 * Path Escape a string
 */
std::string path_escape(std::string::const_iterator begin, std::string::const_iterator end) {
    return str_hex_escape(begin, end,"$-_.+!*'(),;/:@&=");
}

/*
 * hex_unescape
 *
 * unescape a string with hex encoded values
 */
std::string hex_unescape(const std::string& str) {
    return hex_unescape(str.begin(),str.end());
}
/*
 * hex_unescape
 *
 * unescape a string with hex encoded values
 */
std::string hex_unescape(std::string::const_iterator begin, std::string::const_iterator end) {
    std::string unescaped;
    unescaped.reserve(std::distance(begin,end));
    
    // for each character
    std::string::const_iterator s_it;
    for(s_it = begin; s_it != end; ++s_it){
        char c = *s_it;
        if ('%' == *s_it){
            // Check if the two following characters are hexadecimal
            if(std::distance(s_it,end) >= 3){
                if((0 != isxdigit(*(s_it + 1))) && (0 != isxdigit(*(s_it + 2)))){
                    char t[3];
                    t[0] = *(++s_it);
                    t[1] = *(++s_it);
                    t[2] = 0;
                    c = (char)strtol(t,0, 16);
                }
            }
        } 
        unescaped.push_back(c);
    }
    
    return unescaped;
}

/**
 * parse_url
 * parse an url and extract hostname, port, protocol, path and query string
 * @param url [IN] the url to parse
 * @param hostname [OUT]
 * @param port [OUT]
 * @param protocol [OUT]
 * @param path [OUT]
 * @param query_str [OUT]
 */
void parse_url(
    const std::string&  url,
    std::string&        hostname,
    unsigned int&       port,
    std::string&        protocol, 
    std::string&        path, 
    std::string&        query_str) /*throw (LogicException)*/{

    if (! UrlParser::instance().parse(url,hostname,port,protocol,path,query_str)){
        throw InvalidArgumentException("Failed parsing URL");
    }
    if (protocol.empty()){
        throw InvalidArgumentException("Invalid URL: No Schema Specified");
    }
    //get schema position
    std::string::size_type schema_pos = url.find("://");
    if(schema_pos == std::string::npos){
        throw InvalidArgumentException("Invalid URL: No Schema Specified");
    }
    
    path = hex_unescape(path);

    // convert hostname to lower case
    transform(hostname.begin(), hostname.end(), hostname.begin(), tolower); 
    // convert protocol to lower case
    transform(protocol.begin(), protocol.end(), protocol.begin(), tolower); 
    
} // end parse_url

/*
 * parse_url
 *
 * parse an url and extract hostname, port, protocol and path
 */
void parse_url(const std::string& url,std::string& hostname,unsigned int& port,std::string& protocol,std::string& path ) /*throw (LogicError)*/{
    std::string query_str;
    return parse_url(url,hostname,port,protocol,path,query_str);
}

/*
 * parse_url
 *
 * parse an url and extract hostname, port and protocol
 */
void parse_url(const std::string& url,std::string& hostname,unsigned int& port,std::string& protocol) /*throw (LogicError)*/{
    std::string path;
    std::string query_str;
    
    return parse_url(url,hostname,port,protocol,path,query_str);
}

/*
 * parse_url
 *
 * parse an url and extract hostname
 */
void parse_url(const std::string& url,std::string& hostname) /*throw (LogicError)*/{
    unsigned int port = 0;
    std::string protocol;
    std::string path;
    std::string query_str;
    return parse_url(url,hostname,port,protocol,path,query_str);
}


/*
 * parse_query_string
 *
 * parse a query string and return the array of key value pairs
 */
void parse_query_string(const std::string& query_str,std::vector<std::pair<std::string,std::string> >& values) {

    std::string key;
    std::string value;
    
    std::string::const_iterator prop_it = query_str.begin();
    std::string::const_iterator end_it  = query_str.end();
    std::string::const_iterator amp_it;
    std::string::const_iterator eq_it;
    do{
        // Find Key value pair property
        amp_it = std::find(prop_it,end_it, '&');
        eq_it = std::find(prop_it,amp_it,'=');    
        if(eq_it == amp_it){  
            values.push_back(std::pair<std::string,std::string>(
                hex_unescape(prop_it,amp_it),""));
        } else {
            values.push_back(std::pair<std::string,std::string>(
                hex_unescape(prop_it,eq_it),hex_unescape(eq_it + 1,amp_it)));
        }
        // Update property iterator
        prop_it = (amp_it != end_it)?amp_it+1:amp_it;
    } while(amp_it != end_it);
}

/*
 * create_url
 *
 * Create an URL string
 */
std::string create_url(const std::string& protocol, const std::string& hostname,unsigned int port, const std::string& path, const std::string& query_string, bool escape_str) /*throw(LogicError)*/{
    if(protocol.empty()){
        throw InvalidArgumentException("Invalid URL: No Schema Specified");
    }
    
    std::stringstream url;
    url << protocol << "://";
    if(false == hostname.empty()){
        url << hostname;
        if(port > 0){
            url << ':' << port;
        }
    }
    if((false == path.empty()) || (false == query_string.empty())){
        url << '/';
    }
    if(false == path.empty()){
        url << path_escape(path);
    }
    if(false == query_string.empty()){
        url << '?' << ((true == escape_str)?url_escape(query_string):query_string);
    }
    
    return url.str();
}    
 

/*
 * create_url
 *
 * parse a query string and return the array of key value pairs
 */
std::string create_url(const std::string& protocol, const std::string& hostname,unsigned int port, const std::string& path, const std::vector<std::pair<std::string,std::string> >& values, bool escape_str) {
    if(protocol.empty())/*throw(LogicError)*/{
        throw InvalidArgumentException("Invalid URL: No Schema Specified");
    }
    
    std::stringstream url;
    url << protocol << "://";
    if(false == hostname.empty()){
        url << hostname;
        if(port > 0){
            url << ':' << port;
        }
    }
    if((false == path.empty()) || (false == values.empty())){
        url << '/';
    }
    if(false == path.empty()){
        url << path_escape(path);
    }
    if(false == values.empty()){
        url << '?';
        std::vector<std::pair<std::string,std::string> >::const_iterator v_it;
        for(v_it = values.begin(); v_it != values.end(); ++v_it){
            if(v_it != values.begin()){
                url << '&';
            }
            url << ((true == escape_str)?url_escape((*v_it).first):(*v_it).first);
            if(false == (*v_it).second.empty()){
                url << '=' << ((true == escape_str)?url_escape((*v_it).second):(*v_it).second);
            }
        }
    }
    
    return url.str();
}

/*
 * parse_network_endpoint
 *
 * Parse an network endpoint and extract hostname, port.
 */
void parse_network_endpoint(const std::string& str,std::string& hostname,unsigned int& port) /*throw (LogicError)*/{
    
    std::string::const_iterator port_it = std::find(str.begin(),str.end(),':');
    if(port_it != str.end()){
        if(port_it == str.begin()){
            throw InvalidArgumentException("Invalid URL: empty hostname with port specified");
        }
        // Get Port
        port = atoi(std::string(port_it + 1, str.end()).c_str());
        // Strip port from hostname
        hostname = std::string(str.begin(),port_it);
    } else {
        port      = 0;
        hostname = str;
    }
    // convert hostname to lower case
    transform(hostname.begin(), hostname.end(), hostname.begin(), tolower); 
    
} // end parse_network_endpoint

/*
 * parse_endpoint
 *
 * Parse an endpoint and extract hostname, port. The endpoint could be either an
 * URL (<protocol>://<host>[:port][/<path>]) or a network endpoint.
 */
void parse_endpoint(const std::string& endpoint,std::string& hostname,unsigned int& port)/*throw (LogicError)*/{
    // Try to see if it's an endpoint
    std::string tmp_protocol;
    try {
        parse_url(endpoint,hostname,port,tmp_protocol);
    } catch (const AgentException& ){
        hostname.clear();
        // Parse it as a network endpoint
        parse_network_endpoint(endpoint, hostname, port);
    }
}

} // End namespace agents
} // End namespace data
} // End namespace glite
