/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: SurlUtility Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SurlUtilityTestCase.cpp,v 1.4 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/surl-utility.h"

#include <sys/time.h>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>

using namespace glite::data::agents;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class SurlUtilityTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( SurlUtilityTestCase);
    CPPUNIT_TEST( testFullyQualify );
    CPPUNIT_TEST( testFullyQualifyLoop );
    CPPUNIT_TEST( testCompact );
    CPPUNIT_TEST( testCompactWithPort );
    CPPUNIT_TEST( testCompactLoop );
    CPPUNIT_TEST( testPathFromSurl );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the FullyQualify Method
     */
    void testFullyQualify();
    /**
     * Test the FullyQualify Loop Method
     */
    void testFullyQualifyLoop();
    /**
     * Test the Compact Method
     */
    void testCompact();
    /**
     * Test the Compact Method
     */
    void testCompactWithPort();
    /**
     * Test the Compact Loop Method
     */
    void testCompactLoop();
    /**
     * Test Path Extraction Methods
     */
    void testPathFromSurl();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SurlUtilityTestCase, "SurlUtilityTestCase" );

namespace {
const int LOOP_ITERATIONS = 100000;
}
/*
 * setUp
 *
 * set-up test case
 */
void SurlUtilityTestCase::setUp(){
}

/*
 * testFullyQualify Method
 */
void SurlUtilityTestCase::testFullyQualify(){
    
    try{
        std::string endpoint  = "httpg://lxb1401.cern.ch:8443/srm/managerv1";
        std::string surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001";
        std::string qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (1)", qualified == surl);

        surl      = "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001";
        qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (2)", 
            qualified == "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401:8443/dpm/cern.ch/home/dteam/paolo/file_0001";
        qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (3)", 
            qualified == "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401/dpm/cern.ch/home/dteam/paolo/file_0001";
        qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (4)", 
            qualified == "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001";
        qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (5)", 
            qualified == "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001?TEST=true";
        qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (6)", 
            qualified == "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001&TEST=true");

        surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001&TEST=true";
        qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (7)", 
            qualified == surl);

        surl      = "srm://lxb1401.cern.ch//dpm/cern.ch/home/dteam/paolo/file_0001";
        qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (8)", 
            qualified == "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo%20badino/file_0001";
        qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (9)", 
            qualified == "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo%20badino/file_0001");

        surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo%20badino/file_0001";
        qualified = fully_qualify_surl(surl,endpoint);
        CPPUNIT_ASSERT_MESSAGE("Invalid qualification (10)", 
            qualified == surl);

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testFullyQualifyLoop Method
 */
void SurlUtilityTestCase::testFullyQualifyLoop(){
    
    try{
        std::string qualified;
        std::string endpoint  = "httpg://lxb1401.cern.ch:8443/srm/managerv1";
        struct timeval start_time,stop_time;
        long sec_diff,usec_diff;
        
        // Check with a fully qualified SURL
        std::string surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001";
        // Perform iterations
        gettimeofday(&start_time,0);
        for(int i = 0; i < LOOP_ITERATIONS; ++i){
            qualified = fully_qualify_surl(surl,endpoint);
        }
        gettimeofday(&stop_time,0);
        if(start_time.tv_sec < stop_time.tv_sec){
            if(start_time.tv_usec < stop_time.tv_usec){
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
                usec_diff = stop_time.tv_usec - start_time.tv_usec;
            } else {
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
                usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
            }
        } else {
            sec_diff = 0;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        }
        std::cout << "FQ (1) " << LOOP_ITERATIONS << " iterations in " << sec_diff << " secs and " << usec_diff << " usec" << std::endl;
        
        // Check with a Compact SURL
        surl      = "srm://lxb1401/dpm/cern.ch/home/dteam/paolo/file_0001";
        // Perform iterations
        gettimeofday(&start_time,0);
        for(int i = 0; i < LOOP_ITERATIONS; ++i){
            qualified = fully_qualify_surl(surl,endpoint);
        }
        gettimeofday(&stop_time,0);
        if(start_time.tv_sec < stop_time.tv_sec){
            if(start_time.tv_usec < stop_time.tv_usec){
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
                usec_diff = stop_time.tv_usec - start_time.tv_usec;
            } else {
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
                usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
            }
        } else {
            sec_diff = 0;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        }
        std::cout << "FQ (2) " << LOOP_ITERATIONS << " iterations in " << sec_diff << " secs and " << usec_diff << " usec" << std::endl;
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCompact Method
 */
void SurlUtilityTestCase::testCompact(){
    
    try{
        std::string endpoint  = "httpg://lxb1401.cern.ch:8443/srm/managerv1";
        std::string surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001";
        std::string compact   = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (1)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (2)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401:8443/dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (3)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401/dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (4)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (5)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001?TEST=true";
        compact   = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (6)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001?TEST=true");

        surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001&TEST=true";
        compact   = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (7)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001?TEST=true");

        surl      = "srm://lxb1401.cern.ch//dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (8)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo%20badino/file_0001";
        compact = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (9)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo%20badino/file_0001");

        surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo%20badino/file_0001";
        compact = compact_surl(surl,endpoint,false);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (10)", 
            compact == "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo%20badino/file_0001");
            
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCompactWithPort Method
 */
void SurlUtilityTestCase::testCompactWithPort(){
    
    try{
        std::string endpoint  = "httpg://lxb1401.cern.ch:8443/srm/managerv1";
        std::string surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001";
        std::string compact   = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (1)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (2)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401:8443/dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (3)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401/dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (4)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (5)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001?TEST=true";
        compact   = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (6)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001?TEST=true");

        surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001&TEST=true";
        compact   = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (7)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001?TEST=true");

        surl      = "srm://lxb1401.cern.ch//dpm/cern.ch/home/dteam/paolo/file_0001";
        compact   = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (8)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl      = "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo%20badino/file_0001";
        compact = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (9)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo%20badino/file_0001");

        surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo%20badino/file_0001";
        compact = compact_surl(surl,endpoint,true);
        CPPUNIT_ASSERT_MESSAGE("Not compacted (10)", 
            compact == "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo%20badino/file_0001");
            
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCompactLoop Method
 */
void SurlUtilityTestCase::testCompactLoop(){
    
    try{
        std::string compact;
        std::string endpoint  = "httpg://lxb1401.cern.ch:8443/srm/managerv1";
        struct timeval start_time,stop_time;
        long sec_diff,usec_diff;
        
        // Check with a fully qualified SURL
        std::string surl      = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001";
        // Perform iterations
        gettimeofday(&start_time,0);
        for(int i = 0; i < LOOP_ITERATIONS; ++i){
            compact = compact_surl(surl,endpoint,false);
        }
        gettimeofday(&stop_time,0);
        if(start_time.tv_sec < stop_time.tv_sec){
            if(start_time.tv_usec < stop_time.tv_usec){
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
                usec_diff = stop_time.tv_usec - start_time.tv_usec;
            } else {
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
                usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
            }
        } else {
            sec_diff = 0;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        }
        std::cout << "C (1): " << LOOP_ITERATIONS << " iterations in " << sec_diff << " secs and " << usec_diff << " usec" << std::endl;
        
        // Check with a Compact SURL
        surl      = "srm://lxb1401/dpm/cern.ch/home/dteam/paolo/file_0001";
        // Perform iterations
        gettimeofday(&start_time,0);
        for(int i = 0; i < LOOP_ITERATIONS; ++i){
            compact = compact_surl(surl,endpoint,false);
        }
        gettimeofday(&stop_time,0);
        if(start_time.tv_sec < stop_time.tv_sec){
            if(start_time.tv_usec < stop_time.tv_usec){
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
                usec_diff = stop_time.tv_usec - start_time.tv_usec;
            } else {
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
                usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
            }
        } else {
            sec_diff = 0;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        }
        std::cout << "C (2): " << LOOP_ITERATIONS << " iterations in " << sec_diff << " secs and " << usec_diff << " usec" << std::endl;
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testPathFromSurl Method
 */
void SurlUtilityTestCase::testPathFromSurl(){
    
    try{
        std::string surl = "srm://lxb1401.cern.ch:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001";
        std::string path = path_from_surl(surl);
        CPPUNIT_ASSERT_MESSAGE("Invalid path (1)", 
            path == "/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl = "srm://lxb1401.cern.ch:8443/dpm/cern.ch/home/dteam/paolo/file_0001";
        path = path_from_surl(surl);
        CPPUNIT_ASSERT_MESSAGE("Invalid path (2)", 
            path == "/dpm/cern.ch/home/dteam/paolo/file_0001");

        surl = "srm://lxb1401.cern.ch/dpm/cern.ch/home/dteam/paolo/file_0001";
        path = path_from_surl(surl);
        CPPUNIT_ASSERT_MESSAGE("Invalid path (3)", 
            path == "/dpm/cern.ch/home/dteam/paolo/file_0001");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
