/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Url Utility Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: UrlUtilityTestCase.cpp,v 1.6 2010/02/09 14:31:01 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/url-utility.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

#include <iostream>

using namespace glite::data::agents;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class UrlUtilityTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( UrlUtilityTestCase);
#if 0
    CPPUNIT_TEST( testUrlEscape );
    CPPUNIT_TEST( testPathEscape );
    CPPUNIT_TEST( testUrlUnescape );
    CPPUNIT_TEST( testUrlEscape2 );
    CPPUNIT_TEST( testParseUrl );
    CPPUNIT_TEST( testParseUrl2 );
    CPPUNIT_TEST( testParseQueryString );
    CPPUNIT_TEST( testCreateUrl );
    CPPUNIT_TEST( testCreateUrl2 );
    CPPUNIT_TEST( testCreateUrl3 );
    CPPUNIT_TEST( testParseNetworkEndpoint );
    CPPUNIT_TEST( testParseEndpoint );

    CPPUNIT_TEST( test_bug30270 );
#endif
    CPPUNIT_TEST( test_bug59955 );

    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the UrlEscape Method
     */
    void testUrlEscape();
    /**
     * Test the PathEscape Method
     */
    void testPathEscape();
    /**
     * Test the UrlUnescape Method
     */
    void testUrlUnescape();
    /**
     * Test the UrlEscape Method
     */
    void testUrlEscape2();
    /**
     * Test the ParseUrl Method
     */
    void testParseUrl();
    /**
     * Test the ParseUrl Method
     */
    void testParseUrl2();
    /**
     * Test the ParseQueryString Method
     */
    void testParseQueryString();
    /**
     * Test the CreateUrl Method
     */
    void testCreateUrl();
    void testCreateUrl2();
    void testCreateUrl3();
    /**
     * Test the ParseNetworkEndpoint Method
     */
    void testParseNetworkEndpoint();
    /**
     * Test the ParseEndpoint Method
     */
    void testParseEndpoint();

    /**
     * test bug 30270
     */
    void test_bug30270(); 
    void test_bug59955();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( UrlUtilityTestCase, "UrlUtilityTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void UrlUtilityTestCase::setUp(){
}

#define check_value(value,expected,message) \
if (value != expected) { \
    std::stringstream errmsg; \
    errmsg << message << ": expected '" << expected << "' got '" << value << "'"; \
    CPPUNIT_FAIL(errmsg.str().c_str()); \
}

/*
 * testUrlEscape Method
 */
void UrlUtilityTestCase::testUrlEscape(){
    
    try{
        std::string src = "Thequickbrownfoxjumpsoverthelazydog";
        std::string escaped = url_escape(src);
        check_value(escaped,src,"Invalid escaping (1)");

        src = "The quick brown fox jumps over the lazy dog";
        escaped = url_escape(src);
        check_value(escaped,"The%20quick%20brown%20fox%20jumps%20over%20the%20lazy%20dog","Invalid escaping (2)");

        src = "Test ;/?:@&=";
        escaped = url_escape(src);
        CPPUNIT_ASSERT_MESSAGE("Invalid escaping (3)", 
            escaped == "Test%20%3B%2F%3F%3A%40%26%3D");
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testPathEscape Method
 */
void UrlUtilityTestCase::testPathEscape(){
    
    try{
        std::string src = "/dpm/cern.ch/home/dteam/paolo/file_0001";
        std::string escaped = path_escape(src);
        check_value(escaped,src,"Invalid escaping (1)");

        src = "/dpm/cern.ch/home/dteam/paolo badino/file_0001";
        escaped = path_escape(src);
        check_value(escaped,"/dpm/cern.ch/home/dteam/paolo%20badino/file_0001","Invalid escaping (2)");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUrlUnscape Method
 */
void UrlUtilityTestCase::testUrlUnescape(){
    
    try{
        std::string src = "Thequickbrownfoxjumpsoverthelazydog";
        std::string unescaped = hex_unescape(src);
        check_value(unescaped,src,"Invalid unscaping (1)");

        src = "The%20quick%20brown%20fox%20jumps%20over%20the%20lazy%20dog";
        unescaped = hex_unescape(src);
        check_value(unescaped,"The quick brown fox jumps over the lazy dog","Invalid unscaping (2)");

        src = "Test%20%3B%2F%3F%3A%40%26%3D";
        unescaped = hex_unescape(src);
        CPPUNIT_ASSERT_MESSAGE("Invalid escaping (3)", 
            unescaped == "Test ;/?:@&=");
        
        src = "/dpm/cern.ch/home/dteam/paolo%20badino/file_0001";
        unescaped = hex_unescape(src);
        CPPUNIT_ASSERT_MESSAGE("Invalid escaping (3)", 
            unescaped == "/dpm/cern.ch/home/dteam/paolo badino/file_0001");
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUrlEscape Method
 */
void UrlUtilityTestCase::testUrlEscape2(){
    
    try{
        std::string src = "Thequickbrownfoxjumpsoverthelazydog";
        std::string escaped = url_escape(src);
        std::string unescaped = hex_unescape(escaped);
        check_value(unescaped,src,"Invalid escaping (1)");

        src = "The quick brown fox jumps over the lazy dog";
        escaped = url_escape(src);
        unescaped = hex_unescape(escaped);
        check_value(unescaped,src,"Invalid escaping (2)");

        src = "Test ;/?:@&=";
        escaped = url_escape(src);
        unescaped = hex_unescape(escaped);
        check_value(unescaped,src,"Invalid escaping (3)");
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testParseUrl Method
 */
void UrlUtilityTestCase::testParseUrl(){
    
    try{
        std::string  hostname;
        unsigned int port = 0;
        std::string  protocol;
        // Parse Endpoint
        parse_url("https://lxb1405.cern.ch:8443/dteam-fts/glite-data-transfer-fts/services/FileTransfer",hostname,port,protocol);
        check_value(protocol,"https","Invalid Protocol (1)");
        check_value(hostname,"lxb1405.cern.ch","Invalid Hostname (1)");
        check_value(port,8443,"Invalid Port     (1)");
        
        // Parse Endpoint
        protocol.clear();
        hostname.clear();
        port = 0;
        parse_url("myproxy://myproxy.cern.ch",hostname,port,protocol);
        check_value(protocol,"myproxy","Invalid Protocol (2)");
        check_value(hostname,"myproxy.cern.ch","Invalid Hostname (2)");
        check_value(port,0,"Invalid Port     (2)");
        
        // Parse SURL
        protocol.clear();
        hostname.clear();
        port = 0;
        parse_url("srm://lxb1401:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001",hostname,port,protocol);
        check_value(protocol,"srm","Invalid Protocol (3)");
        check_value(hostname,"lxb1401","Invalid Hostname (3)");
        check_value(port,8443,"Invalid Port     (3)");
        
        protocol.clear();
        hostname.clear();
        port = 0;
        parse_url("srm://srm-v2.cr.cnaf.infn.it:8443/srm/managerv2?SFN=/castor/cnaf.infn.it/grid/lcg/lhcb/srm2-test/lhcb/production/DC06/v1-lumi2/00001368/DIGI/0000/00001368_00000132_9.digi",hostname,port,protocol);
        check_value(protocol,"srm","Invalid Protocol (3.1)");
        check_value(hostname,"srm-v2.cr.cnaf.infn.it","Invalid Hostname (3.1)");
        check_value(port,8443,"Invalid Port     (3.2)");
        
        // Parse TURL
        protocol.clear();
        hostname.clear();
        port = 0;
        parse_url("gsiftp://lxb1401.cern.ch/lxb1401:/pool/cg/2005-09-21/test_paolo_05.241.0",hostname,port,protocol);
        check_value(protocol,"gsiftp","Invalid Protocol (4)");
        check_value(hostname,"lxb1401.cern.ch","Invalid Hostname (4)");
        check_value(port,0,"Invalid Port     (4)");

        // Parse URL
        parse_url("http://www.cern.ch",hostname,port,protocol);
        check_value(protocol,"http","Invalid Protocol (5)");
        check_value(hostname,"www.cern.ch","Invalid Hostname (5)");
        check_value(port,0,"Invalid Port     (5)");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testParseUrl Method
 */
void UrlUtilityTestCase::testParseUrl2(){
    
    try{
        std::string  hostname;
        unsigned int port = 0;
        std::string  protocol;
        std::string  path;
        std::string  query;

        // Parse Endpoint
        parse_url("https://lxb1405.cern.ch:8443/dteam-fts/glite-data-transfer-fts/services/FileTransfer",hostname,port,protocol,path,query);
        check_value(protocol,"https","Invalid Protocol (1)");
        check_value(hostname,"lxb1405.cern.ch","Invalid Hostname (1)");
        check_value(port,8443,"Invalid Port     (1)");
        check_value(path,"dteam-fts/glite-data-transfer-fts/services/FileTransfer","Invalid Path     (1)");
        CPPUNIT_ASSERT_MESSAGE("Invalid Query    (1)", query.empty() == true);
        
        // Parse Endpoint
        protocol.clear();
        hostname.clear();
        port = 0;
        path.clear();
        query.clear();
        parse_url("myproxy://myproxy.cern.ch",hostname,port,protocol,path,query);
        check_value(protocol,"myproxy","Invalid Protocol (2)");
        check_value(hostname,"myproxy.cern.ch","Invalid Hostname (2)");
        check_value(port,0,"Invalid Port     (2)");
        CPPUNIT_ASSERT_MESSAGE("Invalid Path     (2)", path.empty()  == true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Query    (2)", query.empty() == true);
        
        // Parse SURL
        protocol.clear();
        hostname.clear();
        port = 0;
        path.clear();
        query.clear();
        parse_url("srm://lxb1401:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001",hostname,port,protocol,path,query);
        check_value(protocol,"srm","Invalid Protocol (3)");
        check_value(hostname,"lxb1401","Invalid Hostname (3)");
        check_value(port,8443,"Invalid Port     (3)");
        check_value(path,"srm/managerv1","Invalid Path     (3)");
        check_value(query,"SFN=/dpm/cern.ch/home/dteam/paolo/file_0001","Invalid Query    (3)");

        // Parse TURL
        protocol.clear();
        hostname.clear();
        port = 0;
        path.clear();
        query.clear();
        parse_url("gsiftp://lxb1401.cern.ch/lxb1401:/pool/cg/2005-09-21/test_paolo_05.241.0",hostname,port,protocol,path,query);
        check_value(protocol,"gsiftp","Invalid Protocol (4)");
        check_value(hostname,"lxb1401.cern.ch","Invalid Hostname (4)");
        check_value(port,0,"Invalid Port     (4)");
        check_value(path,"lxb1401:/pool/cg/2005-09-21/test_paolo_05.241.0","Invalid Path     (4)");
        CPPUNIT_ASSERT_MESSAGE("Invalid Query    (4)", query.empty() == true);

        // Parse URL
        protocol.clear();
        hostname.clear();
        port = 0;
        path.clear();
        query.clear();
        parse_url("http://www.cern.ch",hostname,port,protocol,path,query);
        check_value(protocol,"http","Invalid Protocol (5)");
        check_value(hostname,"www.cern.ch","Invalid Hostname (5)");
        check_value(port,0,"Invalid Port     (5)");
        CPPUNIT_ASSERT_MESSAGE("Invalid Path     (5)", path.empty()  == true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Query    (5)", query.empty() == true);

        // Parse SURL short format
        protocol.clear();
        hostname.clear();
        port = 0;
        path.clear();
        query.clear();
        parse_url("srm://lxb1401/dpm/cern.ch/home/dteam/paolo/file_0001",hostname,port,protocol,path,query);
        check_value(protocol,"srm","Invalid Protocol (6)");
        check_value(hostname,"lxb1401","Invalid Hostname (6)");
        check_value(port,0,"Invalid Port     (6)");
        check_value(path,"dpm/cern.ch/home/dteam/paolo/file_0001","Invalid Path     (6)");
        CPPUNIT_ASSERT_MESSAGE("Invalid Query    (6)", query.empty() == true);

        // Parse SURL short format
        protocol.clear();
        hostname.clear();
        port = 0;
        path.clear();
        query.clear();
        parse_url("srm://lxb1401//dpm/cern.ch/home/dteam/paolo/file_0001",hostname,port,protocol,path,query);
        check_value(protocol,"srm","Invalid Protocol (7)");
        check_value(hostname,"lxb1401","Invalid Hostname (7)");
        check_value(port,0,"Invalid Port     (7)");
        check_value(path,"/dpm/cern.ch/home/dteam/paolo/file_0001","Invalid Path     (7)");
        CPPUNIT_ASSERT_MESSAGE("Invalid Query    (7)", query.empty() == true);

        // Parse SURL short format
        protocol.clear();
        hostname.clear();
        port = 0;
        path.clear();
        query.clear();
        parse_url("srm://lxb1401:8443/dpm/cern.ch/home/dteam/paolo/file_0001",hostname,port,protocol,path,query);
        check_value(protocol,"srm","Invalid Protocol (8)");
        check_value(hostname,"lxb1401","Invalid Hostname (8)");
        check_value(port,8443,"Invalid Port     (8)");
        check_value(path,"dpm/cern.ch/home/dteam/paolo/file_0001","Invalid Path     (8)");
        CPPUNIT_ASSERT_MESSAGE("Invalid Query    (8)", query.empty() == true);

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testParseQueryString Method
 */
void UrlUtilityTestCase::testParseQueryString(){
    
    try{
        // Parse Query String
        std::string  query = "SFN=/dpm/cern.ch/home/dteam/paolo/file_0001";
        std::vector<std::pair<std::string,std::string> > values;
        parse_query_string(query,values);        
        CPPUNIT_ASSERT_MESSAGE("Invalid size (1)", values.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid key1 (1)", values.at(0).first  == "SFN");
        CPPUNIT_ASSERT_MESSAGE("Invalid val1 (1)", values.at(0).second == "/dpm/cern.ch/home/dteam/paolo/file_0001");

        query = "SFN=/dpm/cern.ch/home/dteam/paolo/file_0001&TEST=true";
        values.clear();
        parse_query_string(query,values);
        CPPUNIT_ASSERT_MESSAGE("Invalid size (2)", values.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid key1 (2)", values.at(0).first  == "SFN");
        CPPUNIT_ASSERT_MESSAGE("Invalid val1 (2)", values.at(0).second == "/dpm/cern.ch/home/dteam/paolo/file_0001");        
        CPPUNIT_ASSERT_MESSAGE("Invalid key2 (2)", values.at(1).first  == "TEST");
        CPPUNIT_ASSERT_MESSAGE("Invalid val2 (2)", values.at(1).second == "true");

        query = "SFN=/dpm/cern.ch/home/dteam/paolo%20badino/file_0001";
        values.clear();
        parse_query_string(query,values);
        CPPUNIT_ASSERT_MESSAGE("Invalid size (3)", values.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid key1 (3)", values.at(0).first  == "SFN");
        CPPUNIT_ASSERT_MESSAGE("Invalid val1 (3)", values.at(0).second == "/dpm/cern.ch/home/dteam/paolo badino/file_0001");        

        query = "SFN=/dpm/cern.ch/home/dteam/paolo%20badino/file_0001&TEST=true&T%20E%20ST=%20false%20";
        values.clear();
        parse_query_string(query,values);
        CPPUNIT_ASSERT_MESSAGE("Invalid size (4)", values.size() == 3);
        CPPUNIT_ASSERT_MESSAGE("Invalid key1 (4)", values.at(0).first  == "SFN");
        CPPUNIT_ASSERT_MESSAGE("Invalid val1 (4)", values.at(0).second == "/dpm/cern.ch/home/dteam/paolo badino/file_0001");        
        CPPUNIT_ASSERT_MESSAGE("Invalid key2 (4)", values.at(1).first  == "TEST");
        CPPUNIT_ASSERT_MESSAGE("Invalid val2 (4)", values.at(1).second == "true");
        CPPUNIT_ASSERT_MESSAGE("Invalid key3 (4)", values.at(2).first  == "T E ST");
        CPPUNIT_ASSERT_MESSAGE("Invalid val3 (4)", values.at(2).second == " false ");
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreateUrl Method
 */
void UrlUtilityTestCase::testCreateUrl(){
    try{
        std::string url = create_url("srm","lxb1401",8443,"srm/managerv1","SFN=/dpm/cern.ch/home/dteam/paolo/file_0001",false);
        CPPUNIT_ASSERT_MESSAGE("Invalid url (1)", 
            url == "srm://lxb1401:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001");

        url = create_url("myproxy","myproxy.cern.ch");
        CPPUNIT_ASSERT_MESSAGE("Invalid url (2)", 
            url == "myproxy://myproxy.cern.ch");        
        
        url = create_url("srm","lxb1401",8443,"/srm/managerv1");
        CPPUNIT_ASSERT_MESSAGE("Invalid url (3)", 
            url == "srm://lxb1401:8443//srm/managerv1");

        url = create_url("srm","lxb1401",0,"/dpm/cern.ch/home/dteam/paolo badino/file_0001");
        CPPUNIT_ASSERT_MESSAGE("Invalid url (4)", 
            url == "srm://lxb1401//dpm/cern.ch/home/dteam/paolo%20badino/file_0001");

        url = create_url("srm","lxb1401",8443,"srm/managerv1","SFN=/dpm/cern.ch/home/dteam/paolo/file_0001",true);
        CPPUNIT_ASSERT_MESSAGE("Invalid url (5)", 
            url == "srm://lxb1401:8443/srm/managerv1?SFN%3D%2Fdpm%2Fcern.ch%2Fhome%2Fdteam%2Fpaolo%2Ffile_0001");

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreateUrl2 Method
 */
void UrlUtilityTestCase::testCreateUrl2(){
    try{
        std::vector<std::pair<std::string,std::string> > values;
        values.push_back(std::pair<std::string,std::string>("SFN","/dpm/cern.ch/home/dteam/paolo/file_0001"));
        std::string url = create_url("srm","lxb1401",8443,"srm/managerv1",values);
        CPPUNIT_ASSERT_MESSAGE("Invalid url (1)", 
            url == "srm://lxb1401:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001");
        
        values.push_back(std::pair<std::string,std::string>("TEST","true"));
        url = create_url("srm","lxb1401",8443,"srm/managerv1",values);
        CPPUNIT_ASSERT_MESSAGE("Invalid url (2)", 
            url == "srm://lxb1401:8443/srm/managerv1?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001&TEST=true");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreateUrl3 Method
 */
void UrlUtilityTestCase::testCreateUrl3(){
    try{
        std::vector<std::pair<std::string,std::string> > values;
        values.push_back(std::pair<std::string,std::string>("SFN","/dpm/cern.ch/home/dteam/paolo/file_0001"));
        std::string url = create_url("srm","lxb1401",8443,"",values);
        check_value(url,"srm://lxb1401:8443/?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001","Invalid url (1)");
        
        url = create_url("srm","lxb1401",8443,"","SFN=/dpm/cern.ch/home/dteam/paolo/file_0001");
        check_value(url,"srm://lxb1401:8443/?SFN=/dpm/cern.ch/home/dteam/paolo/file_0001","Invalid url (2)");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testParseNetworkEndpoint Method
 */
void UrlUtilityTestCase::testParseNetworkEndpoint(){
    
    try{
        std::string  hostname;
        unsigned int port = 0;
        // Parse Network Endpoint
        parse_network_endpoint("lxb1434.cern.ch:9999",hostname,port);
        check_value(hostname,"lxb1434.cern.ch","Invalid Hostname (1)");
        check_value(port,9999,"Invalid Port     (1)");
        // Parse Network Endpoint
        hostname.clear();
        port = 0;
        parse_network_endpoint("lxb1434.cern.ch",hostname,port);
        check_value(hostname,"lxb1434.cern.ch","Invalid Hostname (2)");
        check_value(port,0,"Invalid Port     (2)");
         // Parse Network Endpoint
        hostname.clear();
        port = 0;
        parse_network_endpoint("lxb1434:9999",hostname,port);
        check_value(hostname,"lxb1434","Invalid Hostname (3)");
        check_value(port,9999,"Invalid Port     (3)");
         // Parse Network Endpoint
        hostname.clear();
        port = 0;
        parse_network_endpoint("lxb1434",hostname,port);
        check_value(hostname,"lxb1434","Invalid Hostname (4)");
        check_value(port,0,"Invalid Port     (4)");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testParseEndpoint Method
 */
void UrlUtilityTestCase::testParseEndpoint(){
    
    try{
        std::string  hostname;
        unsigned int port = 0;
         // Parse URL
        parse_endpoint("myproxy://myproxy.cern.ch:7521",hostname,port);
        check_value(hostname,"myproxy.cern.ch","Invalid Hostname (1)");
        check_value(port,7521,"Invalid Port     (1)");
         // Parse Network Endpoint
        hostname.clear();
        port = 0;
        parse_endpoint("myproxy.cern.ch:7521",hostname,port);
        check_value(hostname,"myproxy.cern.ch","Invalid Hostname (2)");
        check_value(port,7521,"Invalid Port     (2)");
         // Parse Network Endpoint
        hostname.clear();
        port = 0;
        parse_endpoint("myproxy.cern.ch",hostname,port);
        check_value(hostname,"myproxy.cern.ch","Invalid Hostname (3)");
        check_value(port,0,"Invalid Port     (3)");
         // Parse Network Endpoint
        hostname.clear();
        port = 0;
        parse_endpoint("myproxy",hostname,port);
        check_value(hostname,"myproxy","Invalid Hostname (4)");
        check_value(port,0,"Invalid Port     (4)");
         // Parse Network Endpoint
        hostname.clear();
        port = 0;
        parse_endpoint("myproxy:7521",hostname,port);
        check_value(hostname,"myproxy","Invalid Hostname (5)");
        check_value(port,7521,"Invalid Port     (5)");
         // Parse URL
        hostname.clear();
        port = 0;
        parse_endpoint("myproxy://myproxy",hostname,port);
        check_value(hostname,"myproxy","Invalid Hostname (6)");
        check_value(port,0,"Invalid Port     (6)");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/**
 * test bug 30270
 */
void UrlUtilityTestCase::test_bug30270(){
    try{
        std::string  hostname;
        unsigned int port = 0;
        std::string  protocol;
        std::string  path;
        std::string  query;
        parse_url("httpg://ccsrm.in2p3.fr?sfn=something",hostname,port,protocol,path,query);
        check_value(protocol,   "httpg",            "Invalid protocol (1)");
        check_value(hostname,   "ccsrm.in2p3.fr",   "Invalid hostname (1)");
        check_value(port,       0,                  "Invalid port (1)");
        check_value(path,       "",                 "Invalid path (1)");
        check_value(query,      "sfn=something",    "Invalid query (1)");
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }

}

void UrlUtilityTestCase::test_bug59955()
{
    const std::string url = "srm://srm-atlas.cern.ch//?SFN=valami";
    try{
        std::string  hostname;
        unsigned int port = 987687;
        std::string  protocol;
        std::string  path;
        std::string  query;
        parse_url(url, hostname, port, protocol, path, query);

        std::cout << "Result: (url: " << url << ")" << std::endl
                << "hostname: '" << hostname << "'" << std::endl
                << "port: '" << port << "'" << std::endl
                << "protocol: '" << protocol << "'" << std::endl
                << "path: '" << path << "'" << std::endl
                << "query: '" << query << "'" << std::endl
                << "END RESULT" << std::endl << std::endl;
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
