/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: CredServiceTestCase Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CredServiceTestCase.cpp,v 1.4 2007/01/24 12:08:23 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/cred/CredServiceFactory.h"
#include "glite/data/agents/cred/CredService.h"
#include "glite/data/agents/cred/CredServiceException.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <unistd.h>
 
using namespace glite::data::agents::cred;

using boost::scoped_ptr;

extern const char * g_user_dn;
extern const char * g_passphrase;

/**
 * CPPUnit TestCase
 */
class CredServiceTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( CredServiceTestCase);
    CPPUNIT_TEST( testGet );
    CPPUNIT_TEST( testGet1 );
    CPPUNIT_TEST( testGet2 );
    CPPUNIT_TEST( testGet3 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the get Method
     */
    void testGet();

    /**
     * Test the get Method for an invalid user name
     */
    void testGet1();

    /**
     * Test the get Method for an already valid certificate
     */
    void testGet2();

    /**
     * Test the get Method for an invalid long user name
     */
    void testGet3();

private:
    /**
     * My Proxy Client Instance
     */
    scoped_ptr<CredService> m_cred;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( CredServiceTestCase, "CredServiceTestCase" );

namespace {
const char * const CP_TESTCASE_INVALID_USER       = "/C=NO/O=NOWHERE/OU=UNKNOWN/CN=Invalid 666";
const char * const CP_TESTCASE_INVALID_PASSPHRASE = "bingobongo";
const char * const CP_TESTCASE_INVALID_USER_LONG  = "/C=NO/O=NOWHERE/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/OU=UNKNOWN/"\
    "CN=Invalid 666";
}

/*
 * setUp
 *
 * set-up test case
 */
void CredServiceTestCase::setUp(){

    try{
        CredServiceFactory& factory = CredServiceFactory::instance();
        
        // Create CredService Instance
        m_cred.reset(factory.create(""));
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test get Method
 */
void CredServiceTestCase::testGet(){
    
    try{
        // Check For a valid user
        std::string f;
        m_cred->get(g_user_dn,g_passphrase,f);
        CPPUNIT_ASSERT_MESSAGE("Invalid Proxy File Name", 0 == access(f.c_str(),R_OK));
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test the get Method for an invalid user name
 */
void CredServiceTestCase::testGet1(){

    try{
        // Check for an invalid user
        std::string f;
        m_cred->get(CP_TESTCASE_INVALID_USER,CP_TESTCASE_INVALID_PASSPHRASE,f);
        CPPUNIT_FAIL("It shoudn't be possible to retrive the proxy from an invalid user");
    } catch(const CredServiceException& exc){
        std::string reason = exc.what();
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason", false == reason.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test get Method for an already valid proxy
 */
void CredServiceTestCase::testGet2(){
    
    try{
        // Check For a valid user. It should already be there
        std::string f;
        m_cred->get(g_user_dn,g_passphrase,f);
        CPPUNIT_ASSERT_MESSAGE("Invalid Proxy File Name", 0 == access(f.c_str(),R_OK));
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test the get Method for an invalid long user name
 */
void CredServiceTestCase::testGet3(){

    try{
        // Check for an invalid user
        std::string f;
        m_cred->get(CP_TESTCASE_INVALID_USER_LONG,CP_TESTCASE_INVALID_PASSPHRASE,f);
        CPPUNIT_FAIL("It shoudn't be possible to retrive the proxy from an invalid user");
    } catch(const CredServiceException& exc){
        std::string reason = exc.what();
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason", false == reason.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
