/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: DAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DaoTestCase.cpp,v 1.5 2007/01/24 12:08:23 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/agents/dao/DAOContextFactory.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;
using namespace glite::data::agents::dao;
using boost::scoped_ptr;

/**
 * CPPUnit DaoTestCase
 */
class DaoTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp();

private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( DaoTestCase );
    CPPUNIT_TEST( testSchema_1 );
    CPPUNIT_TEST( testSchema_2 );
    CPPUNIT_TEST( testSchema_3 );
    CPPUNIT_TEST( testSchema_4 );
    CPPUNIT_TEST( testCheck );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Check Schema Version
     */
    void testSchema_1();
    /**
     * Test Check Schema Version
     */
    void testSchema_2();
    /**
     * Test Check Schema Version
     */
    void testSchema_3();
    /**
     * Test Check Schema Version
     */
    void testSchema_4();
    /**
     * Test if the connection is Valid
     */
    void testCheck();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( DaoTestCase, "DaoTestCase" );

namespace{
const char * const SCHEMA_TABLE = "t_test_dao_schema_vers";
unsigned int SCHEMA_MAJOR = 1;
unsigned int SCHEMA_MINOR = 2;
unsigned int SCHEMA_PATCH = 3;
}

/*
 * setUp
 *
 * set-up test case
 */
void DaoTestCase::setUp(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO context", ctx.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Agent DN", ctx->agentDn().empty() == false);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }

}

/*
 * testSchema_1
 *
 * Test Schema Version Method
 */
void DaoTestCase::testSchema_1(){
    try{
        // Check DB Schema
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO context", ctx.get() != 0);
        DAOContextFactory::instance().checkSchema(*ctx,SCHEMA_TABLE,SCHEMA_MAJOR,SCHEMA_MINOR,SCHEMA_PATCH);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testSchema_2
 *
 * Test Schema Version Method
 */
void DaoTestCase::testSchema_2(){
    try{
        // Check DB Schema
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO context", ctx.get() != 0);
        DAOContextFactory::instance().checkSchema(*ctx,SCHEMA_TABLE,SCHEMA_MAJOR,0,0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testSchema_3
 *
 * Test Schema Version Method
 */
void DaoTestCase::testSchema_3(){
    try{
        // Check DB Schema
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO context", ctx.get() != 0);
        DAOContextFactory::instance().checkSchema(*ctx,SCHEMA_TABLE,SCHEMA_MAJOR,SCHEMA_MINOR + 1,0);
        CPPUNIT_FAIL("This schema should have an incompatible version");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testSchema_4
 *
 * Test Schema Version Method
 */
void DaoTestCase::testSchema_4(){
    try{
        // Check DB Schema
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO context", ctx.get() != 0);
        DAOContextFactory::instance().checkSchema(*ctx,SCHEMA_TABLE,SCHEMA_MAJOR + 1,0,0);
        CPPUNIT_FAIL("This schema should have an incompatible version");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * setUp
 *
 * set-up test case
 */
void DaoTestCase::testCheck(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO context", ctx.get() != 0);
        
        //Check if the connection is valid
        ctx->check();
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
