/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: OracleDAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleDaoTestCase.cpp,v 1.8 2010/02/09 14:31:02 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleDaoTestCase.h"

#include "glite/data/agents/dao/DAOContextFactory.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "glite/data/agents/dao/oracle/StatementPtr.h"
#include "glite/data/agents/dao/oracle/ResultSetPtr.h"
#include <boost/scoped_ptr.hpp>


namespace{
const char * const DUMMY_TABLE          = "t_test_dao_dummy";
const char * const DUMMY_ID             = "dummy_id";
const char * const DUMMY_NAME           = "name";
const char * const DUMMY_COUNTER        = "counter";
const char * const DUMMY_INSERTION_TIME = "insertion_time";
const char * const DUMMY_FLAG           = "flag";
const char * const DUMMY_SOME_DATA      = "some_data";
}


using namespace glite::data::agents;
using namespace glite::data::agents::dao;

using boost::scoped_ptr;

namespace glite   {
namespace data    {
namespace agents  {
namespace dao     {
namespace oracle  {

/*
 * setUp
 *
 * set-up test case
 */
void OracleDaoTestCase::setUp(){

    // Create DAO Context
    scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
    CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", dynamic_cast<OracleDAOContext*>(ctx.get()) != 0);

}

void OracleDaoTestCase::tearDown()
{
    // EMPTY
}

/*
 * testQuery_1
 *
 * Test a Query Method
 */
void OracleDaoTestCase::testQuery_1(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

        // Create Statement
        std::stringstream query_stmt;
        query_stmt << "SELECT SYSTIMESTAMP FROM DUAL";
        StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
        // Execute Statement
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
            // Fetch Query Result
            rs->next();
            // Get Timestamp
            time_t timestamp = ora_ctx->toTimeT(rs->getTimestamp(1));
            CPPUNIT_ASSERT_MESSAGE("Invalid Timestamp", timestamp != -1);
        } else {
            CPPUNIT_FAIL("No Result");
        }
    } catch(const ::oracle::occi::SQLException& exc){
        CPPUNIT_FAIL(exc.getMessage().c_str());
    }
}

/*
 * testQuery_2
 *
 * Test a Query Method
 */
void OracleDaoTestCase::testQuery_2(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

        // Create Statement
        std::stringstream query_stmt;
        query_stmt << "SELECT " << DUMMY_ID
                       << " , " << DUMMY_NAME
                       << " , " << DUMMY_COUNTER
                       << " , " << DUMMY_INSERTION_TIME
                       << " , " << DUMMY_FLAG
                       << " , " << DUMMY_SOME_DATA
                       << " FROM " << DUMMY_TABLE;
        StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
        // Execute Statement
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(!(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status())){
            CPPUNIT_FAIL("No Result");
        }
        // Fetch Query Result
        while(rs->next()){
            // Get ID
            int id = rs->getInt(1);
            CPPUNIT_ASSERT_MESSAGE("Invalid ID", id != -1);
            // Get Name
            std::string name = rs->getString(2);
            CPPUNIT_ASSERT_MESSAGE("Invalid Name", name.empty() != true);
            // Get Counter
            longlong counter = ora_ctx->toLongLong(rs->getNumber(3));
            CPPUNIT_ASSERT_MESSAGE("Invalid Counter", counter != -1);
            // Get Insert Time
            time_t insertion_time = ora_ctx->toTimeT(rs->getTimestamp(4));
            CPPUNIT_ASSERT_MESSAGE("Invalid Insertion Time", insertion_time != -1);
            // Get Boolean
            bool b = ora_ctx->toBoolean(rs->getString(5));
            CPPUNIT_ASSERT_MESSAGE("Invalid Boolean", b == ((id%2)==0)?true:false);
            // Get Clob
            std::string clob;
            ora_ctx->toString(rs->getClob(6),clob);
            if(id < 6){
                CPPUNIT_ASSERT_MESSAGE("Empty Clob", clob.empty() == false);
                if(id == 0){
                    CPPUNIT_ASSERT_MESSAGE("Invalid Clob", clob == "The Quick Brown Fox jumps over the Lazy Dog :-)");
                }
            } else {
                CPPUNIT_ASSERT_MESSAGE("Non-Empty Clob", clob.empty() == true);
            }
        }
    } catch(const ::oracle::occi::SQLException& exc){
        CPPUNIT_FAIL(exc.getMessage().c_str());
    }
}

/**
 * Test Time Convertion Method
 */
void OracleDaoTestCase::testTimeConversion_1(){

    // Create DAO Context
    scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
    OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
    CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

    // Get Current Time
    time_t current;
    time(&current);
    
    // Convert it to Timestamp
    ::oracle::occi::Timestamp timestamp = ora_ctx->toTimestamp(current);
    // Convert it to time_t
    time_t time_value = ora_ctx->toTimeT(timestamp);
    CPPUNIT_ASSERT_MESSAGE("Invalid Time value", time_value != -1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", time_value == current);

}

/**
 * Test Time Convertion Method
 */
void OracleDaoTestCase::testTimeConversion_2(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

        // Create Statement
        std::stringstream query_stmt;
        query_stmt << "SELECT SYSTIMESTAMP AT TIME ZONE '+00:00' FROM DUAL";
        StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
        // Execute Statement
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
            // Fetch Query Result
            rs->next();
            // Get Timestamp
            ::oracle::occi::Timestamp timestamp = rs->getTimestamp(1);
            
            // Convert it to time_t
            time_t time_value = ora_ctx->toTimeT(timestamp);
            CPPUNIT_ASSERT_MESSAGE("Invalid Time Value", time_value != -1);

            // Convert it to Timestamp
            ::oracle::occi::Timestamp timestamp_2 = ora_ctx->toTimestamp(time_value);
            std::string t1_str = timestamp.toText("YYYY,MM,DD HH:MI:SS TZH:TZM",0);
            std::string t2_str = timestamp_2.toText("YYYY,MM,DD HH:MI:SS TZH:TZM",0);
            CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", t1_str == t2_str);
        } else {
            CPPUNIT_FAIL("No Result");
        }
    } catch(const ::oracle::occi::SQLException& exc){
        CPPUNIT_FAIL(exc.getMessage().c_str());
    }
}

/**
 * Test LongLong Convertion Method
 */
void OracleDaoTestCase::testLongLongConversion_1(){

    // Create DAO Context
    scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
    OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
    CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

    // Get Current Time
    longlong l = 123456789012345ll;
    
    // Convert it to Number
    ::oracle::occi::Number n = ora_ctx->toNumber(l);
    // Convert it to LongLong
    longlong l_2 = ora_ctx->toLongLong(n);
    CPPUNIT_ASSERT_MESSAGE("Invalid LongLong", l_2 > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", l_2 == l);

}

/**
 * Test LongLong Convertion Method
 */
void OracleDaoTestCase::testLongLongConversion_2(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

        // Create Statement
        std::stringstream query_stmt;
        query_stmt << "SELECT TO_NUMBER('123456789012345') FROM DUAL";
        StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
        // Execute Statement
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
            // Fetch Query Result
            rs->next();
            // Get Timestamp
            ::oracle::occi::Number n = rs->getNumber(1);
            
            // Convert it to longlong
            longlong l = ora_ctx->toLongLong(n);
            CPPUNIT_ASSERT_MESSAGE("Invalid LongLong", l > 0);

            // Convert it to Number
            ::oracle::occi::Number n_2 = ora_ctx->toNumber(l);
            CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", n_2 == n);
        } else {
            CPPUNIT_FAIL("No Result");
        }
    } catch(const ::oracle::occi::SQLException& exc){
        CPPUNIT_FAIL(exc.getMessage().c_str());
    }
}

/**
 * Test Boolean Convertion Method
 */
void OracleDaoTestCase::testBooleanConversion_1(){

    // Create DAO Context
    scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
    OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
    CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

    // Set Boolean value
    bool b = true;
    
    // Convert it to String
    std::string s = ora_ctx->toBoolean(b);
    // Convert it to Boolean
    bool b_2 = ora_ctx->toBoolean(s);
    CPPUNIT_ASSERT_MESSAGE("Invalid Boolean (1)", b_2 == b);

    // Set Boolean value
    b = false;
    
    // Convert it to String
    s = ora_ctx->toBoolean(b);
    // Convert it to Boolean
    b_2 = ora_ctx->toBoolean(s);
    CPPUNIT_ASSERT_MESSAGE("Invalid Boolean (2)", b_2 == b);

}

/**
 * Test Boolean Convertion Method
 */
void OracleDaoTestCase::testBooleanConversion_2(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

        // Create Statement
        {
            std::stringstream query_stmt;
            query_stmt << "SELECT 'Y' FROM DUAL";
            StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
            // Execute Statement
            ResultSetPtr rs(*stmt, stmt->executeQuery());
            if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
                // Fetch Query Result
                rs->next();
                // Get String
                std::string s = rs->getString(1);
                
                // Convert it to boolean
                bool b = ora_ctx->toBoolean(s);
                CPPUNIT_ASSERT_MESSAGE("Invalid Boolean (1)", b == true);
    
                // Convert it to String
                std::string s_2 = ora_ctx->toBoolean(b);
                CPPUNIT_ASSERT_MESSAGE("Invalid Conversion (1)", s_2 == s);
            } else {
                CPPUNIT_FAIL("No Result");
            }
        }
        {
            std::stringstream query_stmt;
            query_stmt << "SELECT 'N' FROM DUAL";
            StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
            // Execute Statement
            ResultSetPtr rs(*stmt, stmt->executeQuery());
            if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
                // Fetch Query Result
                rs->next();
                // Get String
                std::string s = rs->getString(1);
                
                // Convert it to boolean
                bool b = ora_ctx->toBoolean(s);
                CPPUNIT_ASSERT_MESSAGE("Invalid Boolean (2)", b == false);
    
                // Convert it to String
                std::string s_2 = ora_ctx->toBoolean(b);
                CPPUNIT_ASSERT_MESSAGE("Invalid Conversion (2)", s_2 == s);
            } else {
                CPPUNIT_FAIL("No Result");
            }
        }
        {
            std::stringstream query_stmt;
            query_stmt << "SELECT 'y' FROM DUAL";
            StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
            // Execute Statement
            ResultSetPtr rs(*stmt, stmt->executeQuery());
            if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
                // Fetch Query Result
                rs->next();
                // Get String
                std::string s = rs->getString(1);
                
                // Convert it to boolean
                bool b = ora_ctx->toBoolean(s);
                CPPUNIT_ASSERT_MESSAGE("Invalid Boolean (1)", b == true);
    
                // Convert it to String
                std::string s_2 = ora_ctx->toBoolean(b);
                CPPUNIT_ASSERT_MESSAGE("Invalid Conversion (1)", s_2 == "Y");
            } else {
                CPPUNIT_FAIL("No Result");
            }
        }
        {
            std::stringstream query_stmt;
            query_stmt << "SELECT 'n' FROM DUAL";
            StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
            // Execute Statement
            ResultSetPtr rs(*stmt, stmt->executeQuery());
            if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
                // Fetch Query Result
                rs->next();
                // Get String
                std::string s = rs->getString(1);
                
                // Convert it to boolean
                bool b = ora_ctx->toBoolean(s);
                CPPUNIT_ASSERT_MESSAGE("Invalid Boolean (2)", b == false);
    
                // Convert it to String
                std::string s_2 = ora_ctx->toBoolean(b);
                CPPUNIT_ASSERT_MESSAGE("Invalid Conversion (2)", s_2 == "N");
            } else {
                CPPUNIT_FAIL("No Result");
            }
        }
    } catch(const ::oracle::occi::SQLException& exc){
        CPPUNIT_FAIL(exc.getMessage().c_str());
    }
}

/**
 * Test Clob Convertion Method
 */
void OracleDaoTestCase::testClobConversion_1(){

    // Create DAO Context
    scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
    OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
    CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

    {
        std::stringstream query_stmt;
        query_stmt << "SELECT TO_CLOB('Hello World') FROM DUAL";
        StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
        // Execute Statement
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
            // Fetch Query Result
            rs->next();
            // Get Clob
    ::oracle::occi::Clob c = rs->getClob(1);
                                                                                                                                         
            // Convert it to String
            std::string s;
            ora_ctx->toString(c,s);
            CPPUNIT_ASSERT_MESSAGE("Invalid Clob (1)", s == "Hello World");
        } else {
            CPPUNIT_FAIL("No Result");
        }
    }

}

/**
 * testFunction_01
 */
void OracleDaoTestCase::testFunction_01(){
    // select get_dummy_data('entry_0','xxx') from dual;
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

        // Create Statement
        std::stringstream query_stmt;
        query_stmt << "select get_dummy_data('entry_0','fox') from dual";
        StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
        // Execute Statement
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
            // Fetch Query Result
            rs->next();
            // Get value
            std::string value = rs->getString(1);
            CPPUNIT_ASSERT_MESSAGE("Invalid value", value == "brown");
        } else {
            CPPUNIT_FAIL("No Result");
        }
    } catch(const ::oracle::occi::SQLException& exc){
        CPPUNIT_FAIL(exc.getMessage().c_str());
    }
}

/**
 * testFunction_02
 */
void OracleDaoTestCase::testFunction_02(){
    // select get_dummy_data('entry_0','xxx') from dual;
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

        // Create Statement
        std::stringstream query_stmt;
        query_stmt << "select get_dummy_data('entry_0','xxx') from dual";
        StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
        // Execute Statement
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
            // Fetch Query Result
            rs->next();
            // Get value
            std::string value = rs->getString(1);
            CPPUNIT_ASSERT_MESSAGE("Invalid value", value == "");
        } else {
            CPPUNIT_FAIL("No Result");
        }
    } catch(const ::oracle::occi::SQLException& exc){
        CPPUNIT_FAIL(exc.getMessage().c_str());
    }
}

/**
 * testPackage_01
 */
void OracleDaoTestCase::testPackage_01(){
    // select get_dummy_data('entry_0','xxx') from dual;
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

        // Create Statement
        std::stringstream query_stmt;
        query_stmt << "select DUMMY_PACKAGE.package_dummy_data('entry_0','dog') from dual";
        StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
        // Execute Statement
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
            // Fetch Query Result
            rs->next();
            // Get value
            std::string value = rs->getString(1);
            CPPUNIT_ASSERT_MESSAGE("Invalid value", value == "lazy");
        } else {
            CPPUNIT_FAIL("No Result");
        }
    } catch(const ::oracle::occi::SQLException& exc){
        CPPUNIT_FAIL(exc.getMessage().c_str());
    }
}

/**
 * testPackage_02
 */
void OracleDaoTestCase::testPackage_02(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        OracleDAOContext* ora_ctx = dynamic_cast<OracleDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", ora_ctx != 0);

        // Create Statement
        std::stringstream query_stmt;
        query_stmt << "select DUMMY_PACKAGE.package_dummy_data(:1,:2) from dual";
        StatementPtr stmt(ora_ctx->connection(), ora_ctx->connection().createStatement(query_stmt.str()));
        stmt->setString(1,"entry_0");
        stmt->setString(2,"fox");
        // Execute Statement
        ResultSetPtr rs(*stmt, stmt->executeQuery());
        if(::oracle::occi::ResultSet::DATA_AVAILABLE == rs->status()){
            // Fetch Query Result
            rs->next();
            // Get value
            std::string value = rs->getString(1);
            CPPUNIT_ASSERT_MESSAGE("Invalid value", value == "brown");
        } else {
            CPPUNIT_FAIL("No Result");
        }
    } catch(const ::oracle::occi::SQLException& exc){
        CPPUNIT_FAIL(exc.getMessage().c_str());
    }
}

} // End namespace oracle
} // End namespace dao
} // End namespace agenta
} // End namespace data
} // End namespace glite
