/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: PatternList Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PatternListTestCase.cpp,v 1.3 2007/03/15 09:07:15 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/extra/PatternList.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class PatternListTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( PatternListTestCase);
    CPPUNIT_TEST( test1 );
    CPPUNIT_TEST( test2 );
    CPPUNIT_TEST( testLongMessage );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the Methods
     */
    void test1();
    void test2();
    void testLongMessage();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( PatternListTestCase, "PatternListTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void PatternListTestCase::setUp(){
}

/*
 * Test Method
 */
void PatternListTestCase::test1(){
    
    try{
        static const char* p[] = {".*file.*exists.*",".*file.*already.*there.*"};
        static PatternList patters(p,2);
                                                                                                                                          
        bool match = patters.match("This file exists");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (1)", match == true);
        match = patters.match("This FilE maybe eXiStS");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (2)", match == true); 
        match = patters.match("This File may already exist");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (3)", match == false); 
        match = patters.match("This File may be already there");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (4)", match == true); 
        match = patters.match("These files may be already there");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (5)", match == true); 
        match = patters.match("These File maybe already exist");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (6)", match == false); 
        match = patters.match("That string has nothing to do with the test");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (7)", match == false); 
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test Method
 */
void PatternListTestCase::test2(){
    
    try{
        static const char* p[] = {"aabbccddeeff","112233445566"};
        static PatternList patters(p,2);
                                                                                                                                                 
        bool match = patters.match("This file exists",true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (1)", match == false);
        match = patters.match("aabbcc");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (2)", match == false);
        match = patters.match("aabbcc",true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (3)", match == true);
        match = patters.match("112233445566");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (4)", match == true);
        match = patters.match("112233445566",true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (5)", match == true);
        match = patters.match("11223344556");
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (6)", match == false);
        match = patters.match("11223344556",true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (7)", match == true);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void PatternListTestCase::testLongMessage(){
    try{
        static const char* p1[] = {".*a.*long.*test.*message.*"};
        static const char* p2[] = {".*this.*is.*lorem.*ipsum.*"};
        static const char* p3[] = {".*this.*should.*not.*be.*matched.*"};
        static const char* p4[] = {
            ".*do.*not.*match.*this.*stuff.*",
            ".*abcd.*abcd.*abcd.*abcd.*",
            ".*efghhijkl.*mnopqrstuw.*qwertyuiop.*asdfgh.*",
            ".*efghhijkl.*mnopqrstuw.*qwertyuiop.*asdfgh.*123456*",
            ".*file.*does.*not.*exist.*",
            ".*file.*doesn't.*exist.*",
            ".*invalid.*path.*",
            ".*file.*not.*found.*",
            ".*not.*a.*pnfsfile.*",
            ".*source.*retrieval.*of.*TURL",
            ".*no.*such.*file.*or.*directory.*"};
        static PatternList patterns1(p1,1);
        static PatternList patterns2(p2,1);
        static PatternList patterns3(p3,1);
        static PatternList patterns4(p4,11);
        
        std::stringstream longMsgStream;
        longMsgStream << "this is a very long test message" << std::endl;
        for (int i=0;i<1000;i++)
        {
            longMsgStream << "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua." << std::endl;
        }
        longMsgStream << "this final line should not be matched" << std::endl;
        std::string longMsg = longMsgStream.str();
        
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (1)", (true == patterns1.match(longMsg)) );
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (2)", (true == patterns2.match(longMsg)) );
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (3)", (false == patterns3.match(longMsg)) );
        CPPUNIT_ASSERT_MESSAGE("Invalid Match (4)", (false == patterns4.match(longMsg)) );
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
