/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: ActiveObject Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TSSTestCase.cpp,v 1.3 2008/01/07 16:58:50 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/extra/TSS.h"
#include "glite/data/agents/extra/ActiveObject.h"
#include "glite/data/agents/AgentExceptions.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;

using boost::scoped_ptr;

static TSS<unsigned long> g_tss_counter;

/**
 * TSSTestActiveObject: this example is silly since the counter is already 
 * accessible, but it's just an unit test !
 */
class TSSTestActiveObject : public ActiveObject{
public:
    /**
     * Constructor
     */
    TSSTestActiveObject(int sleep_time):ActiveObject("TestTSS"),m_count(0),m_sleepTime(sleep_time){}
    /**
     * Destructor
     */
    virtual ~TSSTestActiveObject(){}
        
    /**
     * Handle Event Start. This method would be called on the ActiveObject thread
     * when the Object is started
     */
    virtual bool onEventStart() throw() {
        // Register the thread-specific counter
        m_count = 0;
        g_tss_counter.set(&m_count);
        return true;
    }
    /**
     * Handle Event Stop. This method would be called on the the ActiveObject thread
     * when the Object is going to stop
     */
    virtual bool onEventStop() throw() {
        // Deregister the thread-specific counter
        g_tss_counter.set(0);
        return true;
    }

    /**
     * Get Count Property
     */
    unsigned long count() const {return m_count;}
protected:
    /**
     * Object Main Function
     */
    virtual void svc() throw(){
        while(this->doRun()){
            // Get TS counter
            unsigned long& counter = g_tss_counter.get();
            ++counter;
            sleep(m_sleepTime);
        }
    }
private:    
    /**
     * Count Property
     */
    unsigned long m_count;

    /**
     * Sleep Time
     */
    int m_sleepTime;
};

/**
 * CPPUnit TestCase
 */
class TSSTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TSSTestCase);
    CPPUNIT_TEST( test_1 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the GetSiteForSurl Method
     */
    void test_1();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TSSTestCase, "TSSTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void TSSTestCase::setUp(){
}

/*
 * Test Method
 */
void TSSTestCase::test_1(){
    
    try{
        // Create ActiveObject
        TSSTestActiveObject obj_1(1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Id (1)", obj_1.id() == (ActiveObject::id_type)-1);
        TSSTestActiveObject obj_2(2);
        CPPUNIT_ASSERT_MESSAGE("Invalid Id (2)", obj_2.id() == (ActiveObject::id_type)-1);
        // Start ActiveObject
        bool s_1 = obj_1.start();
        CPPUNIT_ASSERT_MESSAGE("Not started (1)", s_1 == true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Id (1)", obj_1.id() != (ActiveObject::id_type)-1);
        bool s_2 = obj_2.start();
        CPPUNIT_ASSERT_MESSAGE("Not started (2)", s_2 == true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Id (2)", obj_2.id() != (ActiveObject::id_type)-1);
        // Sleep for a while
        sleep(10);
        // Stop ActiveObject
        obj_1.stop();
        CPPUNIT_ASSERT_MESSAGE("Invalid Id (1)", obj_1.id() == (ActiveObject::id_type)-1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Count (1)", obj_1.count() > 0);
        // Stop ActiveObject
        obj_2.stop();
        CPPUNIT_ASSERT_MESSAGE("Invalid Id (2)", obj_2.id() == (ActiveObject::id_type)-1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Count (2)", obj_2.count() > 0);

        // Check that the count is different
        CPPUNIT_ASSERT_MESSAGE("Count Mismatch", obj_1.count() > obj_2.count());
        
        // Check that in this thread the count is not set
        try{
            /*unsigned long c =*/ g_tss_counter.get();
            CPPUNIT_FAIL("No counter should be registered on the main thread");
        } catch(const LogicError& exc){
            // OK
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
