/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: Extra Utility Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TempFileTestCase.cpp,v 1.4 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/extra/TempFile.h"
#include "glite/data/agents/extra/Handle.h"
#include "glite/data/agents/AgentExceptions.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class TempFileTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TempFileTestCase);
    CPPUNIT_TEST( test_1 );
    CPPUNIT_TEST( test_2 );
    CPPUNIT_TEST( test_3 );
    CPPUNIT_TEST( test_4 );
    CPPUNIT_TEST( test_Error );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the TempFile Method
     */
    void test_1();
    void test_2();
    void test_3();
    void test_4();
    void test_Error();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TempFileTestCase, "TempFileTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void TempFileTestCase::setUp(){
}

/*
 * Test TempFile Method
 */
void TempFileTestCase::test_1(){
    
    try{
        int result;
        std::string fname;
        {
            // Create an Handle 
            Handle h;
            // Generate Temporary File
            TempFile f(TempFile::generate("tempfile-unit-test","/tmp",h.get()));
            CPPUNIT_ASSERT_MESSAGE("Invalid Name", f.name().empty() == false);
            CPPUNIT_ASSERT_MESSAGE("Invalid Handle", h.isNull() == false);
            fname = f.name();
            // Check that the file is accessible for read
            result = access(f.name().c_str(), R_OK); 
            CPPUNIT_ASSERT_MESSAGE("Not Readable", result == 0);
            result = access(f.name().c_str(), R_OK | W_OK); 
            CPPUNIT_ASSERT_MESSAGE("Not Writable", result == 0);
 
            // Try to write soemthing into the file
            char * buffer = "Hello World";
            result = write(h(),buffer, strlen(buffer) + 1);
            CPPUNIT_ASSERT_MESSAGE("Failed To Write", result == (int)(strlen(buffer) +1));

            // Close the Firsst Handle, so the ytes are written
            h.close();            

            // Open a new file For Read
            Handle h2(open(f.name().c_str(),O_RDONLY));
            CPPUNIT_ASSERT_MESSAGE("Cannot Open File for read", false == h2.isNull());
           
            // Read somthing 
            char buf2[128];
            result = read(h2(),buf2,128);
            CPPUNIT_ASSERT_MESSAGE("Failed To read", result != -1);
            CPPUNIT_ASSERT_MESSAGE("Invalid number of bytes read", result == (int)(strlen(buffer) +1));
            CPPUNIT_ASSERT_MESSAGE("Invalid String read", 0 == strcmp(buffer,buf2)); 
        }
        result = access(fname.c_str(), R_OK); 
        CPPUNIT_ASSERT_MESSAGE("Not Deleted", result != 0);

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test TempFile Method
 */
void TempFileTestCase::test_2(){
    
    try{
        int result;
        std::string fname;
        {
            // Create an Handle 
            Handle h;
            {
                // Generate Temporary File
                TempFile f(TempFile::generate("tempfile-unit-test","/tmp",h.get()));
                CPPUNIT_ASSERT_MESSAGE("Invalid Name", f.name().empty() == false);
                CPPUNIT_ASSERT_MESSAGE("Invalid Handle", h.isNull() == false);
                fname = f.name();
                // Check that the file is accessible for read
                result = access(f.name().c_str(), R_OK); 
                CPPUNIT_ASSERT_MESSAGE("Not Readable", result == 0);
                result = access(f.name().c_str(), R_OK | W_OK); 
                CPPUNIT_ASSERT_MESSAGE("Not Writable", result == 0);
            }
            // The file is now delete, but the handle is still valid
            result = access(fname.c_str(), R_OK); 
            CPPUNIT_ASSERT_MESSAGE("Not Deleted", result != 0);
            // Try to write soemthing into the file
            char * buffer = "Hello World";
            result = write(h(),buffer, strlen(buffer) + 1);
            CPPUNIT_ASSERT_MESSAGE("Failed To Write", result == (int)(strlen(buffer) +1));
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test TempFile Method
 */
void TempFileTestCase::test_3(){
    
    try{
        int result;
        std::string fname;
        std::string renamed_file = "/tmp/tempfile-unit-test.renamed";
        {
            {
                // Create an Handle 
                Handle h;
                // Generate Temporary File
                TempFile f(TempFile::generate("tempfile-unit-test","/tmp",h.get()));
                CPPUNIT_ASSERT_MESSAGE("Invalid Name", f.name().empty() == false);
                CPPUNIT_ASSERT_MESSAGE("Invalid Handle", h.isNull() == false);
                fname = f.name();
                // Check that the file is accessible for read
                result = access(f.name().c_str(), R_OK); 
                CPPUNIT_ASSERT_MESSAGE("Not Readable", result == 0);
                result = access(f.name().c_str(), R_OK | W_OK); 
                CPPUNIT_ASSERT_MESSAGE("Not Writable", result == 0);
               
                // Rename the file
                f.rename(renamed_file);
            }
            // The file is not deleted 
            result = access(renamed_file.c_str(), R_OK); 
            CPPUNIT_ASSERT_MESSAGE("Deleted", result == 0);
            {
                // Delete The file
                TempFile f(renamed_file);
            }
            // The file is now deleted 
            result = access(renamed_file.c_str(), R_OK); 
            CPPUNIT_ASSERT_MESSAGE("Not Deleted", result != 0); 
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test TempFile Method
 */
void TempFileTestCase::test_4(){
    
    try{
        int result;
        std::string fname;
        std::string renamed_file = "/tmp/tempfile-unit-test.renamed";
        {
            {
                // Create an Handle 
                Handle h;
                // Generate Temporary File
                TempFile f(TempFile::generate("tempfile-unit-test","/tmp",h.get()));
                CPPUNIT_ASSERT_MESSAGE("Invalid Name", f.name().empty() == false);
                CPPUNIT_ASSERT_MESSAGE("Invalid Handle", h.isNull() == false);
                fname = f.name();
                // Check that the file is accessible for read
                result = access(f.name().c_str(), R_OK); 
                CPPUNIT_ASSERT_MESSAGE("Not Readable", result == 0);
                result = access(f.name().c_str(), R_OK | W_OK); 
                CPPUNIT_ASSERT_MESSAGE("Not Writable", result == 0);
               
                // Rename the file
                f.rename(renamed_file);
            }
            // The file is not deleted 
            result = access(renamed_file.c_str(), R_OK); 
            CPPUNIT_ASSERT_MESSAGE("Deleted", result == 0);
            {
                // Delete The file
                TempFile f(renamed_file);
                f.unlink();
                // The file is now deleted 
                result = access(renamed_file.c_str(), R_OK); 
                CPPUNIT_ASSERT_MESSAGE("Not Deleted", result != 0); 
            }
            // The file is now deleted 
            result = access(renamed_file.c_str(), R_OK); 
            CPPUNIT_ASSERT_MESSAGE("Not Deleted", result != 0); 
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test TempFile Method
 */
void TempFileTestCase::test_Error(){
    
    try{
       // Check Precondition
       try{
            // Without Handle
            TempFile f(TempFile::generate("tempfile-unit-test","/tmp",0));
       } catch(const LogicError& exc){
           // OK
       }
       try{
            // Without Prefix 
            Handle h;
            TempFile f(TempFile::generate("tempfile-unit-test","/tmp",h.get()));
       } catch(const LogicError& exc){
           // OK
       }
       try{
            // With Invalid Dir
            Handle h;
            TempFile f(TempFile::generate("tempfile-unit-test","/",h.get()));
       } catch(const RuntimeError& exc){
           // OK
       }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
