/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: XmlPatternsList Test Case
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: XmlPatternsListTestCase.cpp,v 1.3 2007/07/20 09:30:43 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/extra/XmlPatternsList.h"

#include <exception>
#include <sstream>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using boost::scoped_ptr;
using namespace glite::data::agents;

/**
 * XmlPatternsListTestCase
 */
class XmlPatternsListTestCase : public CppUnit::TestCase{
public:
    /**
     * Test Case setup
     */
    void setUp();
    /**
     * Test Case tearDown
     */
    void tearDown();
    
private:
    
    // Declare Test Methods
    //--------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( XmlPatternsListTestCase );
        CPPUNIT_TEST( test_search_case_insensitive );
        CPPUNIT_TEST( test_match_case_insensitive );
        CPPUNIT_TEST( test_search_1 );
        CPPUNIT_TEST( test_search_2 );
        CPPUNIT_TEST( test_search_3 );
        CPPUNIT_TEST( test_search_4 );
        CPPUNIT_TEST( test_match_1 );
        CPPUNIT_TEST( test_match_2 );
        CPPUNIT_TEST( test_no_category_1 );
        CPPUNIT_TEST( test_no_category_2 );
        CPPUNIT_TEST( test_check_category_1 );
        CPPUNIT_TEST( test_check_category_2 );
        CPPUNIT_TEST( test_properties_1 );
        CPPUNIT_TEST( test_properties_2 );
    CPPUNIT_TEST_SUITE_END();
    
    // Test Methods
    //--------------------------------------------------------------------------
    
    /**
     * test that the patterns are case insensitive
     */
    void test_search_case_insensitive();
    void test_match_case_insensitive();
    
    /**
     * test search type patterns
     */
    void test_search_1();
    void test_search_2();
    void test_search_3();
    void test_search_4();
    
    /**
     * test match type patterns
     */
    void test_match_1();
    void test_match_2();

    /**
     * test that no category is returned if no pattern applies
     */
    void test_no_category_1();
    void test_no_category_2();

    /**
     * test the checkCategory method
     */
    void test_check_category_1();
    void test_check_category_2();

    /**
     * test the patterns properties
     */
    void test_properties_1();
    void test_properties_2();
    
    // Helper Methods
    //--------------------------------------------------------------------------
    
    /**
     * check_category
     * @param message [IN] the message to be matched against the patterns list
     * @param expectedCategory [IN] the expected category (empty = none)
     */
    void check_category(
        const std::string& message,
        const std::string& expectedCategory);

    /**
     * test_property
     * @param message [IN] message
     * @param category [IN] message category
     * @param propertyName [IN] property name
     * @param expectedValue [IN] exptected value of the property
     * @param shouldExist [IN] flag indicating if the property should be found
     */
    void test_property(
        const std::string&  message,
        const std::string&  category,
        const std::string&  propertyName,
        const std::string&  expectedValue,
        bool                shouldExist);
    
    // the object under test
    boost::scoped_ptr<XmlPatternsList> m_XmlPatternsList;
    
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(XmlPatternsListTestCase,"XmlPatternsListTestCase");

// Constants
namespace{
    // file name
    const char * XML_TEST_FILE = "patterns.xml";
    // categories
    const char * AUTHENTICATION     ="AUTHENTICATION";
    const char * CONNECTION         ="CONNECTION";
    const char * INVALID_REQUEST    ="INVALID_REQUEST";
    const char * INVALID_PATH       ="INVALID_PATH";
} // namespace

/**
 * Test Case setUp
 */
void XmlPatternsListTestCase::setUp(){
    try{
	    m_XmlPatternsList.reset(new XmlPatternsList(XML_TEST_FILE));
    } catch (const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch (...){
        CPPUNIT_FAIL("Got unknown exception.");
    }
}

/**
 * Test Case tearDown
 */
void XmlPatternsListTestCase::tearDown(){
}

// Helper Methods
//------------------------------------------------------------------------------

/**
 * check_category
 * @param message [IN] the message to be matched against the patterns list
 * @param category [IN] the expected category
 */
void XmlPatternsListTestCase::check_category(
    const std::string& message,
    const std::string& expectedCategory){
    try{
        std::stringstream err;    
        std::string category = m_XmlPatternsList->getCategory(message);
        if (expectedCategory.empty() && !category.empty()){
            err << "string '" << message << "' matched category '" << category << "'";
            CPPUNIT_FAIL(err.str().c_str());
        }else if (category != expectedCategory){
            err << "string '" << message << "' matched category '" << category << 
                "' instead of '" << expectedCategory << "'";
            CPPUNIT_FAIL(err.str().c_str());
        }
    } catch (const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch (...){
        CPPUNIT_FAIL("Got unknown exception.");
    }
}
 
/**
 * test_property
 * @param message [IN] message
 * @param category [IN] message category
 * @param propertyName [IN] property name
 * @param expectedValue [IN] exptected value of the property
 * @param shouldExist [IN] flag indicating if the property should be found
 */
void XmlPatternsListTestCase::test_property(
    const std::string&  message,
    const std::string&  category,
    const std::string&  propertyName,
    const std::string&  expectedValue,
    bool                shouldExist){
    
    std::stringstream err;
    const XmlPatternsList::PatternMatch * pattern = m_XmlPatternsList->getPatternMatch(message,category);
    if (0 == pattern){
        err << "Pattern object not found for message [" << message << "] in category [" <<
            category << "]";
        CPPUNIT_FAIL(err.str().c_str());
    }
    std::string value;
    bool exists = pattern->getProperty(propertyName,value);
    if (exists != shouldExist){
        err << "getProperty returned " << exists << " for property " << propertyName;
        CPPUNIT_FAIL(err.str().c_str());
    }
    if (value != expectedValue){
        err << "Got '" << value << "' for property '" << propertyName << "' (expected '" << expectedValue << "')";
        CPPUNIT_FAIL(err.str().c_str());
    }
}

// Test Methods
//------------------------------------------------------------------------------

/**
 * test that the patterns are case insensitive
 */
void XmlPatternsListTestCase::test_search_case_insensitive(){
	check_category("gss major status failed",AUTHENTICATION);
	check_category("gss MAJOR status failed",AUTHENTICATION);
	check_category("GSS MAJOR STATUS FAILED",AUTHENTICATION);
}
void XmlPatternsListTestCase::test_match_case_insensitive(){
	check_category("file does not exist",INVALID_PATH);
	check_category("FILE DOES NOT EXIST",INVALID_PATH);
	check_category("fIlE dOeSn'T eXiSt At AlL",INVALID_PATH);
}

/**
 * test search type patterns
 */
void XmlPatternsListTestCase::test_search_1(){
	check_category("An error occurred: GSS Major Status:authentication failed.",AUTHENTICATION);
}
void XmlPatternsListTestCase::test_search_2(){
	check_category("The Connection Has Failed.",CONNECTION);
}
void XmlPatternsListTestCase::test_search_3(){
	check_category("The Given Request Was Not Found.",INVALID_REQUEST);
}
void XmlPatternsListTestCase::test_search_4(){
	check_category("This is an invalid path, the operation failed",INVALID_PATH);
}

/**
 * test match type patterns
 */
void XmlPatternsListTestCase::test_match_1(){
	check_category("file doesn't exist:authentication failed.",INVALID_PATH);
}
void XmlPatternsListTestCase::test_match_2(){
	check_category("file does not want to exist",INVALID_PATH);
}

/**
 * test that no category is returned if no pattern applies
 */
void XmlPatternsListTestCase::test_no_category_1(){
    check_category("this message should match nothing","");
}
void XmlPatternsListTestCase::test_no_category_2(){
    check_category("this file does not exist","");
}

/**
 * test the checkCategory method
 */
void XmlPatternsListTestCase::test_check_category_1(){
    CPPUNIT_ASSERT_MESSAGE("checkCategory failed",
        false == m_XmlPatternsList->checkCategory("file does not want to exist",CONNECTION));
}
void XmlPatternsListTestCase::test_check_category_2(){
    CPPUNIT_ASSERT_MESSAGE("checkCategory failed",
        true == m_XmlPatternsList->checkCategory("file does not want to exist",INVALID_PATH));
}

/**
 * test the patterns properties
 */
void XmlPatternsListTestCase::test_properties_1(){
    test_property("gss major status",AUTHENTICATION,"scope","transfer",true);
    test_property("file does not exist",INVALID_PATH,"scope","source",true);
    test_property("file does not exist",INVALID_PATH,"xxx","yyy",true);
}
void XmlPatternsListTestCase::test_properties_2(){
    test_property("file does not exist",INVALID_PATH,"a","",false);
    test_property("file does not exist",INVALID_PATH,"b","",false);
}
