/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: Python Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PythonTestCase.cpp,v 1.4 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/python/PyHelper.h"
#include "glite/data/agents/python/PythonException.h"
#include <glite/data/config/service/Logger.hxx>

#include <boost/python.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents::python;
using namespace glite::data::agents;

namespace py = boost::python;

extern const char * g_user_dn;
extern const char * g_user_cred;
extern const char * g_cred_server;

/**
 * CPPUnit TestCase
 */
class PythonTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( PythonTestCase);
    CPPUNIT_TEST( testExecute );
    CPPUNIT_TEST( testExecuteFail );
    CPPUNIT_TEST( testImportFail );
    CPPUNIT_TEST( testGetProxy );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test a python module Execution
     */
    void testExecute();
    /**
     * Test a python module Execution
     */
    void testExecuteFail();
    /**
     * Test a python module Import that fails
     */
    void testImportFail();
    /**
     * Test the getproxyFileName method
     */
    void testGetProxy();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( PythonTestCase, "PythonTestCase" );


namespace {
const char * const PYTHON_TEST_MODULE        = "test_python";
const char * const PYTHON_TEST_FAKE_MODULE   = "this_module_doesn_t_exist";
const char * const PYTHON_TEST_FUNCTION      = "foo";
const char * const PYTHON_TEST_FAKE_FUNCTION = "this_function_doesn_t_exist";
const char * const PYTHON_TEST_GET_PROXY_FUNCTION = "test_get_proxy";
}

/*
 * setUp
 *
 * set-up test case
 */
void PythonTestCase::setUp(){
}

/*
 * Test Execute Method
 */
void PythonTestCase::testExecute(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        // Get the function object
        py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_FUNCTION);
        CPPUNIT_ASSERT_MESSAGE("Null function", function.ptr() != 0);
        
        // Invoke the Function
        std::string str = "Hello World :-)";
        int result = py::call<int>(function.ptr(),str);
        CPPUNIT_ASSERT_MESSAGE("Invalid result", result == (int)str.length());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test ExecuteFail Method
 */
void PythonTestCase::testExecuteFail(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        try{
            // Get the function object
            py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_FAKE_FUNCTION);
            CPPUNIT_FAIL("This function should fail");
        } catch(const PythonException& exc){
        // OK
        } 
    }
    catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test Execute Method
 */
void PythonTestCase::testImportFail(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_FAKE_MODULE);
        CPPUNIT_FAIL("This module should fail");
    } catch(const PythonException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test GetProxy Method
 */
void PythonTestCase::testGetProxy(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        // Get the function object
        py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_GET_PROXY_FUNCTION);
        CPPUNIT_ASSERT_MESSAGE("Null function", function.ptr() != 0);
        
        // Invoke the Function
        std::string proxy_file = py::call<std::string>(function.ptr(),g_user_dn,g_user_cred,g_cred_server);
        CPPUNIT_ASSERT_MESSAGE("Invalid proxy file", false == proxy_file.empty());
    } catch(py::error_already_set){
        std::pair<std::string, std::string> err = PyHelper::instance().lastError();
        CPPUNIT_FAIL(err.first + ":" +err.second);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
