/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: ServiceDiscovery Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDCacheTestCase.cpp,v 1.8 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/sd/SDCache.h"
#include "glite/data/agents/sd/SDConfig.h"
#include "glite/data/agents/sd/ServiceDiscoveryException.h"
#include <glite/data/config/service/Logger.hxx>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents::sd;

using boost::scoped_ptr;


/**
 * CPPUnit TestCase
 */
class SDCacheTestCase : public CppUnit::TestCase{
public:
    
    /**
     * set-up test case
     */
    void setUp ();
    /**
     * tear-down test case
     */
    void tearDown ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( SDCacheTestCase);
    CPPUNIT_TEST( testGetByName );
    CPPUNIT_TEST( testGetByType );
    CPPUNIT_TEST( testGetByHost );
    CPPUNIT_TEST( testGetBySite );
    CPPUNIT_TEST( testGetAssociated );
    CPPUNIT_TEST( testGetAssociatedOnSite );
    CPPUNIT_TEST( testGetProperty );
    CPPUNIT_TEST( testMissing );
    CPPUNIT_TEST( testClear );
    CPPUNIT_TEST( testPurge );
    CPPUNIT_TEST( testPurge_2 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the GetServiceByName Method
     */
    void testGetByName();
    /**
     * Test the GetServiceByType Method
     */
    void testGetByType();
    /**
     * Test the GetServiceByName Method For a Given VO 
     */
    void testGetByHost();
    /**
     * Test the GetServiceOnHost Method
     */
    void testGetBySite();
    /**
     * Test the GetAssociated Method
     */
    void testGetAssociated();
    /**
     * Test the GetAssociatedOnSite  Method
     */
    void testGetAssociatedOnSite();
    /**
     * Test the GetProperty Method
     */
    void testGetProperty();
    /**
     * Test the MissingService Method
     */
    void testMissing();
    /**
     * Test the Clear Method
     */
    void testClear();
    /**
     * Test the Purge Method
     */
    void testPurge();
    void testPurge_2();
    
private:

    SDCache * m_service;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SDCacheTestCase, "SDCacheTestCase" );

namespace {
const char * const SD_TYPE_FIREMAN        = "org.glite.FiremanCatalog";
const char * const SD_SITE_FIREMAN        = "cern.ch";
const char * const SD_NAME_FIREMAN        = "CERN-Fireman";
const char * const SD_TYPE_SRM            = "SRM";
const char * const SD_TYPE_GRIDFTP        = "gsiftp";
const char * const SD_TYPE_MYPROXY        = "MyProxy";
const char * const SD_SITE_SRM            = "cern.ch";
const char * const SD_SITE_TEST           = "test.com";
const char * const SD_NAME_SRM            = "CERN-SRM";
const char * const SD_NAME_DPM            = "CERN-DPM";
const char * const SD_NAME_SRM_INFN       = "INFN-SRM";
const char * const SD_NAME_GRIDFTP        = "CERN-GRIDFTP";
const char * const SD_NAME_MYPROXY        = "CERN-MYPROXY";
const char * const SD_NAME_TEST_MYPROXY   = "test-MYPROXY";
const char * const SD_NAME_TEST_FTS       = "test-FileTransfer";
const char * const SD_NAME_TEST_SRM       = "test-SRM";
const char * const SD_NAME_TEST_FIREMAN   = "test-Fireman";
const char * const SD_HOST_SRM            = "griftp05.cern.ch";
const char * const SD_HOST_GRIDFTP        = "griftp05.cern.ch";
const char * const SD_PROP_SRM_MNTPNT_KEY = "SEMountPoint";
const char * const SD_PROP_SRM_MNTPNT     = "/castor/cern.ch/user/g/gproduct/EGEETEST/SE/";
const char * const SD_VO1                 = "VO_1";
const char * const SD_VO2                 = "VO_2";
const char * const SD_SERVICE_NAME_VO1    = "VO_1-Fireman";
const char * const SD_SERVICE_NAME_VO2    = "VO_2-Fireman";
const char * const SD_SRM_NAME_VO1        = "CERN-SRM-VO1";
const char * const SD_SRM_NAME_VO2        = "CERN-SRM-VO2";
const char * const SD_FOO_TYPE            = "foo";
const char * const SD_FOO_HOST            = "host.foo.org";
const char * const SD_FOO_PROPERTY        = "fooProperty";
}

/*
 * setUp
 *
 * set-up test case
 */
void SDCacheTestCase::setUp(){
    m_service = 0;
    try{
        m_service = SDConfig::instance().cache();
        CPPUNIT_ASSERT_MESSAGE("No Cache configured", m_service != 0);
        
        // Set the cache options
        m_service->ttl(1);
        m_service->staleTime(1);
        m_service->obsoleteTime(3);
        m_service->negativeObsoleteTime(3);
        
        m_service->insert(Service(
            "CERN-Fireman",
            "org.glite.FiremanCatalog",
            "https://lxb2028.cern.ch:8443/EGEE/glite-data-catalog-service-fr/services/FiremanCatalog",
            "2.0.0",
            "CERN.CH",
            "lxb2028.cern.ch",
            8443));
        m_service->insert(Service(
            "INFN-SRM",
            "SRM",
            "httpg://griftp.infn.it:8443/srm/managerv1",
            "1.1",
            "infn.it",
            "griftp.infn.it",
            8443));
        m_service->insert(Service(
            "CERN-SRM",
            "SRM",
            "httpg://griftp05.cern.ch:8443/srm/managerv1",
            "1.1",
            "cern.ch",
            "griftp05.cern.ch",
            8443));
        m_service->insert(Service(
            "CERN-DPM",
            "SRM",
            "httpg://griddpm.cern.ch:8443/srm/managerv1",
            "1.1",
            "cern.ch",
            "griddpm.cern.ch",
            8443));
        m_service->insert(Service(
            "CERN-GRIDFTP",
            "gsiftp",
            "gridftp://griftp05.cern.ch/srm/managerv1",
            "1.2.3",
            "cern.ch",
            "griftp05.cern.ch",
            0));
        m_service->insert(Service(
            "CERN-MYPROXY",
            "MyProxy",
            "myproxy://lxb1414.cern.ch",
            "1.14",
            "cern.ch",
            "lxb1414.cern.ch",
            0));
        m_service->insert(Service(
            "VO_1-Fireman",
            "org.glite.FiremanCatalog",
            "https://lxb2028.cern.ch:8443/VO_1/glite-data-catalog-service-fr/services/FiremanCatalog",
            "3.0.0",
            "cern.ch",
            "lxb1414.cern.ch",
            8443),SD_VO1); 
        m_service->insert(Service(
            "VO_2-Fireman",
            "org.glite.FiremanCatalog",
            "https://lxb2028.cern.ch:8443/VO_2/glite-data-catalog-service-fr/services/FiremanCatalog",
            "3.0.0",
            "cern.ch",
            "lxb1414.cern.ch",
            8443),SD_VO2);
        Service srm_vo_1(
            "CERN-SRM-VO1",
            "SRM",
            "httpg://griftp05.cern.ch:8443/srm/managerv1",
            "1.1",
            "cern.ch",
            "griftp05.cern.ch",
            8443);
        m_service->insert(srm_vo_1,SD_VO1); 
        Service srm_vo_2(
            "CERN-SRM-VO2",
            "SRM",
            "httpg://griftp05.cern.ch:8443/srm/managerv1",
            "1.1",
            "cern.ch",
            "griftp05.cern.ch",
            8443);
        m_service->insert(srm_vo_2,SD_VO2);
        Service myproxy_srv(
            "test-MYPROXY",
            "MyProxy",
            "myproxy://myproxy.test.com",
            "1.14",
            "test.com",
            "myproxy.test.com",
            0);
        Service fts_srv(
            "test-FileTransfer",
            "org.glite.FileTransfer",
            "https://filetransfer.test.com:8443/site-fts/glite-data-transfer-fts/services/FileTransfer",
            "3.1.0",
            "test.com",
            "filetransfer.test.com",
            8443);
        Service test_srm(
            "test-SRM",
            "SRM",
            "httpg://host.test.com:8443/srm/managerv1",
            "1.1",
            "test.com",
            "host.test.com",
            8443);
        m_service->insert(test_srm);
            
        // Insert Association
        m_service->insert(myproxy_srv);
        m_service->insert(fts_srv);
        m_service->associate(fts_srv, myproxy_srv);
        
        // Insert Properties
        m_service->insertProperty(srm_vo_1,"SEMountPoint","/castor/cern.ch/user/v/VO1",SD_VO1);
        m_service->insertProperty(srm_vo_2,"SEMountPoint","/castor/cern.ch/user/v/VO2",SD_VO2);
        m_service->insertProperty(test_srm,"VO_1:SEMountPoint","/castor/test.com/SE/VO1",SD_VO1);
        m_service->insertProperty(test_srm,"VO_2:SEMountPoint","/castor/test.com/SE/VO2",SD_VO2);

        // Remember some missing entries
        m_service->rememberMissing(SD_FOO_TYPE);
        m_service->rememberMissing(SD_TYPE_SRM,SD_FOO_HOST);
        m_service->rememberMissingProperty(SD_NAME_TEST_SRM,SD_FOO_PROPERTY);
        m_service->rememberMissingAssociation(SD_NAME_TEST_SRM,SD_FOO_TYPE);
        m_service->rememberMissingAssociation(SD_SERVICE_NAME_VO1,SD_FOO_TYPE,SD_FOO_TYPE);

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * tearDown
 *
 * tear-down test case
 */
void SDCacheTestCase::tearDown(){
    
    try{
        if(0 != m_service){
            m_service->clear();
            
            // reset the cache options
            m_service->ttl(3600);
            m_service->staleTime(900);
            m_service->obsoleteTime(10800);
            m_service->negativeObsoleteTime(300);
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
        
/*
 * Test testGetServiceByName Method
 */
void SDCacheTestCase::testGetByName(){
    
    try{
        // Get Service by Name
        const Service * s = m_service->getByName(SD_NAME_FIREMAN,"");
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (1)", s->name == SD_NAME_FIREMAN);
        
        s = m_service->getByName(SD_SERVICE_NAME_VO1,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (2)", s != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (2)", s->name == SD_SERVICE_NAME_VO1);

        s = m_service->getByName(SD_SERVICE_NAME_VO2,SD_VO2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (3)", s != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (3)", s->name == SD_SERVICE_NAME_VO2);

        s = m_service->getByName(SD_SERVICE_NAME_VO1,SD_VO2);
        CPPUNIT_ASSERT_MESSAGE("Pointer Returned (1)", s == 0);

        s = m_service->getByName(SD_SERVICE_NAME_VO2,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Pointer Returned (2)", s == 0);
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * Test testGetServiceByType Method
 */
void SDCacheTestCase::testGetByType(){
    
    try{
        // Get Service by Type
        SDCache::ServiceArray s = m_service->getByType(SD_TYPE_FIREMAN);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (1)", s.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (1)", s.at(0)->name == SD_NAME_FIREMAN);
        
        s.clear();
        s = m_service->getByType(SD_TYPE_FIREMAN,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (2)", s.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0(2)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0(2)", s.at(0)->name == SD_SERVICE_NAME_VO1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1(2)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1(2)", s.at(1)->name == SD_NAME_FIREMAN);
        
        // Get SRM Service
        s.clear();
        s = m_service->getByType(SD_TYPE_SRM);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (3)", s.size() == 4);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (3)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (3)", s.at(0)->name == SD_NAME_SRM_INFN);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (3)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (3)", s.at(1)->name == SD_NAME_SRM);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #2 (3)", s.at(2) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #2 (3)", s.at(2)->name == SD_NAME_DPM);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #3 (3)", s.at(3) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #3 (3)", s.at(3)->name == SD_NAME_TEST_SRM);

        s.clear();
        s = m_service->getByType(SD_TYPE_SRM,SD_VO2);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (4)", s.size() == 5);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (4)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (4)", s.at(0)->name == SD_SRM_NAME_VO2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (4)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (4)", s.at(1)->name == SD_NAME_SRM_INFN);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #2 (4)", s.at(2) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #2 (4)", s.at(2)->name == SD_NAME_SRM);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #3 (4)", s.at(3) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #3 (4)", s.at(3)->name == SD_NAME_DPM);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #4 (4)", s.at(4) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #4 (4)", s.at(4)->name == SD_NAME_TEST_SRM);
        
        // Get GridFTP Service
        s.clear();
        s = m_service->getByType(SD_TYPE_GRIDFTP);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (5)", s.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (5)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (5)", s.at(0)->name == SD_NAME_GRIDFTP);

        // Get MyProxy Service
        s.clear();
        s = m_service->getByType(SD_TYPE_MYPROXY);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (6)", s.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (6)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (6)", s.at(0)->name == SD_NAME_MYPROXY);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (6)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (6)", s.at(1)->name == SD_NAME_TEST_MYPROXY);

        s.clear();
        s = m_service->getByType(SD_TYPE_MYPROXY,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (7)", s.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (7)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (7)", s.at(0)->name == SD_NAME_MYPROXY);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (7)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (7)", s.at(1)->name == SD_NAME_TEST_MYPROXY);

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetServiceByType Method
 */
void SDCacheTestCase::testGetByHost(){
    
    try{
        SDCache::ServiceArray s = m_service->getByHost(SD_TYPE_GRIDFTP,SD_HOST_GRIDFTP);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (1)", s.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (1)", s.at(0)->name == SD_NAME_GRIDFTP);
        
        s.clear();
        s = m_service->getByHost(SD_TYPE_SRM,SD_HOST_SRM);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (2)", s.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (2)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (2)", s.at(0)->name == SD_NAME_SRM);
        
        s.clear();
        s = m_service->getByHost(SD_TYPE_SRM,SD_HOST_SRM,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (3)", s.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (3)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (3)", s.at(0)->name == SD_SRM_NAME_VO1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (3)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (3)", s.at(1)->name == SD_NAME_SRM);
        
        s.clear();
        s = m_service->getByHost(SD_TYPE_SRM,SD_HOST_SRM,SD_VO2);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (4)", s.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (4)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (4)", s.at(0)->name == SD_SRM_NAME_VO2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (4)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (4)", s.at(1)->name == SD_NAME_SRM);
        
        s.clear();
        std::vector<std::string> vo_list;
        vo_list.push_back(SD_VO2);
        vo_list.push_back(SD_VO1);
        s = m_service->getByHost(SD_TYPE_SRM,SD_HOST_SRM,vo_list);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (5)", s.size() == 3);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (5)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (5)", s.at(0)->name == SD_SRM_NAME_VO2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (5)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (5)", s.at(1)->name == SD_SRM_NAME_VO1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #2 (5)", s.at(2) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #2 (5)", s.at(2)->name == SD_NAME_SRM);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetServiceByName Method For a Given VO
 */
void SDCacheTestCase::testGetBySite(){
    
    try{
        SDCache::ServiceArray s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_SRM);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (1)", s.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (1)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (1)", s.at(0)->name == SD_NAME_SRM);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (1)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (1)", s.at(1)->name == SD_NAME_DPM);
        
        s.clear();
        s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_SRM,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (2)", s.size() == 3);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (2)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (2)", s.at(0)->name == SD_SRM_NAME_VO1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (2)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (2)", s.at(1)->name == SD_NAME_SRM);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #2 (2)", s.at(2) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #2 (2)", s.at(2)->name == SD_NAME_DPM);
        
        s.clear();
        s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_SRM,SD_VO2);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (3)", s.size() == 3);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (3)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (3)", s.at(0)->name == SD_SRM_NAME_VO2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (3)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (3)", s.at(1)->name == SD_NAME_SRM);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #2 (3)", s.at(2) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #2 (3)", s.at(2)->name == SD_NAME_DPM);
        
        s.clear();
        std::vector<std::string> vo_list;
        vo_list.push_back(SD_VO1);
        vo_list.push_back(SD_VO2);
        s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_SRM,vo_list);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (4)", s.size() == 4);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (4)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (4)", s.at(0)->name == SD_SRM_NAME_VO1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #1 (4)", s.at(1) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #1 (4)", s.at(1)->name == SD_SRM_NAME_VO2);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #2 (4)", s.at(2) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #2 (4)", s.at(2)->name == SD_NAME_SRM);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #3 (4)", s.at(3) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #3 (4)", s.at(3)->name == SD_NAME_DPM);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test the GetAssociated Method
 */
void SDCacheTestCase::testGetAssociated(){
    try{
        SDCache::ServiceArray s = m_service->getAssociated(SD_NAME_TEST_FTS,SD_TYPE_MYPROXY);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (1)", s.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (1)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (1)", s.at(0)->name == SD_NAME_TEST_MYPROXY);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
/*
 * Test the GetAssociatedOnSite  Method
 */
void SDCacheTestCase::testGetAssociatedOnSite(){
    try{
        SDCache::ServiceArray s = m_service->getAssociatedOnSite(SD_NAME_TEST_FTS,SD_TYPE_MYPROXY,SD_SITE_TEST);   
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (1)", s.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned #0 (1)", s.at(0) != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned #0 (1)", s.at(0)->name == SD_NAME_TEST_MYPROXY);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
/*
 * Test the GetProperty Method
 */
void SDCacheTestCase::testGetProperty(){
    try{
        std::pair<std::string,bool> p = m_service->getProperty(SD_SRM_NAME_VO1,SD_PROP_SRM_MNTPNT_KEY,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("No Property found  (1)", p.second == true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Property Returned (1)", p.first == "/castor/cern.ch/user/v/VO1");
        
        p = m_service->getProperty(SD_SRM_NAME_VO2,SD_PROP_SRM_MNTPNT_KEY,SD_VO2);
        CPPUNIT_ASSERT_MESSAGE("No Property found  (2)", p.second == true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Property Returned (2)", p.first == "/castor/cern.ch/user/v/VO2");

        p = m_service->getProperty(SD_NAME_TEST_SRM,(std::string)SD_VO1 + ":" + SD_PROP_SRM_MNTPNT_KEY,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("No Property found  (3)", p.second == true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Property Returned (3)", p.first == "/castor/test.com/SE/VO1");

        p = m_service->getProperty(SD_NAME_TEST_SRM,(std::string)SD_VO2 + ":" + SD_PROP_SRM_MNTPNT_KEY,SD_VO2);
        CPPUNIT_ASSERT_MESSAGE("No Property found  (4)", p.second == true);
        CPPUNIT_ASSERT_MESSAGE("Invalid Property Returned (4)", p.first == "/castor/test.com/SE/VO2");

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test the Missing Methods
 */
void SDCacheTestCase::testMissing(){
    try{
        bool is_missing = m_service->isServiceMissing(SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (1)", is_missing == true);
        is_missing = m_service->isServiceMissing(SD_TYPE_SRM);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (2)", is_missing == false);
        is_missing = m_service->isServiceMissing(SD_TYPE_SRM,SD_FOO_HOST);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (3)", is_missing == true);
        is_missing = m_service->isPropertyMissing(SD_NAME_TEST_SRM,SD_FOO_PROPERTY);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (4)", is_missing == true);
        is_missing = m_service->isPropertyMissing(SD_NAME_TEST_SRM,"A new property");
        CPPUNIT_ASSERT_MESSAGE("Invalid result (5)", is_missing == false);
        is_missing = m_service->isAssociationMissing(SD_NAME_TEST_SRM,SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (6)", is_missing == true);
        is_missing = m_service->isAssociationMissing(SD_NAME_TEST_SRM,"anoter type");
        CPPUNIT_ASSERT_MESSAGE("Invalid result (7)", is_missing == false);
        is_missing = m_service->isAssociationMissing(SD_SERVICE_NAME_VO1,SD_FOO_TYPE,SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (8)", is_missing == true);
        is_missing = m_service->isAssociationMissing(SD_SERVICE_NAME_VO1,SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (9)", is_missing == false);
        is_missing = m_service->isAssociationMissing(SD_SERVICE_NAME_VO1,SD_FOO_TYPE,"AnotherSite");
        CPPUNIT_ASSERT_MESSAGE("Invalid result (10)", is_missing == false);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test the Clear Method
 */
void SDCacheTestCase::testClear(){
    try{
        // Clear Service
        m_service->clear();
        
        // Get Service by Name
        const Service * srv = m_service->getByName(SD_NAME_FIREMAN,"");
        CPPUNIT_ASSERT_MESSAGE("Pointer Returned (1)", srv == 0);
        
        // Get Service by Type
        SDCache::ServiceArray s = m_service->getByType(SD_TYPE_FIREMAN);
        CPPUNIT_ASSERT_MESSAGE("Not Empty (2)", s.empty() == true);
        
        // Get Service by Host
        s.clear();
        s = m_service->getByHost(SD_TYPE_GRIDFTP,SD_HOST_GRIDFTP);
        CPPUNIT_ASSERT_MESSAGE("Not Empty (3)", s.empty() == true);
        
        // Get Service by Site
        s.clear();
        s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_SRM);
        CPPUNIT_ASSERT_MESSAGE("Not Empty (4)", s.empty() == true);
        
        // Get Assiciated Services 
        s.clear();
        s = m_service->getAssociated(SD_NAME_TEST_FTS,SD_TYPE_MYPROXY);
        CPPUNIT_ASSERT_MESSAGE("Not Empty (5)", s.empty() == true);
    
        // Get Service Property
        std::pair<std::string,bool> p = m_service->getProperty(SD_SRM_NAME_VO1,SD_PROP_SRM_MNTPNT_KEY,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Property found  (6)", p.second == false);
        
        // Check Missing Service
        bool is_missing = m_service->isServiceMissing(SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (7)", is_missing == false);
        is_missing = m_service->isPropertyMissing(SD_NAME_TEST_SRM,SD_FOO_PROPERTY);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (8)", is_missing == false);
        is_missing = m_service->isAssociationMissing(SD_NAME_TEST_SRM,SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("Invalid result (9)", is_missing == false);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test the Purge Method
 */
void SDCacheTestCase::testPurge(){
    try{
        // Check that entries are there

        // Get Service by Name
        const Service * srv = m_service->getByName(SD_NAME_FIREMAN,"");
        CPPUNIT_ASSERT_MESSAGE("Before Purge: No Pointer Returned (1)", srv != 0);
        
        // Get Service by Type
        SDCache::ServiceArray s = m_service->getByType(SD_TYPE_FIREMAN);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: Empty (2)", s.empty() == false);
        
        // Get Service by Host
        s.clear();
        s = m_service->getByHost(SD_TYPE_GRIDFTP,SD_HOST_GRIDFTP);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: Empty (3)", s.empty() == false);
        
        // Get Service by Site
        s.clear();
        s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_SRM);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: Empty (4)", s.empty() == false);
        
        // Get Assiciated Services 
        s.clear();
        s = m_service->getAssociated(SD_NAME_TEST_FTS,SD_TYPE_MYPROXY);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: Empty (5)", s.empty() == false);
    
        // Get Service Property
        std::pair<std::string,bool> p = m_service->getProperty(SD_SRM_NAME_VO1,SD_PROP_SRM_MNTPNT_KEY,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: No Property found  (6)", p.second == true);
        
        // Check Missing Service
        bool is_missing = m_service->isServiceMissing(SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (7)", is_missing == true);
        is_missing = m_service->isPropertyMissing(SD_NAME_TEST_SRM,SD_FOO_PROPERTY);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (8)", is_missing == true);
        is_missing = m_service->isAssociationMissing(SD_NAME_TEST_SRM,SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (9)", is_missing == true);
        
        // Sleep for a while
        sleep(4);
        
        // Purge Service
        m_service->purge();
 
        // Check that entries have been purged
        
        // Get Service by Name
        srv = m_service->getByName(SD_NAME_FIREMAN,"");
        CPPUNIT_ASSERT_MESSAGE("After Purge: Pointer Returned (1)", srv == 0);
        
        // Get Service by Type
        s.clear();
        s = m_service->getByType(SD_TYPE_FIREMAN);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Not Empty (2)", s.empty() == true);
        
        // Get Service by Host
        s.clear();
        s = m_service->getByHost(SD_TYPE_GRIDFTP,SD_HOST_GRIDFTP);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Not Empty (3)", s.empty() == true);
        
        // Get Service by Site
        s.clear();
        s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_SRM);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Not Empty (4)", s.empty() == true);
        
        // Get Assiciated Services 
        s.clear();
        s = m_service->getAssociated(SD_NAME_TEST_FTS,SD_TYPE_MYPROXY);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Not Empty (5)", s.empty() == true);
    
        // Get Service Property
        p = m_service->getProperty(SD_SRM_NAME_VO1,SD_PROP_SRM_MNTPNT_KEY,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Property found  (6)", p.second == false);
        
        // Check Missing Service
        is_missing = m_service->isServiceMissing(SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (7)", is_missing == false);
        is_missing = m_service->isPropertyMissing(SD_NAME_TEST_SRM,SD_FOO_PROPERTY);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (8)", is_missing == false);
        is_missing = m_service->isAssociationMissing(SD_NAME_TEST_SRM,SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (9)", is_missing == false);
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * Test the Purge Method
 */
void SDCacheTestCase::testPurge_2(){
    try{
        // Check that entries are there
        
        // Get Service by Name
        const Service * srv = m_service->getByName(SD_NAME_FIREMAN,"");
        CPPUNIT_ASSERT_MESSAGE("Before Purge: No Pointer Returned (1)", srv != 0);
        srv = m_service->getByName(SD_SERVICE_NAME_VO1,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: No Pointer Returned (2)", srv != 0);
        
        // Get Service by Type
        SDCache::ServiceArray s = m_service->getByType(SD_TYPE_FIREMAN);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: Empty (3)", s.empty() == false);
        
        // Get Service by Host
        s.clear();
        s = m_service->getByHost(SD_TYPE_GRIDFTP,SD_HOST_GRIDFTP);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: Empty (4)", s.empty() == false);
        
        // Get Service by Site
        s.clear();
        s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_SRM);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: Empty (5)", s.empty() == false);
        
        // Get Assiciated Services 
        s.clear();
        s = m_service->getAssociated(SD_NAME_TEST_FTS,SD_TYPE_MYPROXY);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: Empty (6)", s.empty() == false);
    
        // Get Service Property
        std::pair<std::string,bool> p = m_service->getProperty(SD_SRM_NAME_VO1,SD_PROP_SRM_MNTPNT_KEY,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Before Purge: No Property found  (7)", p.second == true);
        
        // Check Missing Service
        bool is_missing = m_service->isServiceMissing(SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (8)", is_missing == true);
        is_missing = m_service->isPropertyMissing(SD_NAME_TEST_SRM,SD_FOO_PROPERTY);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (9)", is_missing == true);
        is_missing = m_service->isAssociationMissing(SD_NAME_TEST_SRM,SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (10)", is_missing == true);

        // Sleep for a while
        sleep(2);
        
        // Refresh some of the entries
        srv = m_service->getByName(SD_SERVICE_NAME_VO1,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("Just Before Purge: No Pointer Returned (1)", srv != 0);

        p = m_service->getProperty(SD_NAME_TEST_SRM,(std::string)SD_VO2 + ":" + SD_PROP_SRM_MNTPNT_KEY,SD_VO2);
        CPPUNIT_ASSERT_MESSAGE("Just Before Purge: No Property found  (2)", p.second == true);

        // Sleep for a  while
        sleep(2);
        
        // Purge Service
        m_service->purge();
 
        // Check that entries have been purged
        
        // Get Service by Name
        srv = m_service->getByName(SD_NAME_FIREMAN,"");
        CPPUNIT_ASSERT_MESSAGE("After Purge: Pointer Returned (1)", srv == 0);
        srv = m_service->getByName(SD_SERVICE_NAME_VO1,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("After Purge: No Pointer Returned (2)", srv != 0);
        srv = m_service->getByName(SD_NAME_TEST_SRM,""); 
        CPPUNIT_ASSERT_MESSAGE("After Purge: No Pointer Returned (3)", srv != 0);
        
        // Get Service by Type
        s.clear();
        s = m_service->getByType(SD_TYPE_FIREMAN);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Not Empty (4-1)", s.empty() == true);
        s = m_service->getByType(SD_TYPE_FIREMAN,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Empty (4-2)", s.empty() == false);
        s = m_service->getByType(SD_TYPE_GRIDFTP);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Not Empty (5)", s.empty() == true);
        s = m_service->getByType(SD_TYPE_SRM);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Empty (6)", s.empty() == false);
        
        // Get Service by Host
        s.clear();
        s = m_service->getByHost(SD_TYPE_GRIDFTP,SD_HOST_GRIDFTP);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Not Empty (7)", s.empty() == true);
        
        // Get Service by Site
        s.clear();
        s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_TEST);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Empty (8)", s.empty() == false);
        s.clear();
        s = m_service->getBySite(SD_TYPE_SRM,SD_SITE_SRM);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Not Empty (9)", s.empty() == true);
        
        // Get Assiciated Services 
        s.clear();
        s = m_service->getAssociated(SD_NAME_TEST_FTS,SD_TYPE_MYPROXY);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Not Empty (10)", s.empty() == true);
    
        // Get Service Property
        p = m_service->getProperty(SD_SRM_NAME_VO1,SD_PROP_SRM_MNTPNT_KEY,SD_VO1);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Property found  (11)", p.second == false);
        p = m_service->getProperty(SD_NAME_TEST_SRM,(std::string)SD_VO2 + ":" + SD_PROP_SRM_MNTPNT_KEY,SD_VO2);
        CPPUNIT_ASSERT_MESSAGE("No Property found  (12)", p.second == true);

        // Check Missing Service
        is_missing = m_service->isServiceMissing(SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (13)", is_missing == false);
        is_missing = m_service->isPropertyMissing(SD_NAME_TEST_SRM,SD_FOO_PROPERTY);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (14)", is_missing == false);
        is_missing = m_service->isAssociationMissing(SD_NAME_TEST_SRM,SD_FOO_TYPE);
        CPPUNIT_ASSERT_MESSAGE("After Purge: Invalid result (15)", is_missing == false);
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
