/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: SDUtility Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SDUtilityTestCase.cpp,v 1.7 2007/11/05 11:30:09 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/sd/sd-utility.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;
using namespace glite::data::agents::sd;

using boost::scoped_ptr;
extern const char * g_cert_proxy_server;

/**
 * CPPUnit TestCase
 */
class SDUtilityTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( SDUtilityTestCase);
    CPPUNIT_TEST( testGetService );
    CPPUNIT_TEST( testGetServiceByType );
    CPPUNIT_TEST( testGetSiteName );
    CPPUNIT_TEST( testGetAssociatedService );
    CPPUNIT_TEST( testGetProperty );
    CPPUNIT_TEST( testGetSEPath );
    CPPUNIT_TEST( testGetServiceByType_SelectByVersion );
    CPPUNIT_TEST( testGetServiceByType_SelectByPortAndVersion );
    CPPUNIT_TEST( test_bug30268 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------
    /**
     * Test the GetService Method
     */
    void testGetService();
    /**
     * Test the GetServiceByType Method
     */
    void testGetServiceByType();
    void testGetServiceByType_SelectByVersion();
    void testGetServiceByType_SelectByPortAndVersion();
    /**
     * Test the GetSiteName Method
     */
    void testGetSiteName();
    /**
     * Test the GetAssociatedService Method
     */
    void testGetAssociatedService();
    /**
     * Test the GetProperty Method
     */
    void testGetProperty();
    /**
     * Test the GetSEPath Method
     */
    void testGetSEPath();
    
    // test methods for bugs
    void test_bug30268();
        
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SDUtilityTestCase, "SDUtilityTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void SDUtilityTestCase::setUp(){
}

#define FAIL_TEST(msg) {std::stringstream err; err << msg; CPPUNIT_FAIL(err.str().c_str());}
typedef scoped_ptr<Service> ServicePtr;

/**
 * test for bug #30268
 * If both version 1.1 and 2.2 of the same SRM endpoint are published, not always 
 * the correct one is picked (possibly the last one published?). 
 * e.g:
 * SRM for [srm://cmssrm.fnal.gov:8443/srm/managerv1?SFN=/bar] is [httpg://cmssrm.fnal.gov:8443/srm/managerv2]. Version is [2.2.0]
 */
void SDUtilityTestCase::test_bug30268(){
    try{
        
        std::string dteam = "dteam";
        std::string cms = "cms";
        
        std::string srm = "SRM";
        
        std::string path1 = "srm/managerv1";
        std::string path2 = "srm/managerv2";

        std::string v1 = "1.1";
        std::string v2 = "2.2";
                        
        std::string n1 = "httpg://cmssrm.fnal.gov:8443/srm/managerv1";
        std::string n2 = "httpg://cmssrm.fnal.gov:8443/srm/managerv2";
        std::string site = "USCMS-FNAL-WC1";
        std::string host = "cmssrm.fnal.gov";
        int port = 8443;
        
        SelectByPortAndVersion pred_1 (port,path1,v1);
        SelectByPortAndVersion pred_2 (port,path2,v2);
        
        // get service v1 for cms; this must succeed
        ServicePtr cms_s1 (get_service(host,srm,cms,pred_1));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned (cms_s1)", cms_s1.get() != 0);
        if (cms_s1->name != n1){
            FAIL_TEST("Invalid endpoint for cms_s1 : got '" << cms_s1->name << "' exptected '" << n1 << "'");
        }
        // get service v2 for cms; this must fail, and service v1 must *NOT* be returned
        ServicePtr cms_s2 (get_service(host,srm,cms,pred_2));
        CPPUNIT_ASSERT_MESSAGE("Invalid match returned (cms_s2)", cms_s2.get() == 0);
        
        // get service v2 for dteam; the cache must be updated and v2 returned
        ServicePtr dteam_s2 (get_service(host,srm,dteam,pred_2));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned (dteam_s2)", dteam_s2.get() != 0);
        if (dteam_s2->name != n2){
            FAIL_TEST("Invalid endpoint for dteam_s2 : got '" << dteam_s2->name << "' exptected '" << n2 << "'");
        }
        // get service v1 for dteam (must succeed)
        ServicePtr dteam_s1 (get_service(host,srm,dteam,pred_1));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned (dteam_s1)", dteam_s1.get() != 0);
        if (dteam_s1->name != n1){
            FAIL_TEST("Invalid endpoint for dteam_s1 : got '" << dteam_s1->name << "' exptected '" << n1 << "'");
        }

        // Test that if no service path is specified the match on version is applied
        SelectByPortAndVersion pred_v1 (port,"",v1);
        SelectByPortAndVersion pred_v2 (port,"",v2);
        
        ServicePtr s_cms_v1 (get_service(host,srm,cms,pred_v1));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned (s_cms_v1)", s_cms_v1.get() != 0);
        if (s_cms_v1->name != n1){
            FAIL_TEST("Invalid endpoint for s_v1 : got '" << s_cms_v1->name << "' exptected '" << n1 << "'");
        }
        // this must return v1 (v2 is not available to cms)
        ServicePtr s_cms_v2 (get_service(host,srm,cms,pred_v2));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned (s_cms_v2)", s_cms_v2.get() != 0);
        if (s_cms_v2->name != n1){
            FAIL_TEST("Invalid endpoint for s_cms_v2 : got '" << s_cms_v2->name << "' exptected '" << n1 << "'");
        }

        ServicePtr s_dteam_v1 (get_service(host,srm,dteam,pred_v1));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned (s_dteam_v1)", s_dteam_v1.get() != 0);
        if (s_dteam_v1->name != n1){
            FAIL_TEST("Invalid endpoint for s_v1 : got '" << s_dteam_v1->name << "' exptected '" << n1 << "'");
        }
        ServicePtr s_dteam_v2 (get_service(host,srm,dteam,pred_v2));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned (s_dteam_v2)", s_dteam_v2.get() != 0);
        if (s_dteam_v2->name != n2){
            FAIL_TEST("Invalid endpoint for s_dteam_v2 : got '" << s_dteam_v2->name << "' exptected '" << n2 << "'");
        }
        
                        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetSESiteName Method
 */
void SDUtilityTestCase::testGetService(){
    
    try{
        // Get Site Name
        scoped_ptr<Service> s(get_service("host.test.com","SRM",""));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site Name (1)", s->site  == "TEST.COM");
        
        // Get Site Name
        s.reset(get_service("test.com","SRM",""));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (2)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site Name (2)", s->site == "TEST.COM");
        // Get Site Name Uppercase
        s.reset(get_service("TEST.COM","SRM",""));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (3)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site Name (3)", s->site == "TEST.COM");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetServiceByType Method
 */
void SDUtilityTestCase::testGetServiceByType(){
    
  try{
        // Get Service
        scoped_ptr<Service> s(get_service_by_type("SRM",""));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type Name (1)", s->type  == "SRM");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetServiceByType_SelectByVersion Method
 */
void SDUtilityTestCase::testGetServiceByType_SelectByVersion(){
    
  try{
        // Get Service
        SelectByVersion pred_1("2.3.4");
        scoped_ptr<Service> s(get_service_by_type("SELECT_BY_VERSION","",pred_1));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (1)", s->name    == "SVC_SELECT_BY_VERSION_2");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (1)", s->version == "2.3.4");
        // Get Service
        SelectByVersion pred_2("2.0.0");
        s.reset(get_service_by_type("SELECT_BY_VERSION","",pred_2));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (2)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (2)", s->name    == "SVC_SELECT_BY_VERSION_3");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (2)", s->version == "2.5.6");
        // Get Service
        SelectByVersion pred_3("1.0.0");
        s.reset(get_service_by_type("SELECT_BY_VERSION","",pred_3));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (3)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (3)", s->name    == "SVC_SELECT_BY_VERSION_1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (3)", s->version == "1.2.3");
        // Get Service
        SelectByVersion pred_4("1.9.9");
        s.reset(get_service_by_type("SELECT_BY_VERSION","",pred_4));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (4)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (4)", s->name    == "SVC_SELECT_BY_VERSION_1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (4)", s->version == "1.2.3");
        // Get Service
        SelectByVersion pred_5("2.3.0");
        s.reset(get_service_by_type("SELECT_BY_VERSION","",pred_5));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (5)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (5)", s->name    == "SVC_SELECT_BY_VERSION_2");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (5)", s->version == "2.3.4");
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetServiceByType_SelectByPortAndVersion Method
 */
void SDUtilityTestCase::testGetServiceByType_SelectByPortAndVersion(){
    
    try{
        // Get Service
        SelectByPortAndVersion pred_1(123,"test_path","1.2.3");
        scoped_ptr<Service> s(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_1));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (1)", s->name    == "SVC_SELECT_BY_PORT_AND_VERSION_1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (1)", s->version == "1.2.3");
        
        // This one must fail (invalid port, invalid path -> filtered by the predicate)
        SelectByPortAndVersion pred_2(666,"invalid_path","1.2.3");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_2));
        CPPUNIT_ASSERT_MESSAGE("Invalid Pointer Returned (2)", s.get() == 0);
        
        // This one must succeed (0 = don't filter on port,empty string = don't filter on path)
        SelectByPortAndVersion pred_3(0,"","1.2.3");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_3));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (3)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (3)", s->name    == "SVC_SELECT_BY_PORT_AND_VERSION_1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (3)", s->version == "1.2.3");
        
        // This one must fail (invalid path -> filtered by the predicate)
        SelectByPortAndVersion pred_4(0,"invalid_path","2.3.4");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_4));
        CPPUNIT_ASSERT_MESSAGE("Invalid Pointer Returned (4)", s.get() == 0);
        
        // Get Service
        SelectByPortAndVersion pred_5(345,"test_path","3.4.5");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_5));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (5)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (5)", s->name    == "SVC_SELECT_BY_PORT_AND_VERSION_3");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (5)", s->version == "3.4.5");
        
        // Get Service
        SelectByPortAndVersion pred_6(345,"test_path2","3.4.5");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_6));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (6)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (6)", s->name    == "SVC_SELECT_BY_PORT_AND_VERSION_4");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (6)", s->version == "4.5.6");
        // Get Service
        SelectByPortAndVersion pred_7(345,"","3.4.5");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_7));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (7)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (7)", s->name    == "SVC_SELECT_BY_PORT_AND_VERSION_3");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (7)", s->version == "3.4.5");
        // Get Service
        SelectByPortAndVersion pred_8(345,"","4.5.6");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_8));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (8)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (8)", s->name    == "SVC_SELECT_BY_PORT_AND_VERSION_4");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (8)", s->version == "4.5.6");
        // Get Service
        SelectByPortAndVersion pred_9(456,"","5.0.0");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_9));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (9)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (9)", s->name    == "SVC_SELECT_BY_PORT_AND_VERSION_5");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (9)", s->version == "5.6.7");
        
        // Test the filtering on VO name
        SelectByPortAndVersion pred_10(456,"","5.0.0");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","dteam",pred_10));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (10)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (10)", s->name    == "SVC_SELECT_BY_PORT_AND_VERSION_5");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (10)", s->version == "5.6.7");
        
        SelectByPortAndVersion pred_11(456,"","5.0.0");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","cms",pred_10));
        CPPUNIT_ASSERT_MESSAGE("Invalid Pointer Returned (11)", s.get() == 0);
        
        // Test that if no version is specified, the version criteria is ignored
        SelectByPortAndVersion pred_12(456,"","");
        s.reset(get_service_by_type("SELECT_BY_PORT_AND_VERSION","",pred_11));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (12)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name (12)", s->name    == "SVC_SELECT_BY_PORT_AND_VERSION_5");
        CPPUNIT_ASSERT_MESSAGE("Invalid Vers (12)", s->version == "5.6.7");
                
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}



/*
 * testGetSESiteName Method
 */
void SDUtilityTestCase::testGetSiteName(){
    
    try{
        // Get Site Name
        std::string site_name = get_site_name("host.test.com","SRM","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Site Name (1)", site_name  == "TEST.COM");
        
        // Get Site Name
        site_name = get_site_name("test.com","SRM","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Site Name (2)", site_name == "TEST.COM");

        // Test With upper case
        site_name = get_site_name("TEST.COM","SRM","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Site Name (3)", site_name == "TEST.COM");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetAssociatedService Method
 */
void SDUtilityTestCase::testGetAssociatedService(){
    
    try{
        // Get Endpoint
        scoped_ptr<Service> s(get_associated_service("filetransfer.test.com",
            "org.glite.FileTransfer","MyProxy",""));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Endpoint (1)", s->endpoint == "myproxy://myproxy.test.com");
        
        // Get Endpoint
        s.reset(get_associated_service("test.com",
            "org.glite.FileTransfer","MyProxy",""));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (2)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Endpoint (2)", s->endpoint == "myproxy://myproxy.test.com");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetProperty Method
 */
void SDUtilityTestCase::testGetProperty(){
    
    try{
        // Get Endpoint
        scoped_ptr<Service> s(get_service("test.com","SRM",""));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name", s->name == "test-SRM");
        
        // Get Property
        std::string p = get_property(*s,"VO_1:SEMountPoint","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Property (1)", p == "/castor/test.com/SE/VO1");
        p = get_property(*s,"VO_2:SEMountPoint","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Property (2)", p == "/castor/test.com/SE/VO2");
        // Get Property again (test caching)
        p = get_property(*s,"VO_1:SEMountPoint","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Property (3)", p == "/castor/test.com/SE/VO1");
        p = get_property(*s,"VO_2:SEMountPoint","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Property (4)", p == "/castor/test.com/SE/VO2");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetSEPath Method
 */
void SDUtilityTestCase::testGetSEPath(){
    
    try{
        // Get Endpoint
        scoped_ptr<Service> s(get_service("test.com","SRM",""));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name", s->name == "test-SRM");
        
        // Get Property
        std::string p = get_se_path(*s,"VO_1","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Property (1)", p == "/castor/test.com/SE/VO1");
        p = get_se_path(*s,"VO_2","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Property (2)", p == "/castor/test.com/SE/VO2");
        p = get_se_path(*s,"VO_3","");
        CPPUNIT_ASSERT_MESSAGE("Invalid Property (3)", p == "/castor/test.com/SE/default");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
