/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: ServiceDiscovery Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ServiceDiscoveryTestCase.cpp,v 1.4 2007/01/24 12:08:25 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/sd/ServiceDiscovery.h"
#include "glite/data/agents/sd/ServiceDiscoveryException.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents::sd;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class ServiceDiscoveryTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( ServiceDiscoveryTestCase);
    CPPUNIT_TEST( testGetServiceByName );
    CPPUNIT_TEST( testGetServicesByType );
    CPPUNIT_TEST( testGetServicesByTypeForVO );
    CPPUNIT_TEST( testGetServicesOnHost );
    CPPUNIT_TEST( testGetServicesOnHostForVO );
    CPPUNIT_TEST( testGetServicesOnSite );
    CPPUNIT_TEST( testGetServicesOnSiteForVO );
    CPPUNIT_TEST( testGetSite );
    CPPUNIT_TEST( testGetRelatedService );
    CPPUNIT_TEST( testGetProperty );
    CPPUNIT_TEST( testGetAssociatedServices );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the GetServiceByName Method
     */
    void testGetServiceByName();
    /**
     * Test the GetServicesByType Method
     */
    void testGetServicesByType();
    /**
     * Test the GetServiceByName Method For a Given VO 
     */
    void testGetServicesByTypeForVO();
    /**
     * Test the GetServicesOnHost Method
     */
    void testGetServicesOnHost();
    /**
     * Test the GetServicesOnHost Method for a given VO
     */
    void testGetServicesOnHostForVO();
    /**
     * Test the GetServicesOnSite Method
     */
    void testGetServicesOnSite();
    /**
     * Test the GetServicesOnHost Method for a given VO
     */
    void testGetServicesOnSiteForVO();
    /**
     * Test the GetServiceByName Method For a Given VO
     */
    void testGetServiceByTypeForVO();
    /**
     * Test the GetSite Method
     */
    void testGetSite();
    /**
     * Test the GetSite Method
     */
    void testGetRelatedService();
    /**
     * Test the GetProperty Method
     */
    void testGetProperty();
    /**
     * Test the GetAssociatedService Method
     */
    void testGetAssociatedServices();    
private:
    ServiceDiscovery m_service;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ServiceDiscoveryTestCase, "ServiceDiscoveryTestCase" );

namespace {
const char * const SD_TYPE_FIREMAN        = "org.glite.FiremanCatalog";
const char * const SD_SITE_FIREMAN        = "cern.ch";
const char * const SD_SITE_FIREMAN_UPR    = "CERN.CH";
const char * const SD_NAME_FIREMAN        = "CERN-Fireman";
const char * const SD_TYPE_SRM            = "SRM";
const char * const SD_TYPE_GRIDFTP        = "gsiftp";
const char * const SD_TYPE_MYPROXY        = "MyProxy";
const char * const SD_SITE_SRM            = "cern.ch";
const char * const SD_NAME_SRM            = "CERN-SRM";
const char * const SD_NAME_GRIDFTP        = "CERN-GRIDFTP";
const char * const SD_HOST_SRM            = "griftp05.cern.ch";
const char * const SD_HOST_GRIDFTP        = "griftp05.cern.ch";
const char * const SD_PROP_SRM_MNTPNT_KEY = "SEMountPoint";
const char * const SD_PROP_SRM_MNTPNT     = "/castor/cern.ch/user/g/gproduct/EGEETEST/SE/";
const char * const SD_VO1                 = "VO_1";
const char * const SD_VO2                 = "VO_2";
const char * const SD_SERVICE_NAME_VO1    = "VO_1-Fireman";
const char * const SD_SERVICE_NAME_VO2    = "VO_2-Fireman";
const char * const SD_SRM_NAME_VO1        = "CERN-SRM-VO1";
const char * const SD_SRM_NAME_VO2        = "CERN-SRM-VO2";
}

/*
 * setUp
 *
 * set-up test case
 */
void ServiceDiscoveryTestCase::setUp(){
}

/*
 * Test testGetServiceByName Method
 */
void ServiceDiscoveryTestCase::testGetServiceByName(){
    
    try{
        // Get Service by Name
        scoped_ptr<Service> s(m_service.getServiceByName(SD_NAME_FIREMAN));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned", s->name == SD_NAME_FIREMAN);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * Test testGetServiceByType Method
 */
void ServiceDiscoveryTestCase::testGetServicesByType(){
    
    try{
        // Get Service by Type
        ServiceDiscovery::ServiceArray s;
        m_service.getServicesByType(SD_TYPE_FIREMAN,"",s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (FIREMAN)", s.at(0)->type == SD_TYPE_FIREMAN);
        
        // Get SRM Service
        s.clear();
        m_service.getServicesByType(SD_TYPE_SRM,"",s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (SRM)", s.at(0)->type == SD_TYPE_SRM);
        
        // Get GridFTP Service
        s.clear();
        m_service.getServicesByType(SD_TYPE_GRIDFTP,"",s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type Returned (GridFTP)",s.at(0)->type == SD_TYPE_GRIDFTP);

        // Get MyProxy Service
        s.clear();
        m_service.getServicesByType(SD_TYPE_MYPROXY,"",s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type Returned (GridFTP)",s.at(0)->type == SD_TYPE_MYPROXY);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetServiceByName Method For a Given VO
 */
void ServiceDiscoveryTestCase::testGetServicesByTypeForVO(){
    
    try{
        // Get Service by Name
        ServiceDiscovery::ServiceArray s;
        m_service.getServicesByType(SD_TYPE_FIREMAN,SD_VO1,s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (VO1)", s.at(0)->name == SD_SERVICE_NAME_VO1);

        s.clear();
        m_service.getServicesByType(SD_TYPE_FIREMAN,SD_VO2,s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (VO2)", s.at(0)->name == SD_SERVICE_NAME_VO2);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetServiceByType Method
 */
void ServiceDiscoveryTestCase::testGetServicesOnHost(){
    
    try{
        // Get Service by Type
        ServiceDiscovery::ServiceArray s;
        m_service.getServicesOnHost(SD_TYPE_GRIDFTP,SD_HOST_GRIDFTP,0,"",s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned", s.at(0)->name == SD_NAME_GRIDFTP);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetServiceByName Method For a Given VO
 */
void ServiceDiscoveryTestCase::testGetServicesOnHostForVO(){
    
    try{
        // Get Service by Name
        ServiceDiscovery::ServiceArray s;
        m_service.getServicesOnHost(SD_TYPE_SRM,SD_HOST_SRM,0,SD_VO1,s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (VO1)", s.at(0)->name == SD_SRM_NAME_VO1);

        s.clear();
        m_service.getServicesOnHost(SD_TYPE_SRM,SD_HOST_SRM,0,SD_VO2,s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (VO2)", s.at(0)->name == SD_SRM_NAME_VO2);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetServiceByType Method
 */
void ServiceDiscoveryTestCase::testGetServicesOnSite(){
    
    try{
        // Get Service by Type
        ServiceDiscovery::ServiceArray s;
        m_service.getServicesOnSite(SD_TYPE_GRIDFTP,SD_SITE_SRM,"",s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned", s.at(0)->name == SD_NAME_GRIDFTP);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test testGetServiceByName Method For a Given VO
 */
void ServiceDiscoveryTestCase::testGetServicesOnSiteForVO(){
    
    try{
        // Get Service by Name
        ServiceDiscovery::ServiceArray s;
        m_service.getServicesOnSite(SD_TYPE_SRM,SD_SITE_SRM,SD_VO1,s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (VO1)", s.at(0)->name == SD_SRM_NAME_VO1);

        s.clear();
        m_service.getServicesOnSite(SD_TYPE_SRM,SD_SITE_SRM,SD_VO2,s);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size", s.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (VO2)", s.at(0)->name == SD_SRM_NAME_VO2);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test GetSite Method
 */
void ServiceDiscoveryTestCase::testGetSite(){
    
    try{
        // Get Service by Name
        scoped_ptr<Service> s(m_service.getServiceByName(SD_NAME_FIREMAN));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site Returned", s->site == SD_SITE_FIREMAN_UPR);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * Test GetSite Method
 */
void ServiceDiscoveryTestCase::testGetRelatedService(){
    
    try{
        // Get Service by Name
        scoped_ptr<Service> s(m_service.getServiceByName(SD_NAME_FIREMAN));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Site Returned", s->site == SD_SITE_FIREMAN_UPR);
        
        // Get Service by Type & Site
        ServiceDiscovery::ServiceArray sa;
        m_service.getServicesOnSite(SD_TYPE_SRM,s->site,"",sa);
        CPPUNIT_ASSERT_MESSAGE("Invali Array Size", sa.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned", sa.at(0)->name == SD_NAME_SRM);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test GetProperty Method
 */
void ServiceDiscoveryTestCase::testGetProperty(){
    
    try{
        // Get Service by Name
        scoped_ptr<Service> s(m_service.getServiceByName(SD_NAME_SRM));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned", s->name == SD_NAME_SRM);
        
        // Get Service Property
        std::string data = m_service.getProperty(*s,SD_PROP_SRM_MNTPNT_KEY);
        CPPUNIT_ASSERT_MESSAGE("Invalid Proeprty Returned", data == SD_PROP_SRM_MNTPNT);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test GetAssociatedServices Method
 */
void ServiceDiscoveryTestCase::testGetAssociatedServices(){
    
    try{
        // Get Service by Name
        scoped_ptr<Service> s(m_service.getServiceByName(SD_NAME_GRIDFTP));
        CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (GRIDFTP)", s.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (GRIDFTP)", s->name == SD_NAME_GRIDFTP);
        
        // Get Associated Service
        ServiceDiscovery::ServiceArray sa;
        m_service.getAssociatedServices(*s,SD_TYPE_SRM,"",sa);
        CPPUNIT_ASSERT_MESSAGE("Invalid Array Size    (SRM)", sa.size() >= 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Name Returned (SRM)", sa.at(0)->name == SD_NAME_SRM);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type Returned (SRM)", sa.at(0)->type == SD_TYPE_SRM);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
